﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Script.Serialization;

namespace Social.Lib.Activity
{
    public class Attachment
    {
        public class Image
        {
            public string Title { get; set; }
            public string Description { get; set; }
            public string UrlAlbum { get; set; }
            public string ImagePath { get; set; }
            public string ImageUrl { get; set; }

            public Image(){}

            public Image(string title, string description, string urlalbum, string imagepath, string url)
            {
                this.Title = title;
                this.Description = description;
                this.UrlAlbum = urlalbum;
                this.ImagePath = imagepath;
                this.ImageUrl = url;
            }

            public Image(string json)
            {
                JavaScriptSerializer serializer = new JavaScriptSerializer();
                Image image = serializer.Deserialize<Image>(json);
                if (image == null) return;
                this.Title = image.Title;
                this.Description = image.Description;
                this.ImageUrl = image.ImageUrl;
                this.UrlAlbum = image.UrlAlbum;
                this.ImagePath = image.ImagePath;
                if (!string.IsNullOrEmpty(this.ImagePath) && !this.ImagePath.StartsWith("http://") && !this.ImagePath.StartsWith("https://"))
                {
                    this.ImagePath = ConfigManager.FeedCacheServer + this.ImagePath;
                }
            }
        }

        public class Album
        {
            public string Title { get; set; }
            public string Description { get; set; }
            public string Url { get; set; }
            public int TotalImage { get; set; }
            public string[] ImagePath { get; set; }
            public string[] ImageUrl { get; set; }

            public Album(){}
            public Album(string json)
            {
                JavaScriptSerializer serializer = new JavaScriptSerializer();
                Album image = serializer.Deserialize<Album>(json);
                if (image == null) return;
                this.Title = image.Title;
                this.Description = image.Description;
                this.Url = image.Url;
                this.TotalImage = image.TotalImage;
                this.ImageUrl = image.ImageUrl;
                this.ImagePath = image.ImagePath;
                int i = 0;
                foreach (string img in this.ImagePath)
                {
                    if (!string.IsNullOrEmpty(img) && !img.StartsWith("http://") && !img.StartsWith("https://"))
                    {
                        this.ImagePath[i] = ConfigManager.FeedCacheServer + img;
                    }
                    i++;
                }
            }
        }

        public class Video
        {
            public string Title { get; set; }
            public string Description { get; set; }
            public string Url { get; set; }
            public string EmbedUrl { get; set; }
            public string Image { get; set; }
            public string Type { get; set; }
            public int Width { get; set; }
            public int Height { get; set; }

            public string SiteUrl
            {
                get
                {
                    try
                    {
                        Uri linkUrl = new Uri(this.Url);
                        return linkUrl.Host.ToString();
                    }
                    catch (Exception ex)
                    {
                        return this.Url;
                    }
                }
            }

            public Video() { }
            public Video(string json)
            {
                JavaScriptSerializer serializer = new JavaScriptSerializer();
                Video link = serializer.Deserialize<Video>(json);
                if (link == null) return;
                this.Title = link.Title;
                this.Description = link.Description;
                this.Url = link.Url;
                this.Image = link.Image;
                if (!string.IsNullOrEmpty(this.Image) && !this.Image.StartsWith("http://") && !this.Image.StartsWith("https://"))
                {
                    this.Image = ConfigManager.FeedCacheServer + this.Image;
                }
                this.EmbedUrl = link.EmbedUrl;
                this.Type = link.Type;
                this.Width = link.Width;
                this.Height = link.Height;
            }
        }

        public class Link
        {
            public string Title { get; set; }
            public string Description { get; set; }
            public string Url { get; set; }
            public string SiteUrl
            {
                get
                {
                    try
                    {
                        Uri linkUrl = new Uri(this.Url);
                        return linkUrl.Host.ToString();
                    }
                    catch (Exception ex)
                    {
                        return this.Url;
                    }
                }
            }

            public string Image { get; set; }
            public string MimeType { get; set; }

            public Link() { }
            public Link(string json)
            {
                JavaScriptSerializer serializer = new JavaScriptSerializer();
                Link link = serializer.Deserialize<Link>(json);
                if (link == null) return;
                this.Title = link.Title;
                this.Description = link.Description;
                this.Url = link.Url;
                this.Image = link.Image;
                if (!string.IsNullOrEmpty(this.Image) && !this.Image.StartsWith("http://") && !this.Image.StartsWith("https://"))
                {
                    this.Image = ConfigManager.FeedCacheServer + this.Image;
                }
            }
        }

        public class Audio
        {
            public string Image { get; set; }
            public string Url { get; set; }
            public string MimeType { get; set; }
            public string Title { get; set; }
            public string Description { get; set; }
            public string Artist { get; set; }
            public string Album { get; set; }

            public Audio() { }
            public Audio(string json)
            {
                
            }
        }

        public class Status
        {
            
        }
    
        public class Note
        {
            public string Title { get; set; }
            public string Description { get; set; }
            public string Url { get; set; }
            public string Image { get; set; }

            public Note(){}
            
            public Note(string title, string description, string url, string image)
            {
                this.Title = title;
                this.Image = image;
                this.Description = description;
                this.Url = url;
            }

            public Note(string json)
            {
                JavaScriptSerializer serializer = new JavaScriptSerializer();
                Note image = serializer.Deserialize<Note>(json);
                if (image == null) return;
                this.Title = image.Title;
                this.Description = image.Description;
                this.Image = image.Image;
                if (!string.IsNullOrEmpty(this.Image) && !this.Image.StartsWith("http://") && !this.Image.StartsWith("https://"))
                {
                    this.Image = ConfigManager.FeedCacheServer + this.Image;
                }
                this.Url = image.Url;
            }
        }
    }
}
