﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;
using Social.Lib.Photo;

namespace Social.Lib.Activity
{
    public class Feed
    {
        public static string Message(string message)
        {
            string output = HttpUtility.HtmlEncode(message);
            string tagPattern = @"<small id=""([^""]+)""([^>]+)>([^<]+)<\/?small>";
            Regex tagRegex = new Regex(tagPattern,RegexOptions.IgnoreCase);
            foreach (Match match in tagRegex.Matches(message))
            {
                string userid = match.Groups[1].Value;
                string username = match.Groups[3].Value;
                output = output.Replace(HttpUtility.HtmlEncode(match.Value), "@{'" + userid + "','"+username+"'}");
            }
            return output;
        }

        public static List<Guid> ListUserTagged(string message)
        {
            List<Guid> listUser = new List<Guid>();

            string tagPattern = @"<small id=""([\w-]+)""([^>]+)>([^<]+)<\/?small>";
            Regex tagRegex = new Regex(tagPattern, RegexOptions.IgnoreCase);
            foreach (Match match in tagRegex.Matches(message))
            {
                string id = match.Groups[1].Value;
                Guid userid;
                if(Guid.TryParse(id,out userid))
                {
                    listUser.Add(userid);
                }
            }
            return listUser;
        }

        public static string SaveImageUrl(string url, Guid feedid, string preImageName = "")
        {
            if (string.IsNullOrEmpty(url)) return string.Empty;
            string fileName = string.Empty;
            try
            {
                WebRequest request = WebRequest.Create(url);
                if (request != null)
                {
                    using (WebResponse response = request.GetResponse())
                    {
                        if (response != null)
                        {
                            using (Stream responseStream = response.GetResponseStream())
                            {
                                if (responseStream != null)
                                {
                                    int read;
                                    var ms = new MemoryStream();
                                    var buf = new byte[1024];
                                    while ((read = responseStream.Read(buf, 0, buf.Length)) > 0)
                                    {
                                        ms.Write(buf, 0, read);
                                    }
                                    ms.Seek(0, SeekOrigin.Begin);
                                    if (ms.Length > 0)
                                    {

                                        var imagefullsize = System.Drawing.Image.FromStream(ms);
                                        var oldW = imagefullsize.Width;
                                        var oldH = imagefullsize.Height;
                                        double dblCoef = 100 / (double)oldW;
                                        int thumMainW = Convert.ToInt32(dblCoef * oldW);
                                        int thumMainH = Convert.ToInt32(dblCoef * oldH);
                                        var quality = ResizeImage.Quality(response.ContentLength);
                                        var data = ms.GetBuffer();
                                        string pathCreate = DateTime.Now.ToString("yyyyMM") + "/" + DateTime.Now.ToString("ddHH") + "/";
                                        fileName = string.Format("{0}{1}{2}.jpg", pathCreate, feedid, preImageName);

                                        var upload = new MultiUpload(data, string.Format("{0}{1}",ConfigManager.GetKey("FeedCacheResource"), fileName), thumMainW, thumMainH, quality[2], false);
                                        upload.Start();
                                        upload.Join();
                                    }
                                    else
                                    {
                                        return string.Empty;
                                    }
                                }
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                fileName = string.Empty;
            }

            return fileName;
        }

        public static string SaveImage(string imagepath, Guid feedid, string preImageName = "")
        {
            if (string.IsNullOrEmpty(imagepath)) return string.Empty;
            string fileName = string.Empty;
            try
            {
                fileName = string.Format("{0}{1}.jpg",  feedid, preImageName);
                File.Copy(System.Web.HttpContext.Current.Server.MapPath(imagepath), string.Format("{0}{1}", ConfigManager.GetKey("FeedCacheResource"), fileName), true);
            }
            catch (Exception ex)
            {
                fileName = string.Empty;
            }

            return fileName;
        }
        public static bool DeleleImage(string path)
        {
            if (string.IsNullOrEmpty(path)) return true;
            path = System.Web.HttpContext.Current.Server.MapPath(path);
            if (System.IO.File.Exists(path)) System.IO.File.Delete(path);
            return true;
        }


        public static string ReplaceUrl(string message)
        {
            string output =  HttpUtility.HtmlDecode(message);
            //replace url
            //begin by http(s) ftp ...
            Regex regx = new Regex(@"\b((www\.|(http|https|ftp|news|file)+\:\/\/)[_.a-z0-9-]+\.[a-z0-9\/_:@=.+?,#%&~-]*[^.|\'|\# |!|\(|?|,| |>|<|;|\)])\b", RegexOptions.IgnoreCase);
            MatchCollection matches = regx.Matches(output);
            foreach (Match match in matches)
            {
                if (match.Value.StartsWith("http"))
                {
                    output = output.Replace(match.Value, "<a href='" + match.Value + "' target='_blank'>" + match.Value + "</a>");
                }
                if (match.Value.StartsWith("www."))
                {
                    output = output.Replace(match.Value, "<a href='http://" + match.Value + "' target='_blank'>" + match.Value + "</a>");
                }
            }
            return output;
        }
    }
}
