﻿using System;
using System.Configuration;

namespace Social.Lib
{
    public class ConfigManager
    {
        private const double CacheTimeMinutes = 5; 
        
        public static string GetKey(string key)
        {
            var obj = Memcached.MainCache.Get(key);
            if (obj != null)
            {
                return obj.ToString();
            }
            else
            {
                obj = ConfigurationManager.AppSettings[key];
                Memcached.MainCache.Store(Enyim.Caching.Memcached.StoreMode.Set, key, obj.ToString(),
                                          TimeSpan.FromMinutes(CacheTimeMinutes));
                return obj.ToString();
            }
        }

        public static int MaxWallItemLoad
        {
            get
            {
                int max;
                if (int.TryParse(GetKey("MaxWallItemLoad"), out max))
                {
                    return max;
                }
                return 20;
            }
        }
        public static int MaxFeedItemLoad
        {
            get
            {
                int max;
                if (int.TryParse(GetKey("MaxFeedItemLoad"), out max))
                {
                    return max;
                }
                return 30;
            }
        }
        public static string MainDomain
        {
            get { return GetKey("MainDomain"); }
        }
        public static string AvatarServer
        {
            get { return GetKey("AvatarServer"); }
        }
        public static string StaticServer
        {
            get { return GetKey("StaticServer"); }
        }
        public static string FeedCacheServer
        {
            get { return GetKey("FeedCacheServer"); }
        }
        
        //photo
        public static string UrlDataPhoto
        {
            get { return GetKey(Photo.Constants.ConfigKey.URL_DATA_PHOTO); }
        }
        public static string PathResourcePhoto
        {
            get { return GetKey(Photo.Constants.ConfigKey.PATH_RESOURCE_PHOTO); }
        }
        public static string UrlDataMusic
        {
            get { return GetKey(Music.Constants.ConfigKey.URL_DATA_MUSIC); }
        }
        public static string PathResourceMusic
        {
            get { return GetKey(Music.Constants.ConfigKey.PATH_RESOURCE_MUSIC); }
        }
        //music
        public static string UrlPlayerMusic
        {
            get { return GetKey(Music.Constants.ConfigKey.URL_PLAYER_MUSIC); }
        }
        public static string UrlXmlAlbumMusic
        {
            get { return GetKey(Music.Constants.ConfigKey.URL_XML_ALBUM_MUSIC); }
        }
        public static string UrlXmlSongMusic
        {
            get { return GetKey(Music.Constants.ConfigKey.URL_XML_SONG_MUSIC); }
        }
    }
}