﻿using System.Drawing.Imaging;
using System.IO;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace Social.Lib.Photo  
{
    public class ResizeImage
    {
        public static void ImageResize(byte[] data, string desPath, int thumbWidth, int thumbHeight, long quality)
        {
            Stream stream = new MemoryStream(data);
            var sourceBitmap = new Bitmap(stream);
            var thumbBitmap = new Bitmap(thumbWidth, thumbHeight);
            var g = Graphics.FromImage(thumbBitmap);
            try
            {
                g.InterpolationMode = InterpolationMode.HighQualityBicubic;
                g.FillRectangle(Brushes.White, 0, 0, thumbWidth, thumbHeight);
                g.DrawImage(sourceBitmap, 0, 0, thumbWidth, thumbHeight);

                var info = ImageCodecInfo.GetImageEncoders();
                var param = new EncoderParameters(1);
                param.Param[0] = new EncoderParameter(Encoder.Quality, quality);

                //var ms = new MemoryStream();
                //thumbBitmap.Save(ms, info[1], param);
                //Stream xstream = new MemoryStream(ms.GetBuffer());
                //StoreUtility.Instance.Upload(desPath, xstream);

                string folderToUp = desPath.Substring(0, desPath.LastIndexOf("/") + 1);
                try
                {
                    if (Directory.Exists(folderToUp) == false)
                    {
                        Directory.CreateDirectory(folderToUp);
                    }
                }
                catch
                {
                }
                thumbBitmap.Save(desPath, info[1], param);
            }
            finally
            {
                sourceBitmap.Dispose();
                thumbBitmap.Dispose();
                g.Dispose();
            }

        }

        public static void ImageCrop(byte[] data, string desPath, int width, int height, long quality)
        {
            Stream stream = new MemoryStream(data);
            Image imgPhoto = new Bitmap(stream);
            var sourceWidth = imgPhoto.Width;
            var sourceHeight = imgPhoto.Height;
            var destX = 0;
            var destY = 0;

            float nPercent;
            float nPercentW = 0;
            float nPercentH = 0;

            nPercentW = (width / (float)sourceWidth);
            nPercentH = (height / (float)sourceHeight);

            if (nPercentH < nPercentW)
            {
                nPercent = nPercentW;
                destY = (int)((height - (sourceHeight * nPercent)) / 2);
            }
            else
            {
                nPercent = nPercentH;
                destX = (int)((width - (sourceWidth * nPercent)) / 2);
            }

            var destWidth = (int)(sourceWidth * nPercent);
            var destHeight = (int)(sourceHeight * nPercent);

            if (nPercentH < nPercentW)
            {
                destWidth += 1;
            }
            else
            {
                destHeight += 1;
            }

            var bmPhoto = new Bitmap(width, height);
            bmPhoto.SetResolution(imgPhoto.HorizontalResolution, imgPhoto.VerticalResolution);

            try
            {
                Graphics grPhoto = Graphics.FromImage(bmPhoto);
                grPhoto.InterpolationMode = InterpolationMode.HighQualityBicubic;
                grPhoto.DrawImage(imgPhoto,
                                  new Rectangle(destX, destY, destWidth, destHeight),
                                  new Rectangle(0, 0, sourceWidth, sourceHeight),
                                  GraphicsUnit.Pixel);

                var info = ImageCodecInfo.GetImageEncoders();
                var param = new EncoderParameters(1);
                param.Param[0] = new EncoderParameter(Encoder.Quality, 100L);
                //var ms = new MemoryStream();
                //bmPhoto.Save(ms, info[1], param);
                //Stream xstream = new MemoryStream(ms.GetBuffer());
                //StoreUtility.Instance.Upload(desPath, xstream);

                //Check exist folder
                string folderToUp = desPath.Substring(0, desPath.LastIndexOf("/") + 1);
                try
                {
                    if (Directory.Exists(folderToUp) == false)
                    {
                        Directory.CreateDirectory(folderToUp);
                    }
                }
                catch
                {
                }

                bmPhoto.Save(desPath);
            }
            finally
            {
                bmPhoto.Dispose();
                stream.Close();
                stream.Dispose();
                imgPhoto.Dispose();
            }
        }

        public static void Process(byte[] data, string desPath, int width, int height, long quality, bool crop)
        {
            if (crop)
            {
                ImageCrop(data, desPath, width, height, quality);
            }
            else
            {
                ImageResize(data, desPath, width, height, quality);
            }
        }

        public static long[] Quality(long contenLength)
        {
            var quality = new long[6];
            quality[0] = 100L;

            if (contenLength >= 3072000)
            {
                quality[1] = 85L;
                quality[2] = 85L;
                quality[3] = 85L;
                quality[4] = 85L;
                quality[5] = 85L;
            }
            else
            {
                if ((contenLength >= 2048000) && contenLength < 3072000)
                {
                    quality[1] = 87L;
                    quality[2] = 85L;
                    quality[3] = 85L;
                    quality[4] = 85L;
                    quality[5] = 85L;
                }
                else if ((contenLength > 1024000) && (contenLength < 2048000))
                {
                    quality[1] = 95L;
                    quality[2] = 90L;
                    quality[3] = 90L;
                    quality[4] = 90L;
                    quality[5] = 90L;
                }
                else
                {
                    quality[1] = 95L;
                    quality[2] = 95L;
                    quality[3] = 95L;
                    quality[4] = 95L;
                    quality[5] = 95L;
                }
            }
            return quality;
        }
    }
}
