﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Configuration;

namespace Social.Lib
{
    public class SessionManager
    {
        public static object Get(string key)
        {
            return HttpContext.Current.Session[key];
        }
        public static void Set(string key, object obj)
        {
            HttpContext.Current.Session[key] = obj;
        }
        public static bool IsLoggedIn()
        {
            return HttpContext.Current.Session["UserID"] != null;
        }
        public static bool CheckLogin()
        {
            if (HttpContext.Current.Session["UserID"] == null)
            {
                HttpCookie cookie = new HttpCookie("REDIRECT_AFTER_LOGIN", HttpContext.Current.Request.Url.ToString());
                cookie.Expires = DateTime.Now.AddMinutes(1);
                HttpContext.Current.Response.Cookies.Add(cookie);
                return false;
            }
            return true;
        }
        public static Guid CurrentUserID()
        {
            if (IsLoggedIn())
            {
                return new Guid(Get("UserID").ToString());
            }
            else
            {
                return Guid.Empty;
            }
        }
        public static string CurrentUserUserName()
        {
            if (IsLoggedIn())
            {
                return Get("UserName").ToString();
            }
            else
            {
                return string.Empty;
            }
        }

        public static string CurrentUserFullName()
        {
            if (IsLoggedIn())
            {
                return Get("FullName").ToString();
            }
            else
            {
                return string.Empty;
            }
        }

        public static string CurrentSmallAvatar()
        {
            if (IsLoggedIn())
            {
                if(!Get("SmallAvatarUrl").ToString().Contains("http://"))
                    return string.Concat(ConfigurationManager.AppSettings["AvatarServerUrl"], Get("SmallAvatarUrl").ToString());
                return Get("SmallAvatarUrl").ToString();
            }
            else
            {
                return ConfigurationManager.AppSettings["DefaultAvatar"];
            }
        }

        public static string CurrentMediumAvatar()
        {
            if (IsLoggedIn())
            {
                if (!Get("MediumAvatarUrl").ToString().Contains("http://"))
                    return string.Concat(ConfigurationManager.AppSettings["AvatarServerUrl"], Get("MediumAvatarUrl").ToString());

                return Get("MediumAvatarUrl").ToString();
            }

            return ConfigurationManager.AppSettings["DefaultAvatar"];
        }

        public static string CurrentProfileLink()
        {
            if (IsLoggedIn())
            {
                return Get("ProfileLink").ToString();
            }
            return "/";
        }

        public static void UpdateAvatar(string smallAvatar, string mediumAvatar)
        {
            SessionManager.Set("MediumAvatarUrl", mediumAvatar);
            SessionManager.Set("SmallAvatarUrl", smallAvatar);
        }

        public static void UpdateFullName(string fullname)
        {
            SessionManager.Set("FullName", fullname);
        }

        public static bool Abadon()
        {
            HttpContext.Current.Session.Abandon();
            return true;
        }
    }
}