﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Social.Lib
{
    public class Timestamp
    {
        public static string GetFluentTimeSince(DateTime time)
        {
            DateTime now = DateTime.Now;
            return GetFluentTimeSince(now, time);
        }

        public static string GetFluentTimeSince(DateTime now, DateTime time)
        {
            var timeSpan = now.Subtract(time);
            bool isPlus = time > now;

            if(Math.Ceiling(timeSpan.TotalSeconds) < 60)
            {
                return isPlus ? string.Format("trong khoảng {0} giây", Math.Ceiling(timeSpan.TotalSeconds)) : string.Format("{0} giây trước", Math.Ceiling(timeSpan.TotalSeconds));
            }
            else if (Math.Ceiling(timeSpan.TotalMinutes) < 60)
            {
                return isPlus ? string.Format("trong khoảng {0} phút", Math.Ceiling(timeSpan.TotalMinutes)) : string.Format("{0} phút trước", Math.Ceiling(timeSpan.TotalMinutes));
            }
            else if (Math.Ceiling(timeSpan.TotalHours) < 24)
            {
                return isPlus ? string.Format("trong khoảng {0} tiếng", Math.Ceiling(timeSpan.TotalHours)) : string.Format("{0} tiếng trước", Math.Ceiling(timeSpan.TotalHours));
            }
            else if (timeSpan.TotalDays < 7 && GetWeekNumber(time) == GetWeekNumber(now))
            {
                return string.Format("{0} vào lúc {1}", GetWeekDay(time.DayOfWeek), GetTime(time));
            }
            else if (timeSpan.TotalDays < 366 && time.Year == now.Year)
            {
                return string.Format("ngày {0} tháng {1} lúc {2}", time.Day, time.Month, GetTime(time));
            }
            else
            {
                return string.Format("{0}, ngày {1} tháng {2} lúc {3}", GetWeekDay(time.DayOfWeek), time.Day, time.Month, GetTime(time));
            }
        }
        
        public static string GetWeekDay(DayOfWeek day)
        {
            switch (day)
            {
                case DayOfWeek.Monday:
                    return "Thứ hai";
                    break;
                case DayOfWeek.Tuesday:
                    return "Thứ ba";
                    break;
                case DayOfWeek.Wednesday:
                    return "Thứ tư";
                    break;
                case DayOfWeek.Thursday:
                    return "Thứ năm";
                    break;
                case DayOfWeek.Friday:
                    return "Thứ sáu";
                    break;
                case DayOfWeek.Saturday:
                    return "Thứ bẩy";
                    break;
                case DayOfWeek.Sunday:
                    return "Chủ nhật";
                    break;
            }
            return string.Empty;
        }

        public static string GetTime(DateTime time)
        {
            string timeString = time.ToString("h:mm");
            timeString += time.Hour > 12 ? " chiều" : " sáng";
            return timeString;
        }

        public static int GetWeekNumber(DateTime date)
        {
            //Constants
            const int JAN = 1;
            const int DEC = 12;
            const int LASTDAYOFDEC = 31;
            const int FIRSTDAYOFJAN = 1;
            const int THURSDAY = 4;
            bool thursdayFlag = false;

            int dayOfYear = date.DayOfYear;

            int startWeekDayOfYear = (int)(new DateTime(date.Year, JAN, FIRSTDAYOFJAN)).DayOfWeek;
            int endWeekDayOfYear = (int)(new DateTime(date.Year, DEC, LASTDAYOFDEC)).DayOfWeek;

            if (startWeekDayOfYear == 0)
                startWeekDayOfYear = 7;
            if (endWeekDayOfYear == 0)
                endWeekDayOfYear = 7;

            int daysInFirstWeek = 8 - (startWeekDayOfYear);

            if (startWeekDayOfYear == THURSDAY || endWeekDayOfYear == THURSDAY)
                thursdayFlag = true;

            int fullWeeks = (int)Math.Ceiling((dayOfYear - (daysInFirstWeek)) / 7.0);
            int resultWeekNumber = fullWeeks;

            if (daysInFirstWeek >= THURSDAY)
                resultWeekNumber = resultWeekNumber + 1;

            if (resultWeekNumber > 52 && !thursdayFlag)
                resultWeekNumber = 1;

            if (resultWeekNumber == 0)
                resultWeekNumber = GetWeekNumber(new DateTime(date.Year - 1, DEC, LASTDAYOFDEC));
            return resultWeekNumber;
        }
    }
}
