﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Cryptography;
using System.Net.Mail;
using Social.Lib.Activity;
using System.Configuration;

namespace Social.Lib.Utility
{
    public class Utility
    {
        public static string GetMd5Hash(string input)
        {
            MD5 md5Hash = MD5.Create();

            // Convert the input string to a byte array and compute the hash.
            byte[] data = md5Hash.ComputeHash(Encoding.UTF8.GetBytes(input));

            // Create a new Stringbuilder to collect the bytes
            // and create a string.
            StringBuilder sBuilder = new StringBuilder();

            // Loop through each byte of the hashed data 
            // and format each one as a hexadecimal string.
            for (int i = 0; i < data.Length; i++)
            {
                sBuilder.Append(data[i].ToString("x2"));
            }

            // Return the hexadecimal string.
            return sBuilder.ToString();
        }

        // Verify a hash against a string.
        public static bool VerifyMd5Hash(string input, string hash)
        {
            // Hash the input.
            string hashOfInput = GetMd5Hash(input);

            // Create a StringComparer an compare the hashes.
            StringComparer comparer = StringComparer.OrdinalIgnoreCase;

            if (0 == comparer.Compare(hashOfInput, hash))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        private static readonly string[] VietnameseSigns = new string[]

         {
 
        "aAeEoOuUiIdDyY",
 
        "áàạảãâấầậẩẫăắằặẳẵ",

        "ÁÀẠẢÃÂẤẦẬẨẪĂẮẰẶẲẴ",
 
        "éèẹẻẽêếềệểễ",

        "ÉÈẸẺẼÊẾỀỆỂỄ",
 
        "óòọỏõôốồộổỗơớờợởỡ",
 
        "ÓÒỌỎÕÔỐỒỘỔỖƠỚỜỢỞỠ",
 
        "úùụủũưứừựửữ",

        "ÚÙỤỦŨƯỨỪỰỬỮ",
 
        "íìịỉĩ",

        "ÍÌỊỈĨ",
 
        "đ",
 
        "Đ",
 
        "ýỳỵỷỹ",
 
        "ÝỲỴỶỸ"
 
         };

        public static string RemoveSign4VietnameseString(string str)
        {
            for (int i = 1; i < VietnameseSigns.Length; i++)
            {

                for (int j = 0; j < VietnameseSigns[i].Length; j++)

                    str = str.Replace(VietnameseSigns[i][j], VietnameseSigns[0][i - 1]);

            }

            return str;
        }

        public static string UrlThumbAlbumOrPhoto(string urlSource) {
            var retval = "";
            if (!string.IsNullOrEmpty(urlSource))
            {
                retval = ConfigManager.UrlDataPhoto + urlSource + Photo.Constants.ImageInfo.NameThumb;
            }
            else retval = ConfigManager.MainDomain + "/Content/Images/no_image.gif";
            
            return retval;
        }

        public static string UrlMainAlbumOrPhoto(string urlSource)
        {
            var retval = "";
            if (!string.IsNullOrEmpty(urlSource))
            {
                retval = ConfigManager.UrlDataPhoto + urlSource + Photo.Constants.ImageInfo.NameThumMain;
            }
            else retval = ConfigManager.MainDomain + "/Content/Images/no_image.gif";
            return retval;
        }

        public static bool SendMail(string mailTo)
        {
            try
            {
                // To
                System.Net.Mail.MailMessage mailMsg = new MailMessage();
                mailMsg.To.Add(mailTo);

                // From
                MailAddress mailAddress = new MailAddress(Constants.MAIL_ACCOUNT);
                mailMsg.From = mailAddress;

                // Subject and Body
                mailMsg.Subject = Constants.SUBJECT_MAIL;
                mailMsg.IsBodyHtml = true;
                mailMsg.Body = string.Format(Constants.CONTENT_MAIL);

                // Init SmtpClient and send
                SmtpClient smtpClient = new SmtpClient();
                smtpClient.UseDefaultCredentials = true;
                smtpClient.EnableSsl = true;
                smtpClient.Host = "SMTP.FTC.VN";
                smtpClient.Port = 25;

                System.Net.NetworkCredential credentials = new System.Net.NetworkCredential(Constants.MAIL_ACCOUNT, Constants.MAIL_PASSWORD);
                smtpClient.Credentials = credentials;

                smtpClient.Send(mailMsg);
                return true;
            }
            catch (Exception ex)
            {
                return false;
            }
        }

        public static string GetAvatarUrl(string urlSource)
        {
            if (string.IsNullOrEmpty(urlSource)) return ConfigManager.StaticServer + "/Content/Images/avatar_200x200.jpg";

            return urlSource.Contains("http://") ? 
                urlSource 
                : string.Concat(ConfigurationManager.AppSettings["AvatarServerUrl"], urlSource);
        }

        public static string RemoveHtmlTagAndSubString(string val, int length)
        {
            string reVal = RemoveHtmlImageTag(val);
            return SubString(reVal, length);
        }

        public static string RemoveHtmlImageTag(string val)
        {
            //string reval = System.Text.RegularExpressions.Regex.Replace(val, "^<img[^>]*>", string.Empty);
            //return System.Text.RegularExpressions.Regex.Replace(reval, "<.*?>", string.Empty);
            HtmlAgilityPack.HtmlDocument doc = new HtmlAgilityPack.HtmlDocument();
            doc.LoadHtml(val);
            return doc.DocumentNode.InnerText;
        }

        public static string SubString(string val, int length)
        {
            if (string.IsNullOrEmpty(val.Trim())) return string.Empty;

            string[] arrVal = val.Split(' ');
            if (arrVal == null) return string.Empty;

            if (arrVal.Length < length)
            {
                return val;
            }

            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < length; i++)
            {
                sb.AppendFormat("{0} ", arrVal[i]);
            }

            sb.Append("...");

            return sb.ToString();
        }

        public static string GetMarrialStatus(int type)
        {
            switch (type)
            {
                case 13:
                    return "Bí mật";
                case 14:
                    return "Độc thân";
                case 15:
                    return "Đang hẹn hò";
                case 16:
                    return "Đã đính hôn";
                case 17:
                    return "Đã kết hôn";
                case 18:
                    return "Đang yêu";
                case 20:
                    return "Góa (chồng, vợ)";
                case 21:
                    return "Ly hôn";

                default:
                    return "Chưa cập nhật";
            }
        }
    }
}
