﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Web;
using HtmlAgilityPack;

namespace Social.Lib
{
    public class WebCrawnler
    {
        private int MINWITH = 40;
        private int MINHEIGHT = 40;
        private bool AND = true;

        private Encoding ENCODE = Encoding.UTF8;
        private string USER_AGENT = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:5.0) Gecko/20100101 Firefox/5.0";

        public LinkDetails GetDetails(string url)
        {
            LinkDetails linkDetails = new LinkDetails();

            if (url.StartsWith("http://") || url.StartsWith("https://"))
            {
                linkDetails.Url = url;
            }
            else
            {
                linkDetails.Url = "http://" + url;
            }
            Uri linkUrl = new Uri(url);
            linkDetails.SiteUrl = linkUrl.Host.ToString();

            linkDetails = GetHeaders(linkDetails.Url, linkDetails);

            if (linkDetails.MimeType.ToLower().Contains("text/html"))
            {
                linkDetails.LinkType = 1;
                HtmlDocument htmlDocument = new HtmlDocument();

                System.Net.WebClient webClient = new System.Net.WebClient();
                webClient.Headers["User-Agent"] = USER_AGENT;
                webClient.Encoding = ENCODE;
                string download = webClient.DownloadString(linkDetails.Url);
                htmlDocument.LoadHtml(download);

                HtmlNode htmlNode = htmlDocument.DocumentNode.SelectSingleNode("html/head");

                linkDetails = GetStandardInfo(linkDetails, htmlNode);

                linkDetails = GetOpenGraphInfo(linkDetails, htmlNode);

                if (linkDetails.Audio == null)
                {
                    linkDetails = GetAudioSharingInfo(linkDetails, htmlNode);
                }

                if (linkDetails.Video == null)
                {
                    linkDetails = GetVideoSharingInfo(linkDetails, htmlNode);
                }

                if (linkDetails.Image == null)
                {
                    linkDetails = GuessImage(htmlDocument, linkDetails);
                }
            }

            if (linkDetails.MimeType.ToLower().Contains("image/"))
            {
                linkDetails.LinkType = 2;
                linkDetails.Image = new ImageLink(linkDetails.Url);
            }
            return linkDetails;
        }

        private LinkDetails GetStandardInfo(LinkDetails linkDetails, HtmlNode head)
        {
            foreach (HtmlNode headNode in head.ChildNodes)
            {
                switch (headNode.Name.ToLower())
                {
                    case "link": break;
                    case "title":
                        linkDetails.Title = HttpUtility.HtmlDecode(HttpUtility.HtmlDecode(headNode.InnerText));
                        break;
                    case "meta":
                        if (headNode.Attributes["name"] != null && headNode.Attributes["content"] != null)
                        {
                            switch (headNode.Attributes["name"].Value.ToLower())
                            {
                                case "description":
                                    linkDetails.Description = HttpUtility.HtmlDecode(HttpUtility.HtmlDecode(headNode.Attributes["content"].Value));
                                    break;
                            }
                        }
                        break;
                }

            }

            // look for apple touch icon in header
            HtmlNode imageNode = head.SelectSingleNode("link[@rel='apple-touch-icon']");
            if (imageNode != null)
            {
                if (imageNode.Attributes["href"] != null)
                {
                    if (!string.IsNullOrEmpty(imageNode.Attributes["href"].Value.ToString().Trim()))
                    {
                        linkDetails.Image = new ImageLink(imageNode.Attributes["href"].Value, linkDetails.Url);
                        if (!linkDetails.Images.Contains(linkDetails.Image)) { linkDetails.Images.Add(linkDetails.Image); }
                    }
                }
                if (imageNode.Attributes["src"] != null)
                {
                    if (!string.IsNullOrEmpty(imageNode.Attributes["src"].Value.ToString().Trim()))
                    {
                        linkDetails.Image = new ImageLink(imageNode.Attributes["src"].Value, linkDetails.Url);
                        if (!linkDetails.Images.Contains(linkDetails.Image)) { linkDetails.Images.Add(linkDetails.Image); }
                    }
                }
            }

            //look for link image in header
            imageNode = head.SelectSingleNode("link[@rel='image_src']");
            if (imageNode != null)
            {
                if (imageNode.Attributes["href"] != null)
                {
                    if (!string.IsNullOrEmpty(imageNode.Attributes["href"].Value.ToString().Trim()))
                    {
                        linkDetails.Image = new ImageLink(imageNode.Attributes["href"].Value, linkDetails.Url);
                        if (!linkDetails.Images.Contains(linkDetails.Image)) { linkDetails.Images.Add(linkDetails.Image); }
                    }
                }
                if (imageNode.Attributes["src"] != null)
                {
                    if (!string.IsNullOrEmpty(imageNode.Attributes["src"].Value.ToString().Trim()))
                    {
                        linkDetails.Image = new ImageLink(imageNode.Attributes["src"].Value, linkDetails.Url);
                        if (!linkDetails.Images.Contains(linkDetails.Image)) { linkDetails.Images.Add(linkDetails.Image); }
                    }
                }
            }

            return linkDetails;
        }

        private LinkDetails GetAudioSharingInfo(LinkDetails linkDetails, HtmlNode head)
        {
            HtmlNode audioNode = head.SelectSingleNode("link[@rel='audio_src']");
            if (audioNode != null)
            {
                if (audioNode.Attributes["href"] != null)
                {
                    linkDetails.Audio = new AudioLink(audioNode.Attributes["href"].Value, linkDetails.Url);
                }
                if (audioNode.Attributes["src"] != null)
                {
                    linkDetails.Audio = new AudioLink(audioNode.Attributes["href"].Value, linkDetails.Url);
                }
            }

            if (linkDetails.Audio != null)
            {
                //Get audio type
                if (string.IsNullOrEmpty(linkDetails.Audio.MimeType))
                {
                    audioNode = head.SelectSingleNode("meta[@name='audio_type']");
                    if (audioNode != null)
                    {
                        if (audioNode.Attributes["content"] != null)
                        {
                            linkDetails.Audio.MimeType = audioNode.Attributes["content"].Value;
                        }
                    }
                }

                //Get audio title
                audioNode = head.SelectSingleNode("meta[@name='audio_title']");
                if (audioNode != null)
                {
                    if (audioNode.Attributes["content"] != null)
                    {
                        linkDetails.Audio.Title = audioNode.Attributes["content"].Value;
                    }
                }

                //Get audio artist
                audioNode = head.SelectSingleNode("meta[@name='audio_artist']");
                if (audioNode != null)
                {
                    if (audioNode.Attributes["content"] != null)
                    {
                        linkDetails.Audio.Title = audioNode.Attributes["content"].Value;
                    }
                }

                //Get audio album
                audioNode = head.SelectSingleNode("meta[@name='audio_album']");
                if (audioNode != null)
                {
                    if (audioNode.Attributes["content"] != null)
                    {
                        linkDetails.Audio.Title = audioNode.Attributes["content"].Value;
                    }
                }
            }



            return linkDetails;
        }

        private LinkDetails GetVideoSharingInfo(LinkDetails linkDetails, HtmlNode head)
        {
            //Get by Standard Info
            HtmlNode videoNode = head.SelectSingleNode("link[@rel='video_src']");
            if (videoNode != null)
            {
                if (videoNode.Attributes["href"] != null)
                {
                    linkDetails.Video = new VideoLink(videoNode.Attributes["href"].Value, linkDetails.Url);
                }
                if (videoNode.Attributes["src"] != null)
                {
                    linkDetails.Video = new VideoLink(videoNode.Attributes["href"].Value, linkDetails.Url);
                }
            }

            if (linkDetails.Video != null)
            {
                //Get video type
                if (string.IsNullOrEmpty(linkDetails.Video.MimeType))
                {
                    videoNode = head.SelectSingleNode("meta[@name='video_type']");
                    if (videoNode != null)
                    {
                        if (videoNode.Attributes["content"] != null)
                        {
                            linkDetails.Video.MimeType = videoNode.Attributes["content"].Value;
                        }
                    }
                }

                //Get with & height
                videoNode = head.SelectSingleNode("meta[@name='video_height']");
                if (videoNode != null)
                {
                    if (videoNode.Attributes["content"] != null)
                    {
                        int height;
                        linkDetails.Video.Height = int.TryParse(videoNode.Attributes["content"].Value, out height) ? height : 0;
                    }
                }

                videoNode = head.SelectSingleNode("meta[@name='video_width']");
                if (videoNode != null)
                {
                    if (videoNode.Attributes["content"] != null)
                    {
                        int width;
                        linkDetails.Video.Width = int.TryParse(videoNode.Attributes["content"].Value, out width) ? width : 0;
                    }
                }

            }

            return linkDetails;
        }

        private LinkDetails GetOpenGraphInfo(LinkDetails linkDetails, HtmlNode head)
        {
            foreach (HtmlNode headNode in head.ChildNodes)
            {
                switch (headNode.Name.ToLower())
                {
                    case "link": break;

                    case "meta":
                        if (headNode.Attributes["property"] != null && headNode.Attributes["content"] != null)
                        {
                            switch (headNode.Attributes["property"].Value.ToLower())
                            {
                                case "og:title":
                                    linkDetails.Title = HttpUtility.HtmlDecode(HttpUtility.HtmlDecode(headNode.Attributes["content"].Value));
                                    break;
                                case "og:type":
                                    linkDetails.Type = headNode.Attributes["content"].Value;
                                    break;
                                case "og:url":
                                    linkDetails.Url = headNode.Attributes["content"].Value;
                                    break;
                                case "og:image":
                                    linkDetails.Image = new ImageLink(headNode.Attributes["content"].Value, linkDetails.Url);
                                    if (!linkDetails.Images.Contains(linkDetails.Image)) linkDetails.Images.Add(linkDetails.Image);
                                    break;
                                case "og:site_name":
                                    linkDetails.SiteName = HttpUtility.HtmlDecode(HttpUtility.HtmlDecode(headNode.Attributes["content"].Value));
                                    break;
                                case "og:description":
                                    linkDetails.Description = HttpUtility.HtmlDecode(HttpUtility.HtmlDecode(headNode.Attributes["content"].Value));
                                    break;
                                case "og:email":
                                    linkDetails.Email = HttpUtility.HtmlDecode(HttpUtility.HtmlDecode(headNode.Attributes["content"].Value));
                                    break;
                                case "og:phone_number":
                                    linkDetails.PhoneNumber = HttpUtility.HtmlDecode(headNode.Attributes["content"].Value);
                                    break;
                                case "og:fax_number":
                                    linkDetails.FaxNumber = HttpUtility.HtmlDecode(headNode.Attributes["content"].Value);
                                    break;

                                //Open graph for Video
                                case "og:video":
                                    linkDetails.Video = new VideoLink(headNode.Attributes["content"].Value, linkDetails.Url);
                                    break;
                                case "og:video:height":
                                    int height;
                                    linkDetails.Video.Height = int.TryParse(headNode.Attributes["content"].Value, out height) ? height : 0;
                                    break;
                                case "og:video:width":
                                    int width;
                                    linkDetails.Video.Width = int.TryParse(headNode.Attributes["content"].Value, out width) ? width : 0;
                                    break;
                                case "og:video:type":
                                    if (string.IsNullOrEmpty(linkDetails.Video.MimeType)) linkDetails.Video.MimeType = headNode.Attributes["content"].Value;
                                    break;

                                //Open graph for Audio
                                case "og:audio":
                                    linkDetails.Audio = new AudioLink(headNode.Attributes["content"].Value, linkDetails.Url);
                                    break;
                                case "og:audio:title":
                                    linkDetails.Audio.Title = headNode.Attributes["content"].Value;
                                    break;
                                case "og:audio:artist":
                                    linkDetails.Audio.Artist = headNode.Attributes["content"].Value;
                                    break;
                                case "og:audio:album":
                                    linkDetails.Audio.Album = headNode.Attributes["content"].Value;
                                    break;
                                case "og:audio:type":
                                    if (string.IsNullOrEmpty(linkDetails.Audio.MimeType)) linkDetails.Audio.MimeType = headNode.Attributes["content"].Value;
                                    break;
                            }
                        }
                        break;
                }

            }
            return linkDetails;
        }

        private LinkDetails GuessImage(HtmlDocument htmlDocument, LinkDetails linkDetails)
        {
            LinkDetails detail = linkDetails;
            HtmlNodeCollection imageNodes = htmlDocument.DocumentNode.SelectNodes("//img");
            string h1 = string.Empty;
            HtmlNode h1Node = htmlDocument.DocumentNode.SelectSingleNode("//h1");
            if (h1Node != null)
            {
                h1 = h1Node.InnerText;
            }
            int bestScore = -1;
            if (imageNodes != null)
            {
                foreach (HtmlNode imageNode in imageNodes)
                {
                    if (imageNode.Attributes["width"] != null && imageNode.Attributes["height"] != null)
                    {
                        int width;
                        int height;
                        if (int.TryParse(imageNode.Attributes["width"].Value, out width) && int.TryParse(imageNode.Attributes["height"].Value, out height))
                        {
                            bool checkWith = width < MINWITH;
                            bool checkHeight = height < MINHEIGHT;
                            bool checkImage = !AND ? checkWith && checkHeight : checkWith || checkHeight;
                            if (checkImage)
                            {
                                continue;
                            }
                        }
                    }

                    if (imageNode.Attributes["width"] != null)
                    {
                        int width;
                        if (int.TryParse(imageNode.Attributes["width"].Value, out width))
                        {
                            if (width < MINWITH)
                            {
                                continue;
                            }
                        }
                    }

                    if (imageNode.Attributes["height"] != null)
                    {
                        int height;
                        if (int.TryParse(imageNode.Attributes["height"].Value, out height))
                        {
                            if (height < MINHEIGHT)
                            {
                                continue;
                            }
                        }
                    }

                    if (imageNode != null && imageNode.Attributes["src"] != null)
                    {
                        ImageLink imageLink = new ImageLink(imageNode.Attributes["src"].Value, detail.Url);

                        bool checkWith = imageLink.Width >= MINWITH;
                        bool checkHeight = imageLink.Height >= MINHEIGHT;
                        bool checkImage = AND ? checkWith && checkHeight : checkWith || checkHeight;
                        if (checkImage)
                        {
                            int myScore = 0;
                            if (imageNode.Attributes["alt"] != null)
                            {
                                myScore = ScoreImage(imageNode.Attributes["alt"].Value, linkDetails.Title);
                                myScore += ScoreImage(imageNode.Attributes["alt"].Value, h1);
                            }
                            if (myScore > bestScore)
                            {
                                detail.Image = imageLink;
                                bestScore = myScore;
                            }

                            if (!detail.Images.Contains(imageLink)) { detail.Images.Add(imageLink); }
                        }
                    }
                }
            }

            return detail;
        }

        private int ScoreImage(string text, string compare)
        {
            text = text.Replace("\r\n", string.Empty).Replace("\t", string.Empty);
            compare = compare.Replace("\r\n", string.Empty).Replace("\t", string.Empty);
            int score = 0;
            if (!string.IsNullOrEmpty(text) && !string.IsNullOrEmpty(compare))
            {
                string[] c = compare.Split(' ');

                foreach (string test in c)
                {
                    if (text.Contains(test)) { score++; }
                }
            }
            return score;
        }

        public LinkDetails GetHeaders(string link, LinkDetails linkDetails)
        {
            try
            {
                System.Net.WebClient wc = new System.Net.WebClient();
                wc.Headers["User-Agent"] = USER_AGENT;
                wc.Encoding = ENCODE;
                wc.OpenRead(link);
                linkDetails.ContentLength = Convert.ToInt64(wc.ResponseHeaders["Content-Length"]);
                linkDetails.MimeType = wc.ResponseHeaders["Content-Type"];
            }
            catch
            {
                linkDetails.MimeType = "Don't Download";
            }
            return linkDetails;
        }


        public class LinkDetails
        {
            public LinkDetails()
            {
                Images = new List<ImageLink>();
            }
            public string Title { get; set; }
            public string SiteUrl { get; set; }
            public string Url { get; set; }
            public string Type { get; set; }
            public ImageLink Image { get; set; }
            public List<ImageLink> Images { get; set; }
            public AudioLink Audio { get; set; }
            public VideoLink Video { get; set; }
            public string SiteName { get; set; }
            public string Description { get; set; }
            public string Email { get; set; }
            public string PhoneNumber { get; set; }
            public string FaxNumber { get; set; }
            public Int64 ContentLength { get; set; }
            public string MimeType { get; set; }
            public int LinkType { get; set; } // 0=bad, 1=html, 2=image

        }

        public class AudioLink
        {
            public ImageLink Image { get; set; }
            public string Url { get; set; }
            public string MimeType { get; set; }
            public string Title { get; set; }
            public string Artist { get; set; }
            public string Album { get; set; }

            public AudioLink()
            {
            }

            public AudioLink(string url)
            {
                SetAudioLink(url);
            }
            public AudioLink(string url, string siteUrl)
            {
                SetAudioLink(FullyQualifiedAudio(url, siteUrl));
            }

            private void SetAudioLink(string url)
            {
                this.Url = url;
            }

            private string FullyQualifiedAudio(string audioUrl, string siteUrl)
            {
                if (audioUrl.Contains("http:") || audioUrl.Contains("https:"))
                {
                    return audioUrl;
                }

                if (audioUrl.IndexOf("//") == 0)
                {
                    return "http:" + audioUrl;
                }
                try
                {
                    string baseurl = siteUrl.Replace("http://", string.Empty).Replace("https://", string.Empty);
                    baseurl = baseurl.Split('/')[0];
                    return string.Format("http://{0}{1}", baseurl, audioUrl);

                }
                catch { }

                return audioUrl;
            }
        }

        public class VideoLink
        {
            public int Width { get; set; }
            public int Height { get; set; }
            public string Url { get; set; }
            public string MimeType { get; set; }

            public VideoLink()
            {
            }

            public VideoLink(string url)
            {
                SetVideoLink(url);
            }
            public VideoLink(string url, string siteUrl)
            {
                SetVideoLink(FullyQualifiedVideo(url, siteUrl));
            }

            private void SetVideoLink(string url)
            {
                this.Url = url;
            }

            private string FullyQualifiedVideo(string audioUrl, string siteUrl)
            {
                if (audioUrl.Contains("http:") || audioUrl.Contains("https:"))
                {
                    return audioUrl;
                }

                if (audioUrl.IndexOf("//") == 0)
                {
                    return "http:" + audioUrl;
                }
                try
                {
                    string baseurl = siteUrl.Replace("http://", string.Empty).Replace("https://", string.Empty);
                    baseurl = baseurl.Split('/')[0];
                    return string.Format("http://{0}{1}", baseurl, audioUrl);

                }
                catch { }

                return audioUrl;
            }
        }

        public class ImageLink
        {
            public int Width { get; set; }
            public int Height { get; set; }
            public string Url { get; set; }

            public ImageLink()
            {
            }

            public ImageLink(string url, string siteUrl)
            {
                SetImageLink(FullyQualifiedImage(url, siteUrl));
            }

            public ImageLink(string url)
            {
                SetImageLink(url);
            }

            private void SetImageLink(string url)
            {
                this.Url = url;
                try
                {
                    System.Net.WebClient webClient = new System.Net.WebClient();
                    byte[] imageData = webClient.DownloadData(url);
                    MemoryStream stream = new MemoryStream(imageData);
                    Image img = Image.FromStream(stream);
                    stream.Close();
                    this.Width = img.Width;
                    this.Height = img.Height;
                }
                catch
                {

                }
            }


            private string FullyQualifiedImage(string imageUrl, string siteUrl)
            {
                if (imageUrl.Contains("http:") || imageUrl.Contains("https:"))
                {
                    return imageUrl;
                }

                if (imageUrl.IndexOf("//") == 0)
                {
                    return "http:" + imageUrl;
                }
                try
                {
                    string baseurl = siteUrl.Replace("http://", string.Empty).Replace("https://", string.Empty);
                    baseurl = baseurl.Split('/')[0];
                    return string.Format("http://{0}{1}", baseurl, imageUrl);

                }
                catch { }

                return imageUrl;
            }
        }
    }
}