﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Social.DAL.Interface;

namespace Social.Service
{
    public class ServiceBase<T> : IService<T> where T : class, new()
    {
        #region IService Members
        IRepository _repository;
        public ServiceBase(IRepository repository)
        {
            _repository=repository;
        }

        public T Single(System.Linq.Expressions.Expression<Func<T, bool>> whereCondition)
        {
            return _repository.Single<T>(whereCondition);
        }

        public virtual void Add(T entity)
        {
            _repository.Add<T>(entity);
            _repository.CommitChanges();
        }

        public virtual void Delete(T entity)
        {
            _repository.Delete<T>(entity);
            _repository.CommitChanges();
        }

        public virtual void Delete(System.Linq.Expressions.Expression<Func<T, bool>> expression)
        {
            _repository.Delete<T>(expression);
            _repository.CommitChanges();
        }

        public virtual void DeleteAll()
        {
            _repository.DeleteAll<T>();
            _repository.CommitChanges();
        }

        public virtual void Update(T entity)
        {
            _repository.Update<T>(entity);
            _repository.CommitChanges();
        }

        public IList<T> GetAll(System.Linq.Expressions.Expression<Func<T, bool>> whereCondition)
        {
            return _repository.All<T>().Where(whereCondition).ToList();
        }

        public IList<T> GetAll()
        {
            return _repository.All<T>().ToList();
        }

        public IQueryable<T> Query(System.Linq.Expressions.Expression<Func<T, bool>> whereCondition)
        {
            return _repository.All<T>();
        }

        public long Count(System.Linq.Expressions.Expression<Func<T, bool>> whereCondition)
        {
            return _repository.All<T>().Where(whereCondition).Count();
        }

        public long Count()
        {
            return _repository.All<T>().Count();
        }

        #endregion
    }
}
