﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Social.DAL.Interface;
using Social.Entity;
using Social.Entity.Tik;
using Social.Service.Interface.Tik;
using AutoMapper;
using Social.Lib;
using Social.Lib.Utility;
using Webdiyer.WebControls.Mvc;
using Social.Service.Interface.TikFriend;
using Social.Entity.TikFriend;


namespace Social.Service.Tik
{
    public class UserProfileService : ServiceBase<UserProfile>, IUserProfileService
    {
        IRepository _re;
        IUserSearchService _usersearch;
        IUserFriendService _userfriend;
        IFriendSearchService _friendsearch;
        public UserProfileService(ITikRepository repository, IUserSearchService usersearch,IUserFriendService userfriend,IFriendSearchService friendsearch)
            : base(repository)
        {
            _re = repository;
            _usersearch = usersearch;
            _userfriend = userfriend;
            _friendsearch = friendsearch;
        }

        public bool CheckMail(string email)
        {
            var checkuser = from c in _re.All<UserProfile>()
                            where c.Email == email
                            select c;
            if (checkuser != null) if (checkuser.Count() == 1) return true;
            return false;
        }

        public void RegisterUser(UserProfile u)
        {
            u.Password = Utility.GetMd5Hash(u.Password);
            u.UserName = u.Email.Substring(0, u.Email.IndexOf('@'));
            var user_duplicate = _re.Single<UserProfile>(it => it.UserName.Equals(u.UserName, StringComparison.OrdinalIgnoreCase));
            if (user_duplicate == null)
                this.Add(u);
            else
            {
                int i = 0;
                while (true)
                {
                    i++;
                    u.UserName = u.UserName + i;
                    user_duplicate = _re.Single<UserProfile>(it => it.UserName.Equals(u.UserName, StringComparison.OrdinalIgnoreCase));
                    if (user_duplicate == null)
                    {
                        this.Add(u);
                        break;
                    }
                }
            }
        }
        /// <summary>
        /// Lấy danh sách thành viên bắt đầu bằng 1 chữ cái
        /// </summary>
        /// <param name="character"></param>
        /// <returns></returns>
        public List<UserProfile> GetListUserByCharacter(string character)
        {
            var listuser = from it in _re.All<UserProfile>()
                           where it.LastName.StartsWith(character)
                           select it;
            return listuser.ToList();
        }

        /// <summary>
        ///     Đếm số thành viên bắt đầu bằng 1 chữ cái
        /// </summary>
        /// <param name="character"></param>
        /// <returns></returns>
        public int CountListUserByCharacter(string character)
        {
            var countuser = (from it in _re.All<UserProfile>()
                             where it.LastName.StartsWith(character)
                             select it).Count();
            return countuser;
        }

        /// <summary>
        ///     Lay thong tin thanh vien thu index bat dau bang 1 chu cai
        /// </summary>
        /// <param name="character"></param>
        /// <param name="index"></param>
        /// <returns></returns>
        public UserProfile GetUserByCharacterAndIndex(string character, int index)
        {
            var user = (from it in _re.All<UserProfile>()
                        where it.LastName.StartsWith(character)
                        orderby it.LastName
                        select it).Skip(index - 1).Take(1);

            return user.SingleOrDefault();
        }

        public UserProfile GetUserByUsername(string username)
        {
            var user = from userProfile in _re.All<UserProfile>()
                       where userProfile.UserName == username
                       select userProfile;
            if (user.Count() > 0)
            {
                return user.FirstOrDefault();
            }
            return null;
        }
        public UserProfile GetByEmail(string email)
        {
            return _re.Single<UserProfile>(it => it.Email.Equals(email, StringComparison.OrdinalIgnoreCase));
        }

        public List<UserProfileAndCountUserFriend> GetUserProfileCountFriend()
        {
            var qry = from c in _re.All<UserProfile>()
                      select c;
            return qry.ToList().Select(fr => Mapper.Map<UserProfile, UserProfileAndCountUserFriend>(fr)).ToList();
        }



        public UserProfile GetByID(Guid guid)
        {
            return _re.Single<UserProfile>(it => it.ID == guid);
        }


        public PagedList<UserProfile> GetByIDPage(Guid guid, int pageIndex = 0, int pageSize = 10)
        {

            var _userprofile = from c in _re.All<UserProfile>()
                               where c.ID == guid
                               select c;
            return _userprofile.ToPagedList<UserProfile>(pageIndex, pageSize);

            //return _re.Single<UserProfile>(it => it.ID == guid)
        }

        public UserProfile CurrentUser()
        {
            if (Social.Lib.SessionManager.IsLoggedIn())
            {
                Guid currentUserID = SessionManager.CurrentUserID();
                return _re.Single<UserProfile>(it => it.ID == currentUserID);
            }
            else
            {
                return null;
            }
        }
        public override void Update(UserProfile user)
        {
            string fullText = string.Format("{0},{1},{2},{3}", user.UserName, Lib.Utility.Utility.RemoveSign4VietnameseString(user.FullName), user.FullName, user.Email);
            string fullTextName = string.Format("{0},{1}",  Lib.Utility.Utility.RemoveSign4VietnameseString(user.FullName), user.FullName);
            UserSearch us = _usersearch.Single(it => it.UserID == user.ID);
            if (us != null)
            {
                us.SmallAvatar = user.SmallAvatarUrlMap;
                us.Username = user.UserName;
                us.FullName = user.FullName;
                us.FullText = fullText;
                _usersearch.Update(us);
            }
            IList<UserFriend> lstUF = _userfriend.GetAll(it => it.FriendID == user.ID);
            foreach (UserFriend uf in lstUF)
            {
                uf.FriendSmallAvatarUrl = user.SmallAvatarUrlMap;
                uf.FriendUserName = user.UserName;
                uf.FriendFullName = user.FullName;
                _userfriend.Update(uf);
            }
            IList<FriendSearch> lstFS = _friendsearch.GetAll(it => it.FriendID == user.ID);
            foreach (FriendSearch fs in lstFS)
            {
                fs.FriendSmallAvatarUrl = user.SmallAvatarUrlMap;                
                fs.FriendFullName = user.FullName;
                fs.FullText = fullText;
                fs.FullTextName = fullTextName;
                _friendsearch.Update(fs);
            }
            base.Update(user);
        }

        public List<UserProfile> GetAllUserID(List<Guid> userID)
        {
            try {
                var _listuserprofile = from c in _re.All<UserProfile>()
                                       where userID.Contains(c.ID)
                                       select c;
                return _listuserprofile.ToList();
                                      
            }
            catch {
                return new List<UserProfile>();
            }
        }
    }
}
