﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.Script.Serialization;
using Social.DAL.Interface;
using Social.Entity.Tik;
using Social.Entity.TikActivity;
using Social.Entity.TikApplication;
using Social.Entity.TikPhoto;
using Social.Lib.Activity;
using Social.Service.Interface.TikActivity;
using Social.Entity.TikMusic;
using Album = Social.Entity.TikMusic.Album;

namespace Social.Service.TikActivity
{
    public class UserActivityService : ServiceBase<UserActivity>, IUserActivityService
    {
        IRepository _re;
        IRepository _rePhoto;
        IRepository _reApplication;
        IRepository _reMusic;
        public UserActivityService(ITikActivityRepository repository, ITikPhotoRepository photoRepository, ITikApplicationRepository applicationRepository, ITikMusicRepository musicRepository) : base(repository)
        {
            _re = repository;
            _rePhoto = photoRepository;
            _reApplication = applicationRepository;
            _reMusic = musicRepository;
        }

        public UserActivity GetByID(Guid activityid)
        {
            if (Guid.Empty.Equals(activityid)) return null;
            var activity = from a in _re.All<UserActivity>()
                           where a.ID == activityid
                           select a;
            if(activity.Any())
            return activity.FirstOrDefault();
            return null;
        }

        public UserActivity GetByTarget(Constants.Activity.TargetType type, Guid targetid)
        {
            throw new NotImplementedException();
        }

        public List<UserActivity> GetActivity(Guid ownerid, DateTime BeginDate)
        {
            throw new NotImplementedException();
        }
        
        public List<UserActivity> GetWallActivity(Guid userid)
        {
            return GetWallActivity(userid, DateTime.Now, 0);
        }

        public List<UserActivity> GetWallActivity(Guid userid, DateTime beginTime, int top = 0)
        {
            if (Guid.Empty.Equals(userid)) return null;
            var activities = from a in _re.All<UserActivity>()
                             where ((a.OwnerID == userid && a.HideOwner == false && a.ActivityType != (int)Constants.Activity.ActivityType.Tag) || (a.OwnerID != a.TargetID && a.TargetType == (int)Constants.Activity.TargetType.User && a.TargetID == userid && a.HideTarget == false)) 
                                    && (a.ActivityType !=  (int)Constants.Activity.ActivityType.Like 
                                    && a.ActivityType != (int)Constants.Activity.ActivityType.Comment)
                                    && a.CreatedDate < beginTime
                             select a;

            if (activities.Any())
            {
                if(top > 0) return activities.OrderByDescending(a => a.CreatedDate).Take(top).ToList();
                return activities.OrderByDescending(a => a.CreatedDate).ToList();
            }
            return null;
        }

        public List<UserActivity> GetFeedActivity(Guid userid, List<Guid> listFriend, DateTime beginTime, int top = 0)
        {
            if (Guid.Empty.Equals(userid) || listFriend == null) return null;
            if (listFriend.Count == 0) return null;

            var activities = from a in _re.All<UserActivity>()
                             where (a.OwnerID == userid || (a.OwnerID != a.TargetID && a.TargetType == (int)Constants.Activity.TargetType.User && (listFriend.Contains(a.TargetID) || a.TargetID == userid))
                                    || (listFriend.Contains(a.OwnerID) && a.ActivityType != (int)Constants.Activity.ActivityType.Tag))
                                    //&& (a.OwnerID != userid)
                                    && a.CreatedDate < beginTime
                             select a;

            if (activities.Any())
            {
                if (top > 0) return activities.OrderByDescending(a => a.CreatedDate).Take(top).ToList();
                return activities.OrderByDescending(a => a.CreatedDate).ToList();
            }
            return null;
        }

        public List<UserActivity> GetSpecialFeedActivity(Guid userid, List<Guid> listFriend, DateTime beginTime, int top = 0)
        {
            if (Guid.Empty.Equals(userid) || listFriend == null) return null;
            if (listFriend.Count == 0) return null;
            DateTime endTime = DateTime.Now.AddDays(-7);
            var activities = from a in _re.All<UserActivity>()
                             where (listFriend.Contains(a.OwnerID) || (a.OwnerID != a.TargetID && a.TargetType == (int)Constants.Activity.TargetType.User && listFriend.Contains(a.TargetID)))
                                    && (a.OwnerID != userid)
                                    && a.ActivityType != (int)Constants.Activity.ActivityType.Friend
                                    && a.CreatedDate < beginTime
                                    && a.CreatedDate > endTime
                             select a;

            if (activities.Any())
            {
                if (top > 0) return activities.OrderByDescending(a => a.CreatedDate).Take(top).ToList();
                return activities.OrderByDescending(a => a.CreatedDate).ToList();
            }
            return null;
        }   

        /// <summary>
        /// 
        /// </summary>
        /// <param name="currentUser"></param>
        /// <param name="type"></param>
        /// <param name="objectid"></param>
        /// <param name="comment"></param>
        /// <param name="created"></param>
        /// <param name="commentid"></param>
        /// <returns></returns>
        public bool Comment(UserProfile currentUser, Constants.Comment.ObjectType type, Guid objectid, string comment, DateTime created, Guid commentid = new Guid())
        {
            try
            {
                List<Guid> listTagUsers = Feed.ListUserTagged(comment);

                UserComment userComment = new UserComment();
                if(Guid.Empty.Equals(commentid))
                {
                    userComment.ID = Guid.NewGuid();    
                }else
                {
                    userComment.ID = commentid;
                }
                
                userComment.FromID = currentUser.ID;
                userComment.FromName = currentUser.FullName;
                userComment.CommentDate = created;
                userComment.CommentContent = Social.Lib.Activity.Feed.Message(comment);

                UserActivity activity = new UserActivity();
                activity.ID = Guid.NewGuid();
                activity.CreatedDate = created;
                activity.OwnerID = currentUser.ID;
                activity.HideOwner = false;
                activity.ActivityType = (int) Constants.Activity.ActivityType.Comment;
                activity.TargetType = (int) Constants.Activity.TargetType.Comment;
                activity.TargetID = userComment.ID;

                UserNotification notification = new UserNotification();
                notification.ID = Guid.NewGuid();
                notification.CreatedDate = created;
                notification.TargetType = (int) Constants.Notification.TargetType.User;
                notification.TargetID = currentUser.ID;
                notification.NotificationType = (int) Constants.Notification.NotificationType.Comment;
                notification.IsHidden = false;
                notification.IsRead = false;

                bool addNotification = true;

                switch (type)
                {
                    case Constants.Comment.ObjectType.Feed:
                        var feed = _re.Single<UserFeed>(f => f.ID == objectid);
                        if (feed != null)
                        {
                            //if(feed.OwnerID == currentUser.ID)
                            //{
                            //    addNotification = false;
                            //}

                            switch (feed.ObjectType)
                            {
                                case (int)Constants.Feed.ObjectType.AlbumMusic:
                                    userComment.ObjectType = (int)Constants.Comment.ObjectType.AlbumMusic;
                                    userComment.ObjectID = feed.ObjectID;
                                    userComment.OwnerID = feed.TargetID;
                                    activity.ObjectType = (int)Constants.Activity.ObjectType.AlbumMusic;
                                    activity.ObjectID = feed.ObjectID;
                                    notification.ObjectType = (int) Constants.Notification.ObjectType.AlbumMusic;
                                    notification.ObjectID = feed.ObjectID;
                                    notification.OwnerID = feed.OwnerID;
                                     var albumMusicFeed = _reMusic.Single<Album>(p => p.ID == feed.ObjectID);
                                     if (albumMusicFeed!= null)
                                     {
                                         albumMusicFeed.CommentCount++;
                                         userComment.OwnerID = albumMusicFeed.UserID;
                                         notification.OwnerID = albumMusicFeed.UserID;
                                         _reMusic.Update(albumMusicFeed);
                                     }
                                    break;
                                case (int) Constants.Feed.ObjectType.Music:
                                    userComment.ObjectType = (int) Constants.Comment.ObjectType.Music;
                                    userComment.ObjectID = feed.ObjectID;
                                    userComment.OwnerID = feed.TargetID;
                                    activity.ObjectType = (int) Constants.Activity.ObjectType.Music;
                                    activity.ObjectID = feed.ObjectID;
                                    notification.ObjectType = (int)Constants.Notification.ObjectType.Music;
                                    notification.ObjectID = feed.ObjectID;
                                    notification.OwnerID = feed.OwnerID;
                                    var musicFeed = _reMusic.Single<Song>(p => p.ID == feed.ObjectID);
                                    if (musicFeed != null)
                                    {
                                        musicFeed.CommentCount++;
                                        userComment.OwnerID = musicFeed.UserID;
                                        notification.OwnerID = musicFeed.UserID;
                                        _reMusic.Update(musicFeed);
                                    }
                                    break;
                                case (int) Constants.Feed.ObjectType.Image:
                                    userComment.ObjectType = (int) Constants.Comment.ObjectType.Image;
                                    userComment.ObjectID = feed.ObjectID;
                                    activity.ObjectType = (int) Constants.Activity.ObjectType.Image;
                                    activity.ObjectID = feed.ObjectID;
                                    notification.ObjectType = (int)Constants.Notification.ObjectType.Image;
                                    notification.ObjectID = feed.ObjectID;
                                    notification.OwnerID = feed.OwnerID;
                                    var imageFeed = _rePhoto.Single<Photo>(p => p.ID == feed.ObjectID);
                                    if (imageFeed != null)
                                    {
                                        imageFeed.CommentCount++;
                                        userComment.OwnerID = imageFeed.UserID;
                                        notification.OwnerID = imageFeed.UserID;
                                        _rePhoto.Update(imageFeed);
                                    }
                                    break;
                                case (int) Constants.Feed.ObjectType.Album:
                                    userComment.ObjectType = (int) Constants.Comment.ObjectType.Album;
                                    userComment.ObjectID = feed.ObjectID;
                                    activity.ObjectType = (int) Constants.Activity.ObjectType.Album;
                                    activity.ObjectID = feed.ObjectID;
                                    notification.ObjectType = (int)Constants.Notification.ObjectType.Album;
                                    notification.ObjectID = feed.ObjectID;
                                    notification.OwnerID = feed.OwnerID;
                                    var albumFeed = _rePhoto.Single<Social.Entity.TikPhoto.Album>(al => al.ID == feed.ObjectID);
                                    if (albumFeed != null)
                                    {
                                        albumFeed.CommentCount++;
                                        userComment.OwnerID = albumFeed.UserID;
                                        notification.OwnerID = albumFeed.UserID;
                                        _rePhoto.Update(albumFeed);
                                    }
                                    break;
                                case (int)Constants.Feed.ObjectType.Note:
                                    userComment.ObjectType = (int)Constants.Comment.ObjectType.Note;
                                    userComment.ObjectID = feed.ObjectID;
                                    activity.ObjectType = (int)Constants.Activity.ObjectType.Note;
                                    activity.ObjectID = feed.ObjectID;
                                    notification.ObjectType = (int)Constants.Notification.ObjectType.Note;
                                    notification.ObjectID = feed.ObjectID;
                                    notification.OwnerID = feed.OwnerID;
                                    var noteFeed = _reApplication.Single<Note>(n => n.ID == feed.ObjectID);
                                    if (noteFeed != null)
                                    {
                                        noteFeed.CommentCount++;
                                        userComment.OwnerID = noteFeed.UserID;
                                        notification.OwnerID = noteFeed.UserID;
                                        _reApplication.Update(noteFeed);
                                    }
                                    break;
                                case (int) Constants.Feed.ObjectType.Status:
                                    userComment.ObjectType = (int) Constants.Comment.ObjectType.Status;
                                    userComment.ObjectID = feed.ID;
                                    userComment.OwnerID = feed.OwnerID;
                                    activity.ObjectType = (int) Constants.Activity.ObjectType.Status;
                                    activity.ObjectID = feed.ID;
                                    notification.ObjectType = (int)Constants.Notification.ObjectType.Status;
                                    notification.ObjectID = feed.ObjectID;
                                    notification.OwnerID = feed.OwnerID;
                                    break;
                                case (int) Constants.Feed.ObjectType.Post:
                                    userComment.OwnerID = feed.TargetID;
                                    if (feed.AttachmentType == (int) Constants.Feed.AttachmentType.Link ||
                                        feed.AttachmentType == (int) Constants.Feed.AttachmentType.Video)
                                    {
                                        userComment.ObjectType = (int) Constants.Comment.ObjectType.Link;
                                        userComment.ObjectID = feed.ID;
                                        activity.ObjectType = (int) Constants.Activity.ObjectType.Link;
                                        activity.ObjectID = feed.ID;
                                        notification.ObjectType = (int)Constants.Notification.ObjectType.Link;
                                        notification.ObjectID = feed.ID;
                                        notification.OwnerID = feed.OwnerID;
                                    }
                                    else
                                    {
                                        userComment.ObjectType = (int) Constants.Comment.ObjectType.Post;
                                        userComment.ObjectID = feed.ID;
                                        activity.ObjectType = (int) Constants.Activity.ObjectType.Post;
                                        activity.ObjectID = feed.ID;
                                        notification.ObjectType = (int)Constants.Notification.ObjectType.Post;
                                        notification.ObjectID = feed.ID;
                                        notification.OwnerID = feed.OwnerID;
                                    }
                                    break;
                                default:
                                    userComment.ObjectType = (int) Constants.Comment.ObjectType.Feed;
                                    userComment.ObjectID = feed.ID;
                                    userComment.OwnerID = feed.TargetID;
                                    activity.ObjectType = (int) Constants.Activity.ObjectType.Feed;
                                    activity.ObjectID = feed.ID;
                                    notification.ObjectType = (int)Constants.Notification.ObjectType.Feed;
                                    notification.ObjectID = feed.ID;
                                    notification.OwnerID = feed.OwnerID;
                                    break;
                            }

                            feed.CommentCount++;
                            _re.Update<UserFeed>(feed);
                        }

                        break;

                    case Constants.Comment.ObjectType.Image:
                        userComment.ObjectType = (int) Constants.Comment.ObjectType.Image;
                        userComment.ObjectID = objectid;
                        activity.ObjectType = (int) Constants.Activity.ObjectType.Image;
                        activity.ObjectID = objectid;
                        notification.ObjectType = (int)Constants.Notification.ObjectType.Image;
                        notification.ObjectID = objectid;
                        var image = _rePhoto.Single<Photo>(p => p.ID == objectid);
                        if (image != null)
                        {
                            image.CommentCount++;
                            userComment.OwnerID = image.UserID;
                            notification.OwnerID = image.UserID;
                           // if (image.UserID == currentUser.ID) addNotification = false;
                            _rePhoto.Update(image);
                        }
                        break;
                    case Constants.Comment.ObjectType.Album:
                        userComment.ObjectType = (int) Constants.Comment.ObjectType.Album;
                        userComment.ObjectID = objectid;
                        activity.ObjectType = (int) Constants.Activity.ObjectType.Album;
                        activity.ObjectID = objectid;
                        notification.ObjectType = (int)Constants.Notification.ObjectType.Album;
                        notification.ObjectID = objectid;
                        var album = _rePhoto.Single<Social.Entity.TikPhoto.Album>(al => al.ID == objectid);
                        if (album != null)
                        {
                            album.CommentCount++;
                            userComment.OwnerID = album.UserID;
                            notification.OwnerID = album.UserID;
                            //if (album.UserID == currentUser.ID) addNotification = false;
                            _rePhoto.Update(album);
                        }
                        break;
                    case Constants.Comment.ObjectType.Note:
                        userComment.ObjectType = (int) Constants.Comment.ObjectType.Note;
                        userComment.ObjectID = objectid;
                        activity.ObjectType = (int)Constants.Activity.ObjectType.Note;
                        activity.ObjectID = objectid;
                        notification.ObjectType = (int)Constants.Notification.ObjectType.Note;
                        notification.ObjectID = objectid;
                        var note = _reApplication.Single<Note>(n => n.ID == objectid);
                        if (note != null)
                        {
                            note.CommentCount++;
                            userComment.OwnerID = note.UserID;
                            notification.OwnerID = note.UserID;
                            //if (note.UserID == currentUser.ID) addNotification = false;
                            _reApplication.Update(note);
                        }
                        break;
                    case Constants.Comment.ObjectType.AlbumMusic:
                        userComment.ObjectType = (int)Constants.Comment.ObjectType.AlbumMusic;
                        userComment.ObjectID = objectid;
                        activity.ObjectType = (int)Constants.Activity.ObjectType.AlbumMusic;
                        activity.ObjectID = objectid;
                        notification.ObjectType = (int)Constants.Notification.ObjectType.Note;
                        notification.ObjectID = objectid;
                        var albumMusic = _reMusic.Single<Album>(n => n.ID == objectid);
                        if(albumMusic != null)
                        {
                            albumMusic.CommentCount++;
                            userComment.OwnerID = albumMusic.UserID;
                            notification.OwnerID = albumMusic.UserID;
                            //if (albumMusic.UserID == currentUser.ID) addNotification = false;
                            _reMusic.Update(albumMusic);
                        }
                        break;
                    case Constants.Comment.ObjectType.Music:
                        userComment.ObjectType = (int)Constants.Comment.ObjectType.Music;
                        userComment.ObjectID = objectid;
                        activity.ObjectType = (int)Constants.Activity.ObjectType.Music;
                        activity.ObjectID = objectid;
                        notification.ObjectType = (int)Constants.Notification.ObjectType.Note;
                        notification.ObjectID = objectid;
                        var music = _reMusic.Single<Song>(m => m.ID == objectid);
                        if (music != null)
                        {
                            music.CommentCount++;
                            userComment.OwnerID = music.UserID;
                            notification.OwnerID = music.UserID;
                            //if (music.UserID == currentUser.ID) addNotification = false;
                            _reMusic.Update(music);
                        }
                        break;
                }

                _re.Add<UserComment>(userComment);
                _re.Add<UserActivity>(activity);
                //if(addNotification) _re.Add<UserNotification>(notification);
                _re.Add<UserNotification>(notification);

                //TODO: Code Notification
                foreach (Guid userid in listTagUsers.Distinct())
                {
                    if (!userid.Equals(currentUser.ID))
                    {
                        UserNotification notificationTag = new UserNotification();
                        notificationTag.ID = Guid.NewGuid();
                        notificationTag.IsHidden = false;
                        notificationTag.IsRead = false;
                        notificationTag.CreatedDate = created;
                        notificationTag.OwnerID = userid;
                        notificationTag.TargetType = (int)Constants.Notification.TargetType.User;
                        notificationTag.TargetID = currentUser.ID;
                        notificationTag.NotificationType = (int)Constants.Notification.NotificationType.TagPost;
                        notificationTag.ObjectType = (int)Constants.Notification.ObjectType.Comment;
                        notificationTag.ObjectID = userComment.ID;
                        _re.Add(notificationTag);
                    }
                }

                _re.CommitChanges();
                _rePhoto.CommitChanges();
                _reMusic.CommitChanges();
                _reApplication.CommitChanges();
                return true;
            }
            catch (Exception ex)
            {
                return false;
            }
        }

        public bool DeleteComment(Guid commentid)
        {
            try
            {
                var comment = _re.Single<UserComment>(c => c.ID == commentid);
                switch (comment.ObjectType)
                {
                    case (int)Constants.Comment.ObjectType.Feed:
                        var feed = _re.Single<UserFeed>(f => f.ID == comment.ObjectID);
                        if (feed != null)
                        {
                            feed.CommentCount--;
                            if (feed.CommentCount < 0) feed.CommentCount = 0;
                            _re.Update<UserFeed>(feed);
                            switch (feed.ObjectType)
                            {
                                case (int)Constants.Feed.ObjectType.AlbumMusic:
                                    var albumMusicFeed = _reMusic.Single<Album>(p => p.ID == comment.ObjectID);
                                    if (albumMusicFeed != null)
                                    {
                                        albumMusicFeed.CommentCount--;
                                        if (albumMusicFeed.CommentCount < 0) albumMusicFeed.CommentCount = 0;
                                        _reMusic.Update(albumMusicFeed);
                                    }
                                    break;
                                case (int)Constants.Feed.ObjectType.Music:
                                    var musicFeed = _reMusic.Single<Song>(p => p.ID == comment.ObjectID);
                                    if (musicFeed != null)
                                    {
                                        musicFeed.CommentCount--;
                                        if (musicFeed.CommentCount < 0) musicFeed.CommentCount = 0;
                                        _reMusic.Update(musicFeed);
                                    }
                                    break;
                                case (int)Constants.Feed.ObjectType.Image:
                                    var imageFeed = _rePhoto.Single<Photo>(al => al.ID == comment.ObjectID);
                                    if (imageFeed != null)
                                    {
                                        imageFeed.CommentCount--;
                                        if (imageFeed.CommentCount < 0) imageFeed.CommentCount = 0;
                                        _rePhoto.Update(imageFeed);
                                    }
                                    break;
                                case (int)Constants.Feed.ObjectType.Album:
                                    var albumFeed = _rePhoto.Single<Social.Entity.TikPhoto.Album>(al => al.ID == comment.ObjectID);
                                    if (albumFeed != null)
                                    {
                                        albumFeed.CommentCount--;
                                        if (albumFeed.CommentCount < 0) albumFeed.CommentCount = 0;
                                        _rePhoto.Update(albumFeed);
                                    }
                                    break;
                                case (int)Constants.Feed.ObjectType.Note:
                                    var noteFeed = _reApplication.Single<Note>(n => n.ID == comment.ObjectID);
                                    if (noteFeed != null)
                                    {
                                        noteFeed.CommentCount--;
                                        if (noteFeed.CommentCount < 0) noteFeed.CommentCount = 0;
                                        _reApplication.Update(noteFeed);
                                    }
                                    break;
                            }
                        }
                        break;
                    case (int)Constants.Comment.ObjectType.Image:
                        var image = _rePhoto.Single<Photo>(al => al.ID == comment.ObjectID);
                        if (image != null)
                        {
                            image.CommentCount--;
                            if (image.CommentCount < 0) image.CommentCount = 0;
                            _rePhoto.Update(image);
                        }
                        break;
                    case (int)Constants.Comment.ObjectType.Album:
                        var album = _rePhoto.Single<Social.Entity.TikPhoto.Album>(al => al.ID == comment.ObjectID);
                        if (album != null)
                        {
                            album.CommentCount--;
                            if (album.CommentCount < 0) album.CommentCount = 0;
                            _rePhoto.Update(album);
                        }
                        break;
                    case (int)Constants.Comment.ObjectType.Note:
                        var note = _reApplication.Single<Note>(n => n.ID == comment.ObjectID);
                        if (note != null)
                        {
                            note.CommentCount--;
                            if (note.CommentCount < 0) note.CommentCount = 0;
                            _reApplication.Update(note);
                        }
                        break;
                    case (int)Constants.Comment.ObjectType.AlbumMusic:
                        var albumMusic = _reMusic.Single<Album>(n => n.ID == comment.ObjectID);
                        if (albumMusic != null)
                        {
                            albumMusic.CommentCount--;
                            if (albumMusic.CommentCount < 0) albumMusic.CommentCount = 0;
                            _reMusic.Update(albumMusic);
                        }
                        break;
                    case (int)Constants.Comment.ObjectType.Music:
                        var music = _reMusic.Single<Song>(m => m.ID == comment.ObjectID);
                        if (music != null)
                        {
                            music.CommentCount--;
                            if (music.CommentCount < 0) music.CommentCount = 0;
                            _reMusic.Update(music);
                        }
                        break;

                }
                _re.Delete<UserLike>(l=>l.ObjectID == comment.ID);
                _re.Delete<UserComment>(c => c.ID == commentid);
                _re.Delete<UserActivity>(
                    a =>
                    a.ActivityType == (int) Constants.Activity.ActivityType.Comment &&
                    a.TargetType == (int) Constants.Activity.TargetType.Comment 
                    && a.TargetID == comment.ID);
                _re.CommitChanges();
                _rePhoto.CommitChanges();
                _reMusic.CommitChanges();
                _reApplication.CommitChanges();
                return true;
            }catch(Exception ex)
            {
                return false;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="currentUser"></param>
        /// <param name="type"></param>
        /// <param name="objectid"></param>
        /// <param name="created"></param>
        /// <returns></returns>
        public bool Like(UserProfile currentUser, Constants.Like.ObjectType type, Guid objectid, DateTime created)
        {
            try
            {
                UserLike like = new UserLike();
                like.ID = Guid.NewGuid();
                like.LikeDate = created;
                like.FromID = currentUser.ID;
                like.FromName = currentUser.FullName;

                UserActivity activity = new UserActivity();
                activity.ID = Guid.NewGuid();
                activity.CreatedDate = created;
                activity.OwnerID = currentUser.ID;
                activity.HideOwner = false;
                activity.ActivityType = (int)Constants.Activity.ActivityType.Like;
                activity.TargetType = (int)Constants.Activity.TargetType.Like;
                activity.TargetID = like.ID;


                UserNotification notification = new UserNotification();
                notification.ID = Guid.NewGuid();
                notification.CreatedDate = created;
                notification.TargetType = (int)Constants.Notification.TargetType.User;
                notification.TargetID = currentUser.ID;
                notification.NotificationType = (int)Constants.Notification.NotificationType.Like;
                notification.IsHidden = false;
                notification.IsRead = false;

                bool addNotification = true;
                switch (type)
                {
                    case Constants.Like.ObjectType.Feed:
                        var feed = _re.Single<UserFeed>(f => f.ID == objectid);
                        if (feed != null)
                        {
                            if (feed.OwnerID == currentUser.ID)
                            {
                                addNotification = false;
                            }

                            switch (feed.ObjectType)
                            {
                                case (int)Constants.Feed.ObjectType.AlbumMusic:
                                    like.ObjectType = (int)Constants.Like.ObjectType.AlbumMusic;
                                    like.ObjectID = feed.ObjectID;
                                    activity.ObjectType = (int)Constants.Activity.ObjectType.AlbumMusic;
                                    activity.ObjectID = feed.ObjectID;
                                    notification.ObjectType = (int) Constants.Notification.ObjectType.AlbumMusic;
                                    notification.ObjectID = feed.ObjectID;
                                    notification.OwnerID = feed.OwnerID;
                                    var albumMusicFeed = _reMusic.Single<Album>(s => s.ID == feed.ObjectID);
                                    if (albumMusicFeed != null)
                                    {
                                        albumMusicFeed.LikeCount++;
                                        notification.OwnerID = albumMusicFeed.UserID;
                                        _reMusic.Update(albumMusicFeed);
                                    }
                                    break;
                                case (int)Constants.Feed.ObjectType.Music:
                                    like.ObjectType = (int)Constants.Like.ObjectType.Music;
                                    like.ObjectID = feed.ObjectID;
                                    activity.ObjectType = (int)Constants.Activity.ObjectType.Music;
                                    activity.ObjectID = feed.ObjectID;
                                    notification.ObjectType = (int)Constants.Notification.ObjectType.Music;
                                    notification.ObjectID = feed.ObjectID;
                                    notification.OwnerID = feed.OwnerID;
                                    var musicFeed = _reMusic.Single<Song>(s => s.ID == feed.ObjectID);
                                    if (musicFeed != null)
                                    {
                                        musicFeed.LikeCount++;
                                        notification.OwnerID = musicFeed.UserID;
                                        _reMusic.Update(musicFeed);
                                    }
                                    break;
                                case (int)Constants.Feed.ObjectType.Image:
                                    like.ObjectType = (int)Constants.Like.ObjectType.Image;
                                    like.ObjectID = feed.ObjectID;
                                    activity.ObjectType = (int)Constants.Activity.ObjectType.Image;
                                    activity.ObjectID = feed.ObjectID;
                                    notification.ObjectType = (int)Constants.Notification.ObjectType.Image;
                                    notification.ObjectID = feed.ObjectID;
                                    notification.OwnerID = feed.OwnerID;
                                    var imageFeed = _rePhoto.Single<Photo>(p => p.ID == feed.ObjectID);
                                    if (imageFeed != null)
                                    {
                                        imageFeed.LikeCount++;
                                        notification.OwnerID = imageFeed.UserID;
                                        _rePhoto.Update(imageFeed);
                                    }
                                    break;
                                case (int)Constants.Feed.ObjectType.Album:
                                    like.ObjectType = (int)Constants.Like.ObjectType.Album;
                                    like.ObjectID = feed.ObjectID;
                                    activity.ObjectType = (int)Constants.Activity.ObjectType.Album;
                                    activity.ObjectID = feed.ObjectID;
                                    notification.ObjectType = (int)Constants.Notification.ObjectType.Album;
                                    notification.ObjectID = feed.ObjectID;
                                    notification.OwnerID = feed.OwnerID;
                                    var albumFeed = _rePhoto.Single<Social.Entity.TikPhoto.Album>(a => a.ID == feed.ObjectID);
                                    if (albumFeed != null)
                                    {
                                        albumFeed.LikeCount++;
                                        notification.OwnerID = albumFeed.UserID;
                                        _rePhoto.Update(albumFeed);
                                    }
                                    break;
                                case (int)Constants.Feed.ObjectType.Note:
                                    like.ObjectType = (int)Constants.Like.ObjectType.Note;
                                    like.ObjectID = feed.ObjectID;
                                    activity.ObjectType = (int)Constants.Activity.ObjectType.Note;
                                    activity.ObjectID = feed.ObjectID;
                                    notification.ObjectType = (int)Constants.Notification.ObjectType.Note;
                                    notification.ObjectID = feed.ObjectID;
                                    notification.OwnerID = feed.OwnerID;
                                    var noteFeed = _reApplication.Single<Note>(n => n.ID == feed.ObjectID);
                                    if (noteFeed != null)
                                    {
                                        noteFeed.LikeCount++;
                                        notification.OwnerID = noteFeed.UserID;
                                        _reApplication.Update(noteFeed);
                                    }
                                    break;
                                case (int)Constants.Feed.ObjectType.Post:
                                    if (feed.AttachmentType == (int)Constants.Feed.AttachmentType.Link ||
                                        feed.AttachmentType == (int)Constants.Feed.AttachmentType.Video)
                                    {
                                        like.ObjectType = (int)Constants.Comment.ObjectType.Link;
                                        like.ObjectID = feed.ID;
                                        activity.ObjectType = (int)Constants.Activity.ObjectType.Link;
                                        activity.ObjectID = feed.ID;
                                        notification.ObjectType = (int)Constants.Notification.ObjectType.Link;
                                        notification.ObjectID = feed.ID;
                                        notification.OwnerID = feed.OwnerID;
                                    }
                                    else
                                    {
                                        like.ObjectType = (int)Constants.Comment.ObjectType.Post;
                                        like.ObjectID = feed.ID;
                                        activity.ObjectType = (int)Constants.Activity.ObjectType.Post;
                                        activity.ObjectID = feed.ID;
                                        notification.ObjectType = (int)Constants.Notification.ObjectType.Post;
                                        notification.ObjectID = feed.ID;
                                        notification.OwnerID = feed.OwnerID;
                                    }
                                    break;
                                default:
                                    like.ObjectType = (int)Constants.Like.ObjectType.Feed;
                                    like.ObjectID = feed.ID;
                                    activity.ObjectType = (int)Constants.Activity.ObjectType.Feed;
                                    activity.ObjectID = feed.ID;
                                    notification.ObjectType = (int) Constants.Notification.ObjectType.Feed;
                                    notification.ObjectID = feed.ObjectID;
                                    notification.OwnerID = feed.OwnerID;
                                    break;
                            }

                            feed.LikeCount++;
                            _re.Update<UserFeed>(feed);
                        }
                        break;
                    case Constants.Like.ObjectType.Image:
                        like.ObjectType = (int)Constants.Like.ObjectType.Image;
                        like.ObjectID = objectid;
                        activity.ObjectType = (int)Constants.Activity.ObjectType.Image;
                        activity.ObjectID = objectid;
                        notification.ObjectType = (int)Constants.Notification.ObjectType.Image;
                        notification.ObjectID = objectid;
                        var image = _rePhoto.Single<Photo>(a => a.ID == objectid);
                        if (image != null)
                        {
                            image.LikeCount++;
                            notification.OwnerID = image.UserID;
                            if (image.UserID == currentUser.ID) addNotification = false;
                            _rePhoto.Update(image);
                        }
                        break;
                    case Constants.Like.ObjectType.Album:
                        like.ObjectType = (int)Constants.Like.ObjectType.Album;
                        like.ObjectID = objectid;
                        activity.ObjectType = (int)Constants.Activity.ObjectType.Album;
                        activity.ObjectID = objectid;
                        notification.ObjectType = (int)Constants.Notification.ObjectType.Album;
                        notification.ObjectID = objectid;
                        var album = _rePhoto.Single<Social.Entity.TikPhoto.Album>(a => a.ID == objectid);
                        if (album != null)
                        {
                            album.LikeCount++;
                            notification.OwnerID = album.UserID;
                            if (album.UserID == currentUser.ID) addNotification = false;
                            _rePhoto.Update(album);
                        }
                        break;
                    case Constants.Like.ObjectType.Note:
                        like.ObjectType = (int)Constants.Like.ObjectType.Note;
                        like.ObjectID = objectid;
                        activity.ObjectType = (int)Constants.Activity.ObjectType.Note;
                        activity.ObjectID = objectid;
                        notification.ObjectType = (int)Constants.Notification.ObjectType.Note;
                        notification.ObjectID = objectid;
                        var note = _reApplication.Single<Note>(n => n.ID == objectid);
                        if (note != null)
                        {
                            note.LikeCount++;
                            notification.OwnerID = note.UserID;
                            if (note.UserID == currentUser.ID) addNotification = false;
                            _reApplication.Update(note);
                        }
                        break;
                    case Constants.Like.ObjectType.Comment:
                        like.ObjectType = (int)Constants.Like.ObjectType.Comment;
                        like.ObjectID = objectid;
                        activity.ObjectType = (int)Constants.Activity.ObjectType.Comment;
                        activity.ObjectID = objectid;
                        notification.ObjectType = (int)Constants.Notification.ObjectType.Comment;
                        notification.ObjectID = objectid;
                        var comment = _re.Single<UserComment>(c => c.ID == objectid);
                        if (comment != null)
                        {
                            comment.LikeCount++;
                            notification.OwnerID = comment.FromID;
                            if (comment.FromID == currentUser.ID) addNotification = false;
                            _re.Update<UserComment>(comment);
                        }
                        break;
                    case Constants.Like.ObjectType.Music:
                        like.ObjectType = (int)Constants.Like.ObjectType.Music;
                        like.ObjectID = objectid;
                        activity.ObjectType = (int)Constants.Activity.ObjectType.Music;
                        activity.ObjectID = objectid;
                        notification.ObjectType = (int)Constants.Notification.ObjectType.Music;
                        notification.ObjectID = objectid;
                        var music = _reMusic.Single<Song>(s => s.ID == objectid);
                        if (music != null)
                        {
                            music.LikeCount++;
                            notification.OwnerID = music.UserID;
                            if (music.UserID == currentUser.ID) addNotification = false; 
                            _rePhoto.Update(music);
                        }
                        break;
                    case Constants.Like.ObjectType.AlbumMusic:
                        like.ObjectType = (int)Constants.Like.ObjectType.AlbumMusic;
                        like.ObjectID = objectid;
                        activity.ObjectType = (int)Constants.Activity.ObjectType.AlbumMusic;
                        activity.ObjectID = objectid;
                        notification.ObjectType = (int)Constants.Notification.ObjectType.AlbumMusic;
                        notification.ObjectID = objectid;
                        var albumMusic = _reMusic.Single<Album>(s => s.ID == objectid);
                        if (albumMusic != null)
                        {
                            albumMusic.LikeCount++;
                            notification.OwnerID = albumMusic.UserID;
                            if (albumMusic.UserID == currentUser.ID) addNotification = false; 
                            _reMusic.Update(albumMusic);
                        }
                        break;
                }
                _re.Add<UserLike>(like);
                _re.Add<UserActivity>(activity);
                if (addNotification) _re.Add<UserNotification>(notification);
                _re.CommitChanges();
                _reMusic.CommitChanges();
                _rePhoto.CommitChanges();
                _reApplication.CommitChanges();
                return true;
            }catch(Exception ex)
            {
                return false;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="currentUser"></param>
        /// <param name="type"></param>
        /// <param name="objectid"></param>
        /// <returns></returns>
        public bool DisLike(UserProfile currentUser, Constants.Like.ObjectType type, Guid objectid)
        {
            try
            {
                var like = _re.Single<UserLike>(l=>l.ObjectID == objectid && l.FromID == currentUser.ID);
                switch (type)
                {
                    case Constants.Like.ObjectType.Feed:
                        var feed = _re.Single<UserFeed>(f=>f.ID == objectid);
                        if (feed != null)
                        {
                            switch (feed.ObjectType)
                            {
                                case (int)Constants.Feed.ObjectType.AlbumMusic:
                                    var albumMusicFeed = _reMusic.Single<Album>(p => p.ID == like.ObjectID);
                                    if (albumMusicFeed != null)
                                    {
                                        albumMusicFeed.LikeCount--;
                                        if (albumMusicFeed.LikeCount < 0) albumMusicFeed.LikeCount = 0;
                                        _reMusic.Update(albumMusicFeed);
                                    }
                                    break;
                                case (int)Constants.Feed.ObjectType.Music:
                                    var musicFeed = _reMusic.Single<Song>(p => p.ID == like.ObjectID);
                                    if (musicFeed != null)
                                    {
                                        musicFeed.LikeCount--;
                                        if (musicFeed.LikeCount < 0) musicFeed.LikeCount = 0;
                                        _reMusic.Update(musicFeed);
                                    }
                                    break;
                                case (int)Constants.Feed.ObjectType.Image:
                                    var imageFeed = _rePhoto.Single<Photo>(al => al.ID == like.ObjectID);
                                    if (imageFeed != null)
                                    {
                                        imageFeed.LikeCount--;
                                        if (imageFeed.LikeCount < 0) imageFeed.LikeCount = 0;
                                        _rePhoto.Update(imageFeed);
                                    }
                                    break;
                                case (int)Constants.Feed.ObjectType.Album:
                                    var albumFeed = _rePhoto.Single<Social.Entity.TikPhoto.Album>(al => al.ID == like.ObjectID);
                                    if (albumFeed != null)
                                    {
                                        albumFeed.LikeCount--;
                                        if (albumFeed.LikeCount < 0) albumFeed.LikeCount = 0;
                                        _rePhoto.Update(albumFeed);
                                    }
                                    break;
                                case (int)Constants.Feed.ObjectType.Note:
                                    var noteFeed = _reApplication.Single<Note>(n => n.ID == like.ObjectID);
                                    if (noteFeed != null)
                                    {
                                        noteFeed.LikeCount--;
                                        if (noteFeed.LikeCount < 0) noteFeed.LikeCount = 0;
                                        _reApplication.Update(noteFeed);
                                    }
                                    break;
                            }
                            _re.Delete<UserLike>(l=>l.ObjectID == feed.ObjectID && l.FromID == currentUser.ID);
                            feed.LikeCount--;
                            if (feed.LikeCount < 0) feed.LikeCount = 0;
                            _re.Update<UserFeed>(feed);
                        }
                        break;
                    case Constants.Like.ObjectType.Image:
                        var image = _rePhoto.Single<Photo>(al => al.ID == like.ObjectID);
                        if (image != null)
                        {
                            image.LikeCount--;
                            if (image.LikeCount < 0) image.LikeCount = 0;
                            _rePhoto.Update(image);
                        }
                        break;
                    case Constants.Like.ObjectType.Album:
                        var album = _rePhoto.Single<Social.Entity.TikPhoto.Album>(al => al.ID == like.ObjectID);
                        if (album != null)
                        {
                            album.LikeCount--;
                            if (album.LikeCount < 0) album.LikeCount = 0;
                            _rePhoto.Update(album);
                        }
                        break;
                    case Constants.Like.ObjectType.Note:
                        var note = _reApplication.Single<Note>(n => n.ID == like.ObjectID);
                        if (note != null)
                        {
                            note.LikeCount--;
                            if (note.LikeCount < 0) note.LikeCount = 0;
                            _reApplication.Update(note);
                        }
                        break;
                    case Constants.Like.ObjectType.AlbumMusic:
                        var albumMusic = _reMusic.Single<Album>(n => n.ID == like.ObjectID);
                        if (albumMusic != null)
                        {
                            albumMusic.LikeCount--;
                            if (albumMusic.LikeCount < 0) albumMusic.LikeCount = 0;
                            _reMusic.Update(albumMusic);
                        }
                        break;
                    case Constants.Like.ObjectType.Music:
                        var music = _reMusic.Single<Song>(m => m.ID == like.ObjectID);
                        if (music != null)
                        {
                            music.LikeCount--;
                            if (music.LikeCount < 0) music.LikeCount = 0;
                            _reMusic.Update(music);
                        }
                        break;
                    case Constants.Like.ObjectType.Comment:
                        var comment = _re.Single<UserComment>(c=>c.ID == objectid);
                        if (comment != null)
                        {
                            comment.LikeCount--;
                            if (comment.LikeCount < 0) comment.LikeCount = 0;
                            _re.Update<UserComment>(comment);
                        }
                        break;
                }
                _re.Delete<UserLike>(l=>l.ObjectID == objectid && l.FromID == currentUser.ID);
                if (like != null)
                {
                    _re.Delete<UserActivity>(
                        a =>
                        a.ActivityType == (int) Constants.Activity.ActivityType.Like &&
                        a.TargetType == (int) Constants.Activity.TargetType.Like && 
                        a.TargetID == like.ID);
                }
                _re.CommitChanges();
                _rePhoto.CommitChanges();
                _reMusic.CommitChanges();
                _reApplication.CommitChanges();
                return true;
            }catch(Exception ex)
            {
                return false;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="feedid"></param>
        /// <returns></returns>
        public bool DeleteFeed(Guid feedid)
        {
            try
            {
                var feed = _re.Single<UserFeed>(f => f.ID == feedid);
                if (feed == null) return false;

                string[] imagePath = new string[5];
                switch (feed.AttachmentType)
                {
                    case (int) Constants.Feed.AttachmentType.Link:
                        {
                            Attachment.Link link = new Attachment.Link(feed.AttachmentParam);
                            imagePath[0] = link.Image;
                        }
                        break;
                    case (int) Constants.Feed.AttachmentType.Video:
                        {
                            Attachment.Video link = new Attachment.Video(feed.AttachmentParam);
                            imagePath[0] = link.Image;
                        }
                        break;
                    case (int) Constants.Feed.AttachmentType.Image:
                        {
                            Attachment.Image image = new Attachment.Image(feed.AttachmentParam);
                            imagePath[0] = image.ImagePath;
                        }
                        break;
                    case (int) Constants.Feed.AttachmentType.Album:
                        break;
                }
                

                //Delete Comment
                var comments = from c in _re.All<UserComment>()
                               where c.ObjectID == feedid
                               select c.ID;
                //Delete Like comment && like feed
                _re.Delete<UserLike>(l => comments.Contains(l.ObjectID) || l.ObjectID == feedid);
                //Delete all comment
                _re.Delete<UserComment>(c => c.ObjectID == feedid);
                //delete activity
                _re.Delete<UserActivity>(a => a.ObjectType == (int)Constants.Activity.ObjectType.Feed && a.ObjectID == feedid);
                //delete notification
                _re.Delete<UserNotification>(n=>n.ObjectID == feedid);

                _re.CommitChanges();
                //Delete all image cache
                try
                {
                    foreach (string img in imagePath)
                    {
                        Lib.Activity.Feed.DeleleImage(img);
                    }
                }
                catch (Exception ex2)
                {
                }
                return true;
            }catch(Exception ex)
            {
                return false;
            }
        }
        
        public bool DeleteObject(Guid objectid)
        {
            try
            {
                //Delete Feed
                var feeds = from f in _re.All<UserFeed>()
                            where f.ObjectID == objectid
                            select f.ID;
                var feedList = feeds.ToList();
                foreach (Guid feedid in feedList)
                {
                    var feed = _re.Single<UserFeed>(f => f.ID == feedid);
                    if (feed != null)
                    {

                        string[] imagePath = new string[5];
                        switch (feed.AttachmentType)
                        {
                            case (int) Constants.Feed.AttachmentType.Link:
                                {
                                    Attachment.Link link = new Attachment.Link(feed.AttachmentParam);
                                    imagePath[0] = link.Image;
                                }
                                break;
                            case (int) Constants.Feed.AttachmentType.Video:
                                {
                                    Attachment.Video link = new Attachment.Video(feed.AttachmentParam);
                                    imagePath[0] = link.Image;
                                }
                                break;
                            case (int) Constants.Feed.AttachmentType.Image:
                                {
                                    Attachment.Image image = new Attachment.Image(feed.AttachmentParam);
                                    imagePath[0] = image.ImagePath;
                                }
                                break;
                            case (int) Constants.Feed.AttachmentType.Album:
                                break;
                        }
                    }
                }
                //Delete all Feed
                _re.Delete<UserFeed>(f=>feedList.Contains(f.ID));

                //Delete Comment
                var comments = from c in _re.All<UserComment>()
                               where c.ObjectID == objectid
                               select c.ID;
                //Delete Like comment && like feed
                _re.Delete<UserLike>(l => comments.Contains(l.ObjectID) || l.ObjectID == objectid);
                //Delete all comment
                _re.Delete<UserComment>(c => c.ObjectID == objectid);

                //Delete Activity
                _re.Delete<UserActivity>(a => a.ObjectID == objectid);

                //Delete Notification
                _re.Delete<UserNotification>(n => n.ObjectID == objectid);

                return true;
            }catch(Exception ex)
            {
                return false;
            }
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="ownerid"></param>
        /// <param name="friendid"></param>
        /// <param name="created"></param>
        /// <returns></returns>
        public bool MakeFriend(Guid ownerid, Guid friendid, DateTime created)
        {
            if (Guid.Empty.Equals(ownerid) || Guid.Empty.Equals(friendid)) return false;
                
            UserActivity activity = new UserActivity();
            activity.ID = Guid.NewGuid();
            activity.HideOwner = false;
            activity.HideTarget = false;
            activity.OwnerID = ownerid;
            activity.TargetType = (int) Social.Lib.Activity.Constants.Activity.TargetType.User;
            activity.TargetID = friendid;
            activity.ObjectType = (int)Social.Lib.Activity.Constants.Activity.TargetType.User;
            activity.ObjectID = friendid;
            activity.ActivityType = (int) Social.Lib.Activity.Constants.Activity.ActivityType.Friend;
            activity.CreatedDate = created;
            _re.Add(activity);

            UserNotification notification = new UserNotification();
            notification.ID = Guid.NewGuid();
            notification.NotificationType = (int)Constants.Notification.NotificationType.Friend;
            notification.TargetType = (int) Constants.Notification.TargetType.User;
            notification.TargetID = friendid;
            notification.OwnerID = ownerid;
            notification.CreatedDate = created;
            notification.IsHidden = false;
            notification.IsRead = false;
            notification.ObjectType = (int) Constants.Notification.ObjectType.User;
            notification.ObjectID = friendid;
            _re.Add(notification);
            _re.CommitChanges();
            return true;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="ownerid"></param>
        /// <param name="friendid"></param>
        /// <param name="type"></param>
        /// <param name="objectid"></param>
        /// <param name="created"></param>
        /// <returns></returns>
        public bool TagFriend(Guid ownerid, Guid friendid, Constants.Activity.ObjectType type, Guid objectid, DateTime created)
        {
            try
            {
                UserActivity activity = new UserActivity();
                activity.ID = Guid.NewGuid();
                activity.ActivityType = (int) Constants.Activity.ActivityType.Tag;
                activity.HideOwner = false;
                activity.HideTarget = false;
                activity.ObjectType = (byte) type;
                activity.ObjectID = objectid;
                activity.CreatedDate = created;
                activity.TargetType = (int)Constants.Activity.TargetType.User;
                activity.TargetID = friendid;
                _re.Add(activity);

                //TODO: Add Notification here

                _re.CommitChanges();
                return true;
            }catch(Exception ex)
            {
                return false;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="ownerid"></param>
        /// <param name="friendids"></param>
        /// <param name="type"></param>
        /// <param name="objectid"></param>
        /// <param name="created"></param>
        /// <returns></returns>
        public bool TagFriend(Guid ownerid, List<Guid> friendids, Constants.Activity.ObjectType type, Guid objectid, DateTime created, bool removeIfNotExist = false)
        {
            try
            {
                Constants.Notification.ObjectType notificationObjectType = (Constants.Notification.ObjectType)((int)type);
                switch (type)
                {
                    case Constants.Activity.ObjectType.Image:
                        notificationObjectType = Constants.Notification.ObjectType.Image;
                        break;
                    case Constants.Activity.ObjectType.Album:
                        notificationObjectType = Constants.Notification.ObjectType.Album;
                        break;
                    case Constants.Activity.ObjectType.Note:
                        notificationObjectType = Constants.Notification.ObjectType.Note;
                        break;
                }

                if(removeIfNotExist)
                {

                    var listExistActivity = from a in _re.All<UserActivity>()
                                            where a.ObjectType == (int) type && a.ObjectID == objectid
                                                  && a.OwnerID == ownerid
                                                  && !friendids.Contains(a.TargetID) &&
                                                  a.TargetType == (int) Constants.Activity.TargetType.User
                                            select a;
                    if (listExistActivity.Any())
                    {
                        var listExist = listExistActivity.ToList();
                        foreach (var userActivity in listExist)
                        {
                            _re.Delete(userActivity);
                        }                        
                    }

                    var listExistNotification = from a in _re.All<UserNotification>()
                                                where a.ObjectType == (int)notificationObjectType && a.ObjectID == objectid
                                                  && a.TargetID == ownerid
                                                  && !friendids.Contains(a.OwnerID) &&
                                                  a.TargetType == (int)Constants.Notification.TargetType.User
                                            select a;
                    if (listExistNotification.Any())
                    {
                        var listExist = listExistNotification.ToList();
                        foreach (var userNotification in listExist)
                        {
                            _re.Delete(userNotification);
                        }      
                    }
                }

                var listActivity = from a in _re.All<UserActivity>()
                                   where a.ObjectType == (int)type && a.ObjectID == objectid
                                         && a.OwnerID == objectid
                                         && a.TargetType == (int)Constants.Activity.TargetType.User
                                   select a.TargetID;
                var listUserExist = listActivity.ToList();
                var listUserNotExist = friendids.FindAll(u => !listUserExist.Contains(u));

                var listNotification= from a in _re.All<UserActivity>()
                                   where a.ObjectType == (int)type && a.ObjectID == objectid
                                         && a.TargetID == ownerid
                                         && a.TargetType == (int)Constants.Activity.TargetType.User
                                      select a.OwnerID;
                var listUserExistNotification = listNotification.ToList();
                var listUserNotExistNotification = friendids.FindAll(u => !listUserExistNotification.Contains(u));


                //List<UserActivity> activities = new List<UserActivity>();
                foreach (var friendid in listUserNotExist)
                {
                    UserActivity activity = new UserActivity();
                    activity.ID = Guid.NewGuid();
                    activity.ActivityType = (int) Constants.Activity.ActivityType.Tag;
                    activity.OwnerID = ownerid;
                    activity.HideOwner = false;
                    activity.HideTarget = false;
                    activity.ObjectType = (byte) type;
                    activity.ObjectID = objectid;
                    activity.CreatedDate = created;
                    activity.TargetType = (int) Constants.Activity.TargetType.User;
                    activity.TargetID = friendid;
                    _re.Add(activity);

                    if (ownerid != friendid && listUserNotExistNotification.Contains(friendid))
                    {
                        UserNotification notification = new UserNotification();
                        notification.ID = Guid.NewGuid();
                        notification.IsHidden = false;
                        notification.IsRead = false;
                        notification.CreatedDate = created;
                        notification.TargetType = (int) Constants.Notification.TargetType.User;
                        notification.TargetID = ownerid;
                        notification.OwnerID = friendid;
                        notification.NotificationType = (int) Constants.Notification.NotificationType.Tag;
                        notification.ObjectType = (int)notificationObjectType;
                        notification.ObjectID = objectid;
                        _re.Add(notification);
                    }
                }

                var userListNotification = listUserNotExistNotification.FindAll(u => !listUserNotExist.Contains(u));
                foreach (Guid friendid in userListNotification)
                {
                    UserNotification notification = new UserNotification();
                    notification.ID = Guid.NewGuid();
                    notification.IsHidden = false;
                    notification.IsRead = false;
                    notification.CreatedDate = created;
                    notification.TargetType = (int)Constants.Notification.TargetType.User;
                    notification.TargetID = ownerid;
                    notification.OwnerID = friendid;
                    notification.NotificationType = (int)Constants.Notification.NotificationType.Tag;
                    notification.ObjectType = (int)notificationObjectType;
                    notification.ObjectID = objectid;
                    _re.Add(notification);
                }

                //_re.Add(activities);
                _re.CommitChanges();
                return true;
            }
            catch (Exception ex)
            {
                return false;
            }
        }
        
        /// <summary>
        /// Delete Tag friend
        /// </summary>
        /// <param name="ownerid"></param>
        /// <param name="friendids"></param>
        /// <param name="type"></param>
        /// <param name="objectid"></param>
        /// <returns></returns>
        public bool DeleteTagFriend(Guid ownerid, List<Guid> friendids, Constants.Activity.ObjectType type, Guid objectid)
        {
            try
            {
                Constants.Notification.ObjectType notificationObjectType = (Constants.Notification.ObjectType) ((int) type);
                switch (type)
                {
                    case Constants.Activity.ObjectType.Image:
                        notificationObjectType = Constants.Notification.ObjectType.Image;
                        break;
                    case Constants.Activity.ObjectType.Album:
                        notificationObjectType = Constants.Notification.ObjectType.Album;
                        break;
                    case Constants.Activity.ObjectType.Note:
                        notificationObjectType = Constants.Notification.ObjectType.Note;
                        break;
                }

                foreach (var friendid in friendids)
                {
                    var friend = friendid;
                    _re.Delete<UserActivity>(a => a.OwnerID == ownerid
                                                && a.TargetID == friend
                                                && a.TargetType == (int)Constants.Activity.TargetType.User
                                                && a.ObjectID == objectid
                                                && a.ObjectType == (int)type
                                                );

                    _re.Delete<UserNotification>(n => n.OwnerID == friend
                                                && n.TargetType == (int)Constants.Activity.TargetType.User
                                                && n.TargetID ==ownerid
                                                && n.ObjectID == objectid
                                                && n.ObjectType == (int)notificationObjectType
                                                );
                }
                _re.CommitChanges();
                return true;
            }catch(Exception ex)
            {
                return false;
            }
        }

        public bool DeleteTagFriend(List<Guid> friendids, Constants.Activity.ObjectType type, Guid objectid)
        {
            try
            {
                Constants.Notification.ObjectType notificationObjectType = (Constants.Notification.ObjectType)((int)type);
                switch (type)
                {
                    case Constants.Activity.ObjectType.Image:
                        notificationObjectType = Constants.Notification.ObjectType.Image;
                        break;
                    case Constants.Activity.ObjectType.Album:
                        notificationObjectType = Constants.Notification.ObjectType.Album;
                        break;
                    case Constants.Activity.ObjectType.Note:
                        notificationObjectType = Constants.Notification.ObjectType.Note;
                        break;
                }

                foreach (var friendid in friendids)
                {
                    var friend = friendid;
                    _re.Delete<UserActivity>(a =>a.TargetID == friend
                                                && a.TargetType == (int)Constants.Activity.TargetType.User
                                                && a.ObjectID == objectid
                                                && a.ObjectType == (int)type
                                                );

                    _re.Delete<UserNotification>(n => n.OwnerID == friend
                                                && n.ObjectID == objectid
                                                && n.ObjectType == (int)notificationObjectType
                                                );
                }
                _re.CommitChanges();
                return true;
            }
            catch (Exception ex)
            {
                return false;
            }
        }


        /// <summary>
        /// 
        /// </summary>
        /// <param name="ownerid"></param>
        /// <param name="toUserID"></param>
        /// <param name="message"></param>
        /// <param name="created"></param>
        /// <returns></returns>
        public bool Post(Guid ownerid, Guid toUserID,string message, DateTime created, Guid feedid = new Guid(), Attachment.Link link = null , Attachment.Video video = null)
        {
            try
            {
                //Lay ds user da dc tag trong message
                List<Guid> listTagUsers = Feed.ListUserTagged(message);

                //Add to Feed
                UserFeed feed = new UserFeed();
                if (Guid.Empty.Equals(feedid))
                {
                    feed.ID = Guid.NewGuid();
                }
                else
                {
                    feed.ID = feedid;
                }
                feed.CreatedDate = created;
                feed.UpdatedDate = created;
                feed.FeedType = (byte)(ownerid == toUserID ? (int)Constants.Feed.FeedType.Status : (int)Constants.Feed.FeedType.Post);

                feed.CommentCount = 0;
                feed.LikeCount = 0;

                feed.MessageType = 0;
                feed.MessageParam = string.Empty;

                feed.Message = Social.Lib.Activity.Feed.Message(message);

                feed.OwnerID = ownerid;

                feed.TargetType = (int)Constants.Feed.TargetType.User;
                feed.TargetID = toUserID;

                feed.ObjectType = (byte)(ownerid == toUserID ? (int)Constants.Feed.ObjectType.Status : (int)Constants.Feed.ObjectType.Post);
                feed.ObjectID = feed.ID;

                feed.AttachmentType = (int)Constants.Feed.AttachmentType.None;


                //Add to Activity
                UserActivity activity = new UserActivity();
                activity.ID = Guid.NewGuid();
                activity.CreatedDate = created;

                activity.ActivityType = (byte)(feed.FeedType == (int)Constants.Feed.FeedType.Status ? (int)Constants.Activity.ActivityType.Status : (int)Constants.Activity.ActivityType.Post);
                activity.OwnerID = ownerid;
                activity.HideOwner = false;

                activity.TargetType = (int)Constants.Activity.TargetType.User;
                activity.TargetID = toUserID;

                activity.ObjectType = (byte)Constants.Activity.ObjectType.Feed;
                activity.ObjectID = feed.ID;

                JavaScriptSerializer serializer = new JavaScriptSerializer();
                string output;

                if(link != null)
                {
                    string image = Lib.Activity.Feed.SaveImageUrl(link.Image, feed.ID);
                    if (!string.IsNullOrEmpty(image))
                    {
                        link.Image = image;
                    }
                    output = serializer.Serialize(link);
                    feed.ObjectType = (int)Constants.Feed.ObjectType.Link;
                    feed.AttachmentType = (int) Constants.Feed.AttachmentType.Link;
                    feed.AttachmentParam = output;
                }

                if (video != null)
                {
                    string image = Lib.Activity.Feed.SaveImageUrl(video.Image, feed.ID);
                    if (!string.IsNullOrEmpty(image))
                    {
                        video.Image = image;
                    }
                    output = serializer.Serialize(video);
                    feed.ObjectType = (int)Constants.Feed.ObjectType.Video;
                    feed.AttachmentType = (int)Constants.Feed.AttachmentType.Video;
                    feed.AttachmentParam = output;
                }

                _re.Add(feed);
                _re.Add(activity);

                //TODO: Code Notification
                foreach (Guid userid in listTagUsers.Distinct())
                {
                    if (!userid.Equals(ownerid))
                    {
                        UserNotification notification = new UserNotification();
                        notification.ID = Guid.NewGuid();
                        notification.IsHidden = false;
                        notification.IsRead = false;
                        notification.CreatedDate = created;
                        notification.OwnerID = userid;
                        notification.TargetType = (int) Constants.Notification.TargetType.User;
                        notification.TargetID = ownerid;
                        notification.NotificationType = (int) Constants.Notification.NotificationType.TagPost;
                        notification.ObjectType = (int) Constants.Notification.ObjectType.Feed;
                        notification.ObjectID = feed.ID;
                        _re.Add(notification);
                    }
                }
                _re.CommitChanges();
                return true;
            }catch(Exception ex)
            {
                return false;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="ownerid"></param>
        /// <param name="objectid"></param>
        /// <param name="message"></param>
        /// <param name="title"></param>
        /// <param name="description"></param>
        /// <param name="url"></param>
        /// <param name="image"></param>
        /// <returns></returns>
        public bool PostNote(Guid ownerid, Guid objectid, string message, string title, string description, string url, string image)
        {
            try
            {
                Attachment.Note noteAttachment = new Attachment.Note();
                Guid feedid = Guid.NewGuid();

                noteAttachment.Title = title;
                noteAttachment.Description = description;
                noteAttachment.Url = url;
                if (image != null)
                {
                    noteAttachment.Image = image;
                    if (noteAttachment.Image.StartsWith("http"))
                    {
                        string imagepath = Lib.Activity.Feed.SaveImageUrl(noteAttachment.Image, feedid);
                        if (!string.IsNullOrEmpty(imagepath))
                        {
                            noteAttachment.Image = imagepath;
                        }
                    }
                    else
                    {
                        string imagepath = Lib.Activity.Feed.SaveImage(noteAttachment.Image, feedid);
                        if (!string.IsNullOrEmpty(imagepath))
                        {
                            noteAttachment.Image = imagepath;
                        }
                    }
                }

                DateTime created = DateTime.Now;
                UserFeed feed = new UserFeed();
                feed.ID = feedid;
                feed.CreatedDate = created;
                feed.UpdatedDate = created;
                feed.FeedType = (int)Constants.Feed.FeedType.Post;
                feed.CommentCount = 0;
                feed.LikeCount = 0;
                feed.MessageType = 0;
                feed.MessageParam = string.Empty;
                feed.Message = Social.Lib.Activity.Feed.Message(message);
                feed.OwnerID = ownerid;
                feed.TargetType = (int)Constants.Feed.TargetType.User;
                feed.TargetID = ownerid;

                JavaScriptSerializer serializer = new JavaScriptSerializer();
                string output = serializer.Serialize(noteAttachment);

                feed.AttachmentType = (int) Constants.Feed.AttachmentType.Note;
                feed.AttachmentParam = output;

                UserActivity activity = new UserActivity();
                activity.ID = Guid.NewGuid();
                activity.CreatedDate = created;
                activity.ActivityType = (int)Constants.Activity.ActivityType.Post;
                activity.OwnerID = ownerid;
                activity.TargetType = (int)Constants.Activity.TargetType.User;
                activity.TargetID = ownerid;

                if (Guid.Empty.Equals(objectid))
                {
                    feed.ObjectType = (int)Constants.Feed.ObjectType.Note;
                    feed.ObjectID = feedid;
                    activity.ObjectType = (byte)Constants.Activity.ObjectType.Feed;
                    activity.ObjectID = feedid;
                }
                else
                {
                    feed.ObjectType = (int)Constants.Feed.ObjectType.Note;
                    feed.ObjectID = objectid;
                    activity.ObjectType = (int) Constants.Activity.ObjectType.Feed;
                    activity.ObjectID = feedid;
                }

                //TODO: Code Notification

                _re.Add(feed);
                _re.Add(activity);
                _re.CommitChanges();

                return true;
            }catch(Exception ex)
            {
                return false;
            }
        }

        /// <summary>
        ///  Post Music/Video lên tường
        /// </summary>
        /// <param name="ownerid">Owner ID</param>
        /// <param name="objectid">ID của đối tượng Music hoặc Video (để Guid.Empty nếu ko có)</param>
        /// <param name="message">Tin nhắn hiện trên cùng</param>
        /// <param name="title">Tiêu đề của Music/Video</param>
        /// <param name="description">Chú thích</param>
        /// <param name="url">URL đến module Music hoặc đến trang chứa đối tượng này</param>
        /// <param name="image">Đường dẫn đến ảnh minh họa cho đối tượng</param>
        /// <param name="embedurl">URL để cho vào embed flash</param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <returns>true nếu ok or false nếu not ok :D</returns>
        public bool PostMusic(Guid ownerid, Guid objectid, string message, string title, string description,string url, string image, string embedurl, int width=440, int height=0)
        {
            try
            {
                Attachment.Video videoAttachment = new Attachment.Video();
                videoAttachment.Width = width;
                videoAttachment.Height = height;
                videoAttachment.Title = title;
                videoAttachment.Description = description;
                videoAttachment.EmbedUrl = embedurl;
                videoAttachment.Url = url;
                videoAttachment.Image = image;
                videoAttachment.Type = "";
                Guid feedid = Guid.NewGuid();

                if (image.StartsWith("http"))
                {
                    string imagepath = Lib.Activity.Feed.SaveImageUrl(image, feedid);
                    if (!string.IsNullOrEmpty(imagepath))
                    {
                        videoAttachment.Image = imagepath;
                    }
                }
                else
                {
                    string imagepath = Lib.Activity.Feed.SaveImage(image, feedid);
                    if (!string.IsNullOrEmpty(imagepath))
                    {
                        videoAttachment.Image = imagepath;
                    }
                }

                DateTime created = DateTime.Now;
                UserFeed feed = new UserFeed();
                feed.ID = feedid;
                feed.CreatedDate = created;
                feed.UpdatedDate = created;
                feed.FeedType = (int) Constants.Feed.FeedType.Post;
                feed.CommentCount = 0;
                feed.LikeCount = 0;
                feed.MessageType = 0;
                feed.MessageParam = string.Empty;
                feed.Message = Social.Lib.Activity.Feed.Message(message);
                feed.OwnerID = ownerid;
                feed.TargetType = (int) Constants.Feed.TargetType.User;
                feed.TargetID = ownerid;

                JavaScriptSerializer serializer = new JavaScriptSerializer();
                string output = serializer.Serialize(videoAttachment);


                feed.AttachmentType = (byte) Constants.Feed.AttachmentType.Music;
                feed.AttachmentParam = output;


                UserActivity activity = new UserActivity();
                activity.ID = Guid.NewGuid();
                activity.CreatedDate = created;
                activity.ActivityType = (int) Constants.Activity.ActivityType.Post;
                activity.OwnerID = ownerid;
                activity.TargetType = (int) Constants.Activity.TargetType.User;
                activity.TargetID = ownerid;

                if (Guid.Empty.Equals(objectid))
                {
                    feed.ObjectType = (int)Constants.Feed.ObjectType.Music;
                    feed.ObjectID = feedid;
                    activity.ObjectType = (byte) Constants.Activity.ObjectType.Feed;
                    activity.ObjectID = feedid;
                }
                else
                {
                    feed.ObjectType = (int) Constants.Feed.ObjectType.Music;
                    feed.ObjectID = objectid;
                    activity.ObjectType = (byte)Constants.Activity.ObjectType.Feed;
                    activity.ObjectID = feedid;
                }
                
                //TODO: Code Notification


                _re.Add(feed);
                _re.Add(activity);
                _re.CommitChanges();

                return true;
            }catch(Exception ex)
            {
                return false;
            }
        }

        public bool PostImage(Guid ownerid, Guid objectid, string message, string albumtitle, string imagedescription, string urlalbum, string[] image, string[] urlimage, bool isAlbum = false, int totalImage = 1, Guid feedid = new Guid())
        {
            try
            {
                List<Guid> listTagUsers = Feed.ListUserTagged(message);
                Attachment.Image imageAttachment = new Attachment.Image();
                Attachment.Album albumAttachment = new Attachment.Album();
                if(Guid.Empty.Equals(feedid))
                {
                    feedid = Guid.NewGuid();
                }

                if (isAlbum)
                {
                    albumAttachment.Title = albumtitle;
                    albumAttachment.Description = imagedescription;
                    albumAttachment.Url = urlalbum;
                    albumAttachment.ImagePath = image;
                    albumAttachment.ImageUrl = urlimage;
                    albumAttachment.TotalImage = totalImage;
                    int i = 0;
                    foreach (string imageLink in albumAttachment.ImagePath)
                    {
                        if (imageLink.StartsWith("http"))
                        {
                            string imagepath = Lib.Activity.Feed.SaveImageUrl(imageLink, feedid, i.ToString());
                            if (!string.IsNullOrEmpty(imagepath))
                            {
                                albumAttachment.ImagePath[i] = imagepath;
                            }
                        }
                        else
                        {
                            string imagepath = Lib.Activity.Feed.SaveImage(imageLink, feedid, i.ToString());
                            if (!string.IsNullOrEmpty(imagepath))
                            {
                                albumAttachment.ImagePath[i] = imagepath;
                            }
                        }
                        i++;
                    }
                }
                else
                {
                    imageAttachment.Title = albumtitle;
                    imageAttachment.Description = imagedescription;
                    imageAttachment.ImagePath = image[0];
                    imageAttachment.ImageUrl = urlimage[0];
                    imageAttachment.UrlAlbum = urlalbum;

                    if (imageAttachment.ImagePath.StartsWith("http"))
                    {
                        string imagepath = Lib.Activity.Feed.SaveImageUrl(imageAttachment.ImagePath, feedid);
                        if (!string.IsNullOrEmpty(imagepath))
                        {
                            imageAttachment.ImagePath = imagepath;
                        }
                    }
                    else
                    {
                        string imagepath = Lib.Activity.Feed.SaveImage(imageAttachment.ImagePath, feedid);
                        if (!string.IsNullOrEmpty(imagepath))
                        {
                            imageAttachment.ImagePath = imagepath;
                        }
                    }

                }


                DateTime created = DateTime.Now;
                UserFeed feed = new UserFeed();
                feed.ID = feedid;
                feed.CreatedDate = created;
                feed.UpdatedDate = created;
                feed.FeedType = (int) Constants.Feed.FeedType.Post;
                feed.CommentCount = 0;
                feed.LikeCount = 0;
                feed.MessageType = 0;
                feed.MessageParam = string.Empty;
                feed.Message = Social.Lib.Activity.Feed.Message(message);
                feed.OwnerID = ownerid;
                feed.TargetType = (int) Constants.Feed.TargetType.User;
                feed.TargetID = ownerid;

                JavaScriptSerializer serializer = new JavaScriptSerializer();
                string output = isAlbum ? serializer.Serialize(albumAttachment) : serializer.Serialize(imageAttachment);

                feed.AttachmentType =
                    (byte)
                    (isAlbum ? (int) Constants.Feed.AttachmentType.Album : (int) Constants.Feed.AttachmentType.Image);
                feed.AttachmentParam = output;


                UserActivity activity = new UserActivity();
                activity.ID = Guid.NewGuid();
                activity.CreatedDate = created;
                activity.ActivityType = (int) Constants.Activity.ActivityType.Post;
                activity.OwnerID = ownerid;
                activity.TargetType = (int) Constants.Activity.TargetType.User;
                activity.TargetID = ownerid;

                if (Guid.Empty.Equals(objectid))
                {
                    feed.ObjectType =
                        (byte) (isAlbum ? (int) Constants.Feed.ObjectType.Album : (int) Constants.Feed.ObjectType.Image);
                    feed.ObjectID = feedid;
                    activity.ObjectType = (byte) Constants.Activity.ObjectType.Feed;
                    activity.ObjectID = feedid;
                }
                else
                {
                    feed.ObjectType = (byte) (isAlbum ? (int) Constants.Feed.ObjectType.Album : (int) Constants.Feed.ObjectType.Image);
                    feed.ObjectID = objectid;
                    activity.ObjectType = (byte)Constants.Activity.ObjectType.Feed;
                    activity.ObjectID = feedid;
                }

                //TODO: Code Notification
                foreach (Guid userid in listTagUsers.Distinct())
                {
                    if (!userid.Equals(ownerid))
                    {
                        UserNotification notification = new UserNotification();
                        notification.ID = Guid.NewGuid();
                        notification.IsHidden = false;
                        notification.IsRead = false;
                        notification.OwnerID = userid;
                        notification.TargetType = (int)Constants.Notification.TargetType.User;
                        notification.TargetID = ownerid;
                        notification.NotificationType = (int)Constants.Notification.NotificationType.TagPost;
                        notification.ObjectType = (int)Constants.Notification.ObjectType.Feed;
                        notification.ObjectID = feed.ID;
                        notification.CreatedDate = created;
                        _re.Add(notification);
                    }
                }

                _re.Add(feed);
                _re.Add(activity);
                
                _re.CommitChanges();

                return true;
            }catch(Exception ex)
            {
                return false;
            }
        }
        
        public bool PostImage(Guid ownerid, Guid touserid, Guid objectid, string message, string albumtitle, string imagedescription, string urlalbum, string[] image, string[] urlimage, bool isAlbum = false, int totalImage = 1, Guid feedid = new Guid())
        {
            try
            {
                List<Guid> listTagUsers = Feed.ListUserTagged(message);

                Attachment.Image imageAttachment = new Attachment.Image();
                Attachment.Album albumAttachment = new Attachment.Album();
                if (Guid.Empty.Equals(feedid))
                {
                    feedid = Guid.NewGuid();
                }

                if (isAlbum)
                {
                    albumAttachment.Title = albumtitle;
                    albumAttachment.Description = imagedescription;
                    albumAttachment.Url = urlalbum;
                    albumAttachment.ImagePath = image;
                    albumAttachment.ImageUrl = urlimage;
                    albumAttachment.TotalImage = totalImage;
                    int i = 0;
                    foreach (string imageLink in albumAttachment.ImagePath)
                    {
                        if (imageLink.StartsWith("http"))
                        {
                            string imagepath = Lib.Activity.Feed.SaveImageUrl(imageLink, feedid, i.ToString());
                            if (!string.IsNullOrEmpty(imagepath))
                            {
                                albumAttachment.ImagePath[i] = imagepath;
                            }
                        }
                        else
                        {
                            string imagepath = Lib.Activity.Feed.SaveImage(imageLink, feedid, i.ToString());
                            if (!string.IsNullOrEmpty(imagepath))
                            {
                                albumAttachment.ImagePath[i] = imagepath;
                            }
                        }
                        i++;
                    }
                }
                else
                {
                    imageAttachment.Title = albumtitle;
                    imageAttachment.Description = imagedescription;
                    imageAttachment.ImagePath = image[0];
                    imageAttachment.ImageUrl = urlimage[0];
                    imageAttachment.UrlAlbum = urlalbum;

                    if (imageAttachment.ImagePath.StartsWith("http"))
                    {
                        string imagepath = Lib.Activity.Feed.SaveImageUrl(imageAttachment.ImagePath, feedid);
                        if (!string.IsNullOrEmpty(imagepath))
                        {
                            imageAttachment.ImagePath = imagepath;
                        }
                    }
                    else
                    {
                        string imagepath = Lib.Activity.Feed.SaveImage(imageAttachment.ImagePath, feedid);
                        if (!string.IsNullOrEmpty(imagepath))
                        {
                            imageAttachment.ImagePath = imagepath;
                        }
                    }

                }


                DateTime created = DateTime.Now;
                UserFeed feed = new UserFeed();
                feed.ID = feedid;
                feed.CreatedDate = created;
                feed.UpdatedDate = created;
                feed.FeedType = (int)Constants.Feed.FeedType.Post;
                feed.CommentCount = 0;
                feed.LikeCount = 0;
                feed.MessageType = 0;
                feed.MessageParam = string.Empty;
                feed.Message = Social.Lib.Activity.Feed.Message(message);
                feed.OwnerID = ownerid;
                feed.TargetType = (int)Constants.Feed.TargetType.User;
                feed.TargetID = touserid;

                JavaScriptSerializer serializer = new JavaScriptSerializer();
                string output = isAlbum ? serializer.Serialize(albumAttachment) : serializer.Serialize(imageAttachment);

                feed.AttachmentType =
                    (byte)
                    (isAlbum ? (int)Constants.Feed.AttachmentType.Album : (int)Constants.Feed.AttachmentType.Image);
                feed.AttachmentParam = output;


                UserActivity activity = new UserActivity();
                activity.ID = Guid.NewGuid();
                activity.CreatedDate = created;
                activity.ActivityType = (int)Constants.Activity.ActivityType.Post;
                activity.OwnerID = ownerid;
                activity.TargetType = (int)Constants.Activity.TargetType.User;
                activity.TargetID = touserid;

                if (Guid.Empty.Equals(objectid))
                {
                    feed.ObjectType =
                        (byte)(isAlbum ? (int)Constants.Feed.ObjectType.Album : (int)Constants.Feed.ObjectType.Image);
                    feed.ObjectID = feedid;
                    activity.ObjectType = (byte)Constants.Activity.ObjectType.Feed;
                    activity.ObjectID = feedid;
                }
                else
                {
                    feed.ObjectType = (byte)(isAlbum ? (int)Constants.Feed.ObjectType.Album : (int)Constants.Feed.ObjectType.Image);
                    feed.ObjectID = objectid;
                    activity.ObjectType = (byte)Constants.Activity.ObjectType.Feed;
                    activity.ObjectID = feedid;
                }

                //TODO: Code Notification
                foreach (Guid userid in listTagUsers.Distinct())
                {
                    if (!userid.Equals(ownerid))
                    {
                        UserNotification notification = new UserNotification();
                        notification.ID = Guid.NewGuid();
                        notification.IsHidden = false;
                        notification.IsRead = false;
                        notification.OwnerID = userid;
                        notification.TargetType = (int)Constants.Notification.TargetType.User;
                        notification.TargetID = ownerid;
                        notification.NotificationType = (int)Constants.Notification.NotificationType.TagPost;
                        notification.ObjectType = (int)Constants.Notification.ObjectType.Feed;
                        notification.ObjectID = feed.ID;
                        notification.CreatedDate = created;
                        _re.Add(notification);
                    }
                }

                _re.Add(feed);
                _re.Add(activity);
                
                _re.CommitChanges();

                return true;
            }
            catch (Exception ex)
            {
                return false;
            }
        }
        
        public bool UpdateProfile(Guid ownerid, Constants.Profile.Type profileType, Guid objectid)
        {
            try
            {
                return true;
            }catch(Exception ex)
            {
                return false;
            }
        }

        public bool UpdateAvatar(Guid userid, Guid avatarid, DateTime created)
        {
            try
            {
                UserActivity activity = new UserActivity();
                activity.ID = Guid.NewGuid();
                activity.CreatedDate = created;
                activity.ActivityType = (int)Constants.Activity.ActivityType.Profile;
                activity.OwnerID = userid;
                activity.TargetType = (int)Constants.Activity.TargetType.User;
                activity.TargetID = userid;
                activity.ObjectType = (int) Constants.Profile.Type.Avatar;
                activity.ObjectID = avatarid;
                _re.Add(activity);
                _re.CommitChanges();
                return true;
            }
            catch (Exception ex)
            {
                return false;
            }
        }
    }
}
