﻿using System;
using System.Collections.Generic;
using System.Data.Objects;
using System.Linq;
using Social.DAL.Interface;
using Social.Entity.TikActivity;
using Social.Entity.TikFriend;
using Social.Lib;
using Social.Lib.Activity;
using Social.Service.Interface.TikActivity;
using Social.Service.TikFriend;

namespace Social.Service.TikActivity
{
    public class UserNotificationService : ServiceBase<UserNotification>, IUserNotificationService
    {

        #region Notification Type
        public Dictionary<NotificationType, string> NotificationTypes = new Dictionary<NotificationType, string>()
                                              {
                                                 {NotificationType.Commented, "{0} đã bình luận về {1}"},
                                                 {NotificationType.FriendAccepted, "{0} đã nhận lời mời kết bạn của bạn."}
                                              };

        public enum NotificationType
        {
            Commented = 1,
            FriendAccepted = 2
        }
        #endregion

        UserFriendService userFriend;
        IRepository _re;
        ITikFriendRepository _reFriend;
        public UserNotificationService(ITikActivityRepository repository, ITikFriendRepository friendRepository)
            : base(repository)
        {
            _re = repository;
            _reFriend = friendRepository;
            userFriend = new UserFriendService(friendRepository);
        }


        #region IUserNotificationService Members

        public UserNotification Get(Guid id)
        {
            var notification = from n in _re.All<UserNotification>()
                               where n.ID == id
                               select n;
            if (notification.Count() > 0)
            {
                return notification.FirstOrDefault();
            }
            return null;
        }

        public List<UserNotification> GetAll(Guid ownerid, bool isRead = false, bool isHidden = false)
        {
            DateTime now = DateTime.Now.AddDays(-7);
            var notifications = from n in _re.All<UserNotification>()
                                where n.IsHidden == isHidden && n.IsRead == isRead && n.OwnerID == ownerid
                                      && n.CreatedDate >= now
                                select n;
            if (notifications.Any())
            {
                return notifications.ToList();
            }
            return null;
        }
        public List<UserNotification> GetAll(Guid ownerid, bool isHidden = false)
        {
            DateTime now = DateTime.Now.AddDays(-7);
            var listNotification = new List<UserNotification>();

            var notifications = from n in _re.All<UserNotification>()
                                where n.IsHidden == isHidden && n.OwnerID == ownerid && n.TargetID != ownerid
                                      && n.CreatedDate >= now
                                select n;
            if (notifications.Any())
            {
                listNotification.AddRange(notifications.ToList());
            }
            var notificationOtherUserComment = from n in _re.All<UserNotification>()
                                               where n.IsHidden == isHidden && n.TargetType == (int)Constants.Notification.TargetType.User
                                                    && n.TargetID == ownerid
                                                    && n.NotificationType == (int)Constants.Notification.NotificationType.Comment
                                                    && n.CreatedDate >= now
                                               select n;

            var groupNotificationOtherUserComment = notificationOtherUserComment.GroupBy(n => n.ObjectID).ToList();
            List<Guid> listFriends = userFriend.GetByOwnerID(ownerid);
            foreach (IGrouping<Guid, UserNotification> grouping in groupNotificationOtherUserComment)
            {
                var usernotification = grouping.OrderBy(n => n.CreatedDate).ToList()[0];
                var notification = from n in _re.All<UserNotification>()
                                   where n.IsHidden == isHidden && n.ObjectID == usernotification.ObjectID
                                         && n.OwnerID == usernotification.OwnerID
                                         && n.NotificationType == (int)Constants.Notification.NotificationType.Comment
                                         && n.CreatedDate > usernotification.CreatedDate
                                   select n;
                if (notification.Any(n=>listFriends.Contains(n.TargetID))) listNotification.AddRange(notification.ToList());
            }

            if (listNotification.Count > 0)
            {
                return listNotification;
            }
            return null;
        }

        public bool UpdateRead(Guid ownerid, bool isRead = false, bool isHidden = false)
        {
            var unreadNotifications = from n in _re.All<UserNotification>()
                                      where n.IsRead == false && n.OwnerID == ownerid
                                      select n;
            foreach (UserNotification unreadNotification in unreadNotifications)
            {
                unreadNotification.IsRead = true;
                _re.Update(unreadNotification);
            }

            var notificationOtherUserComment = from n in _re.All<UserNotification>()
                                               where n.TargetType == (int)Constants.Notification.TargetType.User
                                                    && n.TargetID == ownerid
                                                    && n.NotificationType == (int)Constants.Notification.NotificationType.Comment
                                               select n;

            var groupNotificationOtherUserComment = notificationOtherUserComment.GroupBy(n => n.ObjectID).ToList();

            foreach (IGrouping<Guid, UserNotification> grouping in groupNotificationOtherUserComment)
            {
                var usernotification = grouping.OrderBy(n => n.CreatedDate).ToList()[0];

                var notification = from n in _re.All<UserNotification>()
                                   where n.ObjectID == usernotification.ObjectID
                                         && n.OwnerID == usernotification.OwnerID
                                         && n.CreatedDate > usernotification.CreatedDate
                                   select n;

                var notifications = notification.ToList();
                foreach (UserNotification userNotification in notifications)
                {
                    userNotification.IsTargetRead = true;
                    _re.Update(userNotification);
                }
            }

            _re.CommitChanges();
            return true;
        }

        public void SetRead(Guid id)
        {
            var notifications = (from n in _re.All<UserNotification>()
                                where n.ID == id
                                select n).Single();
            
            if(notifications.IsRead)
            {
                notifications.IsRead = true;
                _re.CommitChanges();
            }
           
        }

        public void SetHidden(Guid id)
        {
            var notifications = (from n in _re.All<UserNotification>()
                                 where n.ID == id
                                 select n).Single();

            if (notifications.IsHidden)
            {
                notifications.IsHidden = true;
                _re.CommitChanges();
            }
        }
        #endregion

        public int CountUnread(Guid userid)
        {
            //count friend
            var notificationFriend = from n in _re.All<UserNotification>()
                               where n.IsRead == false && n.OwnerID == userid && n.NotificationType == (int)Constants.Notification.NotificationType.Friend
                               select n;
            int count = 0;
            if(notificationFriend.Any())
            {
                count += notificationFriend.Count();
            }

            //count comment
            var notificationComment= from n in _re.All<UserNotification>()
                                     where n.IsRead == false && n.OwnerID == userid && n.NotificationType == (int)Constants.Notification.NotificationType.Comment && n.TargetID != userid
                                     group n by n.ObjectID into ng  
                                     select ng;
            if(notificationComment.Any())
            {
                count += notificationComment.Count();
            }

            //count like
            var notificationLike = from n in _re.All<UserNotification>()
                                      where n.IsRead == false && n.OwnerID == userid && n.NotificationType == (int)Constants.Notification.NotificationType.Like
                                      group n by n.ObjectID into ng
                                      select ng;
            if (notificationLike.Any())
            {
                count += notificationLike.Count();
            }

            //count comment same item

            var notificationOtherUserComment = from n in _re.All<UserNotification>()
                                               where n.TargetType == (int)Constants.Notification.TargetType.User
                                                    && n.TargetID == userid
                                                    && n.NotificationType == (int)Constants.Notification.NotificationType.Comment
                                               select n;

            var groupNotificationOtherUserComment = notificationOtherUserComment.GroupBy(n => n.ObjectID).ToList();

            List<Guid> listFriends = userFriend.GetByOwnerID(userid);
            foreach (IGrouping<Guid, UserNotification> grouping in groupNotificationOtherUserComment)
            {
                var usernotification = grouping.OrderBy(n => n.CreatedDate).ToList()[0];
                var notification = from n in _re.All<UserNotification>()
                                   where n.ObjectID == usernotification.ObjectID
                                         && n.OwnerID == usernotification.OwnerID
                                         && n.CreatedDate > usernotification.CreatedDate
                                         && n.NotificationType == (int)Constants.Notification.NotificationType.Comment
                                         && n.IsTargetRead == false
                                   select n;
                if (notification.Any(n => listFriends.Contains(n.TargetID))) count++;
            }

            //var notificationUserComment = from n in _re.All<UserNotification>()
            //                       where n.IsTargetRead == false && n.TargetID == userid && n.NotificationType == (int)Constants.Notification.NotificationType.Comment
            //                       group n by n.ObjectID into ng
            //                       select ng;
            //if (notificationUserComment.Any())
            //{
            //    count += notificationLike.Count();
            //}

            //count tag in post, comment
            var notificationTagInPost = from n in _re.All<UserNotification>()
                                   where n.IsRead == false && n.OwnerID == userid && n.NotificationType == (int)Constants.Notification.NotificationType.TagPost
                                   group n by n.ObjectID into ng
                                   select ng;
            if (notificationTagInPost.Any())
            {
                count += notificationTagInPost.Count();
            }

            //count tag in post, comment
            var notificationTag = from n in _re.All<UserNotification>()
                                        where n.IsRead == false && n.OwnerID == userid && n.NotificationType == (int)Constants.Notification.NotificationType.Tag
                                        group n by n.ObjectID into ng
                                        select ng;
            if (notificationTag.Any())
            {
                count += notificationTag.Count();
            }

            return count;
        }
        public bool Add(Guid ownerid, Constants.Notification.NotificationType type, Constants.Notification.ObjectType objectType, Guid objectID, Constants.Notification.TargetType targetType, Guid targetID, DateTime created, Guid notificationID = new Guid())
        {
            try
            {
                UserNotification notification = new UserNotification();
                if(Guid.Empty.Equals(notificationID))
                {
                    notification.ID = Guid.NewGuid();
                }else
                {
                    notification.ID = notificationID;
                }
                notification.CreatedDate = created;
                notification.IsHidden = false;
                notification.IsRead = false;
                notification.NotificationType = (int)type;
                notification.ObjectType = (int)objectType;
                notification.ObjectID = objectID;
                notification.TargetType = (int)targetType;
                notification.TargetID = targetID;
                notification.OwnerID = ownerid;
                _re.Add(notification);
                _re.CommitChanges();
                return true;
            }
            catch (Exception exx)
            {
                return false;
            }
        }
    }
}
