using System.Collections.Generic;
using System.Linq;
using Social.DAL.Interface;
using Social.Entity.TikApplication;
using Social.Service.Interface.TikApplication;
using System;

namespace Social.Service.TikApplication
{
    public class NoteService : ServiceBase<Note>, INoteService
    { 
        IRepository _re;
        public NoteService(ITikApplicationRepository repository)
            : base(repository)
        {
            _re = repository;
        }

        public IList<Note> GetListNoteByUserID(Guid userId)
        {
            var retval = from it in _re.All<Note>()
                         where it.UserID == userId
                         orderby it.CreatedDate descending
                         select it;
            return retval.ToList();
        }

        /// <summary>
        ///     Lay so note duoc tao moi nhat
        /// </summary>
        /// <param name="userId"></param>
        /// <param name="top"></param>
        /// <returns></returns>
        public IList<Note> GetTopListNoteByUserID(Guid userId, int top=3)
        {
            var retval = from it in _re.All<Note>()
                         where it.UserID == userId
                         orderby it.CreatedDate descending
                         select it;
            return retval.Take(top).ToList();
        }

        public Note GetNoteByID(Guid noteid)
        {
            if (Guid.Empty.Equals(noteid)) return null;

            var note = from n in _re.All<Note>()
                       where n.ID == noteid
                       select n;
            if (note.Any()) return note.FirstOrDefault();

            return null;
        }

        public IList<Note> GetListNoteTags(Guid userID)
        {
            if (Guid.Empty.Equals(userID)) return null;
            string user = userID.ToString();
            var note = from n in _re.All<Note>()
                       where n.Tags.Contains(user)
                       select n;
            return note.ToList();
        }

        public List<Note> GetListNoteByUserIdAndPageNumber(Guid userId, int pageIndex, int pageSize, out int totalRow)
        {
            var lstAb = from it in _re.All<Note>()
                        where it.UserID == userId
                        orderby it.CreatedDate descending
                        select it;
            totalRow = lstAb.Count();
            var stastRow = (pageIndex - 1) * pageSize;
            var ret = lstAb.Skip(stastRow).Take(pageSize);
            return ret.ToList();
        }
    }
}
