﻿using System.Collections.Generic;
using System.Linq;
using Social.DAL.Interface;
using Social.Entity.TikFriend;
using Social.Service.Interface.TikFriend;
using System;
using AutoMapper;
using Webdiyer.WebControls.Mvc;
using Social.Lib;

namespace Social.Service.TikFriend
{
    public class FriendRequestService : ServiceBase<FriendRequest>, IFriendRequestService
    { 
        IRepository _re;
        public FriendRequestService(ITikFriendRepository repository)
            : base(repository)
        {
            _re = repository;
        }        



        //requet 3 record
        public List<FriendRequest> GetTop3()
        {
            try
            {
                var listTop3 = from c in _re.All<FriendRequest>().Take(3)
                               select c;
                return listTop3.ToList();
            }
            catch { return null; }
                     
        }
        public FriendRequest GetByID(Guid id)
        {
            try {
                var _friendrequest = _re.All<FriendRequest>().Where(c => c.ID == id).Single();
                return _friendrequest;
            }catch{
                return new FriendRequest();
            }
        }

        public List<FriendRequestDTO> Get(Guid id)
        {
            var qry = from c in _re.All<FriendRequest>()
                      where c.ToUserID==id
                       select c;
            IEnumerable<FriendRequestDTO> list=qry.Distinct().ToList().Select(fr => Mapper.Map<FriendRequest, FriendRequestDTO>(fr));
            
            return list.ToList();
        }

        //page
        public PagedList<FriendRequestDTO> GetPage(Guid id, int pageIndex = 0, int pageSize = 10)
        {

                var qry = from c in _re.All<FriendRequest>()
                          where c.ToUserID == id
                          orderby c.CreatedDate descending
                          select c;

                if (pageIndex == 0)
                {
                    pageIndex = 0;
                }
                else if (pageIndex == 1)
                {
                    pageIndex = 0;
                }
                else
                {
                    pageIndex = pageIndex - 1;
                }
                var sc = qry.Skip(pageIndex * pageSize).Take(pageSize).AsEnumerable().Select(fr => Mapper.Map<FriendRequest, FriendRequestDTO>(fr));

                PagedList<FriendRequestDTO> pagedList = new PagedList<FriendRequestDTO>(sc, pageIndex + 1, pageSize, qry.Count());
                return pagedList;
            


        }

        public List<FriendRequestDTO> GetByFromUserID(Guid id)
        {
            var qry = from c in _re.All<FriendRequest>()
                      where c.FromUserID == id
                      orderby c.CreatedDate descending
                      select c;

            IEnumerable<FriendRequestDTO> list = qry.ToList().Select(fr => Mapper.Map<FriendRequest, FriendRequestDTO>(fr));

            return list.ToList();
        }

        //page PagedList<UserFriend> GetListUserFriend_ByOwnerID(Guid ownerID, int pageIndex = 0, int pageSize = 10)

        public PagedList<FriendRequestDTO> GetByFromUserIDPage(Guid id, int pageIndex = 0, int pageSize = 10)
        {
            var qry = from c in _re.All<FriendRequest>()
                      where c.FromUserID == id
                      orderby c.CreatedDate descending
                      select c;

            if (pageIndex == 0)
            {
                pageIndex = 0;
            }
            else if (pageIndex == 1)
            {
                pageIndex = 0;
            }
            else
            {
                pageIndex = pageIndex - 1;
            }
            var sc = qry.Skip(pageIndex * pageSize).Take(pageSize).AsEnumerable().Select(fr => Mapper.Map<FriendRequest, FriendRequestDTO>(fr));
           
            PagedList<FriendRequestDTO> pagedList = new PagedList<FriendRequestDTO>(sc, pageIndex + 1, pageSize, qry.Count());
            return pagedList;
        }


        public List<FriendRequest> GetByFromUserIDAndToUserID(Guid fromuserid, Guid touserid)
        {

            try {
                var friendrequest = from c in _re.All<FriendRequest>()
                                    where c.FromUserID == fromuserid && c.ToUserID == touserid
                                    select c;
                return friendrequest.Distinct().ToList();
            }
            catch {
                return new List<FriendRequest>();
            }
        }
        public int GetCountByFromUserIDAndToUserID(Guid fromuserid, Guid touserid)
        {

            try
            {
                var friendrequest = from c in _re.All<FriendRequest>()
                                    where c.FromUserID == fromuserid && c.ToUserID == touserid
                                    select c;
                return friendrequest.Distinct().Count();
            }
            catch
            {
                return new int();
            }
        }
        public List<FriendRequest> GetByFromToUserIDAndTop(Guid touserid, int top)
        {
            try {
                
                    var friendrequest = from c in _re.All<FriendRequest>()
                                        where c.ToUserID == touserid
                                        select c;
                    if (top != 0)
                    {
                        return friendrequest.Distinct().Take(top).OrderByDescending(c=>c.CreatedDate).ToList();
                    }
                    else {
                        return friendrequest.ToList();
                    }

            }
            catch {
                return new List<FriendRequest>();
            }
        }

        public int GetCountByToUserID(Guid touserID)
        {
            try
            {
                var friendrequest = from c in _re.All<FriendRequestAndUserProfile>()
                                    where c.ToUserID == touserID
                                    select c;
                return friendrequest.Distinct().Count();
            }
            catch {
                return 0;
            }
        }


        public List<FriendRequestAndUserProfile> GetBy_FromToUserIDAndTop(Guid touserid, int top)
        {
            try
            {
                var friendrequest = from c in _re.All<FriendRequestAndUserProfile>()
                                    where c.ToUserID == touserid
                                    select c;
                
                if (top != 0)
                {
                    
                    return friendrequest.Distinct().Take(top).OrderByDescending(c => c.CreatedDate).ToList();
                }
                else
                {
                    return friendrequest.ToList();
                }

            }
            catch
            {
                return new List<FriendRequestAndUserProfile>();
            }
        }
        public bool CheckRequestFriend(Guid friendID, Guid ownerID)
        {
            try {
                var _checkfriend = _re.All<FriendRequest>().Where(c => c.FromUserID == friendID && c.ToUserID == ownerID).Distinct().Count();
                if (_checkfriend > 0)
                    return true;
                else
                    return false;
            }
            catch {
                return false;
            }
        }



        
    }
}
