using System.Collections.Generic;
using System.Linq;
using Social.DAL.Interface;
using Social.Entity.TikFriend;
using Social.Service.Interface.TikFriend;
using System;
using AutoMapper;
using Social.Lib;

namespace Social.Service.TikFriend
{
    public class FriendSearchService : ServiceBase<FriendSearch>, IFriendSearchService
    { 
        IRepository _re;
        public FriendSearchService(ITikFriendRepository repository)
            : base(repository)
        {
            _re = repository;
        }

        public List<FriendSearch> GetByOwnerIDAndKeyWord(Guid ownerID, string keyword, int top)
        {
            try
            {
                var listfriend = (from c in _re.All<FriendSearch>()
                                  where c.OwnerID == ownerID && c.FullText.Contains(keyword)
                                  select c).OrderByDescending(c=>c.Impression).Take(top);
                return listfriend.Distinct().ToList();
            }
            catch(Exception ex) {
                return new List<FriendSearch>();
            }
                        
        }

        //
        // query fulltext name
        //
        public List<FriendSearch> GetByOwnerIDAndKeyWord_FullTextName(Guid ownerID, string keyword, int top)
        {
            try
            {
                if (top != 0)
                {
                    var listfriend = (from c in _re.All<FriendSearch>()
                                      where c.OwnerID == ownerID && c.FullTextName.Contains(keyword)
                                      select c).Distinct().OrderByDescending(c => c.Impression).Take(top);
                    return listfriend.ToList();
                }
                else {
                    var listfriend = (from c in _re.All<FriendSearch>()
                                      where c.OwnerID == ownerID && c.FullTextName.Contains(keyword)
                                      select c).Distinct().OrderByDescending(c => c.Impression);
                    return listfriend.ToList();
                }
            }
            catch
            {
                return new List<FriendSearch>();
            }

        }


        



        public List<FriendSearch> GetByOwnerIDTop(Guid Owner, int top)
        {
            try {
                if (top != 0)
                {
                    var listfriend = (from c in _re.All<FriendSearch>()
                                      where c.OwnerID == Owner
                                      select c).Take(top).Distinct().OrderByDescending(c => c.Impression).ToList();
                    return listfriend;
                }
                else {
                    var listfriendAll = (from c in _re.All<FriendSearch>()
                                         where c.OwnerID == Owner
                                      select c).Distinct().OrderByDescending(c => c.Impression).ToList();
                    return listfriendAll;
                }
            }
            catch {
                return new List<FriendSearch>();
            }
        }

        public bool CheckFriend(Guid friendID,Guid ownerID)
        {
            try {
                var _userfriend = _re.All<UserFriend>().Where(c => c.FriendID == friendID && c.OwnerID == ownerID).Count();
                if (_userfriend > 0)
                { return true; }
                else { return false; }
                
            }
            catch {
                return false;
            }
        }

        public List<FriendSearchUserName> GetByOwnerIDTopIndex(Guid Owner, int top)
        {
            try {
                if (top != 0)
                {
                    var listfriend = (from c in _re.All<FriendSearch>()
                                      where c.OwnerID == Owner
                                      select c).Take(top).Distinct().OrderByDescending(c => c.Impression);
                    IEnumerable<FriendSearchUserName> list = listfriend.ToList().Select(fr => Mapper.Map<FriendSearch, FriendSearchUserName>(fr));
                    return list.ToList();
                }
                else {
                    var listfriendAll = (from c in _re.All<FriendSearch>()
                                         where c.OwnerID == Owner
                                         select c).Distinct().OrderByDescending(c => c.Impression);
                    IEnumerable<FriendSearchUserName> list = listfriendAll.ToList().Select(fr => Mapper.Map<FriendSearch, FriendSearchUserName>(fr));
                    return list.ToList();
                }
            }
            catch {
                return new List<FriendSearchUserName>();
            }
        }

        

    }
}
