using System.Collections.Generic;
using System.Linq;
using Social.DAL.Interface;
using Social.Entity.TikMessage;
using Social.Service.Interface.TikMessage;
using System;
using Webdiyer.WebControls.Mvc;
namespace Social.Service.TikMessage
{
    public class MessageService : ServiceBase<Message>, IMessageService
    { 
        IRepository _re;
        public MessageService(ITikMessageRepository repository)
            : base(repository)
        {
            _re = repository;
        }
        public PagedList<MessageView> GetInboxByUserID(Guid uid, bool? unread = null, bool? isSpam = null, bool? isArchived = null, int pageIndex = 0, int pageSize = 20)
        {
            var listr = _re.All<MessageRecipient>().Where(r=>r.RecipientID==uid);
            if (isSpam.HasValue)
                listr = listr.Where(r=>r.IsSpam == isSpam.Value);
            if (isArchived.HasValue)
                listr = listr.Where(r => r.IsArchived==isArchived.Value);
            if (unread.HasValue)
                listr = listr.Where(r => r.UnRead == unread.Value);
            var lstMsg=listr.Select(r=>r.MessageID);
            var list = from msg in _re.All<MessageView>()
                       where lstMsg.Contains(msg.ID)
                       select msg;
            
            return list.OrderByDescending(mv => mv.UpdatedDate).ToPagedList(pageIndex, pageSize);//.Skip(pageSize * pageIndex).Take(pageSize).ToList();
        }
        public int GetCountUnread(Guid uid)
        {
            var count=_re.All<MessageRecipient>().Where(r=>!r.IsArchived && !r.IsSpam && r.UnRead&&r.RecipientID==uid).Count();
            return count;
        }
        public List<MessageView> GetTopMessageViewByUserID(Guid uid,int top=5)
        {
            try
            {
                var lstMsg = _re.All<MessageRecipient>().Where(r => !r.IsArchived && !r.IsSpam && r.RecipientID == uid).Distinct().Select(r => r.MessageID);
                var list = from msg in _re.All<MessageView>()
                           where lstMsg.Contains(msg.ID)
                           orderby msg.UpdatedDate descending
                           select new { FirstName = msg.FirstName, LastName = msg.LastName, UpdatedDate = msg.UpdatedDate, SmallAvatarUrl = msg.SmallAvatarUrl, Subject = msg.Subject, ID = msg.ID };
                return list.Take(top).AsEnumerable().Select(msg => new MessageView { FirstName = msg.FirstName, LastName = msg.LastName, UpdatedDate = msg.UpdatedDate, SmallAvatarUrl = msg.SmallAvatarUrl, Subject = msg.Subject, ID = msg.ID }).ToList();
            }
            catch { return new List<MessageView>(); }
        }
        public MessageView GetMessageViewByID(Guid msgID)
        {
            return _re.Single<MessageView>(m => m.ID == msgID);
        }
        public void UpdateRead(Guid uid, Guid msgID)
        {
            Message msg = _re.Single<Message>(m => m.ID == msgID);
            MessageRecipient mr = msg.Recipients.SingleOrDefault(r => r.RecipientID == uid && r.UnRead);
            if (mr != null)
                mr.UnRead = false;
            this.Update(msg);
        }
        public PagedList<MessageView> GetSentByUserID(Guid uid, int pageIndex = 0, int pageSize = 20)
        {
            var listc = _re.All<MessageComment>().Where(c => c.FromID == uid).Select(c=>c.MessageID).Distinct();
            var list = from msg in _re.All<MessageView>()
                       where listc.Contains(msg.ID)
                       select msg;
            return list.OrderByDescending(mv => mv.UpdatedDate).ToPagedList(pageIndex, pageSize);//.Skip(pageSize * pageIndex).Take(pageSize).ToList();
        }
    }
}
