using System;
using System.Collections.Generic;
using System.Linq;
using Social.DAL.Interface;
using Social.Entity.TikPhoto;
using Social.Service.Interface.TikPhoto;

namespace Social.Service.TikPhoto
{
    public class AlbumService : ServiceBase<Album>, IAlbumService
    { 
        IRepository _re;
        public AlbumService(ITikPhotoRepository repository)
            : base(repository)
        {
            _re = repository;
        }
        
        public List<Album> GetListAlbumByUserId(Guid userId)
        {
            var retval = from it in _re.All<Album>()
                         where it.UserID == userId
                         orderby it.Type ascending, it.SortOrder ascending, it.CreatedDate descending 
                           select it;
            return retval.ToList();
        }

        public List<Album> GetPageListAlbumByUserId(Guid userId, int pageIndex, int pageSize, out int totalRow, bool isMyPhoto = false)
        {
            if(isMyPhoto)
            {
                var lstAb = from it in _re.All<Album>()
                            where it.UserID == userId
                            orderby it.Type ascending, it.SortOrder ascending, it.CreatedDate descending 
                            select it;
                totalRow = lstAb.Count();

                var stastRow = (pageIndex - 1) * pageSize;
                var ret = lstAb.Skip(stastRow).Take(pageSize);
                return ret.ToList();
            }
            else
            {
                var lstAb = from it in _re.All<Album>()
                            where it.UserID == userId && it.PhotoCount>0
                            orderby it.Type ascending, it.SortOrder ascending, it.CreatedDate descending
                            select it;
                totalRow = lstAb.Count();

                var stastRow = (pageIndex - 1) * pageSize;
                var ret = lstAb.Skip(stastRow).Take(pageSize);
                return ret.ToList();
            }
        }

        public List<Album> GetListAlbumDefaultByUserId(Guid userId)
        {
            var retval = from it in _re.All<Album>()
                         where it.UserID == userId && (it.Type != 8)
                         orderby it.CreatedDate descending
                         select it;

            return retval.ToList();
        }

        public List<Album> GetByAlbumIdUserId(Guid albumId, Guid userId)
        {
            var list = from it in _re.All<Album>()
                       where it.ID == albumId && it.UserID == userId 
                       orderby it.CreatedDate descending 
                         select it;
            return list.ToList();
        }

        public List<Album> GetByUserIdTypeAlbum(Guid userId, int type)
        {
            var lstAb = from it in _re.All<Album>()
                        where it.UserID == userId && it.Type == type
                         orderby it.CreatedDate descending
                         select it; 
            return lstAb.ToList();
        }
    }
}
