using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using Social.DAL.Interface;
using Social.Entity.TikPhoto;
using Social.Service.Interface.TikPhoto;

namespace Social.Service.TikPhoto
{
    public class PhotoService : ServiceBase<Photo>, IPhotoService
    { 
        IRepository _re;
        public PhotoService(ITikPhotoRepository repository)
            : base(repository)
        {
            _re = repository;
        }

        public object GetPhotoById(Guid id)
        {
            var lstAb = from it in _re.All<Photo>()
                        where it.ID == id
                        select new
                                   {
                                       ID = it.ID,
                                       AlbumID = it.AlbumID,
                                       UserID = it.UserID,
                                       Caption = it.Caption,
                                       SourceUrl = it.SourceUrl,
                                       SortOrder = it.SortOrder,
                                       WxH = it.WxH,
                                       ViewCount = it.ViewCount,
                                       LikeCount = it.LikeCount,
                                       CommentCount = it.CommentCount,
                                       Status = it.Status,
                                       PrivacyType = it.PrivacyType,
                                       PrivacyPeopleShowHide = it.PrivacyPeopleShowHide,
                                       CreatedDate = it.CreatedDate,
                                       ModifiedDate = it.ModifiedDate
                                   };
            if (lstAb.Any()) return lstAb.FirstOrDefault();
            return null;
        }

        public List<Photo> GetAllPhotoByAlbumId(Guid albumId)
        {
            var retval = from it in _re.All<Photo>()
                         where it.AlbumID == albumId orderby it.CreatedDate descending 
                         select it;
            return retval.ToList();
        }

        public Photo GetNextPhotoInAnbum(Guid albumId, Guid photoId)
        {
            Photo retval = null;
            var lst = from it in _re.All<Photo>()
                         where it.AlbumID == albumId orderby it.SortOrder, it.CreatedDate descending
                         select it;
            var tmp=lst.ToList();
            for (var i = 0; i < tmp.Count(); i++)
            {
                if (tmp[i].ID != photoId) continue;
                retval = (i + 1) < tmp.Count() ? tmp[i + 1] : tmp[0];
                break;
            }
            return retval;
        }

        public Photo GetPrevPhotoInAnbum(Guid albumId, Guid photoId)
        {
            Photo retval = null;
            var lst = from it in _re.All<Photo>()
                      where it.AlbumID == albumId
                      orderby it.SortOrder, it.CreatedDate descending
                      select it;
            var tmp = lst.ToList();
            for (var i = 0; i < tmp.Count(); i++)
            {
                if (tmp[i].ID != photoId) continue;
                retval = (i - 1) >= 0 ? tmp[i - 1] : tmp[tmp.Count()-1];
                break;
            }
            return retval;
        }

        public int UpdateListPhoto(List<Photo> lstPhoto, Guid albumId)
        {
            var lst = from it in _re.All<Photo>() where it.AlbumID == albumId select it;
            foreach (var p in lst)
            {
                if(lstPhoto.Count<=0) break;
                foreach (var pU in lstPhoto)
                {
                    if (p.ID != pU.ID) continue;
                    p.Caption = pU.Caption;
                    p.AlbumID = pU.AlbumID;
                    p.ModifiedDate = DateTime.Now;
                    lstPhoto.Remove(pU);
                    break;
                }
            }
            try{_re.CommitChanges();}catch (Exception){return 0;}
            return 1;
        }

        public List<Photo> GetPageListPhotoByAlbumId(Guid albumId, int pageIndex, int pageSize, out int totalRow)
        {
            var lstAb = from it in _re.All<Photo>()
                        where it.AlbumID == albumId
                        orderby it.CreatedDate descending
                        select it;
            totalRow = lstAb.Count();
            var stastRow = (pageIndex - 1) * pageSize;
            var ret = lstAb.Skip(stastRow).Take(pageSize);
            return ret.ToList();
        }

        public Photo GetById(Guid pId)
        {
            var lstAb = from it in _re.All<Photo>()
                        where it.ID==pId
                        select it;
            if (lstAb.Any()) return lstAb.First();
            return null;
        }
    }
}
