﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.Mvc;
using System.Web.Script.Serialization;
using Microsoft.Practices.ServiceLocation;
using Social.DAL.EF;
using Social.DAL.Interface;
using Social.Entity.Tik;
using Social.Service;
using Social.Service.Interface;
using Social.Service.Interface.Tik;
using Social.Web.Extends.Helper;
using SolrNet;
using SolrNet.Commands.Parameters;
using SolrNet.Impl;
namespace Social.Web.AjaxHandlers
{
    /// <summary>
    /// Summary description for GetSchools
    /// </summary>
    public class GetSchools : IHttpHandler
    {
        private static readonly string solrURL = ConfigurationManager.AppSettings["SolrUrl"];
        public void ProcessRequest(HttpContext context)
        {
            context.Response.ContentType = "application/json";
            context.Response.ContentEncoding = Encoding.UTF8;

            if (context.Request.UrlReferrer == null) return;
            if (!context.Request.UrlReferrer.Host.Equals(context.Request.Url.Host)) return;

            string query = context.Request.Form["query"];
            int top = string.IsNullOrEmpty(context.Request.Form["limit"]) ? 0 : Convert.ToInt32(context.Request.Form["limit"]);
            if (string.IsNullOrEmpty(query))
            {
                context.Response.Write(" ");
                return;
            }

            var solrWorker = SolrOperationsCache<School>.GetSolrOperations(solrURL);
            QueryOptions options = new QueryOptions();
            if (top > 0) options.Rows = top;

            var solrQuery = new SolrQuery(string.Format("(Name:{0}) AND (type:school)",query));
            ISolrQueryResults<School> matchingSchool = solrWorker.Query(solrQuery, options);

            //var schoolService = DependencyResolver.Current.GetService<ISchoolService>();
            //var schools = schoolService.GetSchools(matchingSchool.Select(school => school.ID));
            var schools = matchingSchool.ToList();

            //var schoolService = DependencyResolver.Current.GetService<ISchoolService>();
            //var schools = schoolService.Search(query, top);
            
            JavaScriptSerializer serializer = new JavaScriptSerializer();
            var output = serializer.Serialize(schools);
            context.Response.Write(output); 
            //context.Response.Write(@"[{""ID"": ""hello world"", ""Name"": ""hello world""}, {""ID"": ""movies"", ""Name"": ""movies""}, {""ID"": ""ski"", ""Name"": ""ski""}, {""ID"": ""snowbord"", ""Name"": ""snowbord""}, {""ID"": ""computer"", ""Name"": ""computer""}, {""ID"": ""apple"", ""Name"": ""apple""}, {""ID"": ""pc"", ""Name"": ""pc""}, {""ID"": ""ipod"", ""Name"": ""ipod""}, {""ID"": ""ipad"", ""Name"": ""ipad""}, {""ID"": ""iphone"", ""Name"": ""iphone""}, {""ID"": ""iphon4"", ""Name"": ""iphone4""}, {""ID"": ""iphone5"", ""Name"": ""iphone5""}, {""ID"": ""samsung"", ""Name"": ""samsung""}, {""ID"": ""blackberry"", ""Name"": ""blackberry""}]");
        }

        public bool IsReusable
        {
            get
            {
                return false;
            }
        }
    }
}