﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Text;
using Social.Lib;
using Social.Service.Interface.Tik;
using System.IO;
using System.Configuration;
using System.Web.Mvc;
using System.Web.SessionState;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;

namespace Social.Web.AjaxHandlers
{
    /// <summary>
    /// Summary description for UploadAvatar
    /// </summary>
    public class UploadAvatar : IHttpHandler, IReadOnlySessionState
    {
        private readonly IUserProfileService _user = DependencyResolver.Current.GetService<IUserProfileService>();

        public void ProcessRequest(HttpContext context)
        {
            var result = "";
            var type = context.Request.QueryString["type"];
            if (string.IsNullOrEmpty(type))
            {
                type = context.Request.Form["type"];
            }
            switch (type)
            {
                case "UploadAvatar":
                    result = UploadAvatarCamera(context);
                    break;

                case "ChangeAvatar":
                    result = ChangeAvatar(context);
                    break;

                default:
                    result = UploadAvatarCamera(context);
                    break;
            }
            context.Response.ContentType = "text/plain";
            context.Response.Write(result);
        }

        private string ChangeAvatar(HttpContext context)
        {
            if (!SessionManager.IsLoggedIn()) return "0";
            Guid userID = Social.Lib.SessionManager.CurrentUserID();
            var user = _user.Single(u => u.ID == userID);

            var urlImageSource = context.Request.Form["UrlImage"];
            urlImageSource = ConfigManager.PathResourcePhoto + Base64String.Decode(urlImageSource) + Lib.Photo.Constants.ImageInfo.NameOriginal;
            if (!string.IsNullOrEmpty(urlImageSource))
            {
                string extension = ".jpg";
                string croppedFileName = user.ID.ToString();

                string pathSmall = string.IsNullOrEmpty(user.LargeAvatarUrl) ?
                    GetAvatarPath("Small", croppedFileName, extension)
                    : user.SmallAvatarUrlMap;

                string pathMedium = string.IsNullOrEmpty(user.LargeAvatarUrl) ?
                    GetAvatarPath("Medium", croppedFileName, extension)
                    : user.MediumAvatarUrlMap;

                string pathLarge = string.IsNullOrEmpty(user.LargeAvatarUrl) ?
                    GetAvatarPath("Large", croppedFileName, extension)
                    : user.LargeAvatarUrl;

                ResizeImage(string.Concat(ConfigurationManager.AppSettings["AvatarResource"], pathLarge), urlImageSource, 720, 720);
                ResizeImage(string.Concat(ConfigurationManager.AppSettings["AvatarResource"], pathMedium), urlImageSource, 200, 0);
                ResizeImage(string.Concat(ConfigurationManager.AppSettings["AvatarResource"], pathSmall), urlImageSource, 50, 0);

                user.LargeAvatarUrl = string.Concat(pathLarge);
                user.MediumAvatarUrlMap = string.Concat(pathMedium);
                user.SmallAvatarUrlMap = string.Concat(pathSmall);
                Social.Lib.SessionManager.UpdateAvatar(pathSmall, pathMedium);
            }

            try
            {
                _user.Update(user);
                return "1";
            }
            catch (Exception ex)
            {
                return "0";
            }
        }

        private string UploadAvatarCamera(HttpContext context)
        {
            context.Response.ContentEncoding = Encoding.UTF8;
            Stream fileContent = context.Request.InputStream;
            var m = new MemoryStream();
            fileContent.CopyTo(m);
            Image image = Image.FromStream(m);
            var bytes = m.ToArray();
            if (!SessionManager.IsLoggedIn()) return "0";
            Guid userID = Social.Lib.SessionManager.CurrentUserID();
            var user = _user.Single(u => u.ID == userID);

            if (image != null)
            {
                string extension = ".jpg";
                string croppedFileName = user.ID.ToString();

                string pathSmall = string.IsNullOrEmpty(user.LargeAvatarUrl) ?
                    GetAvatarPath("Small", croppedFileName, extension)
                    : user.SmallAvatarUrlMap;

                string pathMedium = string.IsNullOrEmpty(user.LargeAvatarUrl) ?
                    GetAvatarPath("Medium", croppedFileName, extension)
                    : user.MediumAvatarUrlMap;

                string pathLarge = string.IsNullOrEmpty(user.LargeAvatarUrl) ?
                    GetAvatarPath("Large", croppedFileName, extension)
                    : user.LargeAvatarUrl;

                image.Save(string.Concat(ConfigurationManager.AppSettings["AvatarResource"], pathLarge), ImageFormat.Jpeg);

                var psv = new Social.Web.Service.PhotoService();
                var jsonVal = psv.UploadAvatar(bytes, userID);

                ResizeImage(string.Concat(ConfigurationManager.AppSettings["AvatarResource"], pathLarge), string.Concat(ConfigurationManager.AppSettings["AvatarResource"], pathLarge), 720, 720);
                ResizeImage(string.Concat(ConfigurationManager.AppSettings["AvatarResource"], pathMedium), string.Concat(ConfigurationManager.AppSettings["AvatarResource"], pathLarge), 200, 720);
                ResizeImage(string.Concat(ConfigurationManager.AppSettings["AvatarResource"], pathSmall), string.Concat(ConfigurationManager.AppSettings["AvatarResource"], pathLarge), 50, 0);

                user.LargeAvatarUrl = string.Concat(pathLarge);
                user.MediumAvatarUrlMap = string.Concat(pathMedium);
                user.SmallAvatarUrlMap = string.Concat(pathSmall);
                Social.Lib.SessionManager.UpdateAvatar(pathSmall, pathMedium);
            }

            try
            {
                _user.Update(user);
                return "1";
            }
            catch (Exception ex)
            {
                return "0";
            }
        }

        private string GetAvatarPath(string type, string filename, string extension)
        {
            string resource = string.Empty;

            switch (type)
            {
                case "Small":
                    resource = ConfigurationManager.AppSettings["SmallAvatarUrl"];
                    break;
                case "Medium":
                    resource = ConfigurationManager.AppSettings["MediumAvatarUrl"];
                    break;
                case "Large":
                    resource = ConfigurationManager.AppSettings["LargeAvatarUrl"];
                    break;
                default:
                    resource = ConfigurationManager.AppSettings["LargeAvatarUrl"];
                    break;
            }

            string path = string.Concat(resource, DateTime.Now.ToString("yyyyMM"), "/");
            string pathSave = string.Concat(ConfigurationManager.AppSettings["AvatarResource"], path);

            try
            {
                if (Directory.Exists(pathSave) == false)
                {
                    Directory.CreateDirectory(pathSave);
                }
            }
            catch { }

            return string.Concat(path, filename, extension);
        }

        public void ResizeImage(string filepath, string filePathSource, int thumbnailSize, int maxSize)
        {
            var newImage = new Bitmap(filePathSource);
            int newWidth = 0, newHeight = 0;
            int scale = maxSize > 0 ? maxSize : thumbnailSize;

            if ((newImage.Width < newImage.Height && maxSize == 0) || (newImage.Width > newImage.Height && maxSize > 0))
            {
                newWidth = newImage.Width > maxSize ? scale : newImage.Width;
                newHeight = newImage.Width > maxSize ? (newImage.Height * scale / newImage.Width) : newImage.Height;
            }
            else
            {
                newWidth = newImage.Height > maxSize ? (newImage.Width * scale / newImage.Height) : newImage.Width;
                newHeight = newImage.Height > maxSize ? scale : newImage.Height;
            }

            var thumbnailBitmap = new Bitmap(maxSize > 0 ? newWidth : thumbnailSize, maxSize > 0 ? newHeight : thumbnailSize);

            var thumbnailGraph = Graphics.FromImage(thumbnailBitmap);
            thumbnailGraph.CompositingQuality = CompositingQuality.HighQuality;
            thumbnailGraph.SmoothingMode = SmoothingMode.HighQuality;
            thumbnailGraph.InterpolationMode = InterpolationMode.HighQualityBicubic;

            var imageRectangle = new Rectangle(0, 0, newWidth, newHeight);
            thumbnailGraph.DrawImage(newImage, imageRectangle);
            newImage.Dispose();
            thumbnailBitmap.Save(filepath);
            thumbnailGraph.Dispose();
            thumbnailBitmap.Dispose();
        }

        public bool IsReusable
        {
            get
            {
                return false;
            }
        }
    }
}