﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Social.Entity.Tik;
using Social.Entity.TikActivity;
using Social.Lib;
using Social.Lib.Activity;
using Social.Service.Interface.Tik;
using Social.Service.Interface.TikActivity;
using Social.Service.Interface.TikFriend;
using Social.Web.Areas.Feed.Models;
using Social.Web.Extends.Helper;

namespace Social.Web.Areas.Ajax.Controllers
{
    public class ActivityController : Controller
    {
        IUserActivityService _userActivity;
        IUserProfileService _userProfile;
        IFriendListService _friendlist;
        IUserFriendService _userfriend;
        public ActivityController(IUserActivityService userActivity, IUserProfileService userProfile, IFriendListService friendlist, IUserFriendService userfriend)
        {
            _userActivity = userActivity;
            _userProfile = userProfile;
            _friendlist = friendlist;
            _userfriend = userfriend;
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult Hide(Guid activityid)
        {
            Result result = new Result();
            result.error = 0;
            result.errorDescription = string.Empty;
            result.result = string.Empty;

            var activity = _userActivity.GetByID(activityid);
            if(activity == null)
            {
                return Json(result, JsonRequestBehavior.AllowGet);
            }

            if(activity.OwnerID != SessionManager.CurrentUserID())
            {
                result.error = 1;
                result.errorDescription = "Bạn không có quyền để thực hiện hành động này!";
            }else
            {
                activity.HideOwner = true;
                _userActivity.Update(activity);
            }
            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post), ValidateInput(false)]
        public JsonResult HideTag(Guid activityid)
        {
            Result result = new Result();
            result.error = 0;
            result.errorDescription = string.Empty;
            result.result = string.Empty;

            var activity = _userActivity.GetByID(activityid);
            if (activity == null)
            {
                return Json(result, JsonRequestBehavior.AllowGet);
            }

            if (activity.TargetType == (int)Constants.Activity.TargetType.User && activity.TargetID == SessionManager.CurrentUserID())
            {
                activity.HideTarget = true;
                _userActivity.Update(activity);
            }
            else
            {
                result.error = 1;
                result.errorDescription = "Bạn không có quyền để thực hiện hành động này!";
            }
            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult LoadFeed(Guid viewid, int type)
        {
            return LoadMore(viewid, type, DateTime.Now);
        }

        [AcceptVerbs(HttpVerbs.Post), ValidateInput(false)]
        public JsonResult LoadMore(Guid viewid, int type, DateTime enddate)
        {
            LoadMoreResult result = new LoadMoreResult();
            result.error = 0;
            result.errorDescription = string.Empty;
            result.isend = false;

            UserProfile viewUser = _userProfile.GetByID(viewid);
            UserProfile currentUser = _userProfile.CurrentUser();
            if (currentUser == null)
            {
                result.error = 1;
                result.errorDescription = "Bạn chưa đăng nhập.";
                return Json(result, JsonRequestBehavior.AllowGet);
            }

            if (viewUser == null)
            {
                result.error = 1;
                result.errorDescription = "Không tìm thấy thành viên này.";
                return Json(result, JsonRequestBehavior.AllowGet);
            }

            switch (type)
            {
                case 1:
                    //Load Activity in Wall
                    var activityWall = _userActivity.GetWallActivity(viewUser.ID, enddate, ConfigManager.MaxWallItemLoad);
                    if (activityWall != null)
                    {
                        var activityWalls = ActivityWall.Convert(viewUser.ID, activityWall);
                        ViewData.Add("CurrentUser", currentUser);
                        ViewData.Add("User", viewUser);
                        result.empty = false;
                        result.dateend = activityWall[activityWall.Count - 1].CreatedDate.ToString();
                        if (activityWall.Count < ConfigManager.MaxWallItemLoad)
                        {
                            result.isend = true;
                        }
                        result.type = type;
                        result.result = PartialView("~/Areas/Feed/Views/Shared/_ActivityInWallList.cshtml", activityWalls.OrderByDescending(a => a.CreatedDate).ToList()).Capture(ControllerContext);
                    }
                    else
                    {
                        result.empty = true;
                        result.result = string.Empty;
                        result.isend = true;
                        result.type = type;
                    }
                    break;
                case 2:
                    //Load Activity in Special Feed
                    List<Guid> listFriends = _userfriend.GetIDByImpressionOrder(currentUser.ID);
                    var activitySpecialFeed = _userActivity.GetSpecialFeedActivity(currentUser.ID, listFriends, enddate, ConfigManager.MaxFeedItemLoad);
                    if (activitySpecialFeed != null)
                    {
                        var activities = ActivityFeed.ConvertSpecial(viewUser.ID, activitySpecialFeed);
                        ViewData.Add("CurrentUser", currentUser);
                        ViewData.Add("User", viewUser);
                        result.empty = false;
                        result.dateend = activitySpecialFeed[activitySpecialFeed.Count - 1].CreatedDate.ToString();
                        if (activitySpecialFeed.Count < ConfigManager.MaxFeedItemLoad)
                        {
                            result.isend = true;
                        }
                        result.type = type;
                        result.result = PartialView("~/Areas/Feed/Views/Shared/_ActivityInFeedList.cshtml", activities.OrderByDescending(a => a.CreatedDate).ToList()).Capture(ControllerContext);
                    }
                    else
                    {
                        result.empty = true;
                        result.result = string.Empty;
                        result.isend = true;
                        result.type = type;
                    }
                    break;
                case 3:
                    //Load Activity in Lasted Feed
                    List<Guid> listFriendLastFeed = _friendlist.GetByOwnerID(currentUser.ID);
                    var activityLastFeed = _userActivity.GetFeedActivity(currentUser.ID, listFriendLastFeed, enddate, ConfigManager.MaxFeedItemLoad);
                    if (activityLastFeed != null)
                    {
                        var activities = ActivityFeed.Convert(viewUser.ID, activityLastFeed);
                        ViewData.Add("CurrentUser", currentUser);
                        ViewData.Add("User", viewUser);
                        result.empty = false;
                        result.dateend = activityLastFeed[activityLastFeed.Count - 1].CreatedDate.ToString();
                        if (activityLastFeed.Count < ConfigManager.MaxFeedItemLoad)
                        {
                            result.isend = true;
                        }
                        result.type = type;
                        result.result = PartialView("~/Areas/Feed/Views/Shared/_ActivityInFeedList.cshtml", activities.OrderByDescending(a=>a.CreatedDate).ToList()).Capture(ControllerContext);
                    }
                    else
                    {
                        result.empty = true;
                        result.result = string.Empty;
                        result.isend = true;
                        result.type = type;
                    }
                    break;
            }

            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [Serializable]
        private class LoadMoreResult
        {
            public int error { get; set; }
            public string errorDescription { get; set; }
            public bool empty { get; set; }
            public bool isend { get; set; }
            public string dateend { get; set; }
            public int type { get; set; }
            public string result { get; set; }
        }

        [Serializable]
        private class Result
        {
            public int error { get; set; }
            public string errorDescription { get; set; }
            public string result { get; set; }
        }
    }
}
