﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Social.Entity.Tik;
using Social.Entity.TikActivity;
using Social.Lib;
using Social.Lib.Activity;
using Social.Service.Interface.Tik;
using Social.Service.Interface.TikActivity;
using Social.Service.TikActivity;
using Social.Web.Areas.Feed.Models;
using Social.Web.Extends.Helper;

namespace Social.Web.Areas.Ajax.Controllers
{
    public class CommentController : Controller
    {
        IUserCommentService _userComment;
        IUserFeedService _userFeed;
        IUserProfileService _userProfile;
        IUserActivityService _userActivity;
        IUserLikeService _userLike;

        public CommentController(IUserCommentService userComment, IUserFeedService userFeed, IUserLikeService userLike, IUserActivityService userActivity, IUserProfileService userProfile)
        {
            _userComment = userComment;
            _userFeed = userFeed;
            _userActivity = userActivity;
            _userProfile = userProfile;
            _userLike = userLike;
        }

        private class CommentResult
        {
            public int error { get; set; }
            public string errorDescription { get; set; }
            public bool empty { get; set; }
            public string result { get; set; }
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Add(int type, Guid objectid, string comment)
        {
            comment = Base64String.Decode(comment);
            CommentResult result = new CommentResult();
            UserProfile currentUser = _userProfile.CurrentUser();
            if (currentUser == null)
            {
                result.error = 1;
                result.errorDescription = "Bạn chưa đăng nhập.";
                return Json(result, JsonRequestBehavior.AllowGet);
            }

            DateTime created = DateTime.Now;
            Guid commentid = Guid.NewGuid();
            if (_userActivity.Comment(currentUser, (Constants.Comment.ObjectType)type, objectid, comment, created, commentid))
            {
                var commentData = new Comment(_userComment.GetComment(commentid));
                result.error = 0;
                result.errorDescription = string.Empty;

                result.result = PartialView("~/Areas/Feed/Views/Shared/_CommentContent.cshtml", commentData).Capture(ControllerContext);
            }else
            {
                result.error = 1;
                result.errorDescription = "Có lỗi xảy ra!";
            }
            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult Delete(Guid commentid)
        {
            CommentResult result = new CommentResult();
            if(_userActivity.DeleteComment(commentid))
            {
                result.error = 0;
                result.errorDescription = string.Empty;
                result.empty = true;
                result.result = string.Empty;
            }
            else
            {
                result.error = 1;
                result.errorDescription = "Có lỗi xảy ra!";
                result.empty = true;
                result.result = string.Empty;
            }

            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult LoadAll(int type, Guid objectid)
        {
            CommentResult result = new CommentResult();
            result.error = 0;
            result.errorDescription = string.Empty;
            result.empty = true;

            Guid targetid = objectid;

            switch (type)
            {
                case (int)Constants.Comment.ObjectType.Feed:
                    var feed = _userFeed.GetFeed(objectid);
                    if(feed != null)
                    {
                        targetid = feed.ObjectID;
                    }
                    break;
                default:
                    targetid = objectid;
                break;
            }

            var comments = _userComment.GetComments(targetid);
            if(comments.Count > 0)
            {
                var newComments = Comment.Convert(comments);
                result.empty = false;
                result.result = PartialView("_AjaxAllCommentContent", newComments).Capture(ControllerContext);
            }else
            {
                result.empty = true;
                result.result = string.Empty;
            }
            return Json(result, JsonRequestBehavior.AllowGet);
        }
    }
}
