﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Mvc;
using System.Web.Script.Serialization;
using Social.Entity.Tik;
using Social.Entity.TikActivity;
using Social.Lib;
using Social.Lib.Photo;
using Social.Service.Interface.Tik;
using Social.Service.Interface.TikActivity;
using Social.Service.Interface.TikFriend;
using Social.Service.TikActivity;
using Social.Web.Areas.Feed.Models;
using Social.Web.Extends.Helper;
using Social.Lib.Activity;
using Constants = Social.Lib.Activity.Constants;

namespace Social.Web.Areas.Ajax.Controllers
{
    public class FeedController : Controller
    {
        IUserFeedService _userFeed;
        IUserCommentService _userComment;
        IUserLikeService _userLike;
        IUserProfileService _userProfile;
        IUserActivityService _userActivity;
        IFriendListService _friendlist;
        public FeedController(IUserFeedService userFeed, IUserCommentService userComment, IUserLikeService userLike, IUserActivityService userActivity, IUserProfileService userProfile, IFriendListService friendlist)
        {
            _userFeed = userFeed;
            _userProfile = userProfile;
            _userComment = userComment;
            _userLike = userLike;
            _userActivity = userActivity;
            _friendlist = friendlist;
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult PostStatus(Guid postid, string message)
        {
            message = Base64String.Decode(message);
            Result result = new Result();
            UserProfile postToUser = _userProfile.GetByID(postid);
            UserProfile currentUser = _userProfile.CurrentUser();
            if(currentUser == null)
            {
                result.error = 1;
                result.errorDescription = "Bạn chưa đăng nhập.";
                return Json(result, JsonRequestBehavior.AllowGet);
            }

            if(postToUser == null)
            {
                result.error = 1;
                result.errorDescription = "Không tìm thấy thành viên này.";
                return Json(result, JsonRequestBehavior.AllowGet);
            }

            //Code check permission
            
            if(string.IsNullOrEmpty(message))
            {
                result.error = 1;
                result.errorDescription = "Nội dung tin không được để rỗng!";
                return Json(result, JsonRequestBehavior.AllowGet);
            }

            DateTime created = DateTime.Now;
            Guid feedid = Guid.NewGuid();

            if(_userActivity.Post(currentUser.ID, postToUser.ID, message, created, feedid))
            {
                var feed = _userFeed.GetFeed(feedid);
                result.error = 0;
                result.errorDescription = string.Empty;
                result.empty = false;
                ViewData.Add("CurrentUser", currentUser);
                Feed.Models.Feed newFeed = new Feed.Models.Feed(feed);
                result.result = PartialView("~/Areas/Feed/Views/Shared/_FeedContent.cshtml", newFeed).Capture(ControllerContext);
            }else
            {
                result.error = 1;
                result.errorDescription = "Có lỗi xảy ra!";
                result.empty = true;
                result.result = string.Empty;
            }
            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult PostLink(Guid postid, string message, string link, string video)
        {
            message = Base64String.Decode(message);
            link = Base64String.Decode(link);
            video = Base64String.Decode(video);
            Result result = new Result();
            UserProfile postToUser = _userProfile.GetByID(postid);
            UserProfile currentUser = _userProfile.CurrentUser();
            
            if (currentUser == null)
            {
                result.error = 1;
                result.errorDescription = "Bạn chưa đăng nhập.";
                return Json(result, JsonRequestBehavior.AllowGet);
            }

            if (postToUser == null)
            {
                result.error = 1;
                result.errorDescription = "Không tìm thấy thành viên này.";
                return Json(result, JsonRequestBehavior.AllowGet);
            }

            //Code check permission

            //if (string.IsNullOrEmpty(message))
            //{
            //    result.error = 1;
            //    result.errorDescription = "Nội dung tin không được để rỗng!";
            //    return Json(result, JsonRequestBehavior.AllowGet);
            //}

            DateTime created = DateTime.Now;
            var feedid = Guid.NewGuid();

            Attachment.Video videoAttachment = null;
            Attachment.Link linkAttachment = null;

            if (!string.IsNullOrEmpty(video))
            {
               videoAttachment = new Attachment.Video(video);
            }
            else
            {
                linkAttachment = new Attachment.Link(link);
            }

            if(_userActivity.Post(currentUser.ID, postToUser.ID, message, created, feedid, linkAttachment, videoAttachment))
            {
                var feed = _userFeed.GetFeed(feedid);
                result.error = 0;
                result.errorDescription = string.Empty;
                result.empty = false;
                ViewData.Add("CurrentUser", currentUser);
                Feed.Models.Feed newFeed = new Feed.Models.Feed(feed);
                result.result = PartialView("~/Areas/Feed/Views/Shared/_FeedContent.cshtml", newFeed).Capture(ControllerContext);
            }else
            {
                result.error = 1;
                result.errorDescription = "Có lỗi xảy ra!";
                result.empty = true;
                result.result = string.Empty;
            }
            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult Delete(Guid feedid)
        {
            Result result = new Result();
            
            if(_userActivity.DeleteFeed(feedid))
            {
                result.error = 0;
                result.errorDescription = string.Empty;
                result.empty = false;
                result.result = string.Empty;
            }else
            {
                result.error = 1;
                result.errorDescription = "Có lỗi xảy ra!";
                result.empty = false;
                result.result = string.Empty;   
            }
            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult ExtractLink(string link)
        {
            LinkResult result = new LinkResult();
            string hashLink = Lib.Utility.Utility.GetMd5Hash(link.Trim());
            var resultCache = Memcached.MainCache.Get<LinkResult>("FeedExtractLink" + hashLink);
            if (resultCache != null)
            {
                result = resultCache;
            }
            else
            {
                result.error = 0;
                result.errorDescription = string.Empty;
                result.empty = false;

                WebCrawnler web = new WebCrawnler();
                WebCrawnler.LinkDetails linkDetails = web.GetDetails(link);

                result.image = linkDetails.Images.Count > 0;
                result.result = PartialView("_AjaxFeedExtractLink", linkDetails).Capture(ControllerContext);

                Memcached.MainCache.Store(Enyim.Caching.Memcached.StoreMode.Set, "FeedExtractLink" + hashLink, result, TimeSpan.FromDays(30));
            }
            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult ViewVideo(Guid feedid)
        {
            Result result = new Result();
            var resultCache = Memcached.MainCache.Get<Result>("FeedAttachmentVideo" + feedid);
            if (resultCache != null)
            {
                result = resultCache;
            }
            else
            {
                result.error = 0;
                result.errorDescription = string.Empty;

                UserFeed feed = _userFeed.GetFeed(feedid);
                if (feed != null)
                {
                    if (feed.AttachmentType == (int)Constants.Feed.AttachmentType.Video || feed.AttachmentType == (int)Constants.Feed.AttachmentType.Music)
                    {
                        Attachment.Video video = new Attachment.Video(feed.AttachmentParam);
                        result.empty = false;
                        result.result = PartialView("_AjaxViewVideo", video).Capture(ControllerContext);
                    }else
                    {
                        result.empty = true;
                        result.result = string.Empty;
                    }
                }
                Memcached.MainCache.Store(Enyim.Caching.Memcached.StoreMode.Set, "FeedAttachmentVideo" + feedid, result, TimeSpan.FromDays(30));
            }
            return Json(result, JsonRequestBehavior.AllowGet);
        }
        
        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult GetFeed(Guid feedid)
        {
            Result result = new Result();
            result.error = 0;
            result.errorDescription = string.Empty;
            UserProfile currentUser = _userProfile.CurrentUser();
            if (currentUser == null)
            {
                result.error = 1;
                result.errorDescription = "Bạn chưa đăng nhập.";
                return Json(result, JsonRequestBehavior.AllowGet);
            }
            var feed = new Feed.Models.Feed(_userFeed.GetFeed(feedid));
            ViewData.Add("CurrentUser", currentUser);
            result.result = PartialView("~/Areas/Feed/Views/Shared/_FeedContent.cshtml", feed).Capture(ControllerContext);
            result.empty = string.IsNullOrEmpty(result.result);
            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult GetCommentLike(int type, Guid objectid)
        {
            Result result = new Result();
            UserProfile currentUser = _userProfile.CurrentUser();
            if (currentUser == null)
            {
                result.error = 1;
                result.errorDescription = "Bạn chưa đăng nhập.";
                return Json(result, JsonRequestBehavior.AllowGet);
            }

            string like = string.Empty;
            var comments = Comment.Convert(_userComment.GetComments(objectid));
            var userlikes = _userLike.GetLikes(objectid);
            var likes = Like.Convert(userlikes);

            var listFriends = _friendlist.GetByOwnerID(SessionManager.CurrentUserID());
            if (listFriends == null) listFriends = new List<Guid>();
            if(userlikes == null) userlikes = new List<UserLike>();
            var friendLikes = Social.Web.Areas.Feed.Models.Like.Convert(userlikes.FindAll(l => listFriends.Contains(l.FromID)));
            var totalLike = likes.Count;
            var youLike = likes.Any(l => l.FromID == SessionManager.CurrentUserID());
            like = Like.GetFeedLikeString((int)Constants.Like.ObjectType.Feed, objectid, totalLike, youLike, friendLikes);


            result.error = 0;
            result.errorDescription = string.Empty;

            ViewData.Add("Type", type.ToString());
            ViewData.Add("ObjectID", objectid);
            ViewData.Add("Like", like);
            ViewData.Add("Likes", likes);
            ViewData.Add("Comments", comments);
            ViewData.Add("CurrentUser", currentUser);
            result.result = PartialView("_AjaxCommentLike").Capture(ControllerContext);
            result.empty = string.IsNullOrEmpty(result.result);
            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [Serializable]
        private class LinkResult
        {
            public int error { get; set; }
            public string errorDescription { get; set; }
            public bool empty { get; set; }
            public bool image { get; set; }
            public string result { get; set; }
        }

        [Serializable]
        private class Result
        {
            public int error { get; set; }
            public string errorDescription { get; set; }
            public bool empty { get; set; }
            public string result { get; set; }
        }

    }
}
