﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Social.Entity.Tik;
using Social.Entity.TikActivity;
using Social.Lib;
using Social.Lib.Activity;
using Social.Service.Interface.Tik;
using Social.Service.Interface.TikActivity;
using Social.Service.Interface.TikFriend;
using Social.Service.TikFriend;
using Social.Web.Areas.Feed.Models;
using Social.Web.Extends.Helper;

namespace Social.Web.Areas.Ajax.Controllers
{
    public class LikeController : Controller
    {
        IUserLikeService _userLike;
        IUserCommentService _userComment;
        IUserFeedService _userFeed;
        IUserActivityService _userActivity;
        IFriendListService _friendlist;
        IUserProfileService _userProfile;
        public LikeController(IUserLikeService userLike, IUserCommentService userComment, IUserFeedService userFeed, IUserActivityService userActivity, IFriendListService friendList, IUserProfileService userProfile)
        {
            _userLike = userLike;
            _userComment = userComment;
            _userFeed = userFeed;
            _userActivity = userActivity;
            _friendlist = friendList;
            _userProfile = userProfile;
        }

        private class LikeResult
        {
            public int error { get; set; }
            public string errorDescription { get; set; }
            public bool empty { get; set; }
            public string result { get; set; }
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult Like(int type, Guid objectid)
        {
            LikeResult result = new LikeResult();
           

            if (!SessionManager.IsLoggedIn())
            {
                result.error = 1;
                result.errorDescription = "Bạn chưa đăng nhập";
                result.empty = true;
                result.result = string.Empty;
                return Json(result, JsonRequestBehavior.AllowGet);
            }
            
            UserProfile currentUser = _userProfile.CurrentUser();
            DateTime created = DateTime.Now;

            if(_userActivity.Like(currentUser, (Constants.Like.ObjectType)type, objectid, created))
            {
                result.empty = false;
                result.error = 0;
                result.errorDescription = string.Empty;
                result.result = GetLikeResult(type, objectid);
            }else
            {
                result.error = 1;
                result.errorDescription = string.Empty;
                result.empty = true;
                result.result = string.Empty;
            }
            return Json(result, JsonRequestBehavior.AllowGet);
        }

        private string GetLikeResult(int type, Guid objectid)
        {
            string result = string.Empty;
            if(type == (int)Constants.Like.ObjectType.Feed)
            {
                var feed = _userFeed.GetFeed(objectid);
                if (feed != null)
                {
                    objectid = feed.ObjectID;
                }
            }

            var likes = _userLike.GetLikes(objectid);
            if (likes != null)
            {
                var listFriends = _friendlist.GetByOwnerID(SessionManager.CurrentUserID());
                if (listFriends == null) listFriends = new List<Guid>();
                var friendLikes = Social.Web.Areas.Feed.Models.Like.Convert(likes.FindAll(l => listFriends.Contains(l.FromID)));
                var totalLike = likes.Count;
                var youLike = likes.Any(l => l.FromID == SessionManager.CurrentUserID());
                switch (type)
                {
                    case (int)Constants.Like.ObjectType.Feed:
                    case (int)Constants.Like.ObjectType.Album:
                    case (int)Constants.Like.ObjectType.Image:
                    case (int)Constants.Like.ObjectType.Music:
                    case (int)Constants.Like.ObjectType.AlbumMusic:
                    case (int)Constants.Like.ObjectType.Note:
                        result = Feed.Models.Like.GetFeedLikeString(type, objectid, totalLike, youLike, friendLikes);
                        break;
                    case (int) Constants.Like.ObjectType.Comment:
                        result = string.Format(@"<a href=""#like"" onclick=""tikStream.showAllLike('{0}','{1}')"">{2} người</a>", type, objectid, likes.Count);
                        break;
                }
            }
            return result;
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult DisLike(int type, Guid objectid)
        {
            LikeResult result = new LikeResult();

            if (!SessionManager.IsLoggedIn())
            {
                result.error = 1;
                result.errorDescription = "Bạn chưa đăng nhập";
                result.empty = true;
                result.result = string.Empty;
                return Json(result, JsonRequestBehavior.AllowGet);
            } 
            
            UserProfile currentUser = _userProfile.CurrentUser();
            if(_userActivity.DisLike(currentUser, (Constants.Like.ObjectType)type, objectid))
            {
                result.error = 0;
                result.errorDescription = string.Empty;
                result.result = GetLikeResult(type, objectid);
                result.empty = string.IsNullOrEmpty(result.result);
            }else
            {
                result.error = 0;
                result.errorDescription = "Có lỗi xảy ra!";
                result.empty = true;
                result.result = string.Empty;
            }

            return Json(result, JsonRequestBehavior.AllowGet);
        }
        
        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult ShowLikeToolTip(int type, Guid objectid)
        {
            LikeResult likeResult = new LikeResult();
            likeResult.empty = false;
            likeResult.error = 0;
            likeResult.errorDescription = string.Empty;

            string result = string.Empty;
            var likes = _userLike.GetLikes(objectid);
            if (likes != null)
            {
                switch (type)
                {
                    case (int)Constants.Like.ObjectType.Comment:
                        #region Feed Like Calculate
                        var listFriends = _friendlist.GetByOwnerID(SessionManager.CurrentUserID());
                        if (listFriends == null) listFriends = new List<Guid>();
                        var friendLikes = Social.Web.Areas.Feed.Models.Like.Convert(likes.FindAll(l => listFriends.Contains(l.FromID)));
                        var totalLike = likes.Count;
                        var youLike = likes.Any(l => l.FromID == SessionManager.CurrentUserID());
                        result = Feed.Models.Like.GetFeedLikeStringNoHTML(type, objectid, totalLike, youLike, friendLikes);
                        #endregion
                        break;
                        
                }
            }
            
            if (string.IsNullOrEmpty(result)) likeResult.empty = true;
            likeResult.result = result;
            return Json(likeResult, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult GetLikes(Guid objectid)
        {
            LikeResult result = new LikeResult();
            result.empty = false;
            result.error = 0;
            result.errorDescription = string.Empty;

            if (!SessionManager.IsLoggedIn())
            {
                result.error = 1;
                result.errorDescription = "Bạn chưa đăng nhập";
                result.empty = true;
                result.result = string.Empty;
                return Json(result, JsonRequestBehavior.AllowGet);
            }
            
            UserProfile currentUser = _userProfile.CurrentUser();
            var listFriends = _friendlist.GetByOwnerID(SessionManager.CurrentUserID()) ?? new List<Guid>();

            var likes = _userLike.GetLikes(objectid);
            List<Like> likeList = Feed.Models.Like.Convert(likes);
            
            ViewData.Add("CurrentUser", currentUser);
            ViewData.Add("FriendList", listFriends);
            result.result = PartialView("_LikeList", likeList).Capture(ControllerContext);
            if (string.IsNullOrEmpty(result.result)) result.empty = true;

            return Json(result, JsonRequestBehavior.AllowGet);
        }
    }
}
