﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Social.Entity.TikActivity;
using Social.Lib;
using Social.Service.Interface.TikActivity;
using Social.Web.Areas.Feed.Models;
using Social.Web.Extends.Helper;
using Social.Service.Interface.TikApplication;
using Social.Entity.TikPhoto;
using Social.Service.Interface.TikPhoto;
using System.Web.Script.Serialization;
using System.Text;
using Social.Service.Interface.TikCms;
using Social.Service.Interface.Tik;
using Social.Entity.Tik;

namespace Social.Web.Areas.Ajax.Controllers
{
    public class NoteController : Controller
    {
        INoteService _note;
        IUserProfileService _user;
        IAlbumService _album;
        IPhotoService _photo;
        IUserActivityService _activity;
        private class NoteResult
        {
            public int error { get; set; }
            public string errorDescription { get; set; }
            public bool empty { get; set; }
            public string result { get; set; }
        }

        public NoteController(INoteService note, IAlbumService album, IPhotoService photo, IUserActivityService activity, IUserProfileService user)
        {
            _note = note;
            _user = user;
            _album = album;
            _photo = photo;
            _activity = activity;
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult ListAlbum()
        {
            IList<Album> listAlbum = _album.GetListAlbumByUserId(Social.Lib.SessionManager.CurrentUserID());
            string result = PartialView("_AjaxAllAlbum", listAlbum).Capture(ControllerContext);

            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult DetailAlbum(string albumid, string albumname)
        {
            Guid albumGuid = Guid.Parse(albumid);
            IList<Social.Entity.TikPhoto.Photo> listAlbum = _photo.GetAllPhotoByAlbumId(albumGuid);

            ViewData.Add("AlbumName", albumname);
            string result = PartialView("_AjaxDetailAlbum", listAlbum).Capture(ControllerContext);

            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public string UploadImage(HttpPostedFileBase file_upload_body)
        {
            file_upload_body = Request.Files[0];
            Guid userID = SessionManager.CurrentUserID();
            var content = new Byte[file_upload_body.ContentLength];
            file_upload_body.InputStream.Read(content, 0, file_upload_body.ContentLength);

            var psv = new Social.Web.Service.PhotoService();
            var jsonVal = psv.UploadNote(content, userID);

            var json_serializer = new JavaScriptSerializer();
            Social.Entity.TikPhoto.Photo photo = json_serializer.Deserialize<Social.Entity.TikPhoto.Photo>(jsonVal);

            // lay album chua anh cua note
            Social.Entity.TikPhoto.Album album = _album.GetByUserIdTypeAlbum(userID, 4).SingleOrDefault();

            string result = string.Format("{{\"albumID\":\"{0}\",\"albumName\":\"{1}\",\"urlimage\":\"{2}\"}}", album.ID, album.AlbumName, Social.Lib.Utility.Utility.UrlMainAlbumOrPhoto(photo.SourceUrl));

            return result;
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public string DeleteNote(Guid noteID)
        {
            //if(string.IsNullOrEmpty(noteID)) 
            try
            {
                Guid currentUserID = SessionManager.CurrentUserID();
                _note.Delete(u => u.ID == noteID && u.UserID == currentUserID);
                _activity.DeleteObject(noteID);
                return "1";
            }
            catch
            {
                return "0";
            }
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public string TagFriend(Guid noteID, string friendIDTags)
        {
            if (string.IsNullOrEmpty(friendIDTags))
                return "0";

            Social.Entity.TikApplication.Note noteUpdate = _note.GetNoteByID(noteID);

            if (noteUpdate == null) return "0";

            if (!noteUpdate.UserID.Equals(SessionManager.CurrentUserID())) return "0";

            StringBuilder noteTags = new StringBuilder();

            if (!string.IsNullOrEmpty(noteUpdate.Tags))
            {
                noteTags.Append(noteUpdate.Tags.Replace('[', ' ').Replace(']', ' ').Trim());
            }

            List<Guid> arrGuidFriend = new List<Guid>();
            List<UserProfile> lstNewFriendTags = new List<UserProfile>();

            string[] arrFriend = friendIDTags.Split(',');

            foreach (string item in arrFriend)
            {
                if (!string.IsNullOrEmpty(item))
                {
                    UserProfile user = _user.GetByID(Guid.Parse(item));
                    bool isAdd = false;

                    if (string.IsNullOrEmpty(noteTags.ToString()))
                    {
                        noteTags.AppendFormat("{{FriendID:'{0}',UserName:'{1}',FriendFullName:'{2}',AvatarSmall:'{3}'}}", item, user.UserName, user.FullName, user.SmallAvatarUrlMap);
                        isAdd = true;
                    }
                    else
                    {
                        if (!noteTags.ToString().Contains(item))
                        {
                            noteTags.AppendFormat(",{{FriendID:'{0}',UserName:'{1}',FriendFullName:'{2}',AvatarSmall:'{3}'}}", item, user.UserName, user.FullName, user.SmallAvatarUrlMap);
                            isAdd = true;
                        }
                    }

                    if (isAdd)
                    {
                        arrGuidFriend.Add(Guid.Parse(item));
                        lstNewFriendTags.Add(user);
                    }
                }
            }

            if (!string.IsNullOrEmpty(noteTags.ToString()))
            {
                noteUpdate.Tags = string.Format("[{0}]", noteTags.ToString());
            }

            try
            {
                _note.Update(noteUpdate);
                _activity.TagFriend(Social.Lib.SessionManager.CurrentUserID(), arrGuidFriend, Lib.Activity.Constants.Activity.ObjectType.Note, noteUpdate.ID, DateTime.Now, false);
                StringBuilder result = new StringBuilder();

                foreach (UserProfile item in lstNewFriendTags)
                {
                    result.AppendFormat(@"<li id=""FriendIDTags-{0}""><a href=""{1}"" class=""tooltip"" title=""{2}"">
                                            <img src=""{3}"" alt="""" width=""32"" height=""32"" />
                                            </a></li>",
                                            item.ID,
                                            Url.Content("~/" + item.UserName),
                                            @item.FullName,
                                            Social.Lib.Utility.Utility.GetAvatarUrl(item.SmallAvatarUrl));
                }
                return result.ToString();
            }
            catch { return "0"; }
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public string RemoveTag(Guid noteID)
        {
            Social.Entity.TikApplication.Note noteUpdate = _note.GetNoteByID(noteID);

            if (noteUpdate == null) return "0";

            if (string.IsNullOrEmpty(noteUpdate.Tags)) return "0";

            var json_serializer = new JavaScriptSerializer();
            List<Social.Web.Areas.Note.Controllers.FriendTag> friendTag = json_serializer.Deserialize<List<Social.Web.Areas.Note.Controllers.FriendTag>>(noteUpdate.Tags);

            StringBuilder sb = new StringBuilder();

            foreach (Social.Web.Areas.Note.Controllers.FriendTag item in friendTag)
            {
                if (!item.FriendID.Equals(SessionManager.CurrentUserID()))
                {
                    sb.AppendFormat(",{{FriendID:'{0}',UserName:'{1}',FriendFullName:'{2}',AvatarSmall:'{3}'}}", item.FriendID, item.UserName, item.FriendFullName, item.AvatarSmall);
                }
            }

            if (!string.IsNullOrEmpty(sb.ToString()))
            {
                sb.Remove(0, 1);
            }

            if (!string.IsNullOrEmpty(sb.ToString()))
            {
                noteUpdate.Tags = string.Format("[{0}]", sb.ToString());
            }

            try
            {
                _note.Update(noteUpdate);
                _activity.DeleteTagFriend(new List<Guid>() { SessionManager.CurrentUserID() }, Lib.Activity.Constants.Activity.ObjectType.Note, noteID);
                return SessionManager.CurrentUserID().ToString();
            }
            catch { return "0"; }
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult LoadMoreNote(string pageNumber, Guid userID, string userNameNote, string userFullNameNote, int totalNote)
        {
            int pageNum = 0;
            try
            {
                pageNum = int.Parse(pageNumber);
            }
            catch
            {
                pageNum = 1;
            }

            var totalRow = 0;
            var lstNote = new List<Social.Entity.TikApplication.Note>();

            lstNote = _note.GetListNoteByUserIdAndPageNumber(userID, pageNum, 10, out totalRow);

            StringBuilder sb = new StringBuilder();
            Guid userIDViewer = SessionManager.CurrentUserID();
            foreach (Social.Entity.TikApplication.Note item in lstNote)
            {
                if (Service.CheckPermission.IsView(SessionManager.CurrentUserID(), userID, item.PrivacyType, item.PrivacyPeople))
                {
                    totalNote++;
                    string url = Url.Content("~/" + userNameNote + Url.Action("Index/" + item.ID, "View", new { area = "Note" }));
                    sb.AppendFormat(@"<li id=""notedetail-{0}"">", item.ID);
                    sb.AppendFormat(@"  <a href=""{0}"" class=""t"">{1}</a>", url, item.Title);
                    sb.AppendFormat(@"  <div class=""time"">Bởi: <a href=""{0}"">{1}</a>, vào lúc {2}·", Url.Content("~/" + userNameNote), Base64String.Decode(userFullNameNote), @item.CreatedDate.ToString("dd/MM/yyyy hh:mm"));
                    if (userIDViewer.Equals(item.UserID))
                    {
                        sb.AppendFormat(@"      <a href=""{0}"">Chỉnh sửa</a>.", Url.Action("Index/" + item.ID, "Edit", new { area = "Note" }));
                        sb.AppendFormat(@"      <a href=""javascript:;"" onclick=""deleteNote('{0}')"">Xóa</a>", item.ID.ToString());
                    }
                    sb.AppendFormat("   </div>");
                    if (!string.IsNullOrEmpty(item.Photo))
                    {
                        sb.AppendFormat(@"<img width=""130"" height=""98"" title="""" alt="""" src=""{0}"" class=""fl""/>", item.PhotoUrl);
                    }

                    sb.AppendFormat(@"  <div {0}>", string.IsNullOrEmpty(item.Photo) ? " " : @"class=""fr if""");
                    sb.AppendFormat(@"      <p>{0}</p>", Social.Lib.Utility.Utility.RemoveHtmlTagAndSubString(item.Body, 50));
                    sb.AppendFormat(@"      <div class=""post_action"">");
                    sb.AppendFormat(@"          <a href=""{0}"">Xem chi tiết</a>", Url.Action("Index/" + item.ID, "View", new { area = "Note" }));
                    sb.AppendFormat(@"          <a href=""{0}"">Bình luận</a>", Url.Action("Index/" + item.ID, "View", new { area = "Note" }));
                    sb.AppendFormat(@"      </div>");
                    sb.AppendFormat(@"  </div>");
                    sb.AppendFormat(@"  <div class=""clr""></div>");
                    sb.AppendFormat(@"</li>");
                }
            }

            ResultListNote result = new ResultListNote();

            result.result = sb.ToString();
            result.isEmpty = false;
            result.isLoadMore = lstNote.Count >= 2;
            result.error = 1;
            result.pagerNumber = pageNum + 1;
            result.errorDescription = string.Empty;
            result.totalNote = totalNote;

            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [Serializable]
        private class ResultListNote
        {
            public int error { get; set; }
            public string errorDescription { get; set; }
            public bool isLoadMore { get; set; }
            public bool isEmpty { get; set; }
            public string result { get; set; }
            public int pagerNumber { get; set; }
            public int totalNote { get; set; }
        }
    }
}
