﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Social.Entity.TikFriend;
using Social.Lib;
using Social.Service.Interface.TikActivity;
using Social.Service.Interface.TikFriend;
using Social.Service.Interface.TikMessage;
using Social.Web.Areas.Feed.Models;
using Social.Web.Extends.Helper;

namespace Social.Web.Areas.Ajax.Controllers
{
    public class NotificationController : Controller
    {
        private IUserNotificationService _userNotification;
        private IMessageService _message;
        private IFriendRequestService _friendRequest;
        public NotificationController(IUserNotificationService userNotification, IMessageService message, IFriendRequestService friendRequest)
        {
            _userNotification = userNotification;
            _message = message;
            _friendRequest = friendRequest;
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GetNotify()
        {
            Result result = new Result();
            if (SessionManager.IsLoggedIn())
            {
                Random random = new Random();
                Guid currentID = SessionManager.CurrentUserID();
                result.message = _message.GetCountUnread(currentID);
                result.request = _friendRequest.GetCountByToUserID(currentID);
                result.anounment = _userNotification.CountUnread(SessionManager.CurrentUserID());
            }
            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult GetNewRequest()
        {
            RequestResult result = new RequestResult();
            if (SessionManager.IsLoggedIn())
            {
                result.error = 0;
                result.errorDescription = string.Empty;
                var listFriendRequest = _friendRequest.GetBy_FromToUserIDAndTop(SessionManager.CurrentUserID(), 0);
                result.result = View("_TopRequestFriend", listFriendRequest).Capture(ControllerContext);
                result.empty = string.IsNullOrEmpty(result.result);
            }
            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult GetNewMessage()
        {
            MessageResult result = new MessageResult();
            if(SessionManager.IsLoggedIn())
            {
                result.error = 0;
                result.errorDescription = string.Empty;
                var listTopMessage = _message.GetTopMessageViewByUserID(SessionManager.CurrentUserID());
                result.result = PartialView("_TopMessage", listTopMessage).Capture(ControllerContext);
                result.empty = string.IsNullOrEmpty(result.result);
            }
            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult GetNewAnounment()
        {
            MessageResult result = new MessageResult();
            if (SessionManager.IsLoggedIn())
            {
                result.error = 0;
                result.errorDescription = string.Empty;
                var notifications = _userNotification.GetAll(SessionManager.CurrentUserID(), false);
                List<Notification> listNotification = Notification.Convert(notifications, true);
                result.result = PartialView("_TopAnounment", listNotification.OrderByDescending(n => n.CreatedDate).Take(5).ToList()).Capture(ControllerContext);
                result.empty = string.IsNullOrEmpty(result.result);
                _userNotification.UpdateRead(SessionManager.CurrentUserID());
            }
            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [Serializable]
        private class AnounmentResult
        {
            public int error { get; set; }
            public string errorDescription { get; set; }
            public string result { get; set; }
            public bool empty { get; set; }
        }

        [Serializable]
        private class RequestResult
        {
            public int error { get; set; }
            public string errorDescription { get; set; }
            public string result { get; set; }
            public bool empty { get; set; }
        }

        [Serializable]
        private class MessageResult
        {
            public int error { get; set; }
            public string errorDescription { get; set; }
            public string result { get; set; }
            public bool empty { get; set; }
        }


        [Serializable]
        private class Result
        {
            public int error { get; set; }
            public string errorDescription { get; set; }
            public int anounment { get; set; }
            public int request { get; set; }
            public int message { get; set; }
        }
    }
}
