﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Mvc;
using Social.Entity.Tik;
using Social.Entity.TikFriend;
using Social.Lib;
using Social.Service.Interface.Tik;
using Social.Service.Interface.TikFriend;
using Social.Web.Extends.Helper;

namespace Social.Web.Areas.Ajax.Controllers
{
    public class UserController : Controller
    {
        private IFriendSearchService _friendSearch;
        IUserProfileService _userProfile;
        IFriendListService _friendList;
        IUserFriendService _userFriend;
        public UserController(IFriendSearchService friendSearch, IUserProfileService userProfile, IFriendListService friendList, IUserFriendService userFriend)
        {
            _friendSearch = friendSearch;
            _userProfile = userProfile;
            _friendList = friendList;
            _userFriend = userFriend;
        }

        private class UserResult
        {
            public int error { get; set; }
            public string errorDescription { get; set; }
            public bool empty { get; set; }
            public string result { get; set; }
            public int total { get; set; }
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult TagFriend(string keyword, string blast)
        {
            keyword = Base64String.Decode(keyword);
            blast = Base64String.Decode(blast);
            List<Guid> listTagged = Social.Lib.Activity.Feed.ListUserTagged(blast);

            var friendList = _friendSearch.GetByOwnerIDAndKeyWord_FullTextName(SessionManager.CurrentUserID(), keyword, 10);
            if(listTagged.Count > 0) friendList.RemoveAll(f => listTagged.Contains(f.FriendID));

            UserResult result = new UserResult();
            result.error = 0;
            result.errorDescription = string.Empty;
            
            result.total = friendList.Count;
            result.result = string.Empty;
            int i = 1;
            foreach (var friend in friendList)
            {
                UserProfile user = _userProfile.GetByID(friend.FriendID);
                if (user != null)
                {
                    //Regex nameRegex = new Regex(keyword);
                    //name = nameRegex.Replace(name, string.Format(@"<b>{0}</b>", keyword));
                    result.result += string.Format(@"<div class=""display_box_tagfriend"" onmouseover=""select({4})"" id=""tagfriend_{4}"" align=""left""><img src=""{0}"" width=""32"" class=""image""/><a id=""{1}"" title=""{2}""><b>{2}</b></a><br /><span class=""info"">{3}</span></div>", user.SmallAvatarUrl, user.ID, user.FullName, user.HomeAddressName, i);
                    i++;
                }
            }
            result.empty = string.IsNullOrEmpty(result.result);
            return Json(result, JsonRequestBehavior.AllowGet);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult ListUser(string listuser)
        {
            Result result = new Result();
            result.error = 0;
            result.errorDescription = string.Empty;
            result.result = string.Empty;

            if (!SessionManager.IsLoggedIn())
            {
                result.error = 1;
                result.errorDescription = "Bạn chưa đăng nhập";
                result.empty = true;
                result.result = string.Empty;
                return Json(result, JsonRequestBehavior.AllowGet);
            }

            UserProfile currentUser = _userProfile.CurrentUser();
            var listFriends = _friendList.GetByOwnerID(SessionManager.CurrentUserID()) ?? new List<Guid>();
            if(!string.IsNullOrEmpty(listuser))
            {
                string[] listusers = Social.Lib.Base64String.Decode(listuser).Split(';');
                List<UserProfile> listUsers = new List<UserProfile>();
                foreach (string user in listusers)
                {
                    Guid u;
                    if (Guid.TryParse(user, out u))
                    {
                        UserProfile us = _userProfile.GetByID(u);
                        if (us != null)
                        {
                         if(!listUsers.Contains(us)) listUsers.Add(us);
                        }
                    }
                }

                ViewData.Add("CurrentUser", currentUser);
                ViewData.Add("FriendList", listFriends);
                result.result = PartialView("_UserList", listUsers).Capture(ControllerContext);
            }
            result.empty = string.IsNullOrEmpty(result.result);
            
            return Json(result, JsonRequestBehavior.AllowGet);
        }

        private class Result
        {
            public int error { get; set; }
            public string errorDescription { get; set; }
            public bool empty { get; set; }
            public string result { get; set; }
        }

    }
}
