﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Social.Entity.Tik;
using Social.Entity.TikActivity;
using Social.Lib;
using Social.Lib.Activity;
using Social.Service.Interface.Tik;
using Social.Service.Interface.TikActivity;
using Social.Service.Interface.TikFriend;
using Social.Service.TikActivity;
using Social.Web.Areas.Feed.Models;
using Social.Web.Extends.Helper;

namespace Social.Web.Areas.Feed.Controllers
{
    public class IndexController : Controller
    {
        IUserFeedService _userFeed;
        IUserProfileService _userProfile;
        IUserActivityService _userActivity;
        IFriendListService _friendlist;
        IUserCommentService _userComment;
        public IndexController(IUserFeedService userFeed, IUserCommentService userComment, IUserActivityService userActivity, IUserProfileService userProfile, IFriendListService friendlist)
        {
            _userFeed = userFeed;
            _userProfile = userProfile;
            _userActivity = userActivity;
            _friendlist = friendlist;
            _userComment = userComment;
        }

        //
        // GET: /Feed/Index/

        public ActionResult Index()
        {
            if (!SessionManager.CheckLogin()) return RedirectToRoute("Register_Index");

            UserProfile currentUser = _userProfile.CurrentUser();

            List<Guid> listFriends = _friendlist.GetByOwnerID(currentUser.ID);
            var activity = _userActivity.GetFeedActivity(currentUser.ID,  listFriends, DateTime.Now, ConfigManager.MaxFeedItemLoad);
            var activities = ActivityFeed.Convert(currentUser.ID, activity);

            ViewData.Add("CurrentUser", _userProfile.CurrentUser());
            ViewData.Add("User", _userProfile.CurrentUser());
            return View(activities);
        }


        public ActionResult View(string objectid, bool comment = false)
        {
            if (!SessionManager.CheckLogin()) return RedirectToRoute("Register_Index");
            UserProfile currentUser = _userProfile.CurrentUser();
            ViewData.Add("CurrentUser", _userProfile.CurrentUser());
            ViewData.Add("User", _userProfile.CurrentUser());
            ViewData.Add("ShowAllComments", "true");
            
            if(comment)
            {
                 Guid commentid;
                 if (Guid.TryParse(objectid, out commentid))
                 {
                     var userComment = _userComment.GetComment(commentid);
                     if (userComment == null) return RedirectToActionPermanent("ServerError", "Error", new { area = "" });
                     var owner = _userProfile.GetByID(userComment.OwnerID);
                     if (owner == null) return RedirectToActionPermanent("ServerError", "Error", new { area = "" });
                     string link = string.Empty;
                     switch (userComment.ObjectType)
                     {
                         case (int) Constants.Comment.ObjectType.AlbumMusic:
                             link = string.Format(@"~/{0}/Music/AlbumDetail/{1}", owner.UserName, userComment.ObjectID);
                             return RedirectPermanent(link);
                             break;
                         case (int) Constants.Comment.ObjectType.Music:
                             link = string.Format(@"~/{0}/Music/SongDetail/{1}", owner.UserName, userComment.ObjectID);
                             return RedirectPermanent(link);
                             break;
                         case (int) Constants.Comment.ObjectType.Album:
                             link = string.Format(@"~/{0}/Photo/ListPhotoInAlbum/{1}", owner.UserName, userComment.ObjectID);
                             return RedirectPermanent(link);
                             break;
                         case (int) Constants.Comment.ObjectType.Image:
                             link = string.Format(@"~/{0}/Photo/PhotoDetail/{1}", owner.UserName, userComment.ObjectID);
                             return RedirectPermanent(link);
                             break;
                         case (int) Constants.Comment.ObjectType.Note:
                             link = string.Format(@"~/{0}/Note/View/Index/{1}", owner.UserName, userComment.ObjectID);
                             return RedirectPermanent(link);
                             break;
                         case (int) Constants.Comment.ObjectType.Link:
                         case (int) Constants.Comment.ObjectType.Post:
                         case (int) Constants.Comment.ObjectType.Status:
                         case (int) Constants.Comment.ObjectType.Feed:
                         default:
                             var userFeed = _userFeed.GetFeed(userComment.ObjectID);
                             if (userFeed == null) return RedirectToActionPermanent("ServerError", "Error", new {area = ""});
                             var feed = new Models.Feed(userFeed, false);
                             return View(feed);
                             break;
                     }
                 }
                 else
                 {
                     return RedirectToActionPermanent("ServerError", "Error", new {area = ""});
                 }
            }else
            {
                Guid feedid;
                if (Guid.TryParse(objectid, out feedid))
                {
                    var userFeed = _userFeed.GetFeed(feedid);
                    if(userFeed==null) return RedirectToActionPermanent("ServerError", "Error", new {area = ""});
                    var feed = new Models.Feed(userFeed, false);
                    return View(feed);
                }else
                {
                    return RedirectToActionPermanent("ServerError", "Error", new {area = ""});
                }
            }
        }


    }
}
