﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Social.Entity.Tik;
using Social.Entity.TikActivity;
using Social.Lib.Activity;

namespace Social.Web.Areas.Feed.Models
{
    public class ActivityFeed : Activity
    {
        public ActivityFeed(Guid ownerid, UserActivity userActivity)
        {
            this.ID = userActivity.ID;
            this.OwnerID = userActivity.OwnerID;
            this.CreatedDate = userActivity.CreatedDate;
            this.Owner = _userProfileService.GetByID(userActivity.OwnerID);
            this.TargetType = userActivity.TargetType;
            this.TargetID = userActivity.TargetID;
            this.ActivityType = userActivity.ActivityType;

            this.TargetUser = new UserProfile();
            if(userActivity.TargetType == (int)Constants.Activity.TargetType.User)
            {
                this.TargetUser = _userProfileService.GetByID(userActivity.TargetID);
            }

            this.ObjectType = userActivity.ObjectType;
            this.ObjectID = userActivity.ObjectID;

            switch (userActivity.ActivityType)
            {
                case (int)Constants.Activity.ActivityType.Post:
                case (int)Constants.Activity.ActivityType.Status:
                    switch (userActivity.ObjectType)
                    {
                        case (int)Constants.Activity.ObjectType.Feed:
                        case (int)Constants.Activity.ObjectType.Music:
                        case (int)Constants.Activity.ObjectType.Image:
                        case (int)Constants.Activity.ObjectType.Album:
                            if (userActivity.TargetType == (int)Constants.Activity.TargetType.User)
                            {
                                this.Type = TypeShow.Feed;
                                var feed = _userFeedService.GetFeed(userActivity.ObjectID);
                                if (feed != null)
                                    this.ObjectFeed = new Feed(feed, false);
                            }
                            break;
                    }
                    break;
                case (int)Constants.Activity.ActivityType.Friend:
                    this.Type = TypeShow.Friend;
                    this.ObjectFeedActivity = new FeedActivity(userActivity);
                    break;
                case (int)Constants.Activity.ActivityType.Tag:
                    this.Type = TypeShow.Tag;
                    switch (userActivity.ObjectType)
                    {
                        case (int)Constants.Activity.ObjectType.Image:
                        case (int)Constants.Activity.ObjectType.Album:
                            this.ObjectTag = new FeedTag(userActivity);
                            break;
                        case (int)Constants.Activity.ObjectType.Note:
                            this.ObjectTag = new FeedTag(userActivity);
                            break;
                    }
                    break;
                case (int)Constants.Activity.ActivityType.Comment:
                    //this.Type = TypeShow.Feed;
                    //this.ObjectFeedActivity = new FeedActivity(userActivity);
                    break;
            }

        }

        public ActivityFeed(Guid ownerid, Constants.Activity.ActivityType type, List<UserActivity> activities)
        {
            if(activities != null && activities.Count > 0)
            {
                var activityList = activities.OrderByDescending(a => a.CreatedDate).ToList();
                this.ID = activityList[0].ID;
                this.OwnerID = ownerid;
                this.CreatedDate = activityList[0].CreatedDate;
                this.Owner = _userProfileService.GetByID(ownerid);
                this.ActivityType = activityList[0].ActivityType;
                
                switch (type)
                {
                    case Constants.Activity.ActivityType.Tag:
                        this.Type = TypeShow.Tag;
                        this.ObjectTag = new FeedTag(activityList);
                        break;
                    case Constants.Activity.ActivityType.Friend:
                        this.Type = TypeShow.Friend;

                        var listTargetFriend = from a in activities
                                               where
                                                   a.OwnerID == ownerid &&
                                                   a.TargetType == (int) Constants.Activity.TargetType.User
                                               select a.TargetID;
                        var listOwnerFriend = from a in activities
                                              where
                                                  a.TargetType == (int) Constants.Activity.TargetType.User &&
                                                  a.TargetID == ownerid
                                              select a.OwnerID;
                        List<Guid> listFriends = new List<Guid>();
                        if(listTargetFriend.Any())
                        {
                            listFriends.AddRange(listTargetFriend.ToList());
                        }
                        if (listOwnerFriend.Any())
                        {
                            listFriends.AddRange(listOwnerFriend.ToList());
                        }
                        listFriends = listFriends.Distinct().ToList();
                        this.ObjectFeedFriend = new FeedFriend(Constants.Activity.ActivityType.Friend, this.ID, this.CreatedDate, ownerid, listFriends);
                        //this.ObjectFeedActivity = new FeedActivity(Constants.Activity.ActivityType.Friend, this.ID, ownerid, listFriends);
                        break;
                    case Constants.Activity.ActivityType.Profile:
                        switch (activityList[0].ObjectType)
                        {
                            case (int)Constants.Profile.Type.Avatar:
                                this.Type = TypeShow.ProfileAvatar;
                                List<Guid> listFriendChangeAvatar = new List<Guid>();
                                var listAvatar = activities.Select(a => a.OwnerID);
                                listFriendChangeAvatar.AddRange(listAvatar);
                                listFriendChangeAvatar = listFriendChangeAvatar.Distinct().ToList();
                                this.ObjectFeedChangeAvatar = new FeedChangeAvatar(Constants.Activity.ActivityType.Profile, this.ID, this.CreatedDate, ownerid, listFriendChangeAvatar);
                                break;
                        }
                        break;
                }
            }
        }
        public static List<ActivityFeed> Convert(Guid ownerid, List<UserActivity> userActivities)
        {
            List<ActivityFeed> activities = new List<ActivityFeed>();
            if (userActivities == null) return activities;

            List<UserActivity> listChangeAvatar = userActivities.FindAll(a => a.ActivityType == (int)Constants.Activity.ActivityType.Profile && a.ObjectType == (int)Constants.Profile.Type.Avatar).OrderByDescending(a=>a.CreatedDate).ToList();
            if(listChangeAvatar.Any())
            {
                ActivityFeed activity = new ActivityFeed(listChangeAvatar[0].OwnerID, Constants.Activity.ActivityType.Profile, listChangeAvatar);
                activities.Add(activity);
            }

            var activitiesTags = userActivities.FindAll(a => a.ActivityType == (int)Constants.Activity.ActivityType.Tag).GroupBy(a => a.ObjectID).ToList();
            foreach (IGrouping<Guid, UserActivity> grouping in activitiesTags)
            {
                List<UserActivity> listTag = grouping.OrderByDescending(n => n.CreatedDate).ToList();
                ActivityFeed activity = new ActivityFeed(listTag[0].OwnerID, Constants.Activity.ActivityType.Tag, listTag);
                activities.Add(activity);
            }

            List<Guid> listOwnerInActivities = new List<Guid>();

            var listActivityNotHaveChangeAvatar = userActivities.FindAll(a => !(a.ActivityType == (int)Constants.Activity.ActivityType.Profile && a.ObjectType == (int)Constants.Profile.Type.Avatar) && a.ActivityType != (int)Constants.Activity.ActivityType.Tag).ToList();
            foreach (UserActivity userActivity in listActivityNotHaveChangeAvatar)
            {
                if (userActivity.ActivityType == (int)Constants.Activity.ActivityType.Friend)
                {
                    if (!listOwnerInActivities.Contains(userActivity.OwnerID))
                    {
                        var userMakeFriends = listActivityNotHaveChangeAvatar.FindAll(a => a.ActivityType == (int)Constants.Activity.ActivityType.Friend && ((a.TargetType == (int)Constants.Activity.TargetType.User && a.TargetID == ownerid) || (a.OwnerID == userActivity.OwnerID)));
                        userMakeFriends = userMakeFriends.OrderByDescending(a => a.CreatedDate).ToList();
                        ActivityFeed activity = new ActivityFeed(userActivity.OwnerID, Constants.Activity.ActivityType.Friend, userMakeFriends);
                        activities.Add(activity);
                        listOwnerInActivities.Add(userActivity.OwnerID);
                    }
                }
                else if (userActivity.ActivityType == (int)Constants.Activity.ActivityType.Friend && userActivity.ObjectType == (int)Constants.Profile.Type.Avatar)
                {
                    
                }else
                {
                    ActivityFeed activity = new ActivityFeed(ownerid, userActivity);
                    activities.Add(activity);
                }
                
            }
            return activities;
        }

        public static List<ActivityFeed> ConvertSpecial(Guid ownerid, List<UserActivity> userActivities)
        {
            List<ActivityFeed> activities = new List<ActivityFeed>();
            if (userActivities == null) return activities;

            List<UserActivity> listChangeAvatar = userActivities.FindAll(a => a.ActivityType == (int)Constants.Activity.ActivityType.Profile && a.ObjectType == (int)Constants.Profile.Type.Avatar).OrderByDescending(a => a.CreatedDate).ToList();
            if (listChangeAvatar.Any())
            {
                ActivityFeed activity = new ActivityFeed(listChangeAvatar[0].OwnerID, Constants.Activity.ActivityType.Profile, listChangeAvatar);
                activities.Add(activity);
            }

            List<Guid> listOwnerInActivities = new List<Guid>();

            var listActivityNotHaveChangeAvatar = userActivities.FindAll(a => !(a.ActivityType == (int)Constants.Activity.ActivityType.Profile && a.ObjectType == (int)Constants.Profile.Type.Avatar)).ToList();
            foreach (UserActivity userActivity in listActivityNotHaveChangeAvatar)
            {
                if (userActivity.ActivityType == (int)Constants.Activity.ActivityType.Friend)
                {
                    if (!listOwnerInActivities.Contains(userActivity.OwnerID))
                    {
                        var userMakeFriends = listActivityNotHaveChangeAvatar.FindAll(a => a.ActivityType == (int)Constants.Activity.ActivityType.Friend && ((a.TargetType == (int)Constants.Activity.TargetType.User && a.TargetID == ownerid) || (a.OwnerID == userActivity.OwnerID)));
                        userMakeFriends = userMakeFriends.OrderByDescending(a => a.CreatedDate).ToList();
                        ActivityFeed activity = new ActivityFeed(userActivity.OwnerID, Constants.Activity.ActivityType.Friend, userMakeFriends);
                        activities.Add(activity);
                        listOwnerInActivities.Add(userActivity.OwnerID);
                    }
                }
                else if (userActivity.ActivityType == (int)Constants.Activity.ActivityType.Friend && userActivity.ObjectType == (int)Constants.Profile.Type.Avatar)
                {

                }
                else
                {
                    ActivityFeed activity = new ActivityFeed(ownerid, userActivity);
                    activities.Add(activity);
                }

            }
            return activities;
        }
    }
}
