﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Social.Entity.Tik;
using Social.Entity.TikActivity;
using Social.Lib.Activity;

namespace Social.Web.Areas.Feed.Models
{
    public class ActivityWall : Activity
    {
        public ActivityWall(Guid ownerid, UserActivity userActivity)
        {
            this.ID = userActivity.ID;
            this.OwnerID = userActivity.OwnerID;
            this.CreatedDate = userActivity.CreatedDate;
            this.Owner = _userProfileService.GetByID(userActivity.OwnerID);
            this.TargetType = userActivity.TargetType;
            this.TargetID = userActivity.TargetID;
            this.ActivityType = userActivity.ActivityType;

            this.TargetUser = new UserProfile();
            if(userActivity.TargetType == (int)Constants.Activity.TargetType.User)
            {
                this.TargetUser = _userProfileService.GetByID(userActivity.TargetID);
            }

            this.ObjectType = userActivity.ObjectType;
            this.ObjectID = userActivity.ObjectID;

            switch (userActivity.ActivityType)
            {
                case (int)Constants.Activity.ActivityType.Post:
                case (int)Constants.Activity.ActivityType.Status:
                    switch (userActivity.ObjectType)
                    {
                        case (int)Constants.Activity.ObjectType.Note:
                            break;
                        case (int)Constants.Activity.ObjectType.Feed:
                        case (int)Constants.Activity.ObjectType.Music:
                            if (userActivity.TargetType == (int)Constants.Activity.TargetType.User && userActivity.TargetID == ownerid)
                            {
                                this.Type = TypeShow.Feed;
                                var feed = _userFeedService.GetFeed(userActivity.ObjectID);
                                if (feed != null)
                                    this.ObjectFeed = new Feed(feed);
                            }
                            if (userActivity.TargetType == (int)Constants.Activity.TargetType.User && userActivity.TargetID != ownerid)
                            {
                                this.Type = TypeShow.Activity;
                                this.ObjectFeedActivity = new FeedActivity(userActivity);
                            }
                            break;
                    }
                    break;
                case (int)Constants.Activity.ActivityType.Friend:
                    this.Type = TypeShow.Activity;
                    this.ObjectFeedActivity = new FeedActivity(userActivity);
                    break;
                case (int)Constants.Activity.ActivityType.Tag:
                    this.Type = TypeShow.Tag;
                    switch (userActivity.ObjectType)
                    {
                        case (int)Constants.Activity.ObjectType.Image:
                        case (int)Constants.Activity.ObjectType.Album:
                        case (int)Constants.Activity.ObjectType.Note:
                            this.ObjectTag = new FeedTag(userActivity);
                            break;
                    }
                    break;
                case (int)Constants.Activity.ActivityType.Profile:
                    this.Type = TypeShow.Activity;
                    this.ObjectFeedActivity = new FeedActivity(userActivity);
                    break;
            }
        }

        public ActivityWall(Guid ownerid, Constants.Activity.ActivityType type, List<UserActivity> activities)
        {
            if(activities != null && activities.Count > 0)
            {
                var activityList = activities.OrderByDescending(a => a.CreatedDate).ToList();
                switch (type)
                {
                    case Constants.Activity.ActivityType.Friend:
                        this.ID = activityList[0].ID;
                        this.OwnerID = ownerid;
                        this.CreatedDate = activityList[0].CreatedDate;
                        this.Owner = _userProfileService.GetByID(ownerid);
                        this.ActivityType = activityList[0].ActivityType;
                        this.Type = TypeShow.Activity;

                        var listTargetFriend = from a in activities
                                               where
                                                   a.OwnerID == ownerid &&
                                                   a.TargetType == (int) Constants.Activity.TargetType.User
                                               select a.TargetID;
                        var listOwnerFriend = from a in activities
                                              where
                                                  a.TargetType == (int) Constants.Activity.TargetType.User &&
                                                  a.TargetID == ownerid
                                              select a.OwnerID;
                        List<Guid> listFriends = new List<Guid>();
                        if(listTargetFriend.Any())
                        {
                            listFriends.AddRange(listTargetFriend.ToList());
                        }
                        if (listOwnerFriend.Any())
                        {
                            listFriends.AddRange(listOwnerFriend.ToList());
                        }
                        listFriends = listFriends.Distinct().ToList();
                        this.ObjectFeedActivity = new FeedActivity(Constants.Activity.ActivityType.Friend, this.ID, ownerid, listFriends);
                        break;
                }
            }
        }

        public static List<ActivityWall> Convert(Guid ownerid, List<UserActivity> userActivities)
        {
            List<ActivityWall> activities = new List<ActivityWall>();
            if (userActivities == null) return activities;

            var userMakeFriends = userActivities.FindAll(a => a.ActivityType == (int)Constants.Activity.ActivityType.Friend && ((a.TargetType == (int)Constants.Activity.TargetType.User && a.TargetID == ownerid) || (a.OwnerID == ownerid)));
            userMakeFriends = userMakeFriends.OrderByDescending(a => a.CreatedDate).ToList();
            //if (userMakeFriends.Count > 0)
            //{
            //    var userMakeFriendIDs = from a in userMakeFriends select a.ID;
            //    userActivities.RemoveAll(a => userMakeFriendIDs.Contains(a.ID) && a.ID != userMakeFriends[0].ID);
            //}


            foreach (UserActivity userActivity in userActivities.FindAll(a => !(a.ActivityType == (int)Constants.Activity.ActivityType.Friend && ((a.TargetType == (int)Constants.Activity.TargetType.User && a.TargetID == ownerid) || (a.OwnerID == ownerid)))))
            {
                if (userActivity.ActivityType == (int)Constants.Activity.ActivityType.Friend)
                {
                    ActivityWall activity = new ActivityWall(ownerid, Constants.Activity.ActivityType.Friend, userMakeFriends);
                    activities.Add(activity);
                }
                else
                {
                    ActivityWall activity = new ActivityWall(ownerid, userActivity);
                    activities.Add(activity);
                }
            }
            return activities;
        }
    }
}
