﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Mvc;
using Social.Entity.Tik;
using Social.Entity.TikActivity;
using Social.Service.Interface.Tik;
using Social.Service.Interface.TikActivity;

namespace Social.Web.Areas.Feed.Models
{
    public class Comment 
    {
        public Guid ID { get; set; }
        public byte ObjectType { get; set; }
        public Guid ObjectID { get; set; }
        public Guid FromID { get; set; }
        public UserProfile FromUser { get; set; }

        public Byte CommentBy { get; set; }
        public DateTime CommentDate { get; set; }
        public String CommentContent { get; set; }
        public Int32 LikeCount { get; set; }

        public Guid OwnerID { get; set; }

        public List<Like> Likes { get; set; }

        private IUserProfileService _userProfileService = DependencyResolver.Current.GetService<IUserProfileService>();
        private IUserLikeService _userLikeService = DependencyResolver.Current.GetService<IUserLikeService>();

        public Comment(){}
        public Comment(UserComment userComment)
        {
            this.ID = userComment.ID;
            this.ObjectType = userComment.ObjectType;
            this.ObjectID = userComment.ObjectID;
            this.FromID = userComment.FromID;
            this.LikeCount = userComment.LikeCount;
            this.FromUser = _userProfileService.GetByID(userComment.FromID);
            if (this.FromUser == null) this.FromUser = new UserProfile();
            this.CommentDate = userComment.CommentDate;
            this.CommentContent = GetComment(userComment.CommentContent);
            this.Likes = Like.Convert(_userLikeService.GetLikes(userComment.ID));
            if(userComment.OwnerID != null)
            {
                this.OwnerID = userComment.OwnerID;
            }else
            {
                this.OwnerID = Guid.Empty;
            }
        }

        public static List<Comment> Convert(List<UserComment> userComments)
        {
            List<Comment> comments = new List<Comment>();
            if(userComments == null) return  comments;
            Comment comment;
            foreach (var userComment in userComments)
            {
                if (userComment != null)
                {
                    comment = new Comment(userComment);
                    comments.Add(comment);
                }
            }
            return comments;
        }

        private string GetComment(string message)
        {
            string output = message;
            //replace url
            output = Lib.Activity.Feed.ReplaceUrl(output);

            //Replace tag user
            string tagPattern = @"@{'([^']+)','([^']*)'}";
            Regex tagRegex = new Regex(tagPattern, RegexOptions.IgnoreCase);
            foreach (Match match in tagRegex.Matches(message))
            {
                string id = match.Groups[1].Value;
                string name = match.Groups[2].Value;
                Guid userid;
                if (Guid.TryParse(id, out userid))
                {
                    UserProfile user = _userProfileService.GetByID(userid);
                    if (user != null)
                    {
                        output = output.Replace(match.Value, string.Format(@"<a class=""fn"" href=""{0}"">{1}</a>", user.ProfileLink, user.FullName));
                    }
                    else
                    {
                        output = output.Replace(match.Value, name);
                    }
                }
            }

           

            return output;
        }
    }
}
