﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Mvc;
using Social.Entity.Tik;
using Social.Entity.TikActivity;
using Social.Lib;
using Social.Lib.Activity;
using Social.Service.Interface.Tik;
using Social.Service.Interface.TikActivity;
using Social.Service.Interface.TikFriend;
using Social.Service.Tik;
using Social.Service.TikActivity;
using Social.Web.Extends.Helper;

namespace Social.Web.Areas.Feed.Models
{
    public class Feed
    {
        public Guid ID { get; set; }
        public Guid OwnerID { get; set; }
        public Guid ObjectID { get; set; }

        public UserProfile Owner { get; set; }
        public byte TargetType { get; set; }
        public UserProfile TargetUser { get; set; }

        public Byte FeedType { get; set; }
        public String MessageTitle { get; set; }
        public String Message { get; set; }
        public Int32 LikeCount { get; set; }
        public Int32 CommentCount { get; set; }
        public DateTime CreatedDate { get; set; }
        public DateTime UpdatedDate { get; set; }

        public bool InWall { get; set; }

        public Byte AttachmentType { get; set; }
        public Attachment.Image Image { get; set; }
        public Attachment.Album Album { get; set; }
        public Attachment.Video Video { get; set; }
        public Attachment.Audio Audio { get; set; }
        public Attachment.Link Link { get; set; }
        public Attachment.Status Status { get; set; }
        public Attachment.Note Note { get; set; }

        public List<Comment> Comments { get; set; }
        public List<Like> Likes { get; set; }
        
        public string Like { get; set; }

        private IUserProfileService _userProfileService = DependencyResolver.Current.GetService<IUserProfileService>();
        private IUserCommentService _userCommentService = DependencyResolver.Current.GetService<IUserCommentService>();
        private IUserLikeService _userLikeService = DependencyResolver.Current.GetService<IUserLikeService>();
        private IFriendListService _friendlist = DependencyResolver.Current.GetService<IFriendListService>();
        public Feed(){}
        public Feed(UserFeed userFeed, bool inWall = true)
        {
            this.InWall = inWall;

            this.ID = userFeed.ID;
            this.OwnerID = userFeed.OwnerID;
            this.ObjectID = Guid.Empty.Equals(userFeed.ObjectID) ? userFeed.ID : userFeed.ObjectID;
            this.Owner = _userProfileService.GetByID(userFeed.OwnerID);
            if(this.Owner == null) this.Owner = new UserProfile();
            this.LikeCount = userFeed.LikeCount;
            this.CommentCount = userFeed.CommentCount;
            this.CreatedDate = userFeed.CreatedDate;
            this.UpdatedDate = userFeed.UpdatedDate;
            this.TargetType = userFeed.TargetType;
            this.AttachmentType = userFeed.AttachmentType;
            
            this.Comments = Comment.Convert(_userCommentService.GetComments(userFeed.ObjectID));
            var likes = _userLikeService.GetLikes(userFeed.ObjectID);
            this.Likes = Models.Like.Convert(likes);
            this.Like = GetLikeString(this.ObjectID, likes);
            
            this.TargetUser = new UserProfile();
            if(userFeed.TargetType == (int) Constants.Feed.TargetType.User)
            {
                this.TargetUser = _userProfileService.GetByID(userFeed.TargetID);
            }

            switch (userFeed.AttachmentType)
            {
                case (int)Constants.Feed.AttachmentType.Image:
                    this.Image = new Attachment.Image(userFeed.AttachmentParam);
                    break;
                case (int)Constants.Feed.AttachmentType.Album:
                    this.Album = new Attachment.Album(userFeed.AttachmentParam);
                    break;
                case (int)Constants.Feed.AttachmentType.Music:
                    this.Video = new Attachment.Video(userFeed.AttachmentParam);
                    break;
                case (int)Constants.Feed.AttachmentType.Link:
                    this.Link = new Attachment.Link(userFeed.AttachmentParam);
                    break;
                case (int)Constants.Feed.AttachmentType.Video:
                    this.Video = new Attachment.Video(userFeed.AttachmentParam);
                    break;
                case (int)Constants.Feed.AttachmentType.Note:
                    this.Note = new Attachment.Note(userFeed.AttachmentParam);
                    break;
                default:
                    break;
            }
            this.Message = GetMessage(userFeed.Message);
            this.MessageTitle = GetMessageTitle(userFeed.MessageType, userFeed.MessageParam);
        }

        public static List<Feed> Convert(List<UserFeed> userFeeds)
        {
            List<Feed> feeds = new List<Feed>();
            Feed feed;
            foreach (var userFeed in userFeeds)
            {
                if (userFeed != null)
                {
                    feed = new Feed(userFeed);
                    feeds.Add(feed);
                }
            }
            return feeds;
        }

        private string GetMessage(string message)
        {
            string output = message;
            //Replace url
            output = Lib.Activity.Feed.ReplaceUrl(output);

            //Replace tag user
            string tagPattern = @"@{'([^']+)','([^']*)'}";
            Regex tagRegex = new Regex(tagPattern, RegexOptions.IgnoreCase);
            foreach (Match match in tagRegex.Matches(message))
            {
                string id = match.Groups[1].Value;
                string name = match.Groups[2].Value;
                Guid userid;
                if (Guid.TryParse(id, out userid))
                {
                    UserProfile user = _userProfileService.GetByID(userid);
                    if(user!=null)
                    {
                        output = output.Replace(match.Value, string.Format(@"<a class=""fn"" href=""{0}"">{1}</a>", user.ProfileLink, user.FullName));
                    }else
                    {
                        output = output.Replace(match.Value, name);
                    }
                }
            }

            

            return output;
        }

        private string GetMessageTitle(byte type, string messageParram)
        {
            if (this.Owner.ID == this.TargetUser.ID)
            {
                if(this.AttachmentType == (int)Constants.Feed.AttachmentType.Album)
                {
                    return string.Format(@"đã thêm {0} ảnh mới vào album <a href=""{1}"">{2}</a>", this.Album.TotalImage, this.Album.Url, this.Album.Title);
                }
                
            }else
            {
                return this.InWall ? string.Empty : string.Format(@" &raquo; <a class=""fn"" href=""{0}"">{1}</a>", this.TargetUser.ProfileLink, this.TargetUser.FullName);
            }
            return string.Empty;
        }

        private string GetLikeString(Guid objectid, List<UserLike> likes)
        {
            string result = string.Empty;
            if (likes != null)
            {
                var listFriends = _friendlist.GetByOwnerID(SessionManager.CurrentUserID());
                if (listFriends == null) listFriends = new List<Guid>();
                var friendLikes = Social.Web.Areas.Feed.Models.Like.Convert(likes.FindAll(l => listFriends.Contains(l.FromID)));
                var totalLike = likes.Count;
                var youLike = likes.Any(l => l.FromID == SessionManager.CurrentUserID());
                result = Models.Like.GetFeedLikeString((int)Constants.Like.ObjectType.Feed, objectid, totalLike, youLike, friendLikes);
            }
            return result;
        }

    }
}
