﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Social.Entity.Tik;
using Social.Entity.TikActivity;
using Social.Lib.Activity;
using Social.Service.Interface.Tik;
using Social.Service.Interface.TikActivity;

namespace Social.Web.Areas.Feed.Models
{
    public class FeedActivity 
    {
        public Guid ID { get; set; }
        public UserProfile Owner { get; set; }
        public Constants.FeedActivity.Type Type { get; set; }
        public string ActivityContent { get; set; }
        public string ListFriends { get; set; }
        private IUserProfileService _userProfileService = DependencyResolver.Current.GetService<IUserProfileService>();
        private IUserFeedService _userFeedService = DependencyResolver.Current.GetService<IUserFeedService>();

        public FeedActivity(){}
        public FeedActivity(UserActivity activity)
        {
            this.ID = activity.ID;
            this.Owner = _userProfileService.GetByID(activity.OwnerID);
            if (this.Owner == null) this.Owner = new UserProfile();
            switch (activity.ActivityType)
            {
                case (int)Constants.Activity.ActivityType.Status:
                case (int)Constants.Activity.ActivityType.Post:
                    if(activity.TargetType == (int)Constants.Activity.TargetType.User)
                    {
                        UserProfile ownerUser = _userProfileService.GetByID(activity.OwnerID);
                        UserProfile targetUser = _userProfileService.GetByID(activity.TargetID);

                        if(activity.ObjectType == (int)Constants.Activity.ObjectType.Feed)
                        {
                            var feed = _userFeedService.GetFeed(activity.ObjectID);
                            if (targetUser != null && ownerUser != null)
                            {
                                switch (feed.AttachmentType)
                                {
                                    case (int)Constants.Feed.AttachmentType.None:
                                        this.ActivityContent = string.Format(@"<a class=""fn"" href=""{0}"">{1}</a> đã viết lên <a href=""{2}"">tường</a> của <a class=""fn"" href=""{3}"">{4}</a>", ownerUser.ProfileLink, ownerUser.FullName, "/Feed/View/" + feed.ID, targetUser.ProfileLink, targetUser.FullName);
                                        this.Type = Constants.FeedActivity.Type.Post;
                                        break;
                                    case (int)Constants.Feed.AttachmentType.Video:
                                    case (int)Constants.Feed.AttachmentType.Link:
                                        this.ActivityContent = string.Format(@"<a class=""fn"" href=""{0}"">{1}</a> đã chia sẻ <a href=""{2}"">liên kết</a> lên tường của <a class=""fn"" href=""{3}"">{4}</a>", ownerUser.ProfileLink, ownerUser.FullName, "/Feed/View/" + feed.ID, targetUser.ProfileLink, targetUser.FullName);
                                        this.Type = Constants.FeedActivity.Type.Link;
                                        break;
                                    case (int)Constants.Feed.AttachmentType.Image:
                                        this.ActivityContent = string.Format(@"<a class=""fn"" href=""{0}"">{1}</a> đã chia sẻ <a href=""{2}"">ảnh</a> lên tường của <a class=""fn"" href=""{3}"">{4}</a>", ownerUser.ProfileLink, ownerUser.FullName, "/Feed/View/" + feed.ID, targetUser.ProfileLink, targetUser.FullName);
                                        this.Type = Constants.FeedActivity.Type.Image;
                                        break;
                                    case (int)Constants.Feed.AttachmentType.Album:
                                        this.ActivityContent = string.Format(@"<a class=""fn"" href=""{0}"">{1}</a> đã chia sẻ <a href=""{2}"">album ảnh</a> lên tường của <a class=""fn"" href=""{3}"">{4}</a>", ownerUser.ProfileLink, ownerUser.FullName, "/Feed/View/" + feed.ID, targetUser.ProfileLink, targetUser.FullName);
                                        this.Type = Constants.FeedActivity.Type.Album;
                                        break;
                                    case (int)Constants.Feed.AttachmentType.Music:
                                        this.ActivityContent = string.Format(@"<a class=""fn"" href=""{0}"">{1}</a> đã chia sẻ <a href=""{2}"">bài nhạc</a> lên tường của <a class=""fn"" href=""{3}"">{4}</a>", ownerUser.ProfileLink, ownerUser.FullName, "/Feed/View/" + feed.ID, targetUser.ProfileLink, targetUser.FullName);
                                        this.Type = Constants.FeedActivity.Type.Music;
                                        break;
                                }
                                
                            }
                        }
                    }
                    break;
                case (int)Constants.Activity.ActivityType.Friend:
                    this.ActivityContent = "Hành động Kết bạn;";
                    this.Type = Constants.FeedActivity.Type.Friend;
                    break;
                case (int)Constants.Activity.ActivityType.Profile:
                    UserProfile owner = _userProfileService.GetByID(activity.OwnerID);
                    switch (activity.ObjectType)
                    {
                        case (int)Constants.Profile.Type.Avatar:
                            this.Type = Constants.FeedActivity.Type.Image;
                            this.ActivityContent = string.Format(@"<a class=""fn"" href=""{0}"">{1}</a> đã  thay đổi ảnh cá nhân của {2}.", owner.ProfileLink, owner.FullName, owner.Gender == 1 ? "anh ấy" : "cô ấy");
                            break;
                    }
                    
                    
                    break;
            }
        }

        public FeedActivity(Constants.Activity.ActivityType type, Guid activityid, Guid ownerid,  List<Guid> friendIDs)
        {
            switch (type)
            {
                case Constants.Activity.ActivityType.Friend:
                    this.ID = activityid;
                    this.Owner = _userProfileService.GetByID(ownerid);
                    if (this.Owner == null) this.Owner = new UserProfile();
                    this.Type = Constants.FeedActivity.Type.Friend;
                    this.ActivityContent = string.Format(string.Format("{0} giờ là bạn của {1}", Owner.FullName, BuildListFriend(ownerid, friendIDs)), activityid);
                    var listFriends = String.Join(";", friendIDs.Select(g => g.ToString()).ToArray());
                    this.ListFriends = Social.Lib.Base64String.Encode(listFriends);
                    break;
            }
        }

        private string BuildListFriend(Guid ownerid, List<Guid> friendIDs)
        {
            string result = string.Empty;
            if(friendIDs.Count > 0 && friendIDs.Count <= 2)
            {
                foreach (var friendID in friendIDs)
                {
                    var user = _userProfileService.GetByID(friendID);
                    if(user != null){
                        if (!string.IsNullOrEmpty(result)) result += " và ";
                        result += string.Format(@"<a class=""fn"" href=""{0}"">{1}</a>", user.ProfileLink, user.FullName);
                    }
                }
            }else
            {
                int i = 0;
                var user = _userProfileService.GetByID(friendIDs[0]);
                while (user == null && i < friendIDs.Count)
                {
                    user = _userProfileService.GetByID(friendIDs[i]);
                    i++;
                }
                if (user != null)
                {
                    result = string.Format(@"<a class=""fn"" href=""{0}"">{1}</a>", user.ProfileLink, user.FullName);
                    result += string.Format(@" và <a class=""fn"" href=""javascript:void(0);"" onclick=""tikStream.getListUser('{{0}}')"">{0} người</a> khác.", friendIDs.Count - 1);
                }
            }

            return result;
        }

    }
}
