﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Social.Entity.Tik;
using Social.Lib.Activity;
using Social.Service.Interface.Tik;
using Social.Service.Interface.TikActivity;

namespace Social.Web.Areas.Feed.Models
{
    public class FeedChangeAvatar
    {
        public Guid ID { get; set; }
        public UserProfile Owner { get; set; }
        public Constants.FeedActivity.Type Type { get; set; }
        public DateTime CreatedDate { get; set; }
        public string ActivityContent { get; set; }
        public string ListFriends { get; set; }
        public  List<UserProfile> Friends { get; set; }
        private IUserProfileService _userProfileService = DependencyResolver.Current.GetService<IUserProfileService>();
        private IUserFeedService _userFeedService = DependencyResolver.Current.GetService<IUserFeedService>();

        public FeedChangeAvatar(Constants.Activity.ActivityType type, Guid activityid, DateTime createdDate, Guid ownerid, List<Guid> friendIDs)
        {
            switch (type)
            {
                case Constants.Activity.ActivityType.Profile:
                    this.ID = activityid;
                    this.Owner = _userProfileService.GetByID(ownerid);
                    this.CreatedDate = createdDate;
                    if (this.Owner == null) this.Owner = new UserProfile();
                    this.Type = Constants.FeedActivity.Type.Friend;
                    this.ActivityContent = string.Format(string.Format("{0} đã thay đổi ảnh cá nhân của mình.", BuildListFriend(ownerid, friendIDs)), activityid);
                    var listFriends = String.Join(";", friendIDs.Select(g => g.ToString()).ToArray());
                    this.ListFriends = Social.Lib.Base64String.Encode(listFriends);
                    this.Friends = new List<UserProfile>();
                    foreach (Guid friendID in friendIDs)
                    {
                        UserProfile friend = _userProfileService.GetByID(friendID);
                        if(friend != null)
                        {
                            if(!this.Friends.Contains(friend)) this.Friends.Add(friend);
                        }
                    }
                    break;
            }
        }

        private string BuildListFriend(Guid ownerid, List<Guid> friendIDs)
        {
            string result = string.Empty;
            if(friendIDs.Count > 0 && friendIDs.Count <= 2)
            {
                foreach (var friendID in friendIDs)
                {
                    var user = _userProfileService.GetByID(friendID);
                    if(user != null){
                        if (!string.IsNullOrEmpty(result)) result += " và ";
                        result += string.Format(@"<a class=""fn"" href=""{0}"">{1}</a>", user.ProfileLink, user.FullName);
                    }
                }
            }else
            {
                int i = 0;
                var user = _userProfileService.GetByID(friendIDs[0]);
                while (user == null && i < friendIDs.Count)
                {
                    user = _userProfileService.GetByID(friendIDs[i]);
                    i++;
                }
                if (user != null)
                {
                    result = string.Format(@"<a class=""fn"" href=""{0}"">{1}</a>", user.ProfileLink, user.FullName);
                    result += string.Format(@" và <a class=""fn"" href=""javascript:void(0);"" onclick=""tikStream.getListUser('{{0}}')"">{0} người</a> khác.", friendIDs.Count - 1);
                }
            }

            return result;
        }
    }
}
