﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Social.Entity.Tik;
using Social.Entity.TikActivity;
using Social.Lib.Activity;
using Social.Service.Interface.Tik;

namespace Social.Web.Areas.Feed.Models
{
    public class FeedComment
    {
        public Guid ID { get; set; }
        public Guid ActivityID { get; set; }
        public UserProfile Owner { get; set; }
        public UserProfile TargetUser { get; set; }

        public string MessageTitle { get; set; }
        public Constants.Tag.AttachmentType AttachmentType { get; set; }
        public Attachment.Image Image { get; set; }
        public Attachment.Album Album { get; set; }
        public DateTime CreatedDate { get; set; }
        
        private IUserProfileService _userProfileService = DependencyResolver.Current.GetService<IUserProfileService>();

        public FeedComment() { }
        public FeedComment(UserActivity activity)
        {
            this.ActivityID = activity.ID;
            this.CreatedDate = activity.CreatedDate;
            this.Owner = _userProfileService.GetByID(activity.OwnerID) ?? new UserProfile();

            if (activity.TargetType == (int)Constants.Activity.TargetType.User)
            {
                this.TargetUser = _userProfileService.GetByID(activity.TargetID) ?? new UserProfile();
            }

            if (this.TargetUser.ID == this.Owner.ID)
            {
                this.MessageTitle = string.Format(@"đã bình luận về {{0}} của {0}.", this.Owner.Gender == 1 ? "anh ấy": "chị ấy");
                
            }else
            {
                this.MessageTitle = string.Format(@"đã bình luận về {{0}} của <a class=""fn"" href=""{0}"">{1}</a>", Owner.ProfileLink, Owner.FullName);
            }

            switch (activity.ObjectType)
            {
                case  (int)Constants.Activity.ObjectType.Image:
                    this.AttachmentType = Constants.Tag.AttachmentType.Image;
                    //this.Image = new Attachment.Image(activity.ObjectID);
                    
                    this.MessageTitle = string.Format(this.MessageTitle, string.Format(@"<a href=""{0}"">hình</a>", this.Image.ImageUrl));
                    break;
                case (int)Constants.Activity.ObjectType.Album:
                    this.AttachmentType = Constants.Tag.AttachmentType.Image;
                    //this.Album = new Attachment.Album(activity.ObjectID);

                    this.MessageTitle = string.Format(this.MessageTitle, "album");
                    break;
                case  (int)Constants.Activity.ObjectType.Status:
                    this.MessageTitle = string.Format(this.MessageTitle, "trạng thái");
                    break;
                case (int)Constants.Activity.ObjectType.Post:
                    this.MessageTitle = string.Format(this.MessageTitle, "trạng thái");
                    break;
                case (int)Constants.Activity.ObjectType.Link:
                    this.MessageTitle = string.Format(this.MessageTitle, "liên kết");
                    break;
                case (int)Constants.Activity.ObjectType.Note:
                    this.MessageTitle = string.Format(this.MessageTitle, "ghi chép");
                    break;
            }
            
           
        }
    }
}
