﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Social.Entity.Tik;
using Social.Entity.TikActivity;
using Social.Lib;
using Social.Lib.Activity;
using Social.Service.Interface.Tik;
using Social.Service.Interface.TikActivity;
using Social.Service.Interface.TikApplication;
using Social.Service.Interface.TikPhoto;

namespace Social.Web.Areas.Feed.Models
{
    public class FeedTag
    {
        public Guid ID { get; set; }
        public Guid ActivityID { get; set; }
        public UserProfile Owner { get; set; }
        public UserProfile TagUser { get; set; }

        public string MessageTitle { get; set; }
        public Constants.Tag.AttachmentType AttachmentType { get; set; }
        public Attachment.Image Image { get; set; }
        public Attachment.Note Note { get; set; }
        public DateTime CreatedDate { get; set; }
        public FeedTag(){}

        private IUserProfileService _userProfileService = DependencyResolver.Current.GetService<IUserProfileService>();
        private INoteService _userNote = DependencyResolver.Current.GetService<INoteService>();
        private IPhotoService _userPhoto = DependencyResolver.Current.GetService<IPhotoService>();
        private IUserFeedService _userFeed = DependencyResolver.Current.GetService<IUserFeedService>();
        
        public FeedTag(List<UserActivity> activities)
        {
            this.ActivityID = activities[0].ID;
            this.CreatedDate = activities[0].CreatedDate;
            this.Owner = _userProfileService.GetByID(activities[0].OwnerID) ?? new UserProfile();
            this.MessageTitle = @"{{{{0}}}} đã được đánh dấu trong {0} của <a class=""fn"" href=""{{1}}"">{{0}}</a>";
            UserProfile owner = null;

            switch (activities[0].ObjectType)
            {
                case (int)Constants.Activity.ObjectType.Image:
                    this.AttachmentType = Constants.Tag.AttachmentType.Image;

                    var resultCachePhoto = Memcached.MainCache.Get<Social.Entity.TikPhoto.Photo>("FeedTagPhoto_" + activities[0].ObjectID);
                    Social.Entity.TikPhoto.Photo photo;
                    if (resultCachePhoto != null)
                    {
                        photo = resultCachePhoto;
                    }
                    else
                    {
                        photo = _userPhoto.GetById(activities[0].ObjectID);
                        Memcached.MainCache.Store(Enyim.Caching.Memcached.StoreMode.Set, "FeedTagPhoto_" + activities[0].ObjectID, photo, TimeSpan.FromDays(1));
                    }
                    if (photo == null)
                    {
                        this.Image = new Attachment.Image();
                    }
                    else
                    {
                        if (photo.UserID != this.Owner.ID)
                        {
                            owner = _userProfileService.GetByID(photo.UserID);
                        }
                        string imageTitle = string.Empty;
                        string imageDescription = string.Empty;
                        string imageAlbumUrl = string.Empty;
                        string imagePath = string.Empty;
                        string imageUrl = string.Empty;

                        if (photo.Album != null)
                        {
                            imageTitle = string.IsNullOrEmpty(photo.Album.AlbumName) ? string.Empty : photo.Album.AlbumName;
                            imageDescription = string.IsNullOrEmpty(photo.Album.Description) ? string.Empty : photo.Album.Description;
                            imageAlbumUrl = string.IsNullOrEmpty(photo.Album.AlbumUrl) ? string.Empty : photo.Album.AlbumUrl;
                        }
                        else
                        {
                            imageAlbumUrl = photo.PhotoUrl;
                        }

                        imagePath = Lib.Utility.Utility.UrlThumbAlbumOrPhoto(photo.SourceUrl);
                        imageUrl = photo.PhotoUrl;

                        this.Image = new Attachment.Image(imageTitle, imageDescription, imageAlbumUrl, imagePath, imageUrl);
                    }
                    this.MessageTitle = string.Format(this.MessageTitle, "hình");
                    break;
                case (int)Constants.Activity.ObjectType.Note:
                    this.AttachmentType = Constants.Tag.AttachmentType.Note;
                    var resultCacheNote = Memcached.MainCache.Get<Social.Entity.TikApplication.Note>("FeedTagNote_" + activities[0].ObjectID);
                    Social.Entity.TikApplication.Note note;
                    if (resultCacheNote != null)
                    {
                        note = resultCacheNote;
                    }
                    else
                    {
                        note = _userNote.GetNoteByID(activities[0].ObjectID);
                        Memcached.MainCache.Store(Enyim.Caching.Memcached.StoreMode.Set, "FeedTagNote_" + activities[0].ObjectID, note, TimeSpan.FromDays(1));
                    }

                    if (note == null)
                    {
                        this.Note = new Attachment.Note();
                    }
                    else
                    {
                        if (note.UserID != this.Owner.ID)
                        {
                            owner = _userProfileService.GetByID(note.UserID);
                        }
                        string noteTitle = note.Title;
                        string noteDescription = note.ShortDescription;
                        string noteUrl = note.Url;
                        string noteImage = note.PhotoUrl;

                        this.Note = new Attachment.Note(noteTitle, noteDescription, noteUrl, noteImage);
                    }
                    this.MessageTitle = string.Format(this.MessageTitle, "bài viết");
                    break;
            }

            if (owner != null)
            {
                this.MessageTitle = string.Format(this.MessageTitle, owner.FullName, owner.ProfileLink);
            }
            else
            {
                this.MessageTitle = string.Format(this.MessageTitle, this.Owner.FullName, this.Owner.ProfileLink);
            }

            if (activities[0].TargetType == (int)Constants.Activity.TargetType.User)
            {
                this.TagUser = _userProfileService.GetByID(activities[0].TargetID) ?? new UserProfile();
            }

            int total = 0;
            string content = string.Empty;

            foreach (UserActivity activity in activities)
            {
                if (total == 0)
                {
                    total++;
                    continue;
                }
                if (total < 2)
                {
                    UserProfile user = _userProfileService.GetByID(activity.TargetID);
                    if (user != null)
                    {
                        content += string.Format(@", <a class=""fn"" href=""{0}"">{1}</a>", user.ProfileLink, user.FullName);
                        total++;
                    }
                }
                else
                {
                    break;
                }
            }
            if (activities.Count - total > 0)
            {
                content += string.Format(" và {0} người khác", activities.Count - total);
            }

            this.MessageTitle = string.Format(this.MessageTitle, content);
        }

        public FeedTag(UserActivity activity)
        {
            this.ActivityID = activity.ID;
            this.CreatedDate = activity.CreatedDate;
            this.Owner = _userProfileService.GetByID(activity.OwnerID) ?? new UserProfile();

            this.MessageTitle = @"đã được đánh dấu trong {0} của <a class=""fn"" href=""{{1}}"">{{0}}</a>";
            UserProfile owner = null;
            switch (activity.ObjectType)
            {
                case  (int)Constants.Activity.ObjectType.Image:
                    this.AttachmentType = Constants.Tag.AttachmentType.Image;

                    var resultCachePhoto = Memcached.MainCache.Get<Social.Entity.TikPhoto.Photo>("FeedTagPhoto_" + activity.ObjectID);
                    Social.Entity.TikPhoto.Photo photo;
                    if (resultCachePhoto != null)
                    {
                        photo = resultCachePhoto;
                    }
                    else
                    {
                        photo = _userPhoto.GetById(activity.ObjectID);
                        Memcached.MainCache.Store(Enyim.Caching.Memcached.StoreMode.Set, "FeedTagPhoto_" + activity.ObjectID, photo, TimeSpan.FromDays(1));
                    }
                    if (photo == null)
                    {
                        this.Image = new Attachment.Image();
                    }
                    else
                    {
                        if(photo.UserID != this.Owner.ID)
                        {
                            owner = _userProfileService.GetByID(photo.UserID);
                        }
                        string imageTitle = string.Empty;
                        string imageDescription = string.Empty;
                        string imageAlbumUrl = string.Empty;
                        string imagePath = string.Empty;
                        string imageUrl = string.Empty;

                        if (photo.Album != null)
                        {
                            imageTitle = string.IsNullOrEmpty(photo.Album.AlbumName) ? string.Empty : photo.Album.AlbumName;
                            imageDescription = string.IsNullOrEmpty(photo.Album.Description) ? string.Empty : photo.Album.Description;
                            imageAlbumUrl = string.IsNullOrEmpty(photo.Album.AlbumUrl) ? string.Empty : photo.Album.AlbumUrl;
                        }
                        else
                        {
                            imageAlbumUrl = photo.PhotoUrl;
                        }

                        imagePath = Lib.Utility.Utility.UrlThumbAlbumOrPhoto(photo.SourceUrl);
                        imageUrl = photo.PhotoUrl;

                        this.Image = new Attachment.Image(imageTitle, imageDescription, imageAlbumUrl, imagePath, imageUrl);
                    }
                    this.MessageTitle = string.Format(this.MessageTitle, "hình");
                    break;
                case (int)Constants.Activity.ObjectType.Note:
                    this.AttachmentType = Constants.Tag.AttachmentType.Note;
                    var resultCacheNote = Memcached.MainCache.Get<Social.Entity.TikApplication.Note>("FeedTagNote_" + activity.ObjectID);
                    Social.Entity.TikApplication.Note note;
                    if (resultCacheNote != null)
                    {
                        note = resultCacheNote;
                    }
                    else
                    {
                        note = _userNote.GetNoteByID(activity.ObjectID);
                        Memcached.MainCache.Store(Enyim.Caching.Memcached.StoreMode.Set, "FeedTagNote_" + activity.ObjectID, note, TimeSpan.FromDays(1));
                    }
                    
                    if(note == null)
                    {
                        this.Note = new Attachment.Note();
                    }else
                    {
                        if (note.UserID != this.Owner.ID)
                        {
                            owner = _userProfileService.GetByID(note.UserID);
                        }
                        string noteTitle = note.Title;
                        string noteDescription = note.ShortDescription;
                        string noteUrl = note.Url;
                        string noteImage = note.PhotoUrl;

                        this.Note = new Attachment.Note(noteTitle, noteDescription, noteUrl, noteImage);
                    }
                    this.MessageTitle = string.Format(this.MessageTitle, "bài viết");
                    break;
            }

            if(owner!= null)
            {
                this.MessageTitle = string.Format(this.MessageTitle, owner.FullName, owner.ProfileLink);
            }else
            {
                this.MessageTitle = string.Format(this.MessageTitle, this.Owner.FullName, this.Owner.ProfileLink);
            }
            
            if (activity.TargetType == (int)Constants.Activity.TargetType.User)
            {
                this.TagUser = _userProfileService.GetByID(activity.TargetID) ?? new UserProfile();
            }
        }

    }
}
