﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Social.Entity.Tik;
using Social.Entity.TikActivity;
using Social.Service.Interface.Tik;

namespace Social.Web.Areas.Feed.Models
{
    public class Like 
    {
        public Guid ID { get; set; }
        public byte ObjectType { get; set; }
        public Guid ObjectID { get; set; }
        public Guid FromID { get; set; }
        public UserProfile FromUser { get; set; }
        public Byte LikeBy { get; set; }
        public DateTime LikeDate { get; set; }
        private IUserProfileService _userProfileService = DependencyResolver.Current.GetService<IUserProfileService>();

        public Like(){}
        public Like(UserLike userLike)
        {
            this.ID = userLike.ID;
            this.ObjectType = userLike.ObjectType;
            this.ObjectID = userLike.ObjectID;
            this.FromID = userLike.FromID;
            this.FromUser = _userProfileService.GetByID(userLike.FromID);
            if (this.FromUser == null) this.FromUser = new UserProfile();
            this.LikeDate = userLike.LikeDate;
        }

        public static List<Like> Convert(List<UserLike> userLikes)
        {
            List<Like> likes = new List<Like>();
            if (userLikes == null) return likes;
            Like like;
            foreach (var userLike in userLikes)
            {
                like = new Like(userLike);
                likes.Add(like);
            }
            return likes;
        }

        public static string GetFeedLikeString(int type, Guid objectid, int totalLike, bool youLike, List<Like> friendLikes)
        {
            string result = string.Empty;
            if (totalLike > 0)
            {
                if (totalLike == 1)
                {
                    if (youLike) { result = "Bạn"; }
                    else if (friendLikes.Count > 0)
                    {
                        result = string.Format(@"<a href=""{0}"">{1}</a>", friendLikes[0].FromUser.ProfileLink, friendLikes[0].FromUser.FullName);
                    }
                    else
                    {
                        result = string.Format(@"<a href=""javascript:void(0);"" onclick=""tikStream.showAllLike('{1}','{2}'); return false;"">{0} người</a>", totalLike, type, objectid);
                    }
                }
                else if (totalLike == 2)
                {
                    if (youLike)
                    {
                        result = "Bạn";

                        if (friendLikes.Count > 0)
                        {
                            result += string.Format(@" và <a href=""{0}"">{1}</a>", friendLikes[0].FromUser.ProfileLink, friendLikes[0].FromUser.FullName);
                        }
                        else
                        {
                            result += string.Format(@" và <a href=""javascript:void(0);"" onclick=""tikStream.showAllLike('{1}','{2}'); return false;"">{0} người khác</a>", totalLike - 1, type, objectid);
                        }
                    }
                    else
                    {
                        if (friendLikes.Count > 0)
                        {
                            if (friendLikes.Count == 1)
                            {
                                result += string.Format(@"<a href=""{0}"">{1}</a>", friendLikes[0].FromUser.ProfileLink, friendLikes[0].FromUser.FullName);
                                result += string.Format(@" và <a href=""javascript:void(0);"" onclick=""tikStream.showAllLike('{1}','{2}'); return false;"">{0} người khác</a>", totalLike - 1, type, objectid);
                            }
                            else
                            {
                                result += string.Format(@"<a href=""{0}"">{1}</a>", friendLikes[0].FromUser.ProfileLink, friendLikes[0].FromUser.FullName);
                                result += string.Format(@" và <a href=""{0}"">{1}</a>", friendLikes[1].FromUser.ProfileLink, friendLikes[1].FromUser.FullName);
                            }

                        }
                        else
                        {
                            result += string.Format(@"<a href=""javascript:void(0);"" onclick=""tikStream.showAllLike('{1}','{2}'); return false;"">{0} người</a>", totalLike, type, objectid);
                        }
                    }
                }
                else
                {
                    if (youLike)
                    {
                        result = "Bạn";
                        if (friendLikes.Count > 0)
                        {
                            result += string.Format(@", <a href=""{0}"">{1}</a>", friendLikes[0].FromUser.ProfileLink, friendLikes[0].FromUser.FullName);
                            result += string.Format(@" và <a href=""javascript:void(0);"" onclick=""tikStream.showAllLike('{1}','{2}');return false;"">{0} người khác</a>", totalLike - 2, type, objectid);
                        }
                        else
                        {
                            result += string.Format(@" và <a href=""javascript:void(0);"" onclick=""tikStream.showAllLike('{1}','{2}');return false;"">{0} người khác</a>", totalLike - 1, type, objectid);
                        }
                    }
                    else
                    {
                        if (friendLikes.Count > 0)
                        {
                            if (friendLikes.Count == 1)
                            {
                                result += string.Format(@"<a href=""{0}"">{1}</a>", friendLikes[0].FromUser.ProfileLink, friendLikes[0].FromUser.FullName);
                                result += string.Format(@" và <a href=""javascript:void(0);"" onclick=""tikStream.showAllLike('{1}','{2}');return false;"">{0} người khác</a>", totalLike - 1, type, objectid);
                            }
                            else
                            {
                                result += string.Format(@"<a href=""{0}"">{1}</a>", friendLikes[0].FromUser.ProfileLink, friendLikes[0].FromUser.FullName);
                                result += string.Format(@", <a href=""{0}"">{1}</a>", friendLikes[1].FromUser.ProfileLink, friendLikes[1].FromUser.FullName);
                                result += string.Format(@" và <a href=""javascript:void(0);"" onclick=""tikStream.showAllLike('{1}','{2}');return false;"">{0} người khác</a>", totalLike - 2, type, objectid);
                            }
                        }
                        else
                        {
                            result += string.Format(@"<a href=""javascript:void(0);"" onclick=""tikStream.showAllLike('{1}','{2}');return false;"">{0} người</a>", totalLike, type, objectid);
                        }
                    }
                }
            }
            if (!string.IsNullOrEmpty(result)) result += " thích điều này";
            return result;
        }

        public static string GetFeedLikeStringNoHTML(int type, Guid objectid, int totalLike, bool youLike, List<Like> friendLikes)
        {
            string result = string.Empty;
            if (totalLike > 0)
            {
                if (totalLike == 1)
                {
                    if (youLike) { result = "Bạn"; }
                    else if (friendLikes.Count > 0)
                    {
                        result = string.Format(@"{1}", friendLikes[0].FromUser.ProfileLink, friendLikes[0].FromUser.FullName);
                    }
                    else
                    {
                        result = string.Format(@"{0} người", totalLike, type, objectid);
                    }
                }
                else if (totalLike == 2)
                {
                    if (youLike)
                    {
                        result = "Bạn";

                        if (friendLikes.Count > 0)
                        {
                            result += string.Format(@" và {0}", friendLikes[0].FromUser.FullName);
                        }
                        else
                        {
                            result += string.Format(@" và {0} người khác", totalLike - 1);
                        }
                    }
                    else
                    {
                        if (friendLikes.Count > 0)
                        {
                            if (friendLikes.Count == 1)
                            {
                                result += string.Format(@"{0}",friendLikes[0].FromUser.FullName);
                                result += string.Format(@" và {0} người khác", totalLike - 1);
                            }
                            else
                            {
                                result += string.Format(@"{0}", friendLikes[0].FromUser.FullName);
                                result += string.Format(@" và {0}", friendLikes[1].FromUser.FullName);
                            }

                        }
                        else
                        {
                            result += string.Format(@"{0} người", totalLike);
                        }
                    }
                }
                else
                {
                    if (youLike)
                    {
                        result = "Bạn";
                        if (friendLikes.Count > 0)
                        {
                            result += string.Format(@", {0}", friendLikes[0].FromUser.FullName);
                            result += string.Format(@" và {0} người khác", totalLike - 2);
                        }
                        else
                        {
                            result += string.Format(@" và {0}", totalLike - 1);
                        }
                    }
                    else
                    {
                        if (friendLikes.Count > 0)
                        {
                            if (friendLikes.Count == 1)
                            {
                                result += string.Format(@"{0}", friendLikes[0].FromUser.FullName);
                                result += string.Format(@" và {0} người khác", totalLike - 1);
                            }
                            else
                            {
                                result += string.Format(@"{0}", friendLikes[0].FromUser.FullName);
                                result += string.Format(@", {0}", friendLikes[1].FromUser.FullName);
                                result += string.Format(@" và {0} người khác", totalLike - 2);
                            }
                        }
                        else
                        {
                            result += string.Format(@"{0} người", totalLike);
                        }
                    }
                }
            }
            if (!string.IsNullOrEmpty(result)) result += " thích điều này";
            return result;
        }
    }
}
