﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Social.Entity.Tik;
using Social.Entity.TikActivity;
using Social.Lib;
using Social.Lib.Activity;
using Social.Service.Interface.Tik;
using Social.Service.Interface.TikFriend;
using Social.Service.Interface.TikActivity;
using Social.Service.Interface.TikPhoto;
using Social.Service.Interface.TikApplication;

namespace Social.Web.Areas.Feed.Models
{
    public class Notification 
    {
        public Guid ID { get; set; }
        public Guid OwnerID { get; set; }
        public UserProfile Owner { get; set; }
        public DateTime CreatedDate { get; set; }
        public Boolean IsRead { get; set; }
        public Boolean IsHidden { get; set; }
        public int NotificationType { get; set; }
        public int ObjectType { get; set; }
        public Guid ObjectID { get; set; }
        public int TargetType { get; set; }
        public Guid TargetID { get; set; }

        public string NotificationLink { get; set; }
        public string NotificationContent { get; set; }

        public UserProfile TargetUser { get; set; }

        private IUserFeedService _userFeed = DependencyResolver.Current.GetService<IUserFeedService>();
        private IPhotoService _photo = DependencyResolver.Current.GetService<IPhotoService>();
        private INoteService _note = DependencyResolver.Current.GetService<INoteService>();

        private IUserProfileService _userProfileService = DependencyResolver.Current.GetService<IUserProfileService>();
        private static IFriendListService _friendlist = DependencyResolver.Current.GetService<IFriendListService>();
        private List<Guid> listFriends = _friendlist.GetByOwnerID(SessionManager.CurrentUserID());

        public Notification(){}

        public Notification(UserNotification userNotification, bool isTop = false)
        {
            this.ID = userNotification.ID;
            this.CreatedDate = userNotification.CreatedDate;
            this.OwnerID = userNotification.OwnerID;
            this.Owner = _userProfileService.GetByID(userNotification.OwnerID);
            this.IsRead = userNotification.IsRead;
            if (this.Owner == null) this.Owner = new UserProfile();

            this.NotificationType = userNotification.NotificationType;
            this.ObjectType = userNotification.ObjectType;
            this.ObjectID = userNotification.ObjectID;

            this.TargetID = userNotification.TargetID;
            this.TargetUser = new UserProfile();
            if (userNotification.TargetType == (int)Constants.Notification.TargetType.User)
            {
                this.TargetUser = _userProfileService.GetByID(userNotification.TargetID);
            }
            switch (userNotification.NotificationType)
            {
                case (int)Constants.Notification.NotificationType.Friend:
                    this.NotificationContent = isTop ? string.Format("<strong>{0}</strong> đã nhận lời mời kết bạn của bạn.", this.TargetUser.FullName) : string.Format(@"<a href=""{0}"">{1}</a> đồng ý kết bạn với bạn.", this.TargetUser.ProfileLink, this.TargetUser.FullName);
                    break;
                case (int)Constants.Notification.NotificationType.TagPost:
                    if (userNotification.ObjectType == (int)Constants.Notification.ObjectType.Feed)
                    {
                        this.NotificationContent = isTop ? string.Format("<strong>{0}</strong> đã nhắc đến bạn trong bài viết trên tường của {1}.", this.TargetUser.FullName, this.TargetUser.Gender == 1 ? "anh ấy" : "cô ấy") : string.Format(@"<a href=""{0}"">{1}</a> đã nhắc đến bạn trong <a href=""{2}"">bài viết trên tường</a> của {3}.", this.TargetUser.ProfileLink, this.TargetUser.FullName, string.Format("/Feed/View/{0}", this.ObjectID), this.TargetUser.Gender == 1 ? "anh ấy" : "cô ấy");
                        this.NotificationLink = string.Format("/Feed/View/{0}", this.ObjectID);
                    }
                    if (userNotification.ObjectType == (int)Constants.Notification.ObjectType.Comment)
                    {
                        this.NotificationContent = isTop ? string.Format("<strong>{0}</strong> đã nhắc đến bạn trong bình luận của {1}.", this.TargetUser.FullName, this.TargetUser.Gender == 1 ? "anh ấy" : "cô ấy") : string.Format(@"<a href=""{0}"">{1}</a> đã nhắc đến bạn trong <a href=""{2}"">bình luận</a> {3}.", this.TargetUser.ProfileLink, this.TargetUser.FullName, string.Format("/Feed/View/{0}", this.ObjectID), this.TargetUser.Gender == 1 ? "anh ấy" : "cô ấy");
                        this.NotificationLink = string.Format("/Feed/View/{0}/true", this.ObjectID);
                    }
                    break;
                case (int)Constants.Notification.NotificationType.Tag:
                    string objectText = string.Empty;
                    string objectLink = string.Empty;
                    string owner = this.TargetUser.Gender == 1 ? "anh ấy" : "cô ấy";
                    GetTypeAndLinkString(userNotification.ObjectType, userNotification.ObjectID, TargetUser.UserName, out objectText, out objectLink);
                    UserProfile user;
                    switch (userNotification.ObjectType)
                    {
                        case (int)Constants.Notification.ObjectType.Feed:
                            var feed = _userFeed.GetFeed(userNotification.ObjectID);
                            if(feed != null)
                            {
                                if(feed.OwnerID != this.TargetUser.ID)
                                {
                                    user = _userProfileService.GetByID(feed.OwnerID);
                                    if (user != null)
                                    {
                                        owner = isTop ? string.Format(@"<strong>{0}</strong>", user.FullName) : string.Format(@"<a href=""{1}"">{0}</a>", user.FullName, user.ProfileLink);
                                    }
                                }
                            }
                            break;
                        case (int)Constants.Notification.ObjectType.Image:
                            var photo = _photo.GetById(userNotification.ObjectID);
                            if (photo != null)
                            {
                                if (photo.UserID != this.TargetUser.ID)
                                {
                                    user = _userProfileService.GetByID(photo.UserID);
                                    if (user != null)
                                    {
                                        owner = isTop ? string.Format(@"<strong>{0}</strong>", user.FullName) : string.Format(@"<a href=""{1}"">{0}</a>", user.FullName, user.ProfileLink);
                                    }
                                }
                            }
                            break;
                        case (int)Constants.Notification.ObjectType.Note:
                            var note = _note.GetNoteByID(userNotification.ObjectID);
                            if (note != null)
                            {
                                if (note.UserID != this.TargetUser.ID)
                                {
                                    user = _userProfileService.GetByID(note.UserID);
                                    if (user != null)
                                    {
                                        owner = isTop ? string.Format(@"<strong>{0}</strong>", user.FullName) : string.Format(@"<a href=""{1}"">{0}</a>", user.FullName, user.ProfileLink);
                                    }
                                }
                            }
                            break; 
                    }
                    this.NotificationContent = isTop ? string.Format("<strong>{0}</strong> đã đánh dấu bạn trong {1} của {2}.", this.TargetUser.FullName, objectText, owner) : string.Format(@"<a href=""{0}"">{1}</a> đã đánh dấu bạn trong <a href=""{3}"">{2}</a> {4}.", this.TargetUser.ProfileLink, this.TargetUser.FullName, objectText, objectLink, owner);
                    this.NotificationLink = objectLink;
                    break;
            }
        }

        private void GetTypeAndLinkString(int type, Guid objectid, string username, out string objectType, out string objectLink)
        {
            string typeString = string.Empty;
            string link = string.Empty;
            switch (type)
            {
                case (int)Constants.Notification.ObjectType.Feed:
                    typeString = "bài đăng trên tường";
                    link = string.Format(@"/Feed/View/{0}", objectid);
                    break;
                case (int)Constants.Notification.ObjectType.Album:
                    typeString = "album ảnh";
                    link = string.Format(@"/{0}/Photo/ListPhotoInAlbum/{1}", username, objectid);
                    break;
                case (int)Constants.Notification.ObjectType.Image:
                    typeString = "ảnh";
                    link = string.Format(@"/{0}/Photo/PhotoDetail/{1}", username, objectid);
                    break;
                case (int)Constants.Notification.ObjectType.Music:
                    typeString = "bài hát";
                    link = string.Format(@"/{0}/Music/SongDetail/{1}", username, objectid);
                    break;
                case (int)Constants.Notification.ObjectType.AlbumMusic:
                    typeString = "album nhạc";
                    link = string.Format(@"/{0}/Music/AlbumDetail/{1}", username, objectid);
                    break;
                case (int)Constants.Notification.ObjectType.Link:
                    typeString = "liên kết";
                    link = string.Format(@"/Feed/View/{0}", objectid);
                    break;
                case (int)Constants.Notification.ObjectType.Post:
                    typeString = "bài đăng";
                    link = string.Format(@"/Feed/View/{0}", objectid);
                    break;
                case (int)Constants.Notification.ObjectType.Status:
                    typeString = "trạng thái";
                    link = string.Format(@"/Feed/View/{0}", objectid);
                    break;
                case (int)Constants.Notification.ObjectType.Comment:
                    typeString = "bình luận";
                    link = string.Format(@"/Feed/View/{0}/true", objectid);
                    break;
                case (int)Constants.Notification.ObjectType.Note:
                    typeString = "bài viết";
                    link = string.Format(@"/{0}/Note/View/Index/{1}", username, objectid);
                    break;
            }

            objectType = typeString;
            objectLink = link;
        }
        
        public Notification(List<UserNotification> userNotifications, Constants.Notification.NotificationType type, bool isTop = false)
        {
            switch (type)
            {
                case Constants.Notification.NotificationType.Comment:
                case Constants.Notification.NotificationType.Like:
                    if (userNotifications.Count > 0)
                    {
                        if (userNotifications[0].OwnerID != SessionManager.CurrentUserID())
                        {
                            var friendNotification = userNotifications.FindAll(n => listFriends.Contains(n.TargetID) && n.TargetID != SessionManager.CurrentUserID());
                            if (friendNotification.Count > 0)
                            {
                                string typeText = "cũng bình luận về";
                                string content = string.Empty;
                                int total = 0;
                                string typeComment = string.Empty;
                                string linkComment = string.Empty;
                                this.ID = friendNotification[0].ID;
                                this.OwnerID = friendNotification[0].OwnerID;
                                this.Owner = _userProfileService.GetByID(friendNotification[0].OwnerID);
                                if (this.Owner == null) this.Owner = new UserProfile();
                                this.CreatedDate = friendNotification[0].CreatedDate;
                                this.TargetID = friendNotification[0].TargetID;
                                this.NotificationType = friendNotification[0].NotificationType;
                                this.TargetUser = _userProfileService.GetByID(friendNotification[0].TargetID);
                                this.IsRead = !friendNotification.Any(n => n.IsTargetRead == false);
                                if (this.TargetUser == null) this.TargetUser = new UserProfile();
                                GetTypeAndLinkString(friendNotification[0].ObjectType, friendNotification[0].ObjectID, this.Owner.UserName, out typeComment, out  linkComment);

                                foreach (UserNotification userNotification in friendNotification.OrderByDescending(n => n.CreatedDate))
                                {
                                    if (total < 2)
                                    {
                                        UserProfile user = _userProfileService.GetByID(userNotification.TargetID);
                                        if (user != null)
                                        {
                                            if (total > 0) content += " và ";
                                            content += isTop ? string.Format(@"<strong>{0}</strong>", user.FullName)
                                                             : string.Format(@"<a href=""{0}"">{1}</a>", user.ProfileLink, user.FullName);
                                            total++;
                                        }
                                    }
                                    else
                                    {
                                        break;
                                    }
                                }
                                if (friendNotification.Count - total > 0)
                                {
                                    content += string.Format(" và {0} người khác", userNotifications.Count - total);
                                }

                                content += isTop ? string.Format(@" {1} <strong>{0}</strong> của <strong>{2}</strong>.", typeComment, typeText, this.Owner.FullName)
                                                 : string.Format(@" {2} <a href=""{0}"">{1}</a> của <a href=""{4}"">{3}</a>.", linkComment, typeComment, typeText, this.Owner.FullName, this.Owner.ProfileLink);
                                this.NotificationContent = content;
                                this.NotificationLink = linkComment;
                            }
                        }
                        else
                        {
                            string typeText = (type == Constants.Notification.NotificationType.Comment) ? "bình luận về" : "thích";
                            string content = string.Empty;
                            int total = 0;
                            string typeComment = string.Empty;
                            string linkComment = string.Empty;
                            this.ID = userNotifications[0].ID;
                            this.OwnerID = userNotifications[0].OwnerID;
                            this.Owner = _userProfileService.GetByID(userNotifications[0].OwnerID);
                            if (this.Owner == null) this.Owner = new UserProfile();
                            this.CreatedDate = userNotifications[0].CreatedDate;
                            this.TargetID = userNotifications[0].TargetID;
                            this.NotificationType = userNotifications[0].NotificationType;
                            this.TargetUser = _userProfileService.GetByID(userNotifications[0].TargetID);
                            this.IsRead = !userNotifications.Any(n => n.IsRead == false);
                            if (this.TargetUser == null) this.TargetUser = new UserProfile();
                            GetTypeAndLinkString(userNotifications[0].ObjectType, userNotifications[0].ObjectID, this.Owner.UserName, out typeComment, out  linkComment);

                            foreach (UserNotification userNotification in userNotifications.OrderByDescending(n => n.CreatedDate))
                            {
                                if (total < 2)
                                {
                                    UserProfile user = _userProfileService.GetByID(userNotification.TargetID);
                                    if (user != null)
                                    {
                                        if (total > 0) content += " và ";
                                        content += isTop ? string.Format(@"<strong>{0}</strong>", user.FullName)
                                                         : string.Format(@"<a href=""{0}"">{1}</a>", user.ProfileLink, user.FullName);
                                        total++;
                                    }
                                }
                                else
                                {
                                    break;
                                }
                            }
                            if (userNotifications.Count - total > 0)
                            {
                                content += string.Format(" và {0} người khác", userNotifications.Count - total);
                            }

                            content += isTop ? string.Format(@" {1} <strong>{0}</strong> của bạn.", typeComment, typeText)
                                             : string.Format(@" {2} <a href=""{0}"">{1}</a> của bạn.", linkComment, typeComment, typeText);
                            this.NotificationContent = content;
                            this.NotificationLink = linkComment;
                        }
                    }
                    break;
            }
        }

        public static List<Notification> Convert(List<UserNotification> userNotifications, bool isTop = false)
        {
            if (userNotifications == null) return new List<Notification>();

            List<Notification> notifications = new List<Notification>();
            Notification notification;

            var x = userNotifications.FindAll(n => n.IsTargetRead == false);
            //Notification Friend
            var notificationFriend = userNotifications.FindAll(n => n.NotificationType == (int)Constants.Notification.NotificationType.Friend);
            foreach (var userNotification in notificationFriend)
            {
                if (userNotification != null)
                {
                    notification = new Notification(userNotification, isTop);
                    notifications.Add(notification);
                }
            }

            //Notification tag in post, comment
            var notificationTagPost = userNotifications.FindAll(n => n.NotificationType == (int)Constants.Notification.NotificationType.TagPost);
            foreach (var userNotification in notificationTagPost)
            {
                if (userNotification != null)
                {
                    notification = new Notification(userNotification, isTop);
                    notifications.Add(notification);
                }
            }

            //Notification tag
            var notificationTag = userNotifications.FindAll(n => n.NotificationType == (int)Constants.Notification.NotificationType.Tag);
            foreach (var userNotification in notificationTag)
            {
                if (userNotification != null)
                {
                    notification = new Notification(userNotification, isTop);
                    notifications.Add(notification);
                }
            }

            //Notification comment
            var notificationOtherUserComment = userNotifications.FindAll(n => n.NotificationType == (int)Constants.Notification.NotificationType.Comment && n.OwnerID != SessionManager.CurrentUserID()).ToList().GroupBy(n => n.ObjectID).ToList();

            foreach (IGrouping<Guid, UserNotification> grouping in notificationOtherUserComment)
            {
                List<UserNotification> listComment = grouping.OrderByDescending(n=>n.CreatedDate).Distinct(new UserNotificationComparer()).ToList();
                notification = new Notification(listComment.OrderByDescending(n => n.CreatedDate).ToList(), Constants.Notification.NotificationType.Comment, isTop);
                notifications.Add(notification);
            }

            //Notification comment
            var notificationComment = userNotifications.FindAll(n => n.NotificationType == (int)Constants.Notification.NotificationType.Comment && n.OwnerID == SessionManager.CurrentUserID()).GroupBy(n => n.ObjectID).ToList();

            foreach (IGrouping<Guid, UserNotification> grouping in notificationComment)
            {
                List<UserNotification> listComment = grouping.OrderByDescending(n => n.CreatedDate).Distinct(new UserNotificationComparer()).ToList();
                notification = new Notification(listComment.OrderByDescending(n => n.CreatedDate).ToList(), Constants.Notification.NotificationType.Comment, isTop);
                notifications.Add(notification);
            }

            //Notification like
            var notificationLike = userNotifications.FindAll(n => n.NotificationType == (int)Constants.Notification.NotificationType.Like).GroupBy(n => n.ObjectID).ToList();

            foreach (IGrouping<Guid, UserNotification> grouping in notificationLike)
            {
                List<UserNotification> listLike = grouping.OrderByDescending(n => n.CreatedDate).Distinct(new UserNotificationComparer()).ToList();
                notification = new Notification(listLike.OrderByDescending(n => n.CreatedDate).ToList(), Constants.Notification.NotificationType.Like, isTop);
                notifications.Add(notification);
            }
            
            return notifications;
        }
    }
    class UserNotificationComparer : IEqualityComparer<UserNotification>
    {
        public bool Equals(UserNotification x, UserNotification y)
        {
            if (Object.ReferenceEquals(x, y)) return true;
            if (Object.ReferenceEquals(x, null) || Object.ReferenceEquals(y, null))
                return false;
            return x.TargetID == y.TargetID;
        }

        public int GetHashCode(UserNotification product)
        {
            if (Object.ReferenceEquals(product, null)) return 0;
            int hashProductName = product.TargetID == null ? 0 : product.TargetID.GetHashCode();
            int hashProductCode = product.TargetID.GetHashCode();
            return hashProductName ^ hashProductCode;
        }

    }
}
