﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Social.Lib;
using Social.Service.Interface.TikMessage;
using System.Web.Script.Serialization;
using Social.Web.Extends.Helper;
using Social.Entity.TikMessage;
using Webdiyer.WebControls.Mvc;
using Social.Web.Areas.Message.Models;
namespace Social.Web.Areas.Message.Controllers
{
    public class IndexController : Controller
    {
        //
        // GET: /Message/Index/
        private IMessageService _messageService;
        private IMessageCommentService _msgCmtService;
        private IMessageRecipientService _msgRcptService;
        public IndexController(IMessageService msgservice, IMessageRecipientService msgRcptService, IMessageCommentService msgCmtService)
        {
            _messageService = msgservice;
            _msgRcptService = msgRcptService;
            _msgCmtService = msgCmtService;
        }
        public ActionResult Index(string query,int p=0)
        {
            if (!SessionManager.CheckLogin())
                return RedirectToAction("Index", "Register", new { area = "Register" });
            Guid uid = SessionManager.CurrentUserID();
            bool? isArchived = null;
            bool? isSpam = null;
            bool? isUnread = null;
            if (String.IsNullOrWhiteSpace(query))
            {
                isArchived = false;
                isSpam = false;
            }
            else if (query == "archived")
                isArchived = true;
            else if (query == "spam")
                isSpam = true;
            else if (query == "unread")
            {
                isUnread = true;
                isArchived = false;
                isSpam = false;
            }
            PagedList<Social.Entity.TikMessage.MessageView> listMsg = _messageService.GetInboxByUserID(uid, isUnread, isSpam, isArchived,p,20);
            ViewData["MSG_QUERY"] = query;
            if (Request.IsAjaxRequest())
                return PartialView("IndexAjax", listMsg);
            return View(listMsg);
        }

        public ActionResult Sent(int p=0)
        {
            if (!SessionManager.CheckLogin())
                return RedirectToAction("Index", "Register", new { area = "Register" });
            Guid uid = SessionManager.CurrentUserID();
            PagedList<Social.Entity.TikMessage.MessageView> listMsg = _messageService.GetSentByUserID(uid,p,20);
            if (Request.IsAjaxRequest())
                return PartialView("SentAjax", listMsg);
            return View(listMsg);
        }
        public ActionResult Read(string id)
        {
            if (!SessionManager.IsLoggedIn())
                return RedirectToAction("Index", "Register", new { area = "Register" });
            Guid msgID=Guid.Parse(id);
            var uid=SessionManager.CurrentUserID();
            List<Social.Entity.TikMessage.MessageCommentView> listMsg = _msgCmtService.GetListByMessageID(msgID, uid);
            MessageView mv = _messageService.GetMessageViewByID(msgID);            
            ReadModel model = new ReadModel();
            model.MessageCommentView = listMsg;
            model.MessageView = mv;
            //Update read
            _messageService.UpdateRead(uid, msgID);
            //
            return View(model);
        }
        [HttpPost]
        public ActionResult Comment(FormCollection frm)
        {
            if (!SessionManager.IsLoggedIn())
                return Content("<li>Bạn cần đăng nhập để gửi trả lời tin nhắn</li>");
            Guid uid = SessionManager.CurrentUserID();
            Guid msgID=Guid.Parse(frm["messageID"]);
            MessageComment cmt = new MessageComment();
            cmt.CreatedDate = DateTime.Now;
            cmt.FromID = uid;
            cmt.ID = Guid.NewGuid();
            cmt.MessageContent = frm["messageContent"];
            cmt.MessageID = msgID;
            _msgCmtService.Add(cmt);
            //Update Message
            Social.Entity.TikMessage.Message msg = _messageService.Single(m=>m.ID==msgID);
            if (msg != null)
            {
                msg.MessageContent = cmt.MessageContent;
                msg.UpdatedDate = cmt.CreatedDate;
                foreach (var r in msg.Recipients)
                {
                    if (r.RecipientID != uid)
                        r.UnRead = true;
                }
                _messageService.Update(msg);
            }
            //
            List<Social.Entity.TikMessage.MessageCommentView> listMsg = _msgCmtService.GetListByMessageID(msgID, uid);
            return PartialView("ReadAjax",listMsg);
        }
        [HttpPost]
        public ActionResult Mark(FormCollection frm)
        {
            try
            {
                if (frm["ids[]"] != null && frm["action"]!=null)
                {
                    string action = frm["action"].ToString();
                    var currentID = SessionManager.CurrentUserID();
                    string[] arrStrID = frm["ids[]"].Split(',');
                    foreach (string strID in arrStrID)
                    {
                        var msgID = Guid.Parse(strID);
                        var msgrcpt = _msgRcptService.Single(mr => mr.RecipientID == currentID && mr.MessageID == msgID);
                        if (action == "archived")
                            msgrcpt.IsArchived = true;
                        else if (action == "read")
                            msgrcpt.UnRead = false;
                        else if (action == "spam")
                            msgrcpt.IsSpam = true;
                        else if (action == "unspam")
                            msgrcpt.IsSpam = false;
                        else if (action == "unarchived")
                            msgrcpt.IsArchived = false;
                        _msgRcptService.Update(msgrcpt);
                    }
                    return Content("1");
                }
                return Content("2");
            }
            catch (Exception ex)
            {
                LogHelper.Error("Archive message:", ex.Message);
                return Content("0");
            }

        }
        [HttpPost]
        public ActionResult Delete(string id)
        {
            try
            {
                var currentID = SessionManager.CurrentUserID();
                var msgID = Guid.Parse(id);
                var msgrcpt = _msgRcptService.Single(mr => mr.RecipientID == currentID && mr.MessageID == msgID);
                msgrcpt.IsArchived = true;
                _msgRcptService.Update(msgrcpt);
                return Content("1");
            }
            catch (Exception ex)
            {
                LogHelper.Error("Archive message:" + id, ex.Message);
                return Content("0");
            }

        }
        [HttpPost]
        public ActionResult Add(FormCollection data)
        {
            try
            {
                Guid fromID = SessionManager.CurrentUserID();
                string msg = data["msg"];
                if (!data["toids"].Contains("fromID"))
                    data["toids"] = data["toids"] + "," + fromID;
                string[] arrToID = data["toids"].Split(',');
                Entity.TikMessage.Message oMessage = new Entity.TikMessage.Message();
                oMessage.ID = Guid.NewGuid();
                oMessage.CreatedDate = DateTime.Now;
                oMessage.UpdatedDate = oMessage.CreatedDate;
                oMessage.FromID = fromID;
                oMessage.MessageContent = msg;
                oMessage.Subject = data["subject"];
                oMessage.Recipients = new List<MessageRecipient>();
                oMessage.Comments = new List<MessageComment>();
                oMessage.Comments.Add(new MessageComment { CreatedDate = DateTime.Now, FromID = fromID, ID = Guid.NewGuid(), MessageContent = msg, MessageID = oMessage.ID });                
                foreach (string strToid in arrToID)
                {
                    if (!String.IsNullOrWhiteSpace(strToid))
                    {
                        Guid toID = Guid.Parse(strToid);
                        MessageRecipient msgrcpt = new MessageRecipient();
                        msgrcpt.ID = Guid.NewGuid();
                        msgrcpt.MessageID = oMessage.ID;
                        msgrcpt.RecipientID = toID;
                        msgrcpt.UnRead = true;
                        msgrcpt.IsArchived = false;
                        if (strToid == fromID.ToString())
                        {
                            msgrcpt.IsSender = true;
                            msgrcpt.UnRead = false;
                        }
                        oMessage.Recipients.Add(msgrcpt);
                    }
                }                
                _messageService.Add(oMessage);
                return Content("1");
            }
            catch (Exception ex)
            {
                LogHelper.Error("send message", ex.Message);
                return Content("0");
            }
        }
    }
}
