﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Mvc;
using System.Web.Script.Serialization;
using HundredMilesSoftware.UltraID3Lib;
using Social.Entity.TikMusic;
using Social.Lib;
using Social.Lib.Music;
using Social.Service.Interface.Tik;
using Social.Service.Interface.TikMusic;

namespace Social.Web.Areas.Music.Controllers
{
    public class MusicController : Controller
    {
        IUserProfileService _user;
        IAlbumService _album;
        ISongService _song;
        ISongQueueService _songQueue;
        ICategoryService _category;
        public MusicController(ISongService song, ISongQueueService songQueueService, IAlbumService album, IUserProfileService user, ICategoryService category)
        {
            _song = song;
            _songQueue = songQueueService;
            _album = album;
            _user = user;
            _category = category;
        }

        public ActionResult UploadSong(string username)
        {
            if (!SessionManager.CheckLogin()) return RedirectToRoute("Register_Index");
            var _uid = SessionManager.CurrentUserID(); 
            JavaScriptSerializer serializer = new JavaScriptSerializer();
            if (_uid!=Guid.Empty)
            { 
                var listAb = _album.GetListAlbumByUserId(_uid);
                var listCt = _category.GetAll();
                ViewData["ListAlbum"] = serializer.Serialize(listAb);
                ViewData["ListCategory"] = serializer.Serialize(listCt); 
                ViewData["uid"] = _uid.ToString();
                ViewData["currentUserName"] = SessionManager.CurrentUserUserName();
            }
            else
            {
                return Redirect(ConfigManager.MainDomain);
            }
            return View();
        }

        public ActionResult SongDetail(string username, string sid)
        {
            if (!SessionManager.CheckLogin()) return RedirectToRoute("Register_Index");
            var _sId = new Guid();
            if (!string.IsNullOrEmpty(sid))
            {
                _sId = Guid.Parse(sid);
                var sInfo = _song.Single(s => s.ID == _sId);
                if (sInfo==null) return Redirect("/Error/NotFound");
                var uInfo = _user.Single(u => u.ID == sInfo.UserID);
                ViewData["author"] = uInfo;
                ViewData["songInfo"] = sInfo;
                ViewData["currentUser"] = _user.CurrentUser();
                ViewData["AlbumId"] = (sInfo.AlbumID!=Guid.Empty?sInfo.AlbumID.ToString():"-");
                if(sInfo.AlbumID!=Guid.Empty)
                {
                    ViewData["AlbumName"] = _album.Single(a => a.ID == sInfo.AlbumID).AlbumName;    
                }
                var urlFile = ConfigManager.UrlDataMusic + sInfo.SourceUrl+ ".mp3";
                var urlXmlSong = ConfigManager.UrlXmlSongMusic + "?urlFile=" + Base64String.Encode(urlFile) +
                                   "&songName=" + Base64String.Encode(sInfo.SongName) +
                                   "&singer=" + Base64String.Encode(sInfo.Singer) +
                                   "&duration=" + sInfo.Duration;
                ViewData["PlayerSong"] = MusicUtil.GeneratePlayerSong(urlXmlSong, Url.Action("SongDetail", "Music",username,
                                                                                             sid), true);
            }
            else
            {
                return Redirect("/Error/NotFound");
            }
            return View();
        }

        public ActionResult EditAlbum(string username, string abId)
        {
            var uid = SessionManager.CurrentUserID();
            if (uid == Guid.Empty) return RedirectToRoute("Register_Index");
            var _abId = new Guid();
            if (!string.IsNullOrEmpty(abId))
            {
                _abId = Guid.Parse(abId);
                var abInfo = _album.Single(ab => ab.ID == _abId);
                if (abInfo == null) return Redirect("/Error/NotFound");
                if (abInfo.UserID != uid) return RedirectToAction("Index", new { username = username });
                
                ViewData["currentUser"] = _user.CurrentUser();
                ViewData["currentUserView"] = username;
                ViewData["abInfo"] = abInfo;
                var key = Constants.MemcacheKey.LIST_MUSIC_ALBUM + _abId;
                var obj = Memcached.MainCache.Get(key);
                List<Song> lstSong;
                if (obj != null)
                {
                    lstSong = (List<Song>)obj;
                }
                else
                {
                    lstSong = _song.GetByAlbumId(_abId);
                    Memcached.MainCache.Store(Enyim.Caching.Memcached.StoreMode.Set, key, lstSong,
                                              TimeSpan.FromMinutes(Constants.MemcacheKey.TIME_CACHE_LMA));
                }
                ViewData["lstSong"] = lstSong;
                var listCt = _category.GetAll();
                ViewData["lstCategory"] = listCt;
            }
            else
            {
                return RedirectToAction("Index", new { username = username });
            }
            return View();
        }

        public ActionResult AlbumDetail(string username, string abId)
        {
            if (!SessionManager.CheckLogin()) return RedirectToRoute("Register_Index");
            var _abId = new Guid();
            if (!string.IsNullOrEmpty(abId))
            {
                _abId = Guid.Parse(abId);
                var abInfo = _album.Single(ab => ab.ID == _abId);
                if (abInfo == null) return Redirect("/Error/NotFound");
                var uInfo = _user.Single(u => u.ID == abInfo.UserID);
                ViewData["author"] = uInfo;

                ViewData["currentUser"] = _user.CurrentUser(); 
                ViewData["AlbumInfo"] = abInfo; 
                var key = Constants.MemcacheKey.LIST_MUSIC_ALBUM + _abId;
                var obj = Memcached.MainCache.Get(key);
                List<Song> lstSong;
                if (obj != null)
                {
                    lstSong = (List<Song>)obj;
                }
                else
                {
                    lstSong = _song.GetByAlbumId(_abId);
                    Memcached.MainCache.Store(Enyim.Caching.Memcached.StoreMode.Set, key, lstSong,
                                              TimeSpan.FromMinutes(Constants.MemcacheKey.TIME_CACHE_LMA));
                }
                var jss = new JavaScriptSerializer();
                var jsonSongs = new StringBuilder();
                jss.Serialize(lstSong, jsonSongs);

                var urlXmlAlbum = ConfigManager.UrlXmlAlbumMusic + "?abId=" + _abId + "&abName=" +
                                  "NameAB";
                ViewData["PlayerAlbum"] = Lib.Music.MusicUtil.GeneratePlayerAlbum(urlXmlAlbum, true,
                                                                                  Url.Action("AlbumDetail", "Music",
                                                                                             _abId), jsonSongs.ToString());
            }
            else
            { 
                return Redirect("/Error/NotFound");
            }
            return View();
        }

        public ActionResult Index(string username)
        {
            if (!SessionManager.CheckLogin()) return RedirectToRoute("Register_Index");
            var totalRowAlbum = 0;
            var totalRowSong = 0;
            if (string.IsNullOrEmpty(username))
            {
                username = SessionManager.CurrentUserUserName();
            }
            var userProfile = _user.GetUserByUsername(username);
            if (userProfile == null)
            {
                return RedirectToActionPermanent("NotFoundUser", "Error", new { area = "" });
            }
            var _uid = userProfile.ID;
            var listAb = new List<Album>();
            
            listAb = username!=SessionManager.CurrentUserUserName() ? _album.GetPageListAlbumByUserId(_uid, 1, 9, out totalRowAlbum) : _album.GetPageListAlbumByUserId(_uid, 1, 9, out totalRowAlbum, true);
            var listSong = _song.GetPageListSongByUserId(_uid, 1, 10, out totalRowSong);
            ViewData["userView"] = userProfile;
            ViewData["currentUserName"] = SessionManager.CurrentUserUserName();
            ViewData["lstAlbum"] = listAb;
            ViewData["lstSong"] = listSong;
            ViewData["totalRowAlbum"] = totalRowAlbum.ToString();
            ViewData["totalRowSong"] = totalRowSong.ToString(); 
            return View();
        } 
    }
}
