﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Mvc;
using Social.Entity;
using Social.Entity.TikApplication;
using Social.Service;
using Social.Service.Interface.TikActivity;
using Social.Service.Interface.TikApplication;
using Social.Web.Extends.Helper;
using Google.GData.Contacts;
using Google.GData.Client;
using Google.GData.Extensions;
using Google.Contacts;
using System.Data;
using System.Collections;
using System.Configuration;
using Social.Lib;
using System.Web.Script.Serialization;
using System.Text;
using Social.Entity.Tik;
using Social.Service.Interface.Tik;

namespace Social.Web.Areas.Note.Controllers
{
    public class DetailController : Controller
    {
        //
        // GET: /Note/Detail/
        INoteService _note;
        IUserProfileService _user;
        IUserActivityService _userActivity;

        public DetailController(INoteService note, IUserProfileService user, IUserActivityService userActivity)
        {
            _note = note;
            _user = user;
            _userActivity = userActivity;
        }

        public ActionResult Index(string id, Social.Web.Areas.Note.Models.NoteModel noteDetail)
        {
            if (!SessionManager.IsLoggedIn()) return RedirectToRoute("Register_Index");

            if (TempData["error"] != null)
            {
                string err = TempData["error"].ToString();
                if (!string.IsNullOrEmpty(err))
                {
                    ViewData["ErrorMessage"] = err;
                }
            }

            if (string.IsNullOrEmpty(id))
            {
                return View(new Social.Web.Areas.Note.Models.NoteModel());
            }

            Guid guid = Guid.Parse(id);
            Social.Entity.TikApplication.Note note  = _note.GetNoteByID(guid);

            if (note != null)
            {
                noteDetail.ID = note.ID;
                noteDetail.UserID = note.UserID;
                noteDetail.Title = note.Title;
                noteDetail.Body = note.Body;
                noteDetail.Tags = note.Tags;
                noteDetail.Photo = note.Photo;
                noteDetail.PrivacyType = note.PrivacyType;
                noteDetail.PrivacyPeople = note.PrivacyPeople;
                noteDetail.Status = note.Status;
                noteDetail.ViewCount = note.ViewCount;
                noteDetail.CreatedDate = note.CreatedDate;
                noteDetail.ModifiedDate = note.ModifiedDate;
                noteDetail.LikeCount = note.LikeCount;
                noteDetail.CommentCount = note.CommentCount;
                noteDetail.IsUserLike = note.IsUserLike;
            }

            return View(noteDetail);
        }

        [HttpPost]
        [ValidateInput(false)]
        public ActionResult Update(Social.Web.Areas.Note.Models.NoteModel note)
        {
            if (!SessionManager.IsLoggedIn()) return RedirectToRoute("Register_Index");
            Guid userID = SessionManager.CurrentUserID();
            Social.Entity.TikApplication.Note noteUpdate = new Entity.TikApplication.Note();
            noteUpdate.ID = Guid.NewGuid();
            HttpPostedFileBase photo = Request.Files["file_upload"];
            if (photo.ContentLength > 0)
            {
                var content = new Byte[photo.ContentLength];
                photo.InputStream.Read(content, 0, photo.ContentLength);

                var psv = new Social.Web.Service.PhotoService();
                var jsonVal = psv.UploadNote(content, userID);

                var json_serializer = new JavaScriptSerializer();
                Social.Entity.TikPhoto.Photo photoInsert = json_serializer.Deserialize<Social.Entity.TikPhoto.Photo>(jsonVal);
                noteUpdate.Photo = photoInsert.SourceUrl;
            }

            List<Guid> arrGuidFriend = new List<Guid>();
            if (!string.IsNullOrEmpty(note.SuggestionFriend))
            {
                StringBuilder sb = new StringBuilder();
                string[] arrFriend = note.SuggestionFriend.Split(',');
                foreach (string item in arrFriend)
                {
                    if (!string.IsNullOrEmpty(item))
                    {
                        UserProfile user = _user.GetByID(Guid.Parse(item));
                        arrGuidFriend.Add(Guid.Parse(item));
                        sb.AppendFormat(",{{FriendID:'{0}',UserName:'{1}',FriendFullName:'{2}',AvatarSmall:'{3}'}}", item, user.UserName, user.FullName, user.SmallAvatarUrlMap);
                    }
                }

                if (!string.IsNullOrEmpty(sb.ToString()))
                {
                    sb.Remove(0, 1);
                }

                if (!string.IsNullOrEmpty(sb.ToString()))
                {
                    noteUpdate.Tags = string.Format("[{0}]", sb.ToString());
                }
            }

            noteUpdate.Body = note.Body;
            noteUpdate.Title = note.Title;
            noteUpdate.ModifiedDate = DateTime.Now;
            noteUpdate.UserID = Social.Lib.SessionManager.CurrentUserID();
            noteUpdate.PrivacyType = note.PrivacyType;

            if (Guid.Empty.Equals(note.ID))
            {
                noteUpdate.CreatedDate = DateTime.Now;
                
                try
                {
                    _note.Add(noteUpdate);
                    string strDes = Social.Lib.Utility.Utility.RemoveHtmlTagAndSubString(noteUpdate.Body, 20);
                    string url = Url.Content("~/" + SessionManager.CurrentUserUserName() + Url.Action("Index/" + noteUpdate.ID, "View", new { area = "Note" }));
                    _userActivity.PostNote(userID, noteUpdate.ID, string.Empty, noteUpdate.Title, strDes, url, string.IsNullOrEmpty(noteUpdate.Photo) ? string.Empty : noteUpdate.PhotoUrl);
                    _userActivity.TagFriend(userID, arrGuidFriend, Lib.Activity.Constants.Activity.ObjectType.Note, noteUpdate.ID, DateTime.Now, false);
                    return RedirectToAction(string.Concat("Index/", noteUpdate.ID), "View");
                }
                catch(Exception ex)
                {
                    TempData["error"] = "Có lỗi trong quá trình cập nhật";
                }
            }

            return RedirectToAction("Index");
        }
    }
}