﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Mvc;
using Social.Entity;
using Social.Entity.TikApplication;
using Social.Service;
using Social.Service.Interface.TikApplication;
using Social.Web.Extends.Helper;
using Google.GData.Contacts;
using Google.GData.Client;
using Google.GData.Extensions;
using Google.Contacts;
using System.Data;
using System.Collections;
using System.Configuration;
using Social.Lib;
using System.Web.Script.Serialization;
using Social.Service.Interface.Tik;
using System.Text;
using Social.Entity.Tik;
using Social.Service.Interface.TikActivity;
namespace Social.Web.Areas.Note.Controllers
{
    public class EditController : Controller
    {
        //
        // GET: /Note/Detail/
        INoteService _note;
        IUserProfileService _user;
        IUserActivityService _activity;

        public EditController(INoteService note, IUserProfileService user, IUserActivityService activity)
        {
            _note = note;
            _user = user;
            _activity = activity;
        }

        public ActionResult Index(string id, Social.Web.Areas.Note.Models.NoteModel noteDetail)
        {
            if (!SessionManager.IsLoggedIn()) return RedirectToRoute("Register_Index");

            if (string.IsNullOrEmpty(id))
            {
                return View(new Social.Web.Areas.Note.Models.NoteModel());
            }

            Guid guidID;
            try
            {
                guidID = Guid.Parse(id);
            }
            catch
            {
                return RedirectToAction("NotFoundPage", "Error", new { area = "" });
            }

            Social.Entity.TikApplication.Note note = _note.GetNoteByID(guidID);
            if (note == null) return RedirectToAction("NotFoundPage", "Error", new { area = "" });

            if(note.UserID != SessionManager.CurrentUserID())
                return RedirectToAction("NotPermission", "Error", new { area = "" });

            noteDetail.ID = note.ID;
            noteDetail.UserID = note.UserID;
            noteDetail.Title = note.Title;
            noteDetail.Body = note.Body;
            noteDetail.Tags = note.Tags;
            noteDetail.Photo = note.Photo;
            noteDetail.PhotoUrl = note.PhotoUrl;
            noteDetail.PrivacyType = note.PrivacyType;
            noteDetail.PrivacyPeople = note.PrivacyPeople;
            noteDetail.Status = note.Status;
            noteDetail.ViewCount = note.ViewCount;
            noteDetail.CreatedDate = note.CreatedDate;
            noteDetail.ModifiedDate = note.ModifiedDate;
            noteDetail.LikeCount = note.LikeCount;
            noteDetail.CommentCount = note.CommentCount;
            noteDetail.IsUserLike = note.IsUserLike;

            return View(noteDetail);
        }

        [HttpPost]
        [ValidateInput(false)]
        public ActionResult Update(string deleteImage, Social.Web.Areas.Note.Models.NoteModel note)
        {
            if (!SessionManager.IsLoggedIn()) return RedirectToRoute("Register_Index");

            Social.Entity.TikApplication.Note noteUpdate = _note.GetNoteByID(note.ID);
            if (noteUpdate == null) return RedirectToAction("NotFoundPage", "Error", new { area = "" });

            Guid userID = SessionManager.CurrentUserID();
            if (userID != noteUpdate.UserID) 
                return RedirectToAction("NotPermission", "Error", new { area = "" });

            noteUpdate.Body = note.Body;
            noteUpdate.Title = note.Title;
            if (deleteImage.Equals("1") || string.IsNullOrEmpty(note.Photo))
            {
                HttpPostedFileBase photo = Request.Files["file_upload"];
                if (photo != null && photo.ContentLength > 0)
                {
                    var content = new Byte[photo.ContentLength];
                    photo.InputStream.Read(content, 0, photo.ContentLength);

                    var psv = new Social.Web.Service.PhotoService();
                    var jsonVal = psv.UploadNote(content, userID);

                    var json_serializer = new JavaScriptSerializer();
                    Social.Entity.TikPhoto.Photo photoInsert = json_serializer.Deserialize<Social.Entity.TikPhoto.Photo>(jsonVal);

                    noteUpdate.Photo = photoInsert.SourceUrl;
                }
            }

            List<Guid> arrGuidFriend = new List<Guid>();

            if (!string.IsNullOrEmpty(note.SuggestionFriend))
            {
                StringBuilder sb = new StringBuilder();
                string[] arrFriend = note.SuggestionFriend.Split(',');
                
                foreach (string item in arrFriend)
                {
                    if (!string.IsNullOrEmpty(item))
                    {
                        UserProfile user = _user.GetByID(Guid.Parse(item));
                        arrGuidFriend.Add(Guid.Parse(item));
                        sb.AppendFormat(",{{FriendID:'{0}',UserName:'{1}',FriendFullName:'{2}',AvatarSmall:'{3}'}}", item, user.UserName, user.FullName, user.SmallAvatarUrlMap);
                    }
                }

                if (!string.IsNullOrEmpty(sb.ToString()))
                {
                    sb.Remove(0, 1);
                }

                if (!string.IsNullOrEmpty(sb.ToString()))
                {
                    noteUpdate.Tags = string.Format("[{0}]", sb.ToString());
                }
            }
            noteUpdate.ModifiedDate = DateTime.Now;
            noteUpdate.UserID = Social.Lib.SessionManager.CurrentUserID();
            noteUpdate.PrivacyType = note.PrivacyType;

            try
            {
                _note.Update(noteUpdate);
                _activity.TagFriend(userID, arrGuidFriend, Lib.Activity.Constants.Activity.ObjectType.Note, noteUpdate.ID, DateTime.Now, true);
            }
            catch { ; }

            return RedirectToAction(string.Format("Index/{0}", note.ID), "View");
        }
    }
}
