﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Social.Service.Interface.TikApplication;
using Social.Lib;
using Social.Service.Interface.TikActivity;
using Social.Service.Interface.TikFriend;
using Social.Entity.TikActivity;
using Social.Service.Interface.Tik;
using Social.Entity.Tik;

namespace Social.Web.Areas.Note.Controllers
{
    public class ViewController : Controller
    {

        INoteService _note;
        IUserCommentService _usercomment;
        IUserLikeService _userlike;
        IFriendListService _friendlist;
        IUserProfileService _userProfile;
        public ViewController(INoteService note, IUserCommentService usercomment, IUserLikeService userlike, IFriendListService friendlist, IUserProfileService userProfile)
        {
            _note = note;
            _usercomment = usercomment;
            _userlike = userlike;
            _friendlist = friendlist;
            _userProfile = userProfile;
        }

        //public ActionResult Index(string id)
        //{
        //    Guid guidID;
        //    try
        //    {
        //        guidID = Guid.Parse(id);
        //    }
        //    catch
        //    {
        //        return RedirectToAction("NotFoundPage", "Error", new { area = "" });
        //    }
        //    Social.Entity.TikApplication.Note note = _note.GetNoteByID(guidID);
        //    if (note == null) return RedirectToAction("NotFoundPage", "Error", new { area = "" });

        //    UserProfile user = _userProfile.GetByID(note.UserID);
        //    if (user == null) return RedirectToAction("NotFoundUser", "Error", new { area = "" });

        //    return RedirectToActionPermanent("Index", "View", new { area = "Note", username = user.UserName, id = id });
        //}

        public ActionResult Index(string username, string id)
        {
            if (!SessionManager.IsLoggedIn()) return RedirectToRoute("Register_Index");
            Guid guidID;
            try
            {
                guidID = Guid.Parse(id);
            }
            catch
            {
                return RedirectToAction("NotFoundPage", "Error", new { area = "" });
            }

            Social.Entity.TikApplication.Note note = _note.GetNoteByID(guidID);
            if (note == null) return RedirectToAction("NotFoundPage", "Error", new { area = "" });

            bool isView = false;

            if (!string.IsNullOrEmpty(note.Tags))
            {
                isView = note.Tags.Contains(SessionManager.CurrentUserID().ToString());
            }

            if(!isView)
            {
                if(!Service.CheckPermission.IsView(SessionManager.CurrentUserID(), note.UserID, note.PrivacyType, note.PrivacyPeople))
                    return RedirectToAction("NotPermission", "Error", new { area = "" });
            }

            if (string.IsNullOrEmpty(username))
            {
                UserProfile user = _userProfile.GetByID(note.UserID);
                return RedirectPermanent(string.Format(@"~/{0}/Note/View/Index/{1}", user.UserName, note.ID));
            }
            else
            {
                var comments = Feed.Models.Comment.Convert(_usercomment.GetComments(guidID));
                var userLike = _userlike.GetLikes(guidID);
                var likes = Feed.Models.Like.Convert(userLike);
                var like = GetLikeString(Social.Lib.Activity.Constants.Like.ObjectType.Note, guidID, userLike);
                ViewData["Like"] = like;
                ViewData.Add("Comments", comments);
                ViewData.Add("Likes", likes);
                return View(note);
            }            
        }

        private string GetLikeString(Social.Lib.Activity.Constants.Like.ObjectType type, Guid objectid, List<UserLike> likes)
        {
            string result = string.Empty;
            if (likes != null)
            {
                var listFriends = _friendlist.GetByOwnerID(SessionManager.CurrentUserID());
                if (listFriends == null) listFriends = new List<Guid>();
                var friendLikes = Social.Web.Areas.Feed.Models.Like.Convert(likes.FindAll(l => listFriends.Contains(l.FromID)));
                var totalLike = likes.Count;
                var youLike = likes.Any(l => l.FromID == SessionManager.CurrentUserID());
                result = Feed.Models.Like.GetFeedLikeString((int)type, objectid, totalLike, youLike, friendLikes);
            }
            return result;
        }
    }
}
