﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Social.Entity.Tik;
using Social.Entity.TikActivity;
using Social.Entity.TikPhoto;
using Social.Lib.Photo;
using Social.Service.Interface.Tik;
using Social.Service.Interface.TikActivity;
using Social.Service.Interface.TikFriend;
using Social.Web.Extends;
using Social.Web.Extends.Helper;
using Social.Service.Interface.TikPhoto;
using Social.Lib;
using SessionManager = Social.Lib.SessionManager;

namespace Social.Web.Areas.Photo.Controllers
{
    public class PhotoController : Controller
    {
        IUserProfileService _user;
        IAlbumService _album;
        IPhotoService _photo;
        IPhotoTagService _photoTag;
        IUserCommentService _comment;
        IUserLikeService _like;
        IFriendListService _friendlist;
        IUserActivityService _useractivity;
        public PhotoController(IPhotoService photo, IPhotoTagService photoTagService, IAlbumService album, IUserProfileService user, IUserCommentService comment, IUserLikeService like, IFriendListService friendlist,IUserActivityService useractivity)
        {
            _photo = photo;
            _album = album;
            _user = user;
            _comment = comment;
            _like = like;
            _friendlist = friendlist;
            _useractivity = useractivity;
            _photoTag = photoTagService;
        }

        public ActionResult Index(string username)
        {
            if (!SessionManager.IsLoggedIn()) return RedirectToRoute("Register_Index");
            if (string.IsNullOrEmpty(username))
            {
                username = SessionManager.CurrentUserUserName(); 
            }
            ViewData["currentUserName"] = SessionManager.CurrentUserUserName();
            var userProfile = _user.GetUserByUsername(username);
            if (userProfile == null)
            {
                return RedirectToActionPermanent("NotFoundUser", "Error", new { area = "" });
            }
            var _uid = userProfile.ID;
            var totalRow = 0;
            var listAb = new List<Album>();
            listAb = username != SessionManager.CurrentUserUserName()
                         ? _album.GetPageListAlbumByUserId(_uid, 1, 9, out totalRow)
                         : _album.GetPageListAlbumByUserId(_uid, 1, 9, out totalRow, true);
            ViewData["currentUserVisit"] = userProfile;
            var lstPhotoTag = _photoTag.GetByUid(_uid);
            ViewData["lstPhotoTag"] = lstPhotoTag;
            ViewData["lstAlbum"] = listAb;
            ViewData["totalRow"] = totalRow.ToString(); 
            return View();
        }

        #region Album, Upload photo

        public ActionResult ListAlbum(string username)
        {  
            if (!SessionManager.IsLoggedIn()) return RedirectToRoute("Register_Index");
            if (string.IsNullOrEmpty(username))
            {
                username = SessionManager.CurrentUserUserName(); 
            }
            ViewData["currentUser"] = SessionManager.CurrentUserUserName();  
            var userProfile = _user.GetUserByUsername(username);
            if (userProfile == null)
            {
                return RedirectToActionPermanent("NotFoundUser", "Error", new { area = "" });
            }
            var _uid = userProfile.ID;
            var totalRow = 0;

            var listAb = _album.GetPageListAlbumByUserId(_uid, 1, 100, out totalRow);
            ViewData["lstAlbum"] = listAb;
            ViewData["totalRow"] = totalRow.ToString();
            var uInfo = _user.Single(u => u.ID == _uid);
            ViewData["author"] = uInfo;  
            ViewData["authorAvatar"] = Lib.Utility.Utility.GetAvatarUrl(uInfo.SmallAvatarUrl);
            return View();
        }

        public ActionResult PhotoDetail(string username, string pId)
        {
            var _pid = Guid.Parse(pId);
            var imageInfo = _photo.Single(p => p.ID == _pid);
            if (imageInfo == null) return Redirect("/Error/NotFound");
            if (!string.IsNullOrEmpty(username)) 
            {
                return Redirect("~/" + username + "/Photo/AlbumDetail/" + imageInfo.AlbumID + "/" + imageInfo.ID);
            }
            var userName = _user.GetByID(imageInfo.UserID).UserName;
            return Redirect("~/" + userName + "/Photo/AlbumDetail/" + imageInfo.AlbumID + "/" + imageInfo.ID);
        }

        public ActionResult EditAlbum(string username, string abid)
        {
            if (!SessionManager.CheckLogin()) return RedirectToRoute("Register_Index");
            var uid = SessionManager.CurrentUserID();
            var _abId = new Guid();
            var lpt = new List<Entity.TikPhoto.Photo>();
            if (!string.IsNullOrEmpty(abid))
            {
                _abId = Guid.Parse(abid);
                var abInfo = _album.Single(ab => ab.ID == _abId);
                if (abInfo.UserID != uid) return RedirectToAction("ListPhotoInAlbum");
                var key = Constants.MemcacheKey.LIST_PHOTO_ALBUM + uid;
                var obj = Memcached.MainCache.Get(key);
                if (obj != null)
                {
                    lpt = (List<Entity.TikPhoto.Photo>)obj;
                }
                else
                {
                    lpt = _photo.GetAllPhotoByAlbumId(_abId);
                    Memcached.MainCache.Store(Enyim.Caching.Memcached.StoreMode.Set, key, lpt,
                                              TimeSpan.FromMinutes(Constants.MemcacheKey.TIME_CACHE_LPA));
                }
                ViewData["albumName"] = abInfo.AlbumName;
                ViewData["currentUserName"] = username;
                ViewData["albumId"] = abInfo.ID.ToString();
                ViewData["albumDescription"] = abInfo.Description;
                ViewData["albumPrivacyType"] = abInfo.PrivacyType.ToString();
                ViewData["albumLocation"] = abInfo.Location;
                ViewData["albumType"] = abInfo.Type.ToString();
                var lstAb = new List<Album>();
                if (abInfo.Type != 8) lstAb.Add(abInfo);
                else
                {
                    lstAb.Clear();
                    lstAb = _album.GetByUserIdTypeAlbum(uid, 8);    
                }
                ViewData["lstAlbum"] = lstAb;
            }
            return View(lpt);
        }

        public ActionResult ListPhotoInAlbum(string username, string abid)
        {
            if (!SessionManager.CheckLogin()) return RedirectToRoute("Register_Index");
            var _abid = new Guid();
            if (!string.IsNullOrEmpty(abid))
            {
                _abid = Guid.Parse(abid);
                var abInfo = _album.Single(a => a.ID == _abid);
                if (abInfo == null) return Redirect("/Error/NotFound");
                var listPt = _photo.GetAllPhotoByAlbumId(_abid);
                ViewData["listPhoto"] = listPt;
                ViewData["albumInfo"] = abInfo;
                var userInfo = _user.GetByID(abInfo.UserID);
                ViewData["author"] = userInfo;
                ViewData["authorAvatar"] = Lib.Utility.Utility.GetAvatarUrl(userInfo.SmallAvatarUrl);
                var userLike = _like.GetLikes(_abid);
                var likes = Feed.Models.Like.Convert(userLike);
                var comments = Feed.Models.Comment.Convert(_comment.GetComments(_abid));
                ViewData["Likes"] = likes;
                ViewData["Comments"] = comments;
                var currentUser = _user.CurrentUser();

                var like = GetLikeString(Lib.Activity.Constants.Like.ObjectType.Album, _abid, userLike);
                ViewData["Like"] = like;
                ViewData["CurrentUser"] = currentUser;
            }
            else return Redirect("/Error/NotFound");
            return View();
        }

        private string GetLikeString(Lib.Activity.Constants.Like.ObjectType type, Guid objectid, List<UserLike> likes)
        {
            string result = string.Empty;
            if (likes != null)
            {
                var listFriends = _friendlist.GetByOwnerID(SessionManager.CurrentUserID());
                if (listFriends == null) listFriends = new List<Guid>();
                var friendLikes = Social.Web.Areas.Feed.Models.Like.Convert(likes.FindAll(l => listFriends.Contains(l.FromID)));
                var totalLike = likes.Count;
                var youLike = likes.Any(l => l.FromID == SessionManager.CurrentUserID());
                result = Feed.Models.Like.GetFeedLikeString((int)type, objectid, totalLike, youLike, friendLikes);
            }
            return result;
        }

        public ActionResult UploadPhotoAlbum(string username, string abid)
        {
            if (!SessionManager.CheckLogin()) return RedirectToAction("ListPhotoInAlbum");

            var currentUser = SessionManager.CurrentUserUserName();
            if (currentUser != username)
                RedirectToAction("UploadPhotoAlbum", new {username = currentUser, abid = abid});

            var userID = SessionManager.CurrentUserID();
            var listAlbum = _album.GetByUserIdTypeAlbum(userID,8);

            var isMyAlbum = false;
            if (!string.IsNullOrEmpty(abid))
            {
                isMyAlbum = listAlbum.Any(album => album.ID.ToString().ToLower().Equals(abid.ToLower()));
                if (!isMyAlbum) return RedirectToAction("ListPhotoInAlbum", new { username = currentUser, abid });
            }

            ViewData["ListAlbum"] = listAlbum;
            ViewData["albumId"] = abid;
            return View();
        }

        public void CreateAlbum(string username, string abName, string abLocation, string abDes, string abPrivacy)
        {
            Guid userID = Guid.Parse(SessionManager.Get("UserID").ToString());
            var abInfo = new Album();
            abInfo.ID = Guid.NewGuid();
            abInfo.UserID = userID;
            abInfo.AlbumName = abName;
            abInfo.Description = abDes;
            abInfo.Type = 8;
            abInfo.Location = abLocation;
            abInfo.PrivacyType = Byte.Parse(abPrivacy);
            abInfo.CreatedDate = DateTime.Now;
            abInfo.ModifiedDate = DateTime.Now;
            _album.Add(abInfo);
            Response.Write(abInfo.ID.ToString());
        }

        public void ActionUploadPhotoAlbum()
        {
            var albumId = "0";
            try { albumId = Request.Form["albumId"]; }catch { albumId = "0"; }
            if (string.IsNullOrEmpty(albumId) || albumId == "0")
            {
                Response.Write("-1");
                return;
            } //Chưa chọn album

            var isFirst = "0";
            try { isFirst = Request.Form["isFirst"]; }catch { isFirst = "0"; }
            var _isFirst = false;
            try { _isFirst = isFirst == "1"; }catch {}

            var isLatest = "0";
            try { isLatest = Request.Form["isLatest"]; }
            catch { isLatest = "0"; }
            var _isLatest = false;
            try { _isLatest = isLatest != "0"; }
            catch { }
            
            Guid userID = Guid.Parse(SessionManager.Get("UserID").ToString());
            var _albumId = Guid.Parse(albumId);

            var thisAb = new List<Album>();
            if (_isFirst || _isLatest)
            {
                thisAb = _album.GetByAlbumIdUserId(_albumId, userID);
                if (thisAb.Count<=0)
                {
                    Response.Write("-2");//Album ko phải của bạn
                    return;
                }
            }
            
            //_useractivity.PostImage(Guid.NewGuid(), Guid.NewGuid(), string.Empty, "Album", string.Empty, "url", new string[]{},new string[]{},true, 1 )

            var htc = Request.Files;
            var posted = htc[0];
            if (htc.Count == 0)
            {
                Response.Write("-3");//Không tồn tại file
                return;
            }

            if (posted.ContentLength <= 0)
            {
                Response.Write("-4"); //Chưa chọn file
                return;
            }

            var filename = DateTime.Now.Ticks.ToString();
            var pathCreate = "";
            var pathSave = "";

            pathCreate = DateTime.Now.ToString("yyyyMM") + "/" + DateTime.Now.ToString("ddHH") + "/";
            pathSave = ConfigManager.PathResourcePhoto + pathCreate;
            string WxH = "0x0";
            lock (posted)
            {
                try
                {
                    #region UploadImage

                    Stream stream = posted.InputStream;
                    int read;
                    var ms = new MemoryStream();
                    var buf = new byte[1024];
                    while ((read = stream.Read(buf, 0, buf.Length)) > 0)
                    {
                        ms.Write(buf, 0, read);
                    }
                    ms.Seek(0, SeekOrigin.Begin);

                    if (ms.Length > 0)
                    {
                        var imagefullsize = System.Drawing.Image.FromStream(ms);
                        var oldW = imagefullsize.Width;
                        var oldH = imagefullsize.Height; 

                        int thumMainW;
                        int thumMainH;
                        if (oldW > Constants.ImageInfo.SizeThumMainW)
                        {
                            double dblCoef = Constants.ImageInfo.SizeThumMainW / (double)oldW;
                            thumMainW = Convert.ToInt32(dblCoef * oldW);
                            thumMainH = Convert.ToInt32(dblCoef * oldH);
                        }
                        else
                        {
                            thumMainW = oldW;
                            thumMainH = oldH;
                        }
                        if (thumMainH > 600)
                        {
                            double dblCoef = 600 / (double)thumMainH;
                            thumMainW = Convert.ToInt32(dblCoef * thumMainW);
                            thumMainH = Convert.ToInt32(dblCoef * thumMainH);
                        }


                        int thumZoomW;
                        int thumZoomH;
                        if (oldW > Constants.ImageInfo.SizeThumbZoomW)
                        {
                            double dblCoef = Constants.ImageInfo.SizeThumbZoomW / (double)oldW;
                            thumZoomW = Convert.ToInt32(dblCoef * oldW);
                            thumZoomH = Convert.ToInt32(dblCoef * oldH);
                        }
                        else
                        {
                            thumZoomW = oldW;
                            thumZoomH = oldH;
                        }
                        WxH = thumZoomW + "x" + thumZoomH;
                        ms.Seek(0, SeekOrigin.Begin);

                        var data = ms.GetBuffer();
                        var w = new[] { oldW, thumZoomW, thumMainW, Constants.ImageInfo.SizeThumbW, Constants.ImageInfo.SizeThumbAlbumSlideW };
                        var h = new[] { oldH, thumZoomH, thumMainH, Constants.ImageInfo.SizeThumbH, Constants.ImageInfo.SizeThumbAlbumSlideH };
                        var nameFiles = new[]
                                           {
                                               filename + Constants.ImageInfo.NameOriginal,
                                               filename + Constants.ImageInfo.NameThumbZoom,
                                               filename + Constants.ImageInfo.NameThumMain,
                                               filename + Constants.ImageInfo.NameThumb,
                                               filename + Constants.ImageInfo.NameThumbAlbumSline
                                           };
                        var crop = new[] { false, false, false, true, true };
                        try
                        {
                            if (Directory.Exists(pathSave) == false)
                            {
                                Directory.CreateDirectory(pathSave);
                            }
                        }
                        catch { }

                        var quality = ResizeImage.Quality(posted.ContentLength);

                        var multi = new MultiUpload[5];
                        for (var j = 0; j < 5; j++)
                        {
                            multi[j] = new MultiUpload(data, pathSave + nameFiles[j], w[j], h[j], quality[j], crop[j]);
                            multi[j].Start();
                        }

                        for (var j = 0; j < 5; j++)
                        {
                            multi[j].Join();
                        }
                    }

                    #endregion

                    var _pathSaveDB = pathCreate + filename;
                    var info = new Entity.TikPhoto.Photo();
                    info.ID = Guid.NewGuid();
                    info.Caption = "";
                    info.AlbumID = _albumId;
                    info.SourceUrl = _pathSaveDB;
                    info.UserID = userID;
                    info.WxH = WxH;
                    info.CreatedDate = DateTime.Now;
                    info.ModifiedDate = DateTime.Now;
                    _photo.Add(info);

                    if (!_isFirst)
                    {
                        if (_isLatest)
                        {
                            var totalUp = 0;
                            int.TryParse(isLatest, out totalUp);
                            thisAb[0].PhotoCount = thisAb[0].PhotoCount + totalUp;
                            _album.Update(thisAb[0]); 
                        }
                    }
                    else
                    {
                        if (string.IsNullOrEmpty(thisAb[0].SourceUrlCoverPhoto))
                        {
                            thisAb[0].CoverPhotoID = info.ID;
                            thisAb[0].SourceUrlCoverPhoto = info.SourceUrl;
                        }
                        if (_isLatest)
                        {
                            var totalUp = 0;
                            int.TryParse(isLatest, out totalUp);
                            thisAb[0].PhotoCount = thisAb[0].PhotoCount + totalUp;
                        }
                        _album.Update(thisAb[0]);
                    }
                }
                catch { Response.Write("0"); }//Không thành công
                
                Response.Write("1");//Thành công
            }
        }

        public JsonResult NextPhoto(string abId, string pId)
        {
            var _abId = Guid.Parse(abId);
            var _pId = Guid.Parse(pId);
            var photoInfo = _photo.GetNextPhotoInAnbum(_abId, _pId);
            var obj = convertPhotoToObject(photoInfo);
            return Json(obj, JsonRequestBehavior.AllowGet);
        }

        public JsonResult PrevPhoto(string abId, string pId)
        {
            var _abId = Guid.Parse(abId);
            var _pId = Guid.Parse(pId);
            var photoInfo = _photo.GetPrevPhotoInAnbum(_abId, _pId);
            var obj = convertPhotoToObject(photoInfo);
            return Json(obj, JsonRequestBehavior.AllowGet);
        }

        public JsonResult PhotoInfo(string pId)
        {
            var _pId = Guid.Parse(pId);
            var pInfo = _photo.GetPhotoById(_pId);
            return Json(pInfo, JsonRequestBehavior.AllowGet);
        }

        public ActionResult AlbumDetail(string username, string abId, string pId)
        {
            //var uid = SessionManager.CurrentUserID();
            if (!SessionManager.CheckLogin()) return RedirectToRoute("Register_Index");

            var _abId = Guid.Parse(abId);
            var aInfo = _album.Single(a => a.ID == _abId);
            if (aInfo == null) return Redirect("/Error/NotFound");
            if (string.IsNullOrEmpty(username))
            {
                var userName = _user.GetByID(aInfo.UserID).UserName;
                return RedirectToActionPermanent("ListPhotoInAlbum", "Photo", new { area = "Photo", username = userName, abid = _abId.ToString()});
            }

            ViewData["aInfo"] = aInfo; 
            ViewData["pId"] = pId;
            var uInfo = _user.GetByID(aInfo.UserID); 
            ViewData["author"] = uInfo;
            ViewData["authorAvatar"] = Lib.Utility.Utility.GetAvatarUrl(uInfo.SmallAvatarUrl); 

            var userLike = _like.GetLikes(_abId);
            var likes = Feed.Models.Like.Convert(userLike);
            var comments = Feed.Models.Comment.Convert(_comment.GetComments(_abId));
            ViewData["Likes"] = likes;
            ViewData["Comments"] = comments;

            var currentUser = _user.CurrentUser();
            var like = GetLikeString(Lib.Activity.Constants.Like.ObjectType.Album, _abId, userLike);
            ViewData["Like"] = like;
            ViewData["CurrentUser"] = currentUser;
            return View();
        }

        private object convertPhotoToObject(Entity.TikPhoto.Photo info)
        {
            return new {
                           ID = info.ID,
                           AlbumID = info.AlbumID,
                           UserID = info.UserID,
                           Caption = info.Caption,
                           SourceUrl = info.SourceUrl,
                           SortOrder = info.SortOrder,
                           WxH = info.WxH,
                           ViewCount = info.ViewCount,
                           LikeCount = info.LikeCount,
                           CommentCount = info.CommentCount,
                           Status = info.Status,
                           PrivacyType = info.PrivacyType,
                           PrivacyPeopleShowHide = info.PrivacyPeopleShowHide,
                           CreatedDate = info.CreatedDate,
                           ModifiedDate = info.ModifiedDate
                       };
        }
        #endregion 
    }
}
