﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Mvc;
using Social.Entity;
using Social.Entity.Tik;
using Social.Lib;
using Social.Service;
using Social.Service.Interface.Tik;
using Social.Web.Extends.Helper;
using Google.GData.Contacts;
using Google.GData.Client;
using Google.GData.Extensions;
using Google.Contacts;
using System.Data;
using System.Collections;
using System.Security.Cryptography;
using Social.Web.Extends.WebService;

namespace Social.Web.Areas.Profile.Controllers
{
    public class AccountController : Controller
    {
        //
        // GET: /Profile/Account/
        IUserProfileService _userProfile;

        public AccountController(IUserProfileService userProfile)
        {
            _userProfile = userProfile;
        }


        public ActionResult Password()
        {
            if (TempData["error"] != null)
            {
                string err = TempData["error"].ToString();
                if (!string.IsNullOrEmpty(err))
                {
                    ViewData["ErrorMessage"] = err;
                }
            }

            if (TempData["errorPassword"] != null)
            {
                string err = TempData["errorPassword"].ToString();
                if (!string.IsNullOrEmpty(err))
                {
                    ViewData["ErrorMessagePassword"] = err;
                }
            }

            if (TempData["errorNewPassword"] != null)
            {
                string err = TempData["errorNewPassword"].ToString();
                if (!string.IsNullOrEmpty(err))
                {
                    ViewData["ErrorMessageNewPassword"] = err;
                }
            }

            if (TempData["errorReNewPassword"] != null)
            {
                string err = TempData["errorReNewPassword"].ToString();
                if (!string.IsNullOrEmpty(err))
                {
                    ViewData["ErrorMessageReNewPassword"] = err;
                }
            }

            return View();
        }

        [HttpPost]
        public ActionResult ChangePassword(string password, string newPassword, string reNewPassword)
        {
            if (string.IsNullOrEmpty(password))
                TempData["errorPassword"] = "Bạn chưa nhập mật khẩu hiện tại";

            if (string.IsNullOrEmpty(newPassword))
                TempData["errorNewPassword"] = "Bạn chưa nhập mật khẩu mới";

            if (string.IsNullOrEmpty(reNewPassword))
                TempData["errorReNewPassword"] = "Bạn chưa nhập lại mật khẩu mới";

            if (string.IsNullOrEmpty(password) || string.IsNullOrEmpty(newPassword) || string.IsNullOrEmpty(reNewPassword))
                return RedirectToAction("Password");

            if (newPassword.Trim().Length < 6)
            {
                TempData["errorNewPassword"] = "Độ dài mật khẩu phải từ 6-32 ký tự.";
                return RedirectToAction("Password");
            }

            if (!newPassword.Trim().Equals(reNewPassword.Trim()))
            {
                TempData["errorReNewPassword"] = "Mật khẩu mới không khớp.";
                return RedirectToAction("Password");
            }

            if (!SessionManager.IsLoggedIn()) return RedirectToRoute("Register_Index");
            Guid userID = SessionManager.CurrentUserID();

            string passwordHash = Social.Lib.Utility.Utility.GetMd5Hash(password);

            UserProfile user = _userProfile.Single(u => u.ID == userID && u.Password == passwordHash);

            if (user == null)
            {
                TempData["errorPassword"] = "Mật khẩu hiện tại không đúng";
                return RedirectToAction("Password");
            }

            string newPasswordHash = Social.Lib.Utility.Utility.GetMd5Hash(newPassword.Trim());

            user.Password = newPasswordHash;

            try
            {
                _userProfile.Update(user);
                TempData["error"] = "Cập nhật mật khẩu thành công.";
            }
            catch
            {
                TempData["error"] = "Quá trình thực hiện không thành công. Xin thử lại sau ít phút.";
            }

            return RedirectToAction("Password");
        }

        [HttpPost]
        public ActionResult Login(string email, string password,string remember)
        {
            UserProfile user = null;
            string passwordHash = Social.Lib.Utility.Utility.GetMd5Hash(password);
            if (ConfigManager.GetKey("UseTikID") != null && ConfigManager.GetKey("UseTikID") == "true")
            {
                Social.Web.Extends.WebService.IDService.ServiceResult res=IDService.Login(email, password);
                if (res.IsSuccess)
                {
                    user = _userProfile.Single(u=>u.Email == email && u.IsActive);
                    if (user == null)
                        TempData["CustomerRegister"] = res.Customer;
                }
            }
            else
                user = _userProfile.Single(u => u.Password == passwordHash && u.Email == email && u.IsActive);
            if (user != null)
            {
                SessionManager.Set("UserID", user.ID);
                SessionManager.Set("UserName", user.UserName);
                SessionManager.Set("FullName", user.FullName);
                SessionManager.Set("Email", user.Email);
                SessionManager.Set("SmallAvatarUrl", user.SmallAvatarUrl);
                SessionManager.Set("MediumAvatarUrl", user.MediumAvatarUrl);
                SessionManager.Set("ProfileLink", user.ProfileLink);
                if (remember=="1")
                {                    
                    string cookieObj=email+"|"+passwordHash;
                    string cookieHash=Lib.Base64String.Encode(cookieObj);
                    HttpCookie rememberCookie = new HttpCookie("REMEMBER_ME", cookieHash);
                    rememberCookie.Expires = DateTime.Now.AddDays(15);
                    Response.Cookies.Add(rememberCookie);
                }
                if (Request.Cookies["REDIRECT_AFTER_LOGIN"] != null)
                    return Redirect(Request.Cookies["REDIRECT_AFTER_LOGIN"].Value);
                return RedirectToAction("Index", "Index", new { area = "Feed" });
            }
            TempData["ErrorLogin"] = "Email hoặc mật khẩu không đúng";
            return RedirectToAction("Index", "Register", new { area = "Register" });
        }

        public ActionResult Logout()
        {
            SessionManager.Abadon();
            if(Response.Cookies["REMEMBER_ME"]!=null)
                Response.Cookies["REMEMBER_ME"].Expires = DateTime.Now.AddDays(-1);
            return RedirectToAction("Index", "Register", new { area = "Register" });
        }
    }
}
