﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Social.Entity;
using Social.Entity.Tik;
using Social.Lib;
using Social.Service;
using Social.Service.Interface.Tik;
using Social.Web.Extends.Helper;
using System.Configuration;
using System.Drawing.Drawing2D;
using System.Net;
using System.Web.Script.Serialization;
using Social.Service.Interface.TikActivity;

namespace Social.Web.Areas.Register.Controllers
{
    public class AvatarController : Controller
    {
        IUserProfileService _user;
        IUserActivityService _userActivity;

        public AvatarController(IUserProfileService user, IUserActivityService userActivity)
        {
            _user = user;
            _userActivity = userActivity;
        }

        public ActionResult Index(UserProfile user)
        {
            if (!SessionManager.IsLoggedIn()) return RedirectToRoute("Register_Index");
            Guid userID = Social.Lib.SessionManager.CurrentUserID();
            user = _user.Single(u => u.ID == userID);

            ViewData["LargeAvatar"] = string.IsNullOrEmpty(user.LargeAvatarUrl) ? ConfigurationManager.AppSettings["DefaultAvatar"] : Social.Lib.Utility.Utility.GetAvatarUrl(user.LargeAvatarUrl);

            return View();
        }

        [HttpPost]
        public ActionResult Upload(HttpPostedFileBase uploadedfile, string actionAvatar)
        {
            if (!SessionManager.IsLoggedIn()) return RedirectToRoute("Register_Index");
            Guid userID = Social.Lib.SessionManager.CurrentUserID();
            var user = _user.Single(u => u.ID == userID);

            if (uploadedfile == null) return RedirectToAction("Index");

            string originalFile = Path.GetFileName(uploadedfile.FileName);

            var posted = Request.Files["uploadedfile"];
            if (posted.FileName != "")
            {
                string extension = Path.GetExtension(originalFile);
                string croppedFileName = user.ID.ToString();

                string pathSmall = string.IsNullOrEmpty(user.LargeAvatarUrl) ?
                    GetAvatarPath("Small", croppedFileName, extension)
                    : user.SmallAvatarUrlMap;

                string pathMedium = string.IsNullOrEmpty(user.LargeAvatarUrl) ?
                    GetAvatarPath("Medium", croppedFileName, extension)
                    : user.MediumAvatarUrlMap;

                string pathLarge = string.IsNullOrEmpty(user.LargeAvatarUrl) ?
                    GetAvatarPath("Large", croppedFileName, extension)
                    : user.LargeAvatarUrl;

                posted.SaveAs(pathLarge.Contains("http://") ? pathLarge : string.Concat(ConfigurationManager.AppSettings["AvatarResource"], pathLarge));

                // upload to avatar album
                var content = new Byte[posted.ContentLength];
                posted.InputStream.Read(content, 0, posted.ContentLength);
                var psv = new Social.Web.Service.PhotoService();
                var jsonVal = psv.UploadAvatar(content, userID);
                var json_serializer = new JavaScriptSerializer();
                Social.Entity.TikPhoto.Photo photoInsert = json_serializer.Deserialize<Social.Entity.TikPhoto.Photo>(jsonVal);

                ResizeImage(string.Concat(ConfigurationManager.AppSettings["AvatarResource"], pathLarge), string.Concat(ConfigurationManager.AppSettings["AvatarResource"], pathLarge), 720, 720);
                ResizeImage(string.Concat(ConfigurationManager.AppSettings["AvatarResource"], pathMedium), string.Concat(ConfigurationManager.AppSettings["AvatarResource"], pathLarge), 200, 720);
                ResizeImage(string.Concat(ConfigurationManager.AppSettings["AvatarResource"], pathSmall), string.Concat(ConfigurationManager.AppSettings["AvatarResource"], pathLarge), 50, 0);

                user.LargeAvatarUrl = string.Concat(pathLarge);
                user.MediumAvatarUrlMap = string.Concat(pathMedium);
                user.SmallAvatarUrlMap = string.Concat(pathSmall);
                Social.Lib.SessionManager.UpdateAvatar(pathSmall, pathMedium); 

                _user.Update(user);
                _userActivity.UpdateAvatar(user.ID, photoInsert.ID, DateTime.Now);
            }

            return string.IsNullOrEmpty(actionAvatar) ?  RedirectToAction("Index") : RedirectToAction("Avatar","Update", new {area = "Profile"});
        }

        private string GetAvatarPath(string type, string filename, string extension)
        {
            string resource = string.Empty;

            switch (type)
            {
                case "Small":
                    resource = ConfigurationManager.AppSettings["SmallAvatarUrl"];
                    break;
                case "Medium":
                    resource = ConfigurationManager.AppSettings["MediumAvatarUrl"];
                    break;
                case "Large":
                    resource = ConfigurationManager.AppSettings["LargeAvatarUrl"];
                    break;
                default:
                    resource = ConfigurationManager.AppSettings["LargeAvatarUrl"];
                    break;
            }
            string path = string.Concat(resource, DateTime.Now.ToString("yyyyMM"), "/");
            string pathSave = string.Concat(ConfigurationManager.AppSettings["AvatarResource"], path);
            try
            {
                if (Directory.Exists(pathSave) == false)
                {
                    Directory.CreateDirectory(pathSave);
                }
            }
            catch { }

            return string.Concat(path, filename, extension);
        }

        public ActionResult EditThumbnail(UserProfile user)
        {
            if (!SessionManager.IsLoggedIn()) return RedirectToRoute("Register_Index");
            Guid userID = Social.Lib.SessionManager.CurrentUserID();
            user = _user.Single(u => u.ID == userID);

            ViewData["LargeAvatar"] = string.IsNullOrEmpty(user.LargeAvatarUrl) ? ConfigurationManager.AppSettings["DefaultAvatar"] : user.LargeAvatarUrl;

            return View();
        }

        [HttpPost]
        public ActionResult CropThumbnail(string filePath, string x1, string y1, string x2, string y2, string actionAvatar)
        {
            if (!SessionManager.IsLoggedIn()) return RedirectToRoute("Register_Index");
            Guid userID = Social.Lib.SessionManager.CurrentUserID();
            var user = _user.Single(u => u.ID == userID);

            //CropImage(string.Concat(ConfigurationManager.AppSettings["AvatarResource"], user.MediumAvatarUrl.Replace(ConfigurationManager.AppSettings["AvatarServerUrl"].ToString(), string.Empty)), filePath, 200, int.Parse(x1), int.Parse(y1), int.Parse(x2), int.Parse(y2));
            CropImage(string.Concat(ConfigurationManager.AppSettings["AvatarResource"], user.SmallAvatarUrl.Replace(ConfigurationManager.AppSettings["AvatarServerUrl"].ToString(), string.Empty)), filePath, 50, int.Parse(x1), int.Parse(y1), int.Parse(x2), int.Parse(y2));

            return string.IsNullOrEmpty(actionAvatar) ? RedirectToAction("Index") : RedirectToAction("Avatar", "Update", new { area = "Profile" });
        }

        public void ResizeImage(string filepath, string filePathSource, int thumbnailSize, int maxSize)
        {
            var newImage = new Bitmap(filePathSource);
            int newWidth = 0, newHeight = 0;
            int scale = maxSize > 0 ? maxSize : thumbnailSize;

            if ((newImage.Width < newImage.Height && maxSize ==0) || (newImage.Width > newImage.Height && maxSize > 0))
            {
                newWidth = newImage.Width > maxSize ? scale : newImage.Width;
                newHeight = newImage.Width > maxSize ? (newImage.Height * scale / newImage.Width) : newImage.Height;
            }
            else
            {
                newWidth = newImage.Height > maxSize ? (newImage.Width * scale / newImage.Height) : newImage.Width;
                newHeight = newImage.Height > maxSize ? scale : newImage.Height;
            }

            var thumbnailBitmap = new Bitmap(maxSize > 0 ? newWidth : thumbnailSize, maxSize > 0 ? newHeight : thumbnailSize);

            var thumbnailGraph = Graphics.FromImage(thumbnailBitmap);
            thumbnailGraph.CompositingQuality = CompositingQuality.HighQuality;
            thumbnailGraph.SmoothingMode = SmoothingMode.HighQuality;
            thumbnailGraph.InterpolationMode = InterpolationMode.HighQualityBicubic;

            var imageRectangle = new Rectangle(0, 0, newWidth, newHeight);
            thumbnailGraph.DrawImage(newImage, imageRectangle);
            newImage.Dispose();
            thumbnailBitmap.Save(filepath);
            thumbnailGraph.Dispose();
            thumbnailBitmap.Dispose();
        }

        public void CropImage(string filepath, string filePathSource, int thumbnailSize, int X1, int Y1, int X2, int Y2)
        {
            if (X2 == 0 || Y2 == 0) return;
            HttpWebRequest req = (HttpWebRequest)WebRequest.Create(filePathSource);
            HttpWebResponse res=(HttpWebResponse)req.GetResponse();

            using (Image img = Image.FromStream(res.GetResponseStream()))
            {
                int w = (System.Math.Max(img.Width, img.Height) < 500) ? (X2 - X1) : (X2 - X1) * System.Math.Max(img.Width, img.Height) / 500;
                int h = (System.Math.Max(img.Width, img.Height) < 500) ? (Y2 - Y1) : (Y2 - Y1) * System.Math.Max(img.Width, img.Height) / 500;
                int x = (System.Math.Max(img.Width, img.Height) < 500) ? System.Math.Min(X1, X2) : System.Math.Min(X1, X2) * System.Math.Max(img.Width, img.Height) / 500;
                int y = (System.Math.Max(img.Width, img.Height) < 500) ? System.Math.Min(Y1, Y2) : System.Math.Min(Y1, Y2) * System.Math.Max(img.Width, img.Height) / 500;

                using (System.Drawing.Bitmap _bitmap = new System.Drawing.Bitmap(w, h))
                {
                    _bitmap.SetResolution(img.HorizontalResolution, img.VerticalResolution);
                    using (Graphics _graphic = Graphics.FromImage(_bitmap))
                    {
                        _graphic.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
                        _graphic.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
                        _graphic.PixelOffsetMode = System.Drawing.Drawing2D.PixelOffsetMode.HighQuality;
                        _graphic.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.HighQuality;
                        _graphic.DrawImage(img, 0, 0, w, h);
                        _graphic.DrawImage(img, new Rectangle(0, 0, w, h), x, y, w, h, GraphicsUnit.Pixel);

                        using (EncoderParameters encoderParameters = new EncoderParameters(1))
                        {
                            encoderParameters.Param[0] = new EncoderParameter(Encoder.Quality, 100L);
                            _bitmap.Save(filepath, GetImageCodec(Path.GetExtension(filepath)), encoderParameters);
                        }
                    }
                }
            }

            ResizeImage(filepath, filepath, thumbnailSize, 0);
        }

        /// <summary>
        /// Find the right codec
        /// </summary>
        /// <param name="extension"></param>
        /// <returns></returns>
        public static ImageCodecInfo GetImageCodec(string extension)
        {
            extension = extension.ToUpperInvariant();
            ImageCodecInfo[] codecs = ImageCodecInfo.GetImageEncoders();
            foreach (ImageCodecInfo codec in codecs)
            {
                if (codec.FilenameExtension.Contains(extension))
                {
                    return codec;
                }
            }
            return codecs[1];
        }
    }

}
