﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Web.Mvc;
using Google.Contacts;
using Google.GData.Client;
using Google.GData.Extensions;
using Social.Entity;
using Social.Entity.Tik;
using Social.Lib;
using Social.Service.Interface.Tik;
using Social.Web.Areas.Register.Models;
using Social.Web.Extends.Helper;
using Social.Web.Extends.WebService;
using System.Linq;
using System.Text;
using Social.Service.Interface.TikFriend;


namespace Social.Web.Areas.Register.Controllers
{
    public class RegisterController : Controller
    {
        IUserProfileService _userservice;
        IUserFriendService _userfriendservice;
        IFriendRequestService _friendrequestservice;
        IUserSearchService _usersearchservice;
        //
        // GET: /UserSchool/
        public RegisterController(IUserProfileService userservice, IUserFriendService userfriendservice, IFriendRequestService friendrequestservice, IUserSearchService usersearchservice)
        {
            _userservice = userservice;
            _userfriendservice = userfriendservice;
            _friendrequestservice = friendrequestservice;
            _usersearchservice = usersearchservice;
        }
        public ActionResult Index()
        {
            //Kiểm tra đăng nhập và đưa về trang feed nếu đã đăng nhập
            if (SessionManager.IsLoggedIn())
                return RedirectToRoute("Feed_Index");
            if (TempData["Message"] != null)
                ViewData["Message"] = TempData["Message"];
            if (TempData["ErrorLogin"] != null)
                ViewData["ErrorLogin"] = TempData["ErrorLogin"];
            if (TempData["CustomerRegister"] != null)
                ViewData["CustomerRegister"] = TempData["CustomerRegister"];
            return View(new RegisterIndexModel());
        }
        public ActionResult InviteFriend()
        {
            if (!SessionManager.IsLoggedIn()) return RedirectToRoute("Register_Index");

            Guid userID = new Guid(SessionManager.CurrentUserID().ToString());
            var userprofile = _userservice.GetByID(userID);
            SessionManager.Set("Email", userprofile.Email);
            ViewData["Email"] = SessionManager.Get("Email").ToString();
            return View();
        }
        public ActionResult GoogleContact()
        {
            if (!SessionManager.IsLoggedIn()) return RedirectToRoute("Register_Index");
            Guid userid = new Guid(SessionManager.Get("UserID").ToString());
            string useremail = SessionManager.Get("Email").ToString();

            List<string> jsonDataTik = new List<string>();
            List<string> jsonDataNotTik = new List<string>();
            if (HttpContext.Request.QueryString.AllKeys.Contains("token"))
            {
                Session["token"] = HttpContext.Request.QueryString.Get("token");
                GoogleService gservice = new GoogleService();
                var contacts = gservice.GetContacts();
                foreach (string email in contacts)
                {

                    var user = _userservice.GetByEmail(email);

                    if (useremail != email)
                    {
                        if (user != null)
                        {
                            Guid id = user.ID;
                            int friendrequest = _friendrequestservice.GetCountByFromUserIDAndToUserID(userid, id);
                            int userfriend = _userfriendservice.GetCountByUserFriend(userid, id);

                            if (userfriend == 0)
                            {
                                if (friendrequest == 0 && userfriend == 0)
                                {
                                    jsonDataTik.Add("{\"id\":\"" + user.ID + "\",\"name\":\"" + user.FullName + "\",\"email\":\"" + email + "\",\"avatar\":\"" + user.SmallAvatarUrl + "\",\"profile_link\":\"" + user.ProfileLink + "\"}");
                                }
                            }
                        }
                        else
                            jsonDataNotTik.Add("{\"email\":\"" + email + "\"}");
                    }
                }
            }
            else
            {
                ViewBag.GoogleUrl = AuthSubUtil.getRequestUrl(HttpContext.Request.Url.AbsoluteUri, "https://www.google.com/m8/feeds/", false, true);
            }
            string jsonObj = "{\"datatik\":[" + string.Join(",", jsonDataTik) + "],\"datanottik\":[" + string.Join(",", jsonDataNotTik) + "]}";
            ViewData["contacts"] = jsonObj;
            return View();
        }
        public ActionResult YahooContact()
        {
            if (!SessionManager.IsLoggedIn()) return RedirectToRoute("Register_Index");

            string oauth_token = Request["oauth_token"];
            string oauth_verifier = Request["oauth_verifier"];
            YahooService ys = new YahooService();
            if (string.IsNullOrEmpty(oauth_token) || string.IsNullOrEmpty(oauth_verifier))
            {
                ViewData["yahoo_url"] = ys.GetRequestToken();
            }
            else
            {
                ys.OauthToken = oauth_token;
                ys.OauthVerifier = oauth_verifier;
                if (string.IsNullOrEmpty(ys.OauthYahooGuid))
                    ys.GetAccessToken(ys.OauthToken, ys.OauthVerifier);
                List<string> contacts = ys.RetriveContacts();
                List<string> jsonDataTik = new List<string>();
                List<string> jsonDataNotTik = new List<string>();
                foreach (string email in contacts)
                {
                    var user = _userservice.GetByEmail(email);
                    if (user != null)
                        jsonDataTik.Add("{\"id\":\"" + user.ID + "\",\"name\":\"" + user.FullName + "\",\"email\":\"" + email + "\",\"avatar\":\"" + user.SmallAvatarUrl + "\",\"profile_link\":\"" + user.ProfileLink + "\"}");
                    else
                        jsonDataNotTik.Add("{\"email\":\"" + email + "\"}");
                }
                string jsonObj = "{\"datatik\":[" + string.Join(",", jsonDataTik) + "],\"datanottik\":[" + string.Join(",", jsonDataNotTik) + "]}";
                ViewData["contacts"] = jsonObj;
            }

            return View();
        }
        [HttpPost]
        public ActionResult Create(RegisterIndexModel model)
        {
            if (ModelState.IsValid)
            {
                try
                {
                    UserProfile user = new UserProfile();
                    user.BirthDay = DateTime.ParseExact(model.Day + "/" + model.Month + "/" + model.Year, "d/M/yyyy", null);
                    user.ID = Guid.NewGuid();
                    user.IsActive = true;
                    user.LastLoginDate = System.DateTime.Now;
                    user.Gender = model.Gender;
                    user.Email = model.Email;
                    user.FirstName = model.FirstName;
                    user.LastName = model.LastName;
                    user.Password = model.Password;
                    user.RegisterDate = DateTime.Now;
                    //Gọi web service để đăng ký với hệ thống TIK
                    bool checkIDOK = true;
                    string message = "Đăng ký không thành công";
                    if (ConfigManager.GetKey("UseTikID") != null && ConfigManager.GetKey("UseTikID") == "true")
                    {
                        var result = IDService.RegisterUser(user.Email, user.Password, user.FirstName, user.LastName, user.BirthDay, user.Gender);
                        checkIDOK = result.IsSuccess;
                        message = result.Message;
                    }
                    if (checkIDOK)
                    {
                        _userservice.RegisterUser(user);
                        //Add to user Search
                        UserSearch us = new UserSearch();
                        us.ID = Guid.NewGuid();
                        us.SmallAvatar = user.SmallAvatarUrlMap;
                        us.UserID = user.ID;
                        us.Username = user.UserName;
                        us.FullName = user.FullName;
                        us.FullText = string.Format("{0},{1},{2},{3}", user.UserName, Lib.Utility.Utility.RemoveSign4VietnameseString(user.FullName), user.FullName, user.Email);
                        _usersearchservice.Add(us);
                        //
                        SessionManager.Set("UserID", user.ID);
                        SessionManager.Set("UserName", user.UserName);
                        SessionManager.Set("FullName", user.FullName);
                        SessionManager.Set("Email", user.Email);
                        SessionManager.Set("SmallAvatarUrl", user.SmallAvatarUrl);
                        SessionManager.Set("ProfileLink", user.ProfileLink);
                        LogHelper.Log("Dang ky thanh cong User voi cac tham so Email:{0};Password:{1};FirstName:{2},LastName:{3},BirthDay:{4},Gender:{5}", user.Email, user.Password, user.FirstName, user.LastName, user.BirthDay, user.Gender);
                        return RedirectToAction("Index", "Join");
                    }
                    else
                    {
                        TempData["Message"] = message;
                        LogHelper.Log("Register", message);
                    }

                }
                catch (Exception ex)
                {
                    LogHelper.Error(ex);
                }
            }
            return RedirectToAction("Index");
        }
        [HttpPost]
        public bool IsCaptchaValid(string challenge, string response)
        {

            // Add your operation implementation here  
            Recaptcha.RecaptchaValidator validator = new Recaptcha.RecaptchaValidator
            {
                PrivateKey = "6Le2i8USAAAAAMZY-pXMRXBLp4xFPgRy-N7QD4mw",
                RemoteIP = Request.UserHostAddress,
                Challenge = challenge,
                Response = response

            };
            Recaptcha.RecaptchaResponse Cresponse = validator.Validate();
            //return Cresponse.IsValid;
            if (Cresponse.IsValid)
            {
                //RedirectToAction("Create");
                return true;
            }
            return false;
        }
    }
}
