﻿using System;
using System.ComponentModel.DataAnnotations;
using Social.Web.Properties;
using System.Web.Mvc;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

namespace Social.Web.Areas.Register.Models
{
    [Serializable]
    public class RegisterIndexModel 
    {
        public Guid ID { get; set; }

        [Required(ErrorMessageResourceType = typeof(Language), ErrorMessageResourceName = "EmailRequired")]
        [Email(ErrorMessageResourceType = typeof(Language), ErrorMessageResourceName = "EmailInvalid")]
        public string Email { get; set; }

        [Required(ErrorMessageResourceType = typeof(Language), ErrorMessageResourceName = "EmailRequired")]
        [Compare("Email", ErrorMessageResourceType = typeof(Language), ErrorMessageResourceName = "EmailMustMatch")]
        public string Email2 { get; set; }

        [Required(ErrorMessageResourceType = typeof(Language), ErrorMessageResourceName = "FirstNameRequired")]
        public string FirstName { get; set; }

        [Required(ErrorMessageResourceType = typeof(Language), ErrorMessageResourceName = "LastNameRequired")]
        public string LastName { get; set; }

        [Range(1,2,ErrorMessageResourceType = typeof(Language), ErrorMessageResourceName = "GenderRequired")]
        public byte Gender { get; set; }
        public IEnumerable<SelectListItem> Genders
        {
            get
            {
                List<SelectListItem> genderList = new List<SelectListItem>();
                genderList.Add(new SelectListItem { Value = "0", Text = "Chọn giới tính", Selected = true });
                genderList.Add(new SelectListItem { Value = "1", Text = "Nam" });
                genderList.Add(new SelectListItem { Value = "2", Text = "Nữ" });
                return genderList;
            }
        }
        [Range(1000, 3000, ErrorMessageResourceType = typeof(Language), ErrorMessageResourceName = "BirthDayRequired")]
        public int Year { get; set; }
        public IEnumerable<SelectListItem> Years
        {
            get
            {
                int currentYear=DateTime.Now.Year;
                List<SelectListItem> yearList = new List<SelectListItem>();
                yearList.Add(new SelectListItem { Value = "0", Text = "Năm:", Selected = true });
                for (int i = currentYear; i >= currentYear - 120; i--)
                {
                    yearList.Add(new SelectListItem { Text = i.ToString(), Value = i.ToString() });
                }
                return yearList;
            }
        }
        [Range(1, 12, ErrorMessageResourceType = typeof(Language), ErrorMessageResourceName = "BirthDayRequired")]
        public int Month { get; set; }
        public IEnumerable<SelectListItem> Months
        {
            get
            {
                List<SelectListItem> monthList = new List<SelectListItem>();
                monthList.Add(new SelectListItem { Value = "0", Text = "Tháng:", Selected = true });
                for (int i = 1; i <= 12; i++)
                {
                    monthList.Add(new SelectListItem { Text = i.ToString(), Value = i.ToString() });
                }
                return monthList;
            }
        }
        [Range(1, 31, ErrorMessageResourceType = typeof(Language), ErrorMessageResourceName = "BirthDayRequired")]
        public int Day { get; set; }
        public IEnumerable<SelectListItem> Days
        {
            get
            {
                List<SelectListItem> dayList = new List<SelectListItem>();
                dayList.Add(new SelectListItem { Value = "0", Text = "Ngày:", Selected = true });
                for (int i = 1; i <= 31; i++)
                {
                    dayList.Add(new SelectListItem { Text = i.ToString(), Value = i.ToString() });
                }
                return dayList;
            }
        }
        
        [Required(ErrorMessageResourceType = typeof(Language), ErrorMessageResourceName = "PasswordRequired")]
        [Password(ErrorMessageResourceType = typeof(Language), ErrorMessageResourceName = "PasswordInvalid")]
        public string Password { get; set; }

    }
    public class PasswordAttribute : ValidationAttribute, IClientValidatable
    {
        private int _minLength=6;
        private int _maxLength=20;
        public PasswordAttribute()
        { }
        public PasswordAttribute(int minLength, int maxLength)
        {
            _minLength = minLength;
            _maxLength = maxLength;
        }
        protected override ValidationResult IsValid(object value, ValidationContext validationContext)
        {
            if (value.ToString().Length >= _minLength && value.ToString().Length <= _maxLength)
                return ValidationResult.Success;
            return new ValidationResult("Mật khẩu từ " + _minLength + "- "+_maxLength+" ký tự", new[] { "Password" });
        }
        IEnumerable<ModelClientValidationRule> IClientValidatable.GetClientValidationRules(ModelMetadata metadata, ControllerContext context)
        {
            yield return new PasswordValidationRule(FormatErrorMessage(metadata.DisplayName), _minLength, _maxLength);
        }
    }
    public class PasswordValidationRule : ModelClientValidationRule
    {
        public PasswordValidationRule(string errorMessage, int minLength,int maxLength)
        {
            ErrorMessage = errorMessage;
            ValidationParameters.Add("minlength", minLength);
            ValidationParameters.Add("maxlength", maxLength);
            ValidationType = "password";
        }
    }
    public class EmailAttribute : RegularExpressionAttribute, IClientValidatable
    {
        public EmailAttribute()
            : base(@"^([a-zA-Z0-9_\-\.]+)@((\[[0-9]{1,3}" +
                   @"\.[0-9]{1,3}\.[0-9]{1,3}\.)|(([a-zA-Z0-9\-]+\" +
                   @".)+))([a-zA-Z]{2,4}|[0-9]{1,3})(\]?)$")
        {
        }
        IEnumerable<ModelClientValidationRule> IClientValidatable.GetClientValidationRules(ModelMetadata metadata, ControllerContext context)
        {
            var errorMessage = FormatErrorMessage(metadata.GetDisplayName());
            yield return new EmailValidationRule(errorMessage);
        }
    }

    public class EmailValidationRule : ModelClientValidationRule
    {
        public EmailValidationRule(string errorMessage)
        {
            ErrorMessage = errorMessage;
            ValidationType = "email";            
        }
    }
}
