﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.Mvc;
using System.Web.Mvc.Html;

namespace Social.Web.Extends.Helper
{
    public class ResponseCapture : IDisposable
    {
        private readonly HttpResponseBase response;
        private readonly TextWriter originalWriter;
        private StringWriter localWriter;

        public ResponseCapture(HttpResponseBase response)
        {
            this.response = response;
            originalWriter = response.Output;
            localWriter = new StringWriter();
            response.Output = localWriter;
        }

        public override string ToString()
        {
            localWriter.Flush();
            return localWriter.ToString();
        }

        public void Dispose()
        {
            if (localWriter != null)
            {
                localWriter.Dispose();
                localWriter = null;
                response.Output = originalWriter;
            }
        }
    }

    public static class ActionResultExtensions
    {
        public static string Capture(this ActionResult result, ControllerContext controllerContext){
            using (var it = new ResponseCapture(controllerContext.RequestContext.HttpContext.Response)){
                result.ExecuteResult(controllerContext);
                return it.ToString();
            }
        }
    }


    //public static class ViewToString 
    //{
    //    /// <summary>Renders a view to string.</summary> 
    //    public static string RenderPartialToString(this HtmlHelper html, string viewName, object viewData)
    //    {
    //        return RenderViewToString(html.ViewContext.Controller.ControllerContext, viewName, viewData);
    //    }
    //    /// <summary>Renders a view to string.</summary> 
    //    public static string RenderViewToString(this Controller controller, string viewName, object viewData)
    //    {
    //        return RenderViewToString(controller.ControllerContext, viewName, viewData);
    //    }

    //    private static string RenderViewToString(ControllerContext context, string viewName, object viewData)
    //    {
    //        //Create memory writer 
    //        var sb = new StringBuilder();
    //        var memWriter = new StringWriter(sb);

    //        //Create fake http context to render the view 
    //        var fakeResponse = new HttpResponse(memWriter);
    //        var fakeContext = new HttpContext(HttpContext.Current.Request, fakeResponse);
    //        var fakeControllerContext = new ControllerContext(
    //            new HttpContextWrapper(fakeContext),
    //            context.RouteData, context.Controller);

    //        var oldContext = HttpContext.Current;
    //        HttpContext.Current = fakeContext;

    //        //Use HtmlHelper to render partial view to fake context 
    //        var html = new HtmlHelper(new ViewContext(fakeControllerContext, new FakeView(), new ViewDataDictionary(), new TempDataDictionary()),
    //            new ViewPage());
    //        html.RenderPartial(viewName, viewData);

    //        //Restore context 
    //        HttpContext.Current = oldContext;

    //        //Flush memory and return output 
    //        memWriter.Flush();
    //        return sb.ToString();
    //    }
    //}

    ///// <summary>Fake IView implementation, only used to instantiate an HtmlHelper.</summary> 
    //public class FakeView : IView
    //{
    //    #region IView Members
    //    public void Render(ViewContext viewContext, System.IO.TextWriter writer)
    //    {
    //        throw new NotImplementedException();
    //    }
    //    #endregion
    //}
}
