﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using Social.Service.Interface.TikFriend;
using System.Web.Script.Serialization;
using Social.Service.Interface.Tik;
using Social.Entity.Tik;
using System.Web.Mvc;

namespace Social.Web.Service
{
    public class CheckPermission
    {
        /// <summary>
        ///     kiem tra quyen view cua mot user
        /// </summary>
        /// <param name="viewerID">ID của người xem</param>
        /// <param name="ownerID">ID của người người được xem</param>
        /// <param name="privacyType">
        ///     1: Tất cả được xem
        ///     2: Chỉ bạn bè tôi
        ///     3: Chỉ mình tôi
        /// </param>
        /// <param name="privacyPeople">
        ///     list user ko được quyền xem (privacyPeople = f77e6087-936a-48ac-ad02-0cb9f8dc7c98,f77e6087-936a-48ac-ad02-0cb9f8dc7c98....)
        /// </param>
        /// <returns>
        ///     true: được xem
        ///     false: không được xem
        /// </returns>
        public static bool IsView(Guid viewerID, Guid ownerID, int privacyType, string privacyPeople)
        {
            if (Guid.NewGuid().Equals(viewerID) || Guid.NewGuid().Equals(ownerID)) return false;

            if (viewerID.Equals(ownerID)) return true;

            if (!string.IsNullOrEmpty(privacyPeople))
            {
                if (privacyPeople.Contains(viewerID.ToString()))
                    return false;
            }

            switch (privacyType)
            {
                case 1:
                    return true;
                case 2:
                    var _userFriend = DependencyResolver.Current.GetService<Social.Service.Interface.TikFriend.IUserFriendService>();
                    try
                    {
                        return _userFriend.CheckExsitFriend(ownerID, viewerID);
                    }
                    catch
                    { return false; }
                case 3:
                    return viewerID.Equals(ownerID);
                default:
                    return false;
            }
        }

        /// <summary>
        ///     Kiem tra quyen được xem thông tin đã được cấu hình trong userconfig
        /// </summary>
        /// <param name="viewerID"></param>
        /// <param name="ownerID"></param>
        /// <param name="configkey">
        ///     Social.Lib.Tik.Constants.ConfigKey
        /// </param>
        /// <returns></returns>
        public static bool IsPermission(Guid viewerID, Guid ownerID, string configkey)
        {
            if (viewerID.Equals(ownerID)) return true;

            var _userConfig = DependencyResolver.Current.GetService<Social.Service.Interface.Tik.IUserConfigService>();
            UserConfig userConfig = _userConfig.GetByUserIdAndConfigKey(ownerID, configkey);

            if (userConfig == null)
            {
                var _userFriend = DependencyResolver.Current.GetService<Social.Service.Interface.TikFriend.IUserFriendService>();
                try
                {
                    return _userFriend.CheckExsitFriend(ownerID, viewerID);
                }
                catch
                { 
                    return false; 
                }
            }

            // Config type is checkbox
            if (userConfig.PrivacyType == 1)
            {
                return userConfig.ConfigValue.Equals("1");
            }

            return IsView(viewerID, ownerID, int.Parse(userConfig.ConfigValue), userConfig.PrivacyPeople);
        }

        /// <summary>
        ///     Kiểm tra xem 2 người có phải bạn bè ko.
        /// </summary>
        /// <param name="viewerID"></param>
        /// <param name="ownerID"></param>
        /// <returns></returns>
        public static bool IsFriend(Guid viewerID, Guid ownerID)
        {
            if (viewerID.Equals(ownerID)) return true;

            var _userFriend = DependencyResolver.Current.GetService<Social.Service.Interface.TikFriend.IUserFriendService>();
            try
            {
                return _userFriend.CheckExsitFriend(ownerID, viewerID);
            }
            catch
            {
                return false;
            }
        }

        //public static Permission GetPermission(Guid viewerID, Guid ownerID)
        //{
        //    if (viewerID.Equals(ownerID)) return new Permission(true);

        //    Social.Lib.Tik.Constants.ConfigKey configKey = new Lib.Tik.Constants.ConfigKey();
        //    var _userConfig = DependencyResolver.Current.GetService<Social.Service.Interface.Tik.IUserConfigService>();
        //    IList<UserConfig> listUserConfig = _userConfig.GetByUserID(ownerID);

        //    if (listUserConfig == null) return new Permission(false);

        //    foreach (UserConfig uconfig in listUserConfig)
        //    {
        //        uconfig.ConfigKey 
        //    }
        //}
    }

    public class Permission
    {
        public bool UserAllowSearch {get;set;}
        public bool UserCity { get; set; }
        public bool UserBirthday { get; set; }
        public bool UserMaridstatus { get; set; }
        public bool UserCurrentAddress { get; set; }
        public bool UserInterested { get; set; }
        public bool UserJob { get; set; }
        public bool UserHobby { get; set; }
        public bool UserEmail { get; set; }
        public bool UserMobile { get; set; }
        public bool UserAddress { get; set; }
        public bool UserNick { get; set; }
        public bool UserWebsite { get; set; }
        public bool UserWall { get; set; }
        public bool UserPostWall { get; set; }

        public Permission(bool defaultValue)
        {
            UserAllowSearch = defaultValue;
            UserCity = defaultValue;
            UserBirthday = defaultValue;
            UserMaridstatus = defaultValue;
            UserCurrentAddress = defaultValue;
            UserInterested = defaultValue;
            UserJob = defaultValue;
            UserHobby = defaultValue;
            UserEmail = defaultValue;
            UserMobile = defaultValue;
            UserAddress = defaultValue;
            UserNick = defaultValue;
            UserWebsite = defaultValue;
            UserWall = defaultValue;
            UserPostWall = defaultValue;
        }
    }
}