﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.Mvc;
using System.Web.Script.Serialization;
using System.Web.SessionState;
using Social.Entity.TikPhoto;
using Social.Lib;
using Social.Lib.Photo;
using Social.Service.Interface.Tik;
using Social.Service.Interface.TikActivity;
using Social.Service.Interface.TikPhoto;
namespace Social.Web.Service
{
    /// <summary>
    /// Summary description for PhotoAlbum
    /// </summary>
    public class PhotoAlbum : IHttpHandler, IReadOnlySessionState
    {
        private readonly IPhotoService _photo = DependencyResolver.Current.GetService<IPhotoService>();
        private readonly IAlbumService _album = DependencyResolver.Current.GetService<IAlbumService>();
        private readonly IPhotoTagService _tag = DependencyResolver.Current.GetService<IPhotoTagService>();
        private readonly IUserActivityService _useractivity = DependencyResolver.Current.GetService<IUserActivityService>();
        private readonly IUserProfileService _user = DependencyResolver.Current.GetService<IUserProfileService>();
        public void ProcessRequest(HttpContext context)
        {
            var result = "";
            var type = context.Request.QueryString["type"];
            if (string.IsNullOrEmpty(type))
            {
                type = context.Request.Form["type"];
            }
            switch (type)
            {
                case "UpdateListPhoto":
                    result = UpdateListPhoto(context);
                    break;
                case "UpdateDesPhoto":
                    result = UpdateDesPhoto(context);
                    break;
                case "EditAlbum":
                    result = EditAlbum(context);
                    break;
                case "PostToWall":
                    result = PostToWall(context);
                    break;
                case "DeleteAlbum":
                    result = DeleteAlbum(context);
                    break;
                case "UploadImgWall":
                    result = UploadImgWall(context);
                    break;
                case "RemovePhoto":
                    result = RemovePhoto(context);
                    break;
                case "SetPhotoAsAvatar":
                    result = SetPhotoAsAvatar(context);
                    break;
                case "AddTagFriend":
                    result = AddTagFriend(context);
                    break;
                case "RemoveTag":
                    result = RemoveTag(context);
                    break;
                case  "LoadFriendTaged":
                    result = LoadFriendTaged(context);
                    break;
                default:
                    result = "";
                    break;
            }
            context.Response.ContentType = "text/plain";
            context.Response.Write(result);
        }

        private string LoadFriendTaged(HttpContext context)
        {
            var currUserInfo = _user.CurrentUser();
            if (currUserInfo == null) return "-1";//Chua dang nhap
            var pid = ""; try { pid = context.Request.Form["pId"]; }
            catch { }
            var _pid = Guid.NewGuid();
            if (!string.IsNullOrEmpty(pid)) _pid = Guid.Parse(pid);
            var lstUserTag = _tag.GetListByPid(_pid);
            var jss = new JavaScriptSerializer();
            var sbControls = new StringBuilder();
            jss.Serialize(lstUserTag, sbControls);
            return sbControls.ToString();
        }

        private string RemoveTag(HttpContext context)
        {
            var retval = "1";
            var currUserInfo = _user.CurrentUser();
            if (currUserInfo == null) return "-1";//Chua dang nhap
            var pid = ""; try { pid = context.Request.Form["pId"]; }
            catch { }
            var _pid = Guid.NewGuid();
            if (!string.IsNullOrEmpty(pid)) _pid = Guid.Parse(pid);
            var uId = ""; try { uId = context.Request.Form["uId"]; }
            catch { }
            var _uId = Guid.NewGuid();
            if (!string.IsNullOrEmpty(uId)) _uId = Guid.Parse(uId);
            //Check quyen
            _tag.DeleteByUidAndPid(_uId, _pid);
            var lstUser = new List<Guid>();
            lstUser.Add(_uId);
            _useractivity.DeleteTagFriend(lstUser, Lib.Activity.Constants.Activity.ObjectType.Image, _pid);
            return retval;
        }

        private string AddTagFriend(HttpContext context)
        {
            var retval = "({{InfoTag: {0}, Return: '{1}'}})";
            var currUserInfo = _user.CurrentUser();
            if (currUserInfo == null) return string.Format(retval, "-1", "-1");//Chua dang nhap
            var pid = ""; try { pid = context.Request.Form["pId"]; }
            catch { }
            var _pid = Guid.NewGuid();
            if (!string.IsNullOrEmpty(pid)) _pid = Guid.Parse(pid);
            //Check quyen
            var uId = ""; try { uId = context.Request.Form["uId"]; }
            catch { }
            var uName = ""; try { uName = context.Request.Form["uName"]; }
            catch { }
            var uFullName = ""; try { uFullName = context.Request.Form["uFullName"]; }
            catch { }
            var _left = ""; try { _left = context.Request.Form["_left"]; }
            catch { }
            var _top = ""; try { _top = context.Request.Form["_top"]; }
            catch { }
            var urlP = ""; try { urlP = Base64String.Decode(context.Request.Form["urlP"]); }
            catch { }
            var photoTag = new PhotoTag();
            photoTag.ID = Guid.NewGuid();
            photoTag.PhotoID = _pid;
            photoTag.PhotoUrl = urlP;
            photoTag.UserID = Guid.Parse(uId);
            photoTag.UserName = uName;
            photoTag.UserFullName = uFullName;
            photoTag.AuthorUserIDTag = currUserInfo.ID;
            photoTag.AuthorUserNameTag = currUserInfo.UserName;
            photoTag.AuthorUserFullNameTag = currUserInfo.FullName;
            photoTag.TagLeft =int.Parse(_left);
            photoTag.TagTop = int.Parse(_top);
            _tag.Add(photoTag);
            var lstUser = new List<Guid>();
            lstUser.Add(photoTag.UserID);
            _useractivity.TagFriend(currUserInfo.ID, lstUser, Lib.Activity.Constants.Activity.ObjectType.Image, _pid, DateTime.Now, false);
            var oSerializer = new JavaScriptSerializer();
            var sJSON = oSerializer.Serialize(photoTag);
            return string.Format(retval, sJSON, "1");
        }

        private string SetPhotoAsAvatar(HttpContext context)
        {
            var retval = "({{PathFile: '{0}', Return: '{1}'}})";
            var uid = ""; try { uid = context.Session["UserID"].ToString(); }
            catch { }
            if (string.IsNullOrEmpty(uid)) return string.Format(retval, "-1", "-1");//Chua dang nhap
            var pid = ""; try { pid = context.Request.Form["pid"]; }
            catch { }
            var _pid = Guid.NewGuid();
            if (!string.IsNullOrEmpty(pid)) _pid = Guid.Parse(pid);
            var pInfo = _photo.Single(p => p.ID == _pid);
            if (pInfo == null) return string.Format(retval, "-1", "-2");//Ko ton tai photo
            var _uid = Guid.Parse(uid);
            //if (pInfo.UserID != _uid) return string.Format(retval, "-1", "-3");//Anh khong phai cua ban
            //var abId = ""; try { abId = context.Request.Form["abId"]; }
            //catch { }
            //if (string.IsNullOrEmpty(abId)) return string.Format(retval, "-1", "-2");// Album ko tồn tại
            var pathFile = ConfigManager.PathResourcePhoto + pInfo.SourceUrl + Constants.ImageInfo.NameOriginal;
            var content = Utility.FileToByteArray(pathFile);
            var photoSV = new PhotoService();
            var retV = photoSV.UploadAvatar(content, _uid);
            return string.Format(retval, pInfo.SourceUrl, "1");
        }

        private string RemovePhoto(HttpContext context)
        {
            var uid = ""; try { uid = context.Session["UserID"].ToString(); }
            catch { }
            if (string.IsNullOrEmpty(uid)) return "-1";//Chua dang nhap
            var pid = ""; try { pid = context.Request.Form["pid"]; }
            catch { }
            var _pid = Guid.NewGuid();
            if (!string.IsNullOrEmpty(pid)) _pid = Guid.Parse(pid);
            var pInfo = _photo.Single(p => p.ID == _pid);
            if (pInfo == null) return "-2";//Ko ton tai photo
            var _uid = Guid.Parse(uid);
            if (pInfo.UserID != _uid) return "-3";//Anh khong phai cua ban
            var abId = ""; try { abId = context.Request.Form["abId"]; }
            catch { }
            if (string.IsNullOrEmpty(abId)) return "-2";// Album ko tồn tại
            Album abInfo;
            var _abId_ = Guid.Parse(abId);
            try { abInfo = _album.Single(ab => ab.ID == _abId_); }
            catch { abInfo = null; }
            if (abInfo == null) return "-3";// Album ko tồn tại
            if (_uid != abInfo.UserID) return "-4"; // Album ko phải của user hiện tại
            abInfo.PhotoCount = abInfo.PhotoCount - 1;
            if(abInfo.PhotoCount==0)
            {
                abInfo.CoverPhotoID = Guid.Empty;
                abInfo.SourceUrlCoverPhoto = "";
            }
            else
            {
                if (_pid == abInfo.CoverPhotoID)
                {
                    var listPhoto = _photo.GetAllPhotoByAlbumId(_abId_);
                    foreach (var photo in listPhoto.Where(photo => photo.ID != abInfo.CoverPhotoID))
                    {
                        abInfo.CoverPhotoID = photo.ID;
                        abInfo.SourceUrlCoverPhoto = photo.SourceUrl;
                        break;
                    }
                }
            }
            _album.Update(abInfo);
            var lstUser = new List<Guid>();
            lstUser = _tag.DeleteByPid(pInfo.ID);
            if (lstUser.Count > 0)
                _useractivity.DeleteTagFriend(lstUser, Lib.Activity.Constants.Activity.ObjectType.Image, pInfo.ID);
            _useractivity.DeleteObject(pInfo.ID);
            _photo.Delete(pInfo);
            Utility.DeleteFile(ConfigManager.PathResourcePhoto + pInfo.SourceUrl + Constants.ImageInfo.NameOriginal);
            Utility.DeleteFile(ConfigManager.PathResourcePhoto + pInfo.SourceUrl + Constants.ImageInfo.NameThumbZoom);
            Utility.DeleteFile(ConfigManager.PathResourcePhoto + pInfo.SourceUrl + Constants.ImageInfo.NameThumMain);
            Utility.DeleteFile(ConfigManager.PathResourcePhoto + pInfo.SourceUrl + Constants.ImageInfo.NameThumb);
            Utility.DeleteFile(ConfigManager.PathResourcePhoto + pInfo.SourceUrl + Constants.ImageInfo.NameThumbAlbumSline);
            return "1";
        }

        private string DeleteAlbum(HttpContext context)
        {
            var uid = SessionManager.CurrentUserID();
            if (uid == Guid.Empty) return "-1";//Chua dang nhap
            var abId = ""; try { abId = context.Request.Form["abId"]; }
            catch { }
            if (string.IsNullOrEmpty(abId)) return "-2";// Album ko tồn tại
            Album abInfo;
            var _abId_ = Guid.Parse(abId);
            try { abInfo = _album.Single(ab => ab.ID == _abId_); }
            catch { abInfo = null; }
            if (abInfo == null) return "-3";// Album ko tồn tại
            if (uid != abInfo.UserID) return "-4"; // Album ko phải của user hiện tại
            if (abInfo.Type != 8) return "-5"; //Album ko được phép xóa
            _album.Delete(abInfo);
            _useractivity.DeleteObject(abInfo.ID);
            var lstPhotoDel = _photo.GetAllPhotoByAlbumId(_abId_);
            var lstUser = new List<Guid>();
            foreach (var photo in lstPhotoDel)
            {
                lstUser = _tag.DeleteByPid(photo.ID);
                if (lstUser.Count > 0)
                    _useractivity.DeleteTagFriend(lstUser, Lib.Activity.Constants.Activity.ObjectType.Image, photo.ID);
                _useractivity.DeleteObject(photo.ID);
                _photo.Delete(photo);
                Utility.DeleteFile(ConfigManager.PathResourcePhoto + photo.SourceUrl + Constants.ImageInfo.NameOriginal);
                Utility.DeleteFile(ConfigManager.PathResourcePhoto + photo.SourceUrl + Constants.ImageInfo.NameThumbZoom);
                Utility.DeleteFile(ConfigManager.PathResourcePhoto + photo.SourceUrl + Constants.ImageInfo.NameThumMain);
                Utility.DeleteFile(ConfigManager.PathResourcePhoto + photo.SourceUrl + Constants.ImageInfo.NameThumb);
                Utility.DeleteFile(ConfigManager.PathResourcePhoto + photo.SourceUrl + Constants.ImageInfo.NameThumbAlbumSline);
            }
            return "1";
        }

        private string UploadImgWall(HttpContext context)
        {
            var retval = "({{FeedId: '{0}', Return: '{1}'}})";
            var mess = context.Request.Form["mess"];
            if (mess != "-")
            {
                mess = Base64String.Decode(mess);
            }
            else mess = "";
            
            var userView = context.Request.Form["userView"];
            userView = Base64String.Decode(userView);
            var _userView = Guid.Parse(userView);
            var userId = SessionManager.CurrentUserID();
            if (userId == Guid.Empty) return string.Format(retval, "-1", "-1");//Bạn chưa đăng nhập
            var htc = context.Request.Files;
            var posted = htc[0];
            if (htc.Count == 0)
            {
                return string.Format(retval, "-1", "-2");//Không tồn tại file
            }
            if (posted.ContentLength <= 0)
            {
                return string.Format(retval, "-1", "-3");//Chưa chọn file
            }
            var photoSV = new PhotoService();
            var content = new Byte[posted.ContentLength];
            posted.InputStream.Read(content, 0, posted.ContentLength);
            var retV = photoSV.UploadWall(content, userId);

            var json_serializer = new JavaScriptSerializer();
            Photo photoInfo = json_serializer.Deserialize<Photo>(retV);
            var aInfo = _album.Single(a => a.ID == photoInfo.AlbumID);
            var lstImage = new string[1];
            var lstUrl = new string[1];
            lstImage[0] = Lib.Utility.Utility.UrlThumbAlbumOrPhoto(photoInfo.SourceUrl);
            lstUrl[0] = "/" + SessionManager.CurrentUserUserName() + "/photo/albumdetail/" + photoInfo.AlbumID + "/" + photoInfo.ID;
            var feedid = Guid.NewGuid();
            var ret = _useractivity.PostImage(userId, _userView, photoInfo.ID, mess, aInfo.AlbumName,
                                  aInfo.Description, "/" + SessionManager.CurrentUserUserName() + "/Photo/ListPhotoInAlbum/" + aInfo.ID,
                                  lstImage, lstUrl, false, 1, feedid);
            return ret ? string.Format(retval, feedid, "1") : string.Format(retval, "0", "0");
        }

        private string PostToWall(HttpContext context)
        {
            var retval = "1";
            var userId = SessionManager.CurrentUserID();
            if (userId == Guid.Empty) return "-1"; 
            var albumId = ""; try { albumId = context.Request.Form["aId"]; }
            catch { }
            var totalUp = ""; try { totalUp = context.Request.Form["totalUp"];}
            catch { }
            var _albumId = Guid.Empty;
            try {
                _albumId = Guid.Parse(albumId);
            }
            catch {} 
            var _totalUp = 0;
            try
            {
                _totalUp = Int32.Parse(totalUp);
            }
            catch { }

            var iImg = _totalUp;
            if (_totalUp > 3) iImg = 3;
            var totalR = 0;
            var lstImgNews = _photo.GetPageListPhotoByAlbumId(_albumId, 1, iImg, out totalR);
            var lstImage = new string[lstImgNews.Count];
            var lstUrl = new string[lstImgNews.Count];
            var abInfo = _album.Single(a => a.ID == _albumId);
            for (var j = 0; j < lstImage.Count(); j++)
            {
                lstImage[j] = Lib.Utility.Utility.UrlThumbAlbumOrPhoto(lstImgNews[j].SourceUrl);
                lstUrl[j] = "/" + SessionManager.CurrentUserUserName() + "/photo/albumdetail/" + _albumId + "/" + lstImgNews[j].ID;
            }

            _useractivity.PostImage(userId, abInfo.ID, string.Empty, abInfo.AlbumName,
                                    abInfo.Description, "/" + SessionManager.CurrentUserUserName() + "/Photo/ListPhotoInAlbum/" + abInfo.ID,
                                    lstImage, lstUrl, true, _totalUp);
            return retval;
        }

        private string UpdateDesPhoto(HttpContext context)
        {
            var uid = ""; try { uid = context.Session["UserID"].ToString(); }
            catch { }
            if (string.IsNullOrEmpty(uid)) return "-1";//Chua dang nhap
            var pid = ""; try { pid = context.Request.Form["pid"]; }
            catch { }
            var _pid = Guid.NewGuid();
            if (!string.IsNullOrEmpty(pid)) _pid = Guid.Parse(pid);
            var pInfo = _photo.Single(p => p.ID == _pid);
            if (pInfo == null) return "-2";//Ko ton tai photo
            var _uid = Guid.Parse(uid);
            if (pInfo.UserID != _uid) return "-3";//Anh khong phai cua ban
            var desP = ""; try { desP = Base64String.Decode(context.Request.Form["des"]); }
            catch { }
            pInfo.Caption = desP;
            _photo.Update(pInfo);
            return "1";
        }

        private string UpdateListPhoto(HttpContext context)
        {
            var retval = "-2";
            var uid = ""; try { uid = context.Session["UserID"].ToString(); }
            catch { }
            if (string.IsNullOrEmpty(uid)) return "1";//Chua dang nhap
            var albumId = ""; try { albumId = context.Request.Form["albumId"]; }
            catch { }
            if (string.IsNullOrEmpty(albumId)) return "-2";// Album ko tồn tại
            var strlstP = context.Request.Form["lstP"];
            var key1 = context.Request.Form["k1"];
            var key2 = context.Request.Form["k2"];
            Album abInfo;
            var _abId_ = Guid.Parse(albumId);
            try { abInfo = _album.Single(ab => ab.ID == _abId_); }
            catch { abInfo = null; }
            if (abInfo == null) return "-3";// Album ko tồn tại
            var _uid = Guid.Parse(uid);
            if (_uid != abInfo.UserID) return "-4"; // Album ko phải của user hiện tại
            if (!string.IsNullOrEmpty(strlstP))
            {
                var lstP = strlstP.Replace(key1, "|").Split('|').ToList();
                if (lstP.Count <= 0) return "-2";//Album chua co anh

                var key = Constants.MemcacheKey.LIST_PHOTO_ALBUM + uid;
                var obj = Memcached.MainCache.Get(key);
                var lstPr = new List<Photo>();
                if (obj != null)
                {
                    lstPr = (List<Photo>)obj;
                }
                else
                {
                    lstPr = _photo.GetAllPhotoByAlbumId(_abId_);
                    Memcached.MainCache.Store(Enyim.Caching.Memcached.StoreMode.Set, key, lstPr,
                                              TimeSpan.FromMinutes(Constants.MemcacheKey.TIME_CACHE_LPA));
                }
                if (lstPr.Count <= 0) return "-2";//Abum ko co anh nao
                var isDel = "false";
                var pId = "0";
                var pDes = "";
                var aId = "";
                var isAvaA = "false";
                var _tmp = 0;
                for (var i = 0; i < lstPr.Count; i++)
                {
                    _tmp = 1;
                    foreach (var p in lstP)
                    {
                        var strItemP = p.Replace(key2, "|").Split('|');
                        try
                        {
                            isDel = Base64String.Decode(strItemP[0]);
                            pId = Base64String.Decode(strItemP[1]);
                            pDes = Base64String.Decode(strItemP[2]);
                            isAvaA = Base64String.Decode(strItemP[3]);
                            aId = Base64String.Decode(strItemP[4]);
                        }
                        catch { continue; }
                        if (string.IsNullOrEmpty(pId)) continue;
                        var _pId = Guid.Parse(pId);
                        if (lstPr[i].ID != _pId) continue;
                        if (isDel.ToLower() == "true")
                        {
                            var currPId = lstPr[i].ID;
                            Utility.DeleteFile(ConfigManager.PathResourcePhoto + lstPr[i].SourceUrl + Constants.ImageInfo.NameOriginal);
                            Utility.DeleteFile(ConfigManager.PathResourcePhoto + lstPr[i].SourceUrl + Constants.ImageInfo.NameThumbZoom);
                            Utility.DeleteFile(ConfigManager.PathResourcePhoto + lstPr[i].SourceUrl + Constants.ImageInfo.NameThumMain);
                            Utility.DeleteFile(ConfigManager.PathResourcePhoto + lstPr[i].SourceUrl + Constants.ImageInfo.NameThumb);
                            Utility.DeleteFile(ConfigManager.PathResourcePhoto + lstPr[i].SourceUrl + Constants.ImageInfo.NameThumbAlbumSline);
                            _photo.Delete(lstPr[i]);
                            _useractivity.DeleteObject(lstPr[i].ID);
                            lstPr.Remove(lstPr[i]);
                            i = i - 1;
                            if (currPId == abInfo.CoverPhotoID)
                            {
                                if (lstPr.Count > 0)
                                {
                                    abInfo.CoverPhotoID = lstPr[lstPr.Count - 1].ID;
                                    abInfo.SourceUrlCoverPhoto = lstPr[lstPr.Count - 1].SourceUrl;
                                }
                                else
                                {
                                    abInfo.CoverPhotoID = Guid.Empty;
                                    abInfo.SourceUrlCoverPhoto = "";
                                }
                            }
                            abInfo.PhotoCount = abInfo.PhotoCount - 1;
                            _album.Update(abInfo);
                        }
                        else
                        {
                            var newAbId = Guid.Parse(aId);
                            lstPr[i].Caption = pDes;
                            lstPr[i].AlbumID = newAbId;
                            if (newAbId != abInfo.ID)
                            {
                                //update count album
                                var currPId = lstPr[i].ID;
                                if (_tmp==0) abInfo = _album.Single(ab => ab.ID == _abId_);
                                if (currPId == abInfo.CoverPhotoID)
                                {
                                    var isExistImg = false;
                                    foreach (var pIr in lstPr.Where(pIr => pIr.ID != currPId && pIr.AlbumID == _abId_))
                                    {
                                        abInfo.CoverPhotoID = pIr.ID;
                                        abInfo.SourceUrlCoverPhoto = pIr.SourceUrl;
                                        isExistImg = true;
                                        _tmp = 0;
                                        break;
                                    }
                                    if(!isExistImg)
                                    {
                                        abInfo.CoverPhotoID = Guid.Empty;
                                        abInfo.SourceUrlCoverPhoto = "";
                                    }
                                }
                                abInfo.PhotoCount = abInfo.PhotoCount - 1;
                                _album.Update(abInfo);

                                var newAb = _album.Single(ab => ab.ID == newAbId);
                                newAb.PhotoCount = newAb.PhotoCount + 1;
                                _album.Update(newAb);
                                lstP.Remove(p);
                                break;
                            }
                            
                            if (isAvaA.ToLower() == "true")
                            {
                                abInfo.CoverPhotoID = lstPr[i].ID;
                                abInfo.SourceUrlCoverPhoto = lstPr[i].SourceUrl;
                                _album.Update(abInfo);
                            }
                        }
                        lstP.Remove(p);
                        break;
                    }
                }
                retval = _photo.UpdateListPhoto(lstPr, abInfo.ID).ToString();
                if (retval == "1") Memcached.MainCache.Remove(key);
            }
            return retval;
        }

        private string EditAlbum(HttpContext context)
        {
            var retval = "1";
            var uid = ""; try { uid = context.Session["UserID"].ToString(); }
            catch { }
            if (string.IsNullOrEmpty(uid)) return "-1";//Chua dang nhap
            var albumId = ""; try { albumId = context.Request.Form["aId"]; }
            catch { }
            if (string.IsNullOrEmpty(albumId)) return "-2";// Album ko tồn tại
            Album abInfo;
            var _abId_ = Guid.Parse(Base64String.Decode(albumId));
            try { abInfo = _album.Single(ab => ab.ID == _abId_); }
            catch { abInfo = null; }
            if (abInfo == null) return "-3";// Album ko tồn tại
            var _uid = Guid.Parse(uid);
            if (_uid != abInfo.UserID) return "-4"; // Album ko phải của user hiện tại
            var abName = Base64String.Decode(context.Request.Form["abName"]);
            var abLocation = Base64String.Decode(context.Request.Form["abLocation"]);
            var abDes = Base64String.Decode(context.Request.Form["abDes"]);
            var abPrivacy = Base64String.Decode(context.Request.Form["abPrivacy"]);
            abInfo.AlbumName = abName;
            abInfo.Location = abLocation;
            abInfo.Description = abDes;
            abInfo.PrivacyType = Byte.Parse(abPrivacy);
            abInfo.ModifiedDate = DateTime.Now;
            _album.Update(abInfo);
            return retval;
        }

        public bool IsReusable
        {
            get
            {
                return false;
            }
        }
    }
}