﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Mvc;
using System.Web.Script.Serialization;
using System.Web.Security;
using System.Web.SessionState;
using Social.Entity.TikMusic;
using Social.Lib;
using Social.Lib.Music;
using Social.Lib.Photo;
using Social.Service.Interface.TikActivity;
using Social.Service.Interface.TikMusic;
using HundredMilesSoftware.UltraID3Lib;

namespace Social.Web.Service
{
    /// <summary>
    /// Summary description for SongService
    /// </summary>
    public class SongService : IHttpHandler, IReadOnlySessionState
    {
        private readonly ISongService _song = DependencyResolver.Current.GetService<ISongService>();
        private readonly ISongQueueService _songQueue = DependencyResolver.Current.GetService<ISongQueueService>();
        private readonly IAlbumService _album = DependencyResolver.Current.GetService<IAlbumService>();
        private readonly IUserActivityService _useractivity = DependencyResolver.Current.GetService<IUserActivityService>();
        public void ProcessRequest(HttpContext context)
        { 
            var result = "";
            var type = context.Request.QueryString["type"];
            if (string.IsNullOrEmpty(type))
            {
                type = context.Request.Form["type"];

            }
            switch (type)
            {
                case "UploadQueue":
                    result = UploadQueue(context);
                    break;
                case "CreateAlbum":
                    result = CreateAlbum(context);
                    break;
                case "AddSong":
                    result = AddSong(context);
                    break;
                case "CancelAddSong":
                    result = CancelAddSong(context);
                    break;
                case "UpdateAlbum":
                    result = UpdateAlbum(context);
                    break;
                case "RemoveAlbum":
                    result = RemoveAlbum(context);
                    break;
                case "RemoveSong":
                    result = RemoveSong(context);
                    break;
                case "LoadMoreAlbum":
                    result = LoadMoreAlbum(context);
                    break;
                case "LoadMoreSong":
                    result = LoadMoreSong(context);
                    break;
                default:
                    result = "";
                    break;
            }
            context.Response.ContentType = "text/plain";
            context.Response.Write(result);
        }

        private string CreateAlbum(HttpContext context)
        {
            var retval = "({{Album: {0}, retVal: {1}}})";
            var userId = SessionManager.CurrentUserID();
            if (userId == Guid.Empty) return string.Format(retval, "{}", "-1");//Bạn chưa đăng nhập
            var abName =  Base64String.Decode(context.Request.Form["abName"]);  
            var singer = Base64String.Decode(context.Request.Form["singer"]);
            var catID = Base64String.Decode(context.Request.Form["catID"]);
            var _catID = Guid.Empty;
            try{_catID = Guid.Parse(catID);}catch{}
            var catName = Base64String.Decode(context.Request.Form["catName"]);
            var desc = Base64String.Decode(context.Request.Form["desc"]);
            var privacy = Convert.ToByte(context.Request.Form["privacy"]);

            #region UploadImage
            var htc = context.Request.Files;
            var _pathSaveDB = "";
            if (htc.Count > 0)
            {
                var posted = htc[0];
                if(posted.ContentLength > 0)
                {
                    var filename = DateTime.Now.Ticks.ToString();
                    var pathCreate = "";
                    var pathSave = "";

                    pathCreate = DateTime.Now.ToString("yyyyMM") + "/" + DateTime.Now.ToString("ddHH") + "/";
                    pathSave = ConfigManager.PathResourcePhoto + pathCreate;
                    string WxH = "0x0";
                    lock (posted)
                    {
                        try
                        {
                            #region UploadImage

                            Stream stream = posted.InputStream;
                            int read;
                            var ms = new MemoryStream();
                            var buf = new byte[1024];
                            while ((read = stream.Read(buf, 0, buf.Length)) > 0)
                            {
                                ms.Write(buf, 0, read);
                            }
                            ms.Seek(0, SeekOrigin.Begin);

                            if (ms.Length > 0)
                            {
                                var imagefullsize = System.Drawing.Image.FromStream(ms);
                                var oldW = imagefullsize.Width;
                                var oldH = imagefullsize.Height;

                                int thumMainW;
                                int thumMainH;
                                if (oldW > Lib.Photo.Constants.ImageInfo.SizeThumMainW)
                                {
                                    double dblCoef = Lib.Photo.Constants.ImageInfo.SizeThumMainW / (double)oldW;
                                    thumMainW = Convert.ToInt32(dblCoef * oldW);
                                    thumMainH = Convert.ToInt32(dblCoef * oldH);
                                }
                                else
                                {
                                    thumMainW = oldW;
                                    thumMainH = oldH;
                                }
                                if (thumMainH > 600)
                                {
                                    double dblCoef = 600 / (double)thumMainH;
                                    thumMainW = Convert.ToInt32(dblCoef * thumMainW);
                                    thumMainH = Convert.ToInt32(dblCoef * thumMainH);
                                }

                                int thumZoomW;
                                int thumZoomH;
                                if (oldW > Lib.Photo.Constants.ImageInfo.SizeThumbZoomW)
                                {
                                    double dblCoef = Lib.Photo.Constants.ImageInfo.SizeThumbZoomW / (double)oldW;
                                    thumZoomW = Convert.ToInt32(dblCoef * oldW);
                                    thumZoomH = Convert.ToInt32(dblCoef * oldH);
                                }
                                else
                                {
                                    thumZoomW = oldW;
                                    thumZoomH = oldH;
                                }
                                WxH = thumZoomW + "x" + thumZoomH;
                                ms.Seek(0, SeekOrigin.Begin);

                                var data = ms.GetBuffer();
                                var w = new[] { oldW, thumZoomW, thumMainW, Lib.Photo.Constants.ImageInfo.SizeThumbW, Lib.Photo.Constants.ImageInfo.SizeThumbAlbumSlideW };
                                var h = new[] { oldH, thumZoomH, thumMainH, Lib.Photo.Constants.ImageInfo.SizeThumbH, Lib.Photo.Constants.ImageInfo.SizeThumbAlbumSlideH };
                                var nameFiles = new[]
                                           {
                                               filename + Lib.Photo.Constants.ImageInfo.NameOriginal,
                                               filename + Lib.Photo.Constants.ImageInfo.NameThumbZoom,
                                               filename + Lib.Photo.Constants.ImageInfo.NameThumMain,
                                               filename + Lib.Photo.Constants.ImageInfo.NameThumb,
                                               filename + Lib.Photo.Constants.ImageInfo.NameThumbAlbumSline
                                           };
                                var crop = new[] { false, false, false, true, true };
                                try
                                {
                                    if (Directory.Exists(pathSave) == false)
                                    {
                                        Directory.CreateDirectory(pathSave);
                                    }
                                }
                                catch { }

                                var quality = ResizeImage.Quality(posted.ContentLength);

                                var multi = new MultiUpload[5];
                                for (var j = 0; j < 5; j++)
                                {
                                    multi[j] = new MultiUpload(data, pathSave + nameFiles[j], w[j], h[j], quality[j], crop[j]);
                                    multi[j].Start();
                                }

                                for (var j = 0; j < 5; j++)
                                {
                                    multi[j].Join();
                                }
                            }
                            #endregion

                            _pathSaveDB = pathCreate + filename;
                            var info = new Entity.TikPhoto.Photo();
                            info.ID = Guid.NewGuid();
                            info.Caption = "";
                            var _albumPhoto = DependencyResolver.Current.GetService<Social.Service.Interface.TikPhoto.IAlbumService>();
                            var _photo = DependencyResolver.Current.GetService<Social.Service.Interface.TikPhoto.IPhotoService>();
                            var abRv = _albumPhoto.GetByUserIdTypeAlbum(userId, 5);
                            Entity.TikPhoto.Album abMusic;
                            abMusic = abRv.Count <= 0 ? null : abRv[0];
                            if (abMusic != null)
                            {
                                info.AlbumID = abMusic.ID;
                                if (string.IsNullOrEmpty(abMusic.SourceUrlCoverPhoto))
                                {
                                    abMusic.CoverPhotoID = info.ID;
                                    abMusic.SourceUrlCoverPhoto = _pathSaveDB;
                                }
                                abMusic.PhotoCount = abMusic.PhotoCount + 1;
                                _albumPhoto.Update(abMusic);
                            }
                            else
                            {
                                var abW = new Entity.TikPhoto.Album
                                              {
                                                  AlbumName = "Ảnh album nhạc",
                                                  CoverPhotoID = info.ID,
                                                  SourceUrlCoverPhoto = _pathSaveDB,
                                                  ID = Guid.NewGuid(),
                                                  UserID = userId,
                                                  Description = "Ảnh album nhạc",
                                                  Location = "",
                                                  PrivacyType = 1,
                                                  Type = 5,
                                                  PhotoCount = 1,
                                                  CreatedDate = DateTime.Now,
                                                  ModifiedDate = DateTime.Now
                                              };
                                _albumPhoto.Add(abW);
                                info.AlbumID = abW.ID;
                            }
                            info.SourceUrl = _pathSaveDB;
                            info.UserID = userId;
                            info.WxH = WxH;
                            info.CreatedDate = DateTime.Now;
                            info.ModifiedDate = DateTime.Now;
                            _photo.Add(info);
                        }
                        catch { return string.Format(retval, "{}", "-2"); }//Không thành công
                    }
                }
            }
            #endregion

            var abInfo=new Album();
            abInfo.ID = Guid.NewGuid();
            abInfo.UserID = userId;
            abInfo.AlbumName = abName;
            abInfo.Description = desc;
            abInfo.Singer = singer;
            abInfo.CategoryID = _catID;
            abInfo.CategoryName = catName;
            abInfo.ImageURL = _pathSaveDB;
            abInfo.LikeCount = 0;
            abInfo.ViewCount = 1;
            abInfo.CommentCount = 0;
            abInfo.PrivacyType = privacy;
            abInfo.CreatedDate = DateTime.Now;
            abInfo.ModifiedDate = DateTime.Now;
            _album.Add(abInfo);
            var oSerializer = new JavaScriptSerializer();
            var sJSON = oSerializer.Serialize(abInfo); 
            return string.Format(retval, sJSON, "1");
        }
        
        private string UploadQueue(HttpContext context)
        {
            var retval = "({{SongInfo: {0}, Return: {1}}})";
            var userId = SessionManager.CurrentUserID();
            if (userId==Guid.Empty) return string.Format(retval, "{}", "-1");//Bạn chưa đăng nhập
            var htc = context.Request.Files;
            var posted = htc[0];
            if (htc.Count == 0)
            {
                return string.Format(retval, "{}", "-2");//Không tồn tại file
            }
            if (posted.ContentLength <= 0)
            {
                return string.Format(retval, "{}", "-3"); //Chưa chọn file
            }
            var ext = Path.GetExtension(posted.FileName).ToLower();
            var bytes = new Byte[posted.ContentLength];
            posted.InputStream.Read(bytes, 0, posted.ContentLength);
            if (ext.ToLower() != ".mp3")
            {
                return string.Format(retval, "{}", "-4"); //Không đúng định dạng
            }

            if (bytes.Length > 0)
            {
                if (bytes.Length > 15728640)
                {
                    return string.Format(retval, "{}", "-5"); //File lớn hơn 15MB
                }
            }
            else
            {
                return string.Format(retval, "{}", "-3"); //Chưa chọn file
            }

            lock (posted)
            {
                try
                {
                    var urlFile = UploadMusicUtility.UploadFileMp3(bytes, ext.ToLower());
                    if (string.IsNullOrEmpty(urlFile))
                    {
                        return string.Format(retval, "{}", "-7"); //Chưa up được file
                    }

                    var mp3File = new UltraID3();
                    mp3File.Read(ConfigManager.PathResourceMusic + urlFile + ext);
                    var _bitRate = mp3File.FirstMPEGFrameInfo.Bitrate;
                    var _duration = mp3File.Duration.TotalSeconds;
                    //var drtH = mp3File.Duration.Hours.ToString();
                    //var drtM = mp3File.Duration.Minutes.ToString();
                    //var drtS = mp3File.Duration.Seconds.ToString();
                    //if(mp3File.Duration.Hours>0)
                    //    _duration = drtH.Length == 1 ? ("0" + drtH) : drtH;
                    //if (_duration != "") _duration = _duration + ":" + (drtM.Length == 1 ? "0" + drtM : drtM);
                    //else _duration = _duration + (drtM.Length == 1 ? "0" + drtM : drtM);
                    //_duration = _duration + ":" + (drtS.Length == 1 ? "0" + drtS : drtS);
                        
                    var info = new SongQueue();
                    info.ID = Guid.NewGuid();
                    info.SongName = GetSongTitle(mp3File, Regex.Replace(posted.FileName, ".mp3", "", RegexOptions.IgnoreCase));
                    info.SourceUrl = urlFile;
                    info.UserID = userId;
                    info.BitRate = _bitRate+ "kb/s";
                    info.FileSize = bytes.Length.ToString();
                    info.Duration = _duration;
                    _songQueue.Add(info);
                    var oSerializer = new JavaScriptSerializer();
                    var sJSON = oSerializer.Serialize(info);
                    return string.Format(retval, sJSON, "1");//Thành công
                }
                catch { return string.Format(retval, "{}", "0"); }//Không thành công
            }
        }

        private string LoadMoreAlbum(HttpContext context)
        {
            var retval = "({{lstAlbum: {0}, Return: '{1}'}})";
            var uid = SessionManager.CurrentUserID();
            if (uid==Guid.Empty) return string.Format(retval, "[{}]", "-1");//Bạn chưa đăng nhập
            var pager = 0; try { pager = Convert.ToInt32(context.Request.Form["pager"]); }
            catch { }
            var currU = Guid.Empty; try { currU =Guid.Parse(context.Request.Form["uid"]); }
            catch { }
            var totalRow = 0;
            var lstAlbum = new List<Album>();
            if(currU!=Guid.Empty)
            {
                var isMyMusic = false;
                if (uid == currU) isMyMusic = true;
                //if(pager==1)
                //{
                //    lstSong = _album.GetPageListAlbumByUserId(currU, pager, 9, out totalRow, true);
                //    var tmp = 1;
                //    while (lstSong.Count > 0 && tmp<6)
                //    {
                //        lstSong.Remove(lstSong[0]);
                //        tmp = tmp + 1;
                //    }
                //}
                //else
                //{
                lstAlbum = _album.GetPageListAlbumByUserId(currU, pager, 9, out totalRow, isMyMusic);
                //}
            }
            foreach (var album in lstAlbum)
            {
                album.ImageURL = Lib.Utility.Utility.UrlThumbAlbumOrPhoto(album.ImageURL);
            }
            var oSerializer = new JavaScriptSerializer();
            var sJSON = oSerializer.Serialize(lstAlbum);
            return string.Format(retval, sJSON, "1");//Thành công
        }

        private string LoadMoreSong(HttpContext context)
        {
            var retval = "({{lstSong: {0}, Return: '{1}'}})";
            var uid = ""; try { uid = context.Session["UserID"].ToString(); }
            catch { }
            if (string.IsNullOrEmpty(uid)) return string.Format(retval, "[{}]", "-1");//Bạn chưa đăng nhập
            var pager = 0; try { pager = Convert.ToInt32(context.Request.Form["pager"]); }
            catch { }
            var currU = Guid.Empty; try { currU = Guid.Parse(context.Request.Form["uid"]); }
            catch { }
            var totalRow = 0;
            var lstSong = new List<Song>();
            if (currU != Guid.Empty)
            {
                //if(pager==1)
                //{
                //    lstSong = _album.GetPageListAlbumByUserId(currU, pager, 9, out totalRow, true);
                //    var tmp = 1;
                //    while (lstSong.Count > 0 && tmp<6)
                //    {
                //        lstSong.Remove(lstSong[0]);
                //        tmp = tmp + 1;
                //    }
                //}
                //else
                //{
                lstSong = _song.GetPageListSongByUserId(currU, pager, 10, out totalRow);
                //}
            }
            var oSerializer = new JavaScriptSerializer();
            var sJSON = oSerializer.Serialize(lstSong);
            return string.Format(retval, sJSON, "1");//Thành công
        }

        private string AddSong(HttpContext context)
        {
            var retval = "1";
            var _userID = SessionManager.CurrentUserID();
            if (_userID == Guid.Empty) return "-1";//Bạn chưa đăng nhập
            var ID = ""; try { ID = Base64String.Decode(context.Request.Form["ID"]); }
            catch { }
            if (string.IsNullOrEmpty(ID)) return "-2";//Lỗi Upload
            var _iD = Guid.Parse(ID);
            var songQueueInfo = _songQueue.Single(s => s.ID == _iD);
            if (songQueueInfo == null || songQueueInfo.UserID != _userID) return "-2";//Lỗi Upload
            var CategoryID = ""; try { CategoryID = Base64String.Decode(context.Request.Form["CategoryID"]); }
            catch { }
            var _categoryID = Guid.Empty;
            try{_categoryID = Guid.Parse(CategoryID);}catch { }  
            var CategoryName = ""; try { CategoryName = Base64String.Decode(context.Request.Form["CategoryName"]); }
            catch { }
            var AlbumID = ""; try { AlbumID = Base64String.Decode(context.Request.Form["AlbumID"]); }
            catch { }
            var _albumID = Guid.Empty;
            try{_albumID = Guid.Parse(AlbumID);}
            catch{}
            if (_albumID != Guid.Empty)
            {
                var abInfo = _album.Single(ab => ab.ID == _albumID);
                if (abInfo.UserID != _userID) return "-3"; //Album ko phải của ấy
                abInfo.SongCount = abInfo.SongCount+1;
                _album.Update(abInfo);
            }
            var SourceUrl = ""; try { SourceUrl = Base64String.Decode(context.Request.Form["SourceUrl"]); }
            catch { }
            var BitRate = ""; try { BitRate = Base64String.Decode(context.Request.Form["BitRate"]); }
            catch { }
            var FileSize = ""; try { FileSize = Base64String.Decode(context.Request.Form["FileSize"]); }
            catch { }
            var Duration = ""; try { Duration = Base64String.Decode(context.Request.Form["Duration"]); }
            catch { }
            var _duration = Convert.ToDouble(Duration);
            var Lyric = ""; try { Lyric = Base64String.Decode(context.Request.Form["Lyric"]); }
            catch { }
            var Singer = ""; try { Singer = Base64String.Decode(context.Request.Form["Singer"]); }
            catch { }
            var SongName = ""; try { SongName = Base64String.Decode(context.Request.Form["SongName"]); }
            catch { }
            var Writer = ""; try { Writer = Base64String.Decode(context.Request.Form["Writer"]); }
            catch { }

            var songInfo = new Song();
            songInfo.ID = _iD;
            songInfo.SongName = SongName;
            songInfo.CategoryID = _categoryID;
            songInfo.CategoryName = CategoryName;
            songInfo.SourceUrl = SourceUrl;
            songInfo.UserID = _userID;
            songInfo.AlbumID = _albumID;
            songInfo.Writer = Writer;
            songInfo.BitRate = BitRate;
            songInfo.Duration = _duration;
            songInfo.FileSize = FileSize;
            songInfo.Singer = Singer;
            songInfo.Lyric = Lyric;
            songInfo.Description = ""; 
            songInfo.CreatedDate = DateTime.Now;
            songInfo.ModifiedDate = DateTime.Now;
            _song.Add(songInfo);
            _songQueue.Delete(s=>s.ID==_iD);

            var urlDetail = "/" + SessionManager.CurrentUserUserName() + "/Music/SongDetail/" + _iD;
            var urlImageThumb =string.Empty;
            var urlFile = ConfigManager.UrlDataMusic + SourceUrl + ".mp3";
            var urlXmlSong = ConfigManager.UrlXmlSongMusic + "?urlFile=" +Base64String.Encode(urlFile) + 
                                    "&songName=" + Base64String.Encode(SongName) + 
                                    "&singer=" + Base64String.Encode(Singer) + 
                                    "&duration=" + _duration;
            var embed = MusicUtil.GeneratePlayerSongToWall(urlXmlSong, urlDetail, true);
            var desC = "Ca sỹ: " + (string.IsNullOrEmpty(songInfo.Singer) ? "Chưa có thông tin" : songInfo.Singer) + "<br />" + "Nhạc sỹ: " + (string.IsNullOrEmpty(songInfo.Writer) ? "Chưa có thông tin" : songInfo.Writer) + "<br />" + "Thể loại: " + songInfo.CategoryName + "<br />" + songInfo.BitRate;
            _useractivity.PostMusic(_userID, _iD, string.Empty, songInfo.SongName, desC, urlDetail,
                                    urlImageThumb, embed, 390, 30);

            return retval;
        }

        private string CancelAddSong(HttpContext context)
        {
            var retVal = "1";
            var UserID = ""; try { UserID = context.Session["UserID"].ToString(); }
            catch { }
            if (string.IsNullOrEmpty(UserID)) return "-1";//Chưa đăng nhập
            var _userID = Guid.Parse(UserID);

            var ID = ""; try { ID = context.Request.Form["ID"]; }
            catch { }
            if (string.IsNullOrEmpty(ID)) return "-2";//Ko tồn tại ID
            var _iD = Guid.Empty;
            try{_iD = Guid.Parse(ID);}
            catch { }
            if (_iD == Guid.Empty) return "-2";//Ko tồn tại ID
            var songQueueInfo = _songQueue.Single(s => s.ID == _iD);
            if (songQueueInfo != null && songQueueInfo.UserID == _userID)
            {
                Utility.DeleteFile(ConfigManager.PathResourceMusic + songQueueInfo.SourceUrl + ".mp3");
                _songQueue.Delete(s => s.ID == _iD);
            }
            return retVal;
        }
         
        private string UpdateAlbum(HttpContext context)
        {
            var retval = "1";
            var uid = ""; try { uid = context.Session["UserID"].ToString(); }
            catch { }
            if (string.IsNullOrEmpty(uid)) return "-1";//Chua dang nhap
            var abName = Base64String.Decode(context.Request.Form["abName"]);
            var singer = Base64String.Decode(context.Request.Form["singer"]);
            var catID = Base64String.Decode(context.Request.Form["catID"]);
            var _catID = Guid.Empty;
            try { _catID = Guid.Parse(catID); }
            catch { }
            var catName = Base64String.Decode(context.Request.Form["catName"]);
            var desc = Base64String.Decode(context.Request.Form["desc"]);
            var privacy = Convert.ToByte(context.Request.Form["privacy"]);
            var albumId = ""; try { albumId = context.Request.Form["aId"]; }
            catch { }
            if (string.IsNullOrEmpty(albumId)) return "-2";// Album ko tồn tại
            Album abInfo;
            var _abId_ = Guid.Parse(albumId);
            try { abInfo = _album.Single(ab => ab.ID == _abId_); }
            catch { abInfo = null; }
            if (abInfo == null) return "-3";// Album ko tồn tại
            var _uid = Guid.Parse(uid);
            if (_uid != abInfo.UserID) return "-4"; // Album ko phải của user hiện tại

            #region UploadImage
            var htc = context.Request.Files;
            var _pathSaveDB = "";
            if (htc.Count > 0)
            {
                var posted = htc[0];
                if (posted.ContentLength > 0)
                {
                    var filename = DateTime.Now.Ticks.ToString();
                    var pathCreate = "";
                    var pathSave = "";

                    pathCreate = DateTime.Now.ToString("yyyyMM") + "/" + DateTime.Now.ToString("ddHH") + "/";
                    pathSave = ConfigManager.PathResourcePhoto + pathCreate;
                    string WxH = "0x0";
                    lock (posted)
                    {
                        try
                        {
                            #region UploadImage
                            Stream stream = posted.InputStream;
                            int read;
                            var ms = new MemoryStream();
                            var buf = new byte[1024];
                            while ((read = stream.Read(buf, 0, buf.Length)) > 0)
                            {
                                ms.Write(buf, 0, read);
                            }
                            ms.Seek(0, SeekOrigin.Begin);

                            if (ms.Length > 0)
                            {
                                var imagefullsize = System.Drawing.Image.FromStream(ms);
                                var oldW = imagefullsize.Width;
                                var oldH = imagefullsize.Height;

                                int thumMainW;
                                int thumMainH;
                                if (oldW > Lib.Photo.Constants.ImageInfo.SizeThumMainW)
                                {
                                    double dblCoef = Lib.Photo.Constants.ImageInfo.SizeThumMainW / (double)oldW;
                                    thumMainW = Convert.ToInt32(dblCoef * oldW);
                                    thumMainH = Convert.ToInt32(dblCoef * oldH);
                                }
                                else
                                {
                                    thumMainW = oldW;
                                    thumMainH = oldH;
                                }
                                if (thumMainH > 600)
                                {
                                    double dblCoef = 600 / (double)thumMainH;
                                    thumMainW = Convert.ToInt32(dblCoef * thumMainW);
                                    thumMainH = Convert.ToInt32(dblCoef * thumMainH);
                                }

                                int thumZoomW;
                                int thumZoomH;
                                if (oldW > Lib.Photo.Constants.ImageInfo.SizeThumbZoomW)
                                {
                                    double dblCoef = Lib.Photo.Constants.ImageInfo.SizeThumbZoomW / (double)oldW;
                                    thumZoomW = Convert.ToInt32(dblCoef * oldW);
                                    thumZoomH = Convert.ToInt32(dblCoef * oldH);
                                }
                                else
                                {
                                    thumZoomW = oldW;
                                    thumZoomH = oldH;
                                }
                                WxH = thumZoomW + "x" + thumZoomH;
                                ms.Seek(0, SeekOrigin.Begin);

                                var data = ms.GetBuffer();
                                var w = new[] { oldW, thumZoomW, thumMainW, Lib.Photo.Constants.ImageInfo.SizeThumbW, Lib.Photo.Constants.ImageInfo.SizeThumbAlbumSlideW };
                                var h = new[] { oldH, thumZoomH, thumMainH, Lib.Photo.Constants.ImageInfo.SizeThumbH, Lib.Photo.Constants.ImageInfo.SizeThumbAlbumSlideH };
                                var nameFiles = new[]
                                           {
                                               filename + Lib.Photo.Constants.ImageInfo.NameOriginal,
                                               filename + Lib.Photo.Constants.ImageInfo.NameThumbZoom,
                                               filename + Lib.Photo.Constants.ImageInfo.NameThumMain,
                                               filename + Lib.Photo.Constants.ImageInfo.NameThumb,
                                               filename + Lib.Photo.Constants.ImageInfo.NameThumbAlbumSline
                                           };
                                var crop = new[] { false, false, false, true, true };
                                try
                                {
                                    if (Directory.Exists(pathSave) == false)
                                    {
                                        Directory.CreateDirectory(pathSave);
                                    }
                                }
                                catch { }

                                var quality = ResizeImage.Quality(posted.ContentLength);

                                var multi = new MultiUpload[5];
                                for (var j = 0; j < 5; j++)
                                {
                                    multi[j] = new MultiUpload(data, pathSave + nameFiles[j], w[j], h[j], quality[j], crop[j]);
                                    multi[j].Start();
                                }

                                for (var j = 0; j < 5; j++)
                                {
                                    multi[j].Join();
                                }
                            }
                            #endregion

                            _pathSaveDB = pathCreate + filename;
                            var info = new Entity.TikPhoto.Photo();
                            info.ID = Guid.NewGuid();
                            info.Caption = "";
                            var _albumPhoto = DependencyResolver.Current.GetService<Social.Service.Interface.TikPhoto.IAlbumService>();
                            var _photo = DependencyResolver.Current.GetService<Social.Service.Interface.TikPhoto.IPhotoService>();
                            var abRv = _albumPhoto.GetByUserIdTypeAlbum(_uid, 5);
                            Entity.TikPhoto.Album abMusic;
                            abMusic = abRv.Count <= 0 ? null : abRv[0];
                            if (abMusic != null)
                            {
                                info.AlbumID = abMusic.ID;
                                if (string.IsNullOrEmpty(abMusic.SourceUrlCoverPhoto))
                                {
                                    abMusic.CoverPhotoID = info.ID;
                                    abMusic.SourceUrlCoverPhoto = _pathSaveDB;
                                }
                                abMusic.PhotoCount = abMusic.PhotoCount + 1;
                                _albumPhoto.Update(abMusic);
                            }
                            else
                            {
                                var abW = new Entity.TikPhoto.Album
                                {
                                    AlbumName = "Ảnh album nhạc",
                                    CoverPhotoID = info.ID,
                                    SourceUrlCoverPhoto = _pathSaveDB,
                                    ID = Guid.NewGuid(),
                                    UserID = _uid,
                                    Description = "Ảnh album nhạc",
                                    Location = "",
                                    PrivacyType = 1,
                                    Type = 5,
                                    PhotoCount = 1,
                                    CreatedDate = DateTime.Now,
                                    ModifiedDate = DateTime.Now
                                };
                                _albumPhoto.Add(abW);
                                info.AlbumID = abW.ID;
                            }
                            info.SourceUrl = _pathSaveDB;
                            info.UserID = _uid;
                            info.WxH = WxH;
                            info.CreatedDate = DateTime.Now;
                            info.ModifiedDate = DateTime.Now;
                            _photo.Add(info);
                        }
                        catch { return string.Format(retval, "{}", "-2"); }//Không thành công
                    }
                }
            }
            #endregion

            abInfo.UserID = _uid;
            abInfo.AlbumName = abName;
            abInfo.Description = desc;
            abInfo.Singer = singer;
            abInfo.CategoryID = _catID;
            abInfo.CategoryName = catName;
            if (!string.IsNullOrEmpty(_pathSaveDB)) abInfo.ImageURL = _pathSaveDB;
            abInfo.PrivacyType = privacy;
            abInfo.ModifiedDate = DateTime.Now;
            _album.Update(abInfo);
            return retval;
        }

        private string RemoveAlbum(HttpContext context)
        {
            var retval = "1";
            var uid = ""; try { uid = context.Session["UserID"].ToString(); }
            catch { }
            if (string.IsNullOrEmpty(uid)) return "-1";//Chua dang nhap
            var albumId = ""; try { albumId = context.Request.Form["aId"]; }
            catch { }
            if (string.IsNullOrEmpty(albumId)) return "-2";// Album ko tồn tại
            Album abInfo;
            var _abId_ = Guid.Parse(albumId);
            try { abInfo = _album.Single(ab => ab.ID == _abId_); }
            catch { abInfo = null; }
            if (abInfo == null) return "-3";// Album ko tồn tại
            var _uid = Guid.Parse(uid);
            if (_uid != abInfo.UserID) return "-4"; // Album ko phải của user hiện tại
            //delete All Song in image?
            var lstSong = _song.GetByAlbumId(_abId_);
            if(lstSong.Count>0)
            {
                foreach (var song in lstSong)
                {
                    Utility.DeleteFile(ConfigManager.PathResourceMusic + song.SourceUrl + ".mp3");
                }
            }
            Utility.DeleteFile(ConfigManager.PathResourcePhoto + abInfo.ImageURL + Lib.Photo.Constants.ImageInfo.NameOriginal);
            Utility.DeleteFile(ConfigManager.PathResourcePhoto + abInfo.ImageURL + Lib.Photo.Constants.ImageInfo.NameThumbZoom);
            Utility.DeleteFile(ConfigManager.PathResourcePhoto + abInfo.ImageURL + Lib.Photo.Constants.ImageInfo.NameThumMain);
            Utility.DeleteFile(ConfigManager.PathResourcePhoto + abInfo.ImageURL + Lib.Photo.Constants.ImageInfo.NameThumb);
            Utility.DeleteFile(ConfigManager.PathResourcePhoto + abInfo.ImageURL + Lib.Photo.Constants.ImageInfo.NameThumbAlbumSline);
            _album.Delete(abInfo);
            return retval;
        }

        private string RemoveSong(HttpContext context)
        {
            var retval = "1";
            var uid = ""; try { uid = context.Session["UserID"].ToString(); }
            catch { }
            if (string.IsNullOrEmpty(uid)) return "-1";//Chua dang nhap
            var sId = ""; try { sId = context.Request.Form["sId"]; }
            catch { }
            if (string.IsNullOrEmpty(sId)) return "-2";// Song ko tồn tại
            Song sInfo;
            var _sId_ = Guid.Parse(sId);
            try { sInfo = _song.Single(s => s.ID == _sId_); }
            catch { sInfo = null; }
            if (sInfo == null) return "-3";// Song ko tồn tại
            var _uid = Guid.Parse(uid);
            if (_uid != sInfo.UserID) return "-4"; // Song ko phải của user hiện tại
            var abId = sInfo.AlbumID;
            Utility.DeleteFile(ConfigManager.PathResourceMusic + sInfo.SourceUrl + ".mp3");
            _song.Delete(sInfo);
            if (abId != Guid.Empty)
            {
                var albumInfo = _album.Single(a => a.ID == sInfo.AlbumID);
                albumInfo.SongCount = albumInfo.SongCount - 1;
                _album.Update(albumInfo);
            }
            return retval;
        }

        private string GetSongTitle(UltraID3 mp3File, string defaultName)
        {
            string title = "";
            if (mp3File.Title.Trim() != "")
                title = mp3File.Title;
            else if (mp3File.ID3v1Tag != null && !string.IsNullOrEmpty(mp3File.ID3v1Tag.Title))
                title = mp3File.ID3v1Tag.Title;
            else if (mp3File.ID3v2Tag != null && !string.IsNullOrEmpty(mp3File.ID3v2Tag.Title))
                title = mp3File.ID3v2Tag.Title;
            else
                title = defaultName;
            //title = Common.RemoveBadKeywordFromSongProperties(title);
            return title;
        }

        public bool IsReusable
        {
            get
            {
                return false;
            }
        }
    }
}